/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <cstdlib>
#include <vector>
#include <iterator>
#include <iostream>
#include <string>
#include <typeinfo>

#include "../update_I.h"
#include "../core_I.h"
#include "trans.h"
#include "state.h"
#include "hypertrans.h"
#include "link.h"
#include "fsm.h"
#include "core.h"


using namespace std;


Core::Core() {
  fsmpointer = fv.end();
  numberOfFsms = 0;
  name = "Projekt";
  this->currentFSM = NULL;
}
void Core::projectName(string pname) {
  name = pname;
}

string Core::getProjectName() {
  return name;
}

bool Core::getFirstFsm() {
  if(fv.empty()) {
    return false;
  } else {
    fsmpointer = fv.begin();
    return true;
  }
}

bool Core::getNextFsm() {
  if(fsmpointer == fv.end()) {
    return false;
   } else {
    fsmpointer++;
    if(fsmpointer == fv.end()) {
      return false;
    } else {
      return true;
    }
   }
}

bool Core::setCurrentFsm(int id) {

    vector <Fsm>::iterator iter;
    if (id >=0)
    {
        for(iter = fv.begin(); iter!=fv.end(); iter++) {
            if(id == iter->getFsmID())
            {
                fsmpointer = iter;
                this->currentFSM = iter.base();
                return true;
            }
            else
            {
            	this->currentFSM = NULL;
                return false;
            }
        }
    }
    else
    {
        return false;
    }

}

Fsm_I& Core::getCurrentFsm() {
	return (*(fsmpointer));
}

Fsm_I& Core::getFsm(int i) {
  
  vector <Fsm>::iterator iter;

  for(iter = fv.begin(); iter!=fv.end(); iter++) {
    if(i == iter->getFsmID()) {
      fsmpointer = iter;
      return (*iter);
    }
  }
  fsmpointer = fv.begin();
}

bool Core::isFsm() {

	// Prepare result
	bool result = !fv.empty();

	if (result && (fsmpointer.base()==NULL || fsmpointer==fv.end())) {

		// If fsmpointer not set, then set it to first
		this->getFirstFsm();
	}


	return result;
}

void Core::addFsm() {
  fv.push_back(Fsm(numberOfFsms++));
  fsmpointer = fv.end();
  fsmpointer--;
}

void Core::deleteFsm(int id) {
   
  vector <Fsm>::iterator i;

  for(i=fv.begin(); i!=fv.end(); i++){
    if(i->getFsmID() == id) {

		// Erase the FSM
		fv.erase(i);
		// Set pointer to end
		fsmpointer = fv.end();

		return;
    }
  }
}

void Core::reset() {
  while(getFirstFsm()) {
    deleteFsm(getCurrentFsm().getFsmID());
  }
  numberOfFsms = 0;
}

/*

int main() {
  Core c;
  Xmlsave xs;
 

  xs.createdtd();
  c.addFsm();
  c.addFsm();
  c.addFsm();
  c.addFsm();
  c.addFsm();

  c.deleteFsm(2);

  vector <Fsm>::iterator i;



  for(i=c.fv.begin(); i!=c.fv.end(); i++){
    i->numberOfIns(8);
    i->numberOfOuts(5);
    i->addState();
    i->resetState(0);
    i->addState();
    i->addState();
    i->addTrans(0,1);
    i->addTrans(0,2);
    i->addTrans(1,2);
    i->addTrans(0,1);
    i->addTrans(2,1);
    i->addLink(1.1,1.1,2,0);
    i->addLink(1.1,1.1,27,0);
    i->addHypertrans(true,1.2,1.2,2,0);
  }
  
  cout << "CNAME: " << c.name <<endl;

  for(i=c.fv.begin(); i!=c.fv.end(); i++){
    cout << "FSM: " << i->getFsmName() << " outnr: " << i->getNumberOfOutputs() << " innr: " << i->getNumberOfInputs() <<endl;
    
    vector <State>::iterator iter;
    
    for(iter=i->sv.begin(); iter!=i->sv.end(); iter++){
      cout << "STATE: " << iter->sname << " SID: " << iter->sid <<endl; 
    }


  }  
for(i=c.fv.begin(); i!=c.fv.end(); i++){
    i->addState();
    i->addState();
    i->addState();
  }
  
  cout << "CNAME: " << c.name <<endl;

  for(i=c.fv.begin(); i!=c.fv.end(); i++){

    cout << "FSM: " << i->getFsmName() <<endl;
    if(i->getFsmID() == 0){
      i->deleteState(2);
    }
    i->modifyState(3,15);
    i->modifyState(0,"idle");
    i->stateLevel(4,1);  
    i->modifyState(0,2.3,4.78);
    if(i->getFsmID() == 1){
      //i->deleteState(5);
    }
    if(i->getFsmID()==0){
      //i->deleteTrans(1);i
    }
    if(i->getFsmID() == 1){
      //i->deleteTrans(3);
      //i->deleteTrans(4);
    }
  }

  

  for(i=c.fv.begin(); i!=c.fv.end(); i++){
    
    if(i->getFsmID() == 3){
      i->numberOfIns(3);
      i->numberOfOuts(3);

    }

    if(i->getFsmID() == 4){
      i->numberOfIns(10);
      i->numberOfOuts(10);
    }

    cout << "FSM: " << i->getFsmName() <<endl;

        vector <State>::iterator iter;


    for(iter=i->sv.begin(); iter!=i->sv.end(); iter++){

      if(iter->sid != 0) {
	i->modifyState(iter->sid,(iter->sid) - 1, '1');
      }

      cout << "STATE: " << iter->sname <<  " SID: " << iter->sid <<" outputvector: " ; 
      /*for (b = 0; b!=iter->output.size(); b++) {
	 cout << iter->output[b];
	 }*/
/*  cout <<  iter->output;
      cout << " Pos: " << "(" << iter->pos.first << "," << iter->pos.second << ")";
      cout << " Color: " << iter->color << " Level: " << iter->hlevel << endl;

    }   
  
    
    
    vector <Trans>::iterator it;


    for(it=i->tv.begin(); it!=i->tv.end(); it++){
      
      
      it->addCondition(i->getNumberOfInputs());
      
      
      if(it->tid == 3) {
	i->addConditionToTrans(3);
	i->addConditionToTrans(3);
	//it->addCondition(i->numberOfInputs);
	//it->addCondition(i->numberOfInputs);
	}
      if(it->tid == 3) {
	it->modifyCondition(1,3,1);
      }
      if((it->tid == 3) && (i->getFsmID() == 0)) {
	//it->deleteCondition(1);
	i->deleteConditionOfTrans(3,1);
      }
      if(it->tid == 3) {
	it->modifyCondition(1,3,1);
      }
      i->modifyTrans(1,true);
      i->modifyTrans(3,66);
      i->modifyTrans(4,0,"go");
      i->modifyTrans(0,0,2,'1');
      //i->modifyTrans(0,0,20,'1');
      i->addTrackpointToTrans(0, 0, false, 13.5, 7.7, 0);
      i->addTrackpointToTrans(0, 2006, false, 12, 2.3, 0);
      i->addTrackpointToTrans(0, 2006, false, 18.1, 19, 0);
      i->addTrackpointToTrans(0, 0, true, 3, 8, 0);
      if(i->getFsmID() == 4) {
	i->deleteTrackpointOfTrans(0,3);
      }
      i->addTrackpointToTrans(0, 1, true, 1.2, 1.3, 0);
      i->addTrackpointToTrans(0, 2, true, 33.3, 53.7, 0);
      i->addTrackpointToTrans(0, 2, false, 7, 4, 0);
      if(i->getFsmID() == 1) {
	i->deleteTrackpointOfTrans(0,5);
      }
      if(it->tid == 0) {
	//it->modifyCondition(0,2,1);
	i->modifyTrans(0,0,4,'X');
	it->modifyCondition(0,6,'1');
      }

    if(i->getFsmID() == 3){
      i->numberOfIns(3);
      //i->numberOfOuts(3);
    }

    if(i->getFsmID() == 4){
      i->numberOfIns(10);
      //i->numberOfOuts(10);
    }
      
    cout << "Trans (von,nach): (" << it->start << "," << it->end << ")" << " Color: "<< it->color << " TID: " << it->tid << " bitvector: "; 
   
      if(it->d) {
	
	cout << "default" ;
      } else {
      
	vector <Condition>::iterator ci;
      
	for(ci = it->cv.begin(); ci!=it->cv.end(); ci++) {

	  cout <<" ";
	
	  if(it->tid == 4) {
	    cout << ci->cname << ":";
	  }
	  
	  //for(vector <int>::size_type civ = 0; civ < ci->input.size(); civ++) {
	  
	  //  cout << ci->input[civ];

	  //}
	  cout << ci->input;
	  }
      }
      cout << endl;
      
      list <Trackpoint>::iterator tri;

      for(tri = it->tl.begin(); tri!=it->tl.end(); tri++) {
	cout << "(" << tri->pos.first << "," << tri->pos.second << ") ";
      }
      cout << endl;
    }
 

    
    
   vector <Link>::iterator li;

  for(li=i->lv.begin();li!=i->lv.end();li++) {
    if(i->getFsmID() == 4){
       i->modifyLink(0, 13);
       i->modifyLink(0, 0.815, 47.11);
       i->linkLevel(0, 1);
    }
    cout << "Link: " << li->lid << " Nach: " << li->goal << " Pos: (" << li->linkpos.first; 
    cout << "," << li->linkpos.second << ") Color: " << li->color << " Level: " << li->hlevel << endl;
    li = i->lv.end();
    li--;
    }

 
  vector <Hypertrans>::iterator hi;

  for(hi=i->htv.begin();hi!=i->htv.end();hi++) {

   	hi->addCondition(i->getNumberOfInputs());
	hi->addCondition(i->getNumberOfInputs());
	hi->addCondition(i->getNumberOfInputs());
	hi->addCondition(i->getNumberOfInputs());
	
	i->modifyHypertrans(0,0,5,'1');
	i->modifyHypertrans(0,0,6,'1');	
	//i->modifyHypertrans(0,0,20,'1');
	hi->modifyCondition(0,0,'1');    
	hi->modifyCondition(1,1,'1');
	hi->modifyCondition(2,3,'1');
	hi->modifyCondition(2,5,'1');
	hi->modifyCondition(3,5,'1');

	hi->deleteCondition(2);
	if(i->getFsmID() == 4){
	  i->modifyHypertrans(0, 17);
	  i->modifyHypertrans(0, 0.815, 47.11);
	  i->modifyHypertrans(0, false);
	  i->hypertransLevel(0, 1);
	}
	if(i->getFsmID() == 1){
	  i->modifyHypertrans(0, 13);
	  i->modifyHypertrans(0, 0.07, 0.0, 3);
	  i->modifyHypertrans(0, false);
	  i->hypertransLevel(0, 1);
	}
	
	cout << "Hypertrans: " << hi->hid << " nach: " << hi->goal << " Code(id bits): ";

	vector <Condition>::iterator ci;
       
	for(ci = hi->cv.begin(); ci!=hi->cv.end(); ci++) {

	  cout <<" "<< ci->cid <<" ";
	
	  /*for(vector <int>::size_type civ = 0; civ < ci->input.size(); civ++) {
	  
	  cout << ci->input[civ];

	  }*/
/*	  cout << ci->input;
     }


	cout << endl;
	cout << "Color: " << hi->color << " Pos: (" << hi->hpos.first << "," << hi->hpos.second;
	cout << ") all/rest (true/false): " << hi->c << " Level: " << hi->hlevel << endl;
	}
  }
  return 1;
}

*/


