/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <cstdlib>
#include <vector>
#include <iterator>
#include <iostream>
#include <sstream>
#include <string>

#include "../update_I.h"
#include "condition.h"
#include "state.h"
#include "trans.h"
#include "hypertrans.h"
#include "link.h"
#include "fsm.h"

using namespace std;


string Fsm::itos(int i) {
  stringstream s;
  s << i;
  return s.str();
}


Fsm::Fsm(int f) {
  fid = f;
  numberOfStates = 0;
  numberOfTrans = 0;
  numberOfHypertrans = 0;
  numberOfLinks = 0;
  numberOfParameters = 0;
  fname = "fsm_";
  fname = fname + itos(fid);
  numberOfInputs = 0;
  numberOfOutputs = 0;
  resetstate = 0;
  statepointer = sv.end();
  transpointer = tv.end();
  hypertranspointer = htv.end();
  linkpointer = lv.end();
  parameterpointer = parameter.end();
  globals.first = "clk";
  globals.second = "res_n";
}

Fsm& Fsm::operator=(const Fsm &fsm){
  this->fname = fsm.fname;
  this->fid = fsm.fid;
  this->numberOfStates = fsm.numberOfStates;
  this->numberOfTrans = fsm.numberOfTrans;
  this->numberOfHypertrans = fsm.numberOfHypertrans;
  this->numberOfLinks = fsm.numberOfLinks;
  this->numberOfParameters = fsm.numberOfParameters;
//   this->numberOfInputNames = fsm.numberOfInputNames;
//   this->numberOfOutputNames = fsm.numberOfOutputNames;
  this->numberOfInputs = fsm.numberOfInputs;
  this->numberOfOutputs = fsm.numberOfOutputs;
  this->statepointer = fsm.statepointer;
  this->transpointer = fsm.transpointer;
  this->hypertranspointer = fsm.hypertranspointer;
  this->linkpointer = fsm.linkpointer;
  this->parameterpointer = fsm.parameterpointer;
  this->sv = fsm.sv;
  this->tv = fsm.tv;
  this->htv = fsm.htv;
  this->lv = fsm.lv;
  this->parameter = fsm.parameter;
  this->inputnames = fsm.inputnames;
  this->outputnames = fsm.outputnames;
  this->views = fsm.views;
  this->resetstate = fsm.resetstate;
  return *this;
}

int Fsm::operator==(const Fsm &fsm) const{
  if(fid == fsm.fid) return 1;
  return 0;

}

int Fsm::operator<(const Fsm &fsm) const{
  if(fid < fsm.fid) return 0;
  return 1;

}

void Fsm::registerView(Update_I* u) {
	views.push_back(u);
}
 
void Fsm::signoffView(Update_I* u) {
	vector <Update_I*>::iterator i;
 for(i = views.begin(); i!=views.end(); i++) {
   if (*i==u) {
	views.erase(i);
	return;
   }
 }
}

void Fsm::update() {
  
  vector <Update_I*>::iterator i;

  for(i = views.begin(); i!=views.end(); i++) {
    (**i)();
  }

}

void Fsm::addInputName() {
  inputnames.push_back("input");
  update();
}

void Fsm::addOutputName() {
  outputnames.push_back("output"); 
  update();
}

void Fsm::deleteInputName() {
  if(numberOfInputs > 0) {
    inputnames.pop_back();
  }
  update();
}

void Fsm::deleteOutputName() {
  if(numberOfOutputs > 0) {
    outputnames.pop_back(); 
  }
  update();
}

void Fsm::inputName(int i, string name) {
  if(i < numberOfInputs) {
    inputnames[i] = name;
    update();
  }    
}

void Fsm::outputName(int i, string name) {
  if(i < numberOfOutputs) {
    outputnames[i] = name;
    update();
  }
}

void Fsm::deleteInput(int i) {
  if(i < numberOfInputs) {
    
    vector <Hypertrans>::iterator iter;
    vector <Trans>::iterator ii;
    vector <Condition>::iterator ci;
    
    for(ci=parameter.begin(); ci!=parameter.end(); ci++){

      string::iterator help;
      
      int countfordelete = 0;
      
      for(help = ci->input.begin(); help!=ci->input.end(); help++) {
	if(countfordelete == i) {
	  ci->input.erase(help);
	  help = ci->input.end();
	  help--;
	}
	countfordelete++;
      }
    }
      
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      for(ci=iter->cv.begin(); ci!=iter->cv.end(); ci++) {
	
	string::iterator help;
	
	int countfordelete = 0;
	
	for(help = ci->input.begin(); help!=ci->input.end(); help++) {
	  if(countfordelete == i) {
	    ci->input.erase(help);
	    help = ci->input.end();
	    help--;
	  }
	  countfordelete++;
	}
      }
    }
      
    for(ii=tv.begin(); ii!=tv.end(); ii++){
      for(ci=ii->cv.begin(); ci!=ii->cv.end(); ci++) {
	 	
	string::iterator help;
	
	int countfordelete = 0;
	
	for(help = ci->input.begin(); help!=ci->input.end(); help++) {
	  if(countfordelete == i) {
	    ci->input.erase(help);
	    help = ci->input.end();
	    help--;
	  }
	  countfordelete++;
	}	
      }
    }
    if(i < (numberOfInputs - 1) ) {
      for(int f = i;f < numberOfInputs - 1;f++) {
	inputName(f,getInputName((f+1)));
      }
    }
    deleteInputName();
    numberOfInputs--;
    update();
  }
}

void Fsm::deleteOutput(int i) {
  if(i < numberOfOutputs) {
    
    vector <State>::iterator iter;
    
    for(iter=sv.begin(); iter!=sv.end(); iter++){ 
      
      string::iterator help;
      
      int countfordelete = 0;
      
      for(help = iter->output.begin(); help!=iter->output.end(); help++) {
	if(countfordelete == i) {
	  iter->output.erase(help);
	  help = iter->output.end();
	  help--;
	}
	countfordelete++;
      }
    }
    if(i < (numberOfOutputs - 1) ) {
      for(int f = i;f < numberOfOutputs - 1;f++) {
	outputName(f,getOutputName((f+1)));
      }
    }
    deleteOutputName();
    numberOfOutputs--;
    update();
  }
}

void Fsm::moveupInput(int i) {
  if(i < numberOfInputs && i > 0) {
    
    vector <Hypertrans>::iterator iter;
    vector <Trans>::iterator ii;
    vector <Condition>::iterator ci;
    
    for(ci=parameter.begin(); ci!=parameter.end(); ci++){
      char hvar;
      
      hvar = ci->input[i - 1];
      ci->input[i - 1] = ci->input[i];
      ci->input[i] = hvar;
    }
    
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      for(ci=iter->cv.begin(); ci!=iter->cv.end(); ci++) {
	char hvar;
	
	hvar = ci->input[i - 1];
	ci->input[i - 1] = ci->input[i];
	ci->input[i] = hvar;
      }
    }
    
    for(ii=tv.begin(); ii!=tv.end(); ii++){
      for(ci=ii->cv.begin(); ci!=ii->cv.end(); ci++) {
	char hvar;
	
	hvar = ci->input[i - 1];
	ci->input[i - 1] = ci->input[i];
	ci->input[i] = hvar;
      }
    }
    
    string helpname;
    helpname = getInputName(i - 1);
    inputName(i - 1,getInputName(i));
    inputName(i,helpname);
    update();
  }  
}

void Fsm::moveupOutput(int i) {
 if(i < numberOfOutputs && i > 0) {

    vector <State>::iterator iter;
    
    for(iter=sv.begin(); iter!=sv.end(); iter++){ 

      char hvar;

      hvar = iter->output[i - 1];
      iter->output[i - 1] = iter->output[i];
      iter->output[i] = hvar;
    }
    string helpname;
    helpname = getOutputName(i - 1);
    outputName(i - 1,getOutputName(i));
    outputName(i,helpname);
    update();
  }
}

void Fsm::movedownInput(int i) {
  if(i < numberOfInputs - 1) {
    
    vector <Hypertrans>::iterator iter;
    vector <Trans>::iterator ii;
    vector <Condition>::iterator ci;
    
    for(ci=parameter.begin(); ci!=parameter.end(); ci++){
      char hvar;
      
      hvar = ci->input[i + 1];
      ci->input[i + 1] = ci->input[i];
      ci->input[i] = hvar;
    }
    
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      for(ci=iter->cv.begin(); ci!=iter->cv.end(); ci++) {
      char hvar;
      
      hvar = ci->input[i + 1];
      ci->input[i + 1] = ci->input[i];
      ci->input[i] = hvar;
      }
    }
    
    for(ii=tv.begin(); ii!=tv.end(); ii++){
      for(ci=ii->cv.begin(); ci!=ii->cv.end(); ci++) {
	char hvar;
	
	hvar = ci->input[i + 1];
	ci->input[i + 1] = ci->input[i];
	ci->input[i] = hvar;
      }
    }
    
    string helpname;
    helpname = getInputName(i + 1);
    inputName(i + 1,getInputName(i));
    inputName(i,helpname);
    update();
  }  
}

void Fsm::movedownOutput(int i) {
  if(i < numberOfOutputs - 1) {

    vector <State>::iterator iter;
    
    for(iter=sv.begin(); iter!=sv.end(); iter++){ 

      char hvar;

      hvar = iter->output[i + 1];
      iter->output[i + 1] = iter->output[i];
      iter->output[i] = hvar;
    }
    string helpname;
    helpname = getOutputName(i + 1);
    outputName(i + 1,getOutputName(i));
    outputName(i,helpname);
    update();

  }
}

void Fsm::addState() {
  sv.push_back(State(numberOfStates++, numberOfOutputs));
  statepointer = sv.end();
  statepointer--;
  update();
}

void Fsm::deleteState(int id) {

   vector <State>::iterator iter;
     
   statepointer = sv.end();
     
    for(iter=sv.begin(); iter!=sv.end(); iter++){
      if(iter->sid == id) { 


      	 sv.erase(iter);
	 statepointer = sv.end();

	 // delete connected transitions

	 vector <Trans>::iterator i;
     
	 for(i=tv.begin(); i!=tv.end(); i++){
	   if(i->start == id || i->end == id ) { 
	     transpointer = tv.end();
	     tv.erase(i);
	     i--;
	     transpointer = tv.end();
	   }
	 }
	 
	 // delete connected transitions
	 
	 // delete connected Link
	 
	  vector <Link>::iterator it;
     
	  for(it=lv.begin(); it!=lv.end(); it++){
	    if(it->goal == id) { 
	      linkpointer = lv.end();
	      lv.erase(it);
	      it--;
	      linkpointer = lv.end();
	    }
	  }

	 // delete connected Link

	 // delete connected Hypertrans
	 
	  vector <Hypertrans>::iterator ii;
     
	  for(ii=htv.begin(); ii!=htv.end(); ii++){
	    if(ii->goal == id) { 
	      hypertranspointer = htv.end();
	      htv.erase(ii);
	      ii--; 
	      hypertranspointer = htv.end();
	    }
	  }
    
	 // delete connected Hypertrans
	 update(); 
	 return;
	 //iter--;

	
       }
    }
   
}
 
void Fsm::modifyState(int id, unsigned int colour) {

  vector <State>::iterator iter;     
     
    for(iter=sv.begin(); iter!=sv.end(); iter++){
      if(iter->sid == id) { 
	iter->color = colour;
	update();
	return;
      }
    }
}

void Fsm::modifyState(int id, string name) {
 
  vector <State>::iterator iter;    
     
    for(iter=sv.begin(); iter!=sv.end(); iter++){
      if(iter->sid == id) { 
	iter->sname = name;
	update();
	return;
      }
    }
}

void Fsm::modifyState(int id, double posx, double posy) {
   
  vector <State>::iterator iter;    
     
    for(iter=sv.begin(); iter!=sv.end(); iter++){
      if(iter->sid == id) { 
	iter->pos.first = posx;
	iter->pos.second = posy;
	update();
	return;
      }
    }
}

void Fsm::modifyState(int id, int position, char c) {

  if(!(c == '1' || c == '0')) {
    cerr << "Error illegal use of character" << endl;
    return;
  }

  if(position < numberOfOutputs) {

    vector <State>::iterator iter;    
     
    for(iter=sv.begin(); iter!=sv.end(); iter++){
      if(iter->sid == id) { 
	iter->output[position] = c;
	update();
	return;
      }
    }
  } else {
    cerr << "Error modification is out of range" << endl;
  }
}

void Fsm::stateOutput(int id, string output) {
  for(int i = 0; i < numberOfOutputs; i++) {
    char c = output[i];
    modifyState(id, i, c);
  }
}

void Fsm::stateLevel(int id, int hierarchylevel) {
   
  vector <State>::iterator iter;    
     
    for(iter=sv.begin(); iter!=sv.end(); iter++){
      if(iter->sid == id) { 
	iter->hlevel = hierarchylevel;
	
	// if there are connected links the link will be deleted

	vector <Link>::iterator i;

	for(i=lv.begin(); i!=lv.end(); i++ ) {
	  if(i->goal == id){
	    deleteLink(i->lid);
	  }
	}

	// if there are connected links the link will be deleted
	update();
	return;
      }
    }
}

void Fsm::modifyTrans(int id, string name) {
 
  vector <Trans>::iterator iter;    
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	iter->tname = name;
	update();
	return;
      }
    }
}

void Fsm::modifyTrans(int id, double posx, double posy, int level) {
 
  vector <Trans>::iterator iter;    
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	iter->textpos.first = posx;
	iter->textpos.second = posy;
	iter->level = level;
	update();
	return;
      }
    }
}

void Fsm::addTrans(int start, int end) {
   tv.push_back(Trans(numberOfTrans++, start, end));
   transpointer = tv.end();
   transpointer--;
   update();
} 

void Fsm::deleteTrans(int id) {

 vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	transpointer = tv.end();
      	tv.erase(iter);
	transpointer = tv.end();
	update();
	return;
	//iter--;
       }
    }
}


void Fsm::modifyTrans(int id, int colour){
   
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) {
	iter->color = colour;
	update();
	return;
      }
    }
}

void Fsm::modifyTransLevel(int id, int newlevel){

  vector <Trans>::iterator iter;

    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) {
        iter->level = newlevel;
        update();
        return;
      }
    }
}

void Fsm::modifyTrans(int id, bool b) {
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) {
	iter->d = b;
	update();
	return;
      }
    }
}

void Fsm::addConditionToTrans(int id) {
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) {
	iter->addCondition(numberOfInputs);
	update();
	return;
      }
    }
}

void Fsm::deleteConditionOfTrans(int id, int cid) {
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) {
	iter->deleteCondition(cid);
	update();
	return;
      }
    }
 
}

void Fsm::modifyTrans(int id, int cid, int position, char value) {
  
  if(!(value == '1' || value == '0' || value == 'X' || value == 'x' || value == '-')) {
    cerr << "Error illegal use of character" << endl;
    return;
  }

  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	if(position < numberOfInputs) {
	  iter->modifyCondition(cid, position, value);
	} else {
	  cerr << "Error modification is out of range" << endl;
	}
	update();
	return;
      }
    } 
}

void Fsm::modifyTransConditionValue(int id, int cid, string value) {
  for(int i = 0; i < numberOfInputs; i++) {
    char c = value[i];
    modifyTrans(id, cid, i, c);
  }
}


void Fsm::modifyTrans(int id, int cid, string name) {
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	iter->modifyCondition(cid, name);
	update();
	return;
      }
    } 
}

void Fsm::addTrackpointToTrans(int id, int posid, bool b, double posx, double posy, int hl) {
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	iter->addTrackpoint(posid, b, posx, posy, hl);
	update();
	return;
      }
    } 
}

void Fsm::deleteTrackpointOfTrans(int id, int trid) {
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	iter->deleteTrackpoint(trid);
	update();
	return;
      }
    } 
}

void Fsm::modifyTrans(int id, int trid, double posx, double posy){
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	iter->modifyTrackpoint(trid, posx, posy);
	update(); 
	return;
      }
    } 
}

void Fsm::modifyTrans(int id, int trid, bool b, int linkid){
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	iter->modifyTrackpoint(trid, b, linkid);
	update();
	return;
      }
    } 
}

void Fsm::modifyTrans(int id,int trid, bool b) {

  vector <Trans>::iterator iter;
  
  for(iter=tv.begin(); iter!=tv.end(); iter++){
    if(iter->tid == id) { 
      iter->modifyTrackpoint(trid, b);
      update();
      return;
    }
  } 
}

void Fsm::modifyTransTrackpointColor(int id,int trid, int color) {

  vector <Trans>::iterator iter;
  
  for(iter=tv.begin(); iter!=tv.end(); iter++){
    if(iter->tid == id) { 
      iter->modifyTrackpoint(trid, color);
      update();
      return;
    }
  } 
}

void Fsm::transLevel(int id, int trid, int hldir){
  
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) { 
	iter->trackpointLevel(trid, hldir);
	update();
	return;
      }
    } 
}

void Fsm::modifyTrans(int id, int start ,int end){
   
  vector <Trans>::iterator iter;
     
    for(iter=tv.begin(); iter!=tv.end(); iter++){
      if(iter->tid == id) {
	iter->start = start;
	iter->end = end;
	update();
	return;
      }
    }
 
}

void Fsm::numberOfOuts(int i, bool defaultValue) {
  if(i > -1) {

    vector <State>::iterator iter;
    
    if(i < numberOfOutputs) {
      
      int b = numberOfOutputs - i;
      
      for(int a = 0; a < b; a++) {
	deleteOutputName();
      }
      
      for(iter=sv.begin(); iter!=sv.end(); iter++){
	for(int a = 0; a < b; a++) {
	  iter->output.assign(iter->output,0,iter->output.size() - 1);
	}
      }
    } else {
      
      int b = i - numberOfOutputs;
      
      for(int a = 0; a < b; a++) {
	addOutputName();
      }
      
      for(iter=sv.begin(); iter!=sv.end(); iter++){
          if (defaultValue == true){
              iter->output.append(b,'1');
          }
          else {
              iter->output.append(b,'0');
          }
      }    
  }
    numberOfOutputs = i;
    update();
  }
}

void Fsm::numberOfIns(int i, int defaultValue) {
  if(i > -1) {
    
    vector <Hypertrans>::iterator iter;
    vector <Trans>::iterator ii;
    vector <Condition>::iterator ci;
    
    
    if (i < numberOfInputs) {    
      
      int b = numberOfInputs - i;
      
      for(int a = 0; a < b; a++) {
	deleteInputName();
      }
      
      for(ci=parameter.begin(); ci!=parameter.end(); ci++){
	for(int a = 0; a < b; a++) {
	  ci->input.assign(ci->input,0,ci->input.size() - 1);
	}
      }
      
      for(iter=htv.begin(); iter!=htv.end(); iter++){
	for(ci=iter->cv.begin(); ci!=iter->cv.end(); ci++) {
	  for(int a = 0; a < b; a++) {
	    ci->input.assign(ci->input,0,ci->input.size() - 1);
	  }
	}
      }
      
      for(ii=tv.begin(); ii!=tv.end(); ii++){
	for(ci=ii->cv.begin(); ci!=ii->cv.end(); ci++) {
	  for(int a = 0; a < b; a++) {
	    ci->input.assign(ci->input,0,ci->input.size() - 1);
	  }
	}
      }
    } else {
      
      int b = i - numberOfInputs;
      
      for(int a = 0; a < b; a++) {
	addInputName();
      }
      
      for(ci=parameter.begin(); ci!=parameter.end(); ci++) {
          for(int a = 0; a < b; a++) {
              if (defaultValue == 0){
                  ci->input.append(1,'0');}
              else if (defaultValue == 1){
                  ci->input.append(1,'1');}
              else if (defaultValue == -1000){
                  ci->input.append(1,'-');}
          }
      } 
      
      for(iter=htv.begin(); iter!=htv.end(); iter++){
          for(ci=iter->cv.begin(); ci!=iter->cv.end(); ci++) {
              for(int a = 0; a < b; a++) {
                  if (defaultValue == 0){
                      ci->input.append(1,'0');}
                  else if (defaultValue == 1){
                      ci->input.append(1,'1');}
                  else if (defaultValue == -1000){
                      ci->input.append(1,'-');}
              }
          }
      } 
      
      for(ii=tv.begin(); ii!=tv.end(); ii++){
          for(ci=ii->cv.begin(); ci!=ii->cv.end(); ci++) {
              for(int a = 0; a < b; a++) {
                  if (defaultValue == 0){
                      ci->input.append(1,'0');}
                  else if (defaultValue == 1){
                      ci->input.append(1,'1');}
                  else if (defaultValue == -1000){
                      ci->input.append(1,'-');}
              }
          }
      }
  }
    numberOfInputs = i;
    update();
  }
}

void Fsm::resetState(int id) {
  resetstate = id;
  update();
}



void Fsm::addHypertrans(bool type, double posx, double posy, int target, int level) {
   htv.push_back(Hypertrans(numberOfHypertrans++, type, posx, posy, target, level));
   hypertranspointer = htv.end();
   hypertranspointer--;
   update();
}

void Fsm::deleteHypertrans(int id) {

 vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) { 


      	 htv.erase(iter);
	 update();
	 return;
	 //iter--;

	
       }
    }

}

void Fsm::fsmName(string str) {
  fname = str;
  update();
}


void Fsm::modifyHypertrans(int id, int colour) {
  
  vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) { 
	iter->color = colour;
	update();
	return;
      }
    }
}

void Fsm::modifyHypertrans(int id, bool b) {
  
  vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) { 
	iter->c = b;
	update();
	return;
      }
    } 
}

void Fsm::addConditionToHypertrans(int id) {
  
  vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) { 
	iter->addCondition(numberOfInputs);
	update();
	return;
      }
    } 
}

void Fsm::deleteConditionOfHypertrans(int id, int cid) {

  vector <Hypertrans>::iterator iter;
     
  for(iter=htv.begin(); iter!=htv.end(); iter++){
    if(iter->hid == id) {
      iter->deleteCondition(cid);
      update();
      return;
      }
    } 
}

void Fsm::modifyHypertrans(int id, int cid, int position, char value) {
  
  if(!(value == '1' || value == '0' || value == 'X' || value == 'x' || value == '-')) {
    cerr << "Error illegal use of character" << endl;
    return;
  }

  vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) { 
	if(position < numberOfInputs) {
	  iter->modifyCondition(cid, position, value);
	} else {
	  cerr << "Error modification is out of range" << endl;
	}
	update();
	return;
      }
    } 
}

void Fsm::modifyHypertrans(int id, double posx, double posy, int target) {

  vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) {
	iter->goal = target;
	iter->hpos.first = posx;
	iter->hpos.second = posy;
	update();
	return;
      }
    }
}

void Fsm::modifyHypertrans(int id, double posx, double posy) {  
 
  vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) { 
	iter->hpos.first = posx;
	iter->hpos.second = posy;
	update();
	return;
      }
    }
}

void Fsm::modifyHypertransConditionValue(int id, int cid, string value) {
  for(int i = 0; i < numberOfInputs; i++) {
    char c = value[i];
    modifyHypertrans(id, cid, i, c);
  }
}

void Fsm::modifyHypertrans(int id, int cid, string name) {
   
  vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) { 
	iter->modifyCondition(cid, name);
	update();
	return;
      }
    }
}

void Fsm::hypertransLevel(int id, int hierarchylevel) {
   
  vector <Hypertrans>::iterator iter;
     
    for(iter=htv.begin(); iter!=htv.end(); iter++){
      if(iter->hid == id) { 
	iter->hlevel = hierarchylevel;
	update();
	return;
      }
    }
}

void Fsm::addLink(double posx, double posy, int target, int level) {
     
  vector <State>::iterator iter;    
     
    for(iter=sv.begin(); iter!=sv.end(); iter++){
      if(iter->sid == target) { 
	 lv.push_back(Link(numberOfLinks++, posx, posy, target, level));
	 linkpointer = lv.end();
	 linkpointer--;
	 update();
	 return;
      }
    }
}

void Fsm::deleteLink(int id) {

  vector <Link>::iterator iter;
     
    for(iter=lv.begin(); iter!=lv.end(); iter++){
      if(iter->lid == id) { 


      	 lv.erase(iter);

	  // if the Link is used by a transition => it must turn off linkoptions
    
	 vector <Trans>::iterator i;
	 list <Trackpoint>::iterator tri;
    
	 for(i=tv.begin(); i!=tv.end(); i++) {
	   for(tri=i->tl.begin(); tri!=i->tl.end(); tri++) {
	     if((tri->lid == id) && tri->link) {
	       tri->link = false;
	       i->end = -1;
	     }
	   }
	 }
	 
	 // if the Link is used by a transition => it must turn off linkoptions

	 update();
	 return;
	 //iter--;
       }
    }
}

void Fsm::modifyLink(int id, int colour) {

 vector <Link>::iterator iter;
     
    for(iter=lv.begin(); iter!=lv.end(); iter++){
      if(iter->lid == id) { 
	iter->color = colour;
	update();
	return;
       }
    }
}

void Fsm::modifyLink(int id, double posx, double posy) {

  vector <Link>::iterator iter;
     
    for(iter=lv.begin(); iter!=lv.end(); iter++){
      if(iter->lid == id) { 
	iter->linkpos.first = posx;
	iter->linkpos.second = posy;
	update();
	return;
       }
    }
}

void Fsm::linkLevel(int id, int hierarchylevel) {

  vector <Link>::iterator iter;
     
    for(iter=lv.begin(); iter!=lv.end(); iter++){
      if(iter->lid == id) { 
	iter->hlevel = hierarchylevel;
	update();
	return;
       }
    }
}

void Fsm::linkGoal(int id, int target) {

  vector <Link>::iterator iter;
     
    for(iter=lv.begin(); iter!=lv.end(); iter++){
      if(iter->lid == id) {  
	
	// if the Link is used by a transition => target must be adapted
    
	 vector <Trans>::iterator i;
	 list <Trackpoint>::iterator tri;
    
	 for(i=tv.begin(); i!=tv.end(); i++) {
	   for(tri=i->tl.begin(); tri!=i->tl.end(); tri++) {
	     if((tri->lid == id) && tri->link) {
	       i->end = target;
	     }
	   }
	 }
	 
	 // if the Link is used by a transition => target must be adapted 

	iter->goal = target;
	update();
	return;
       }
    }
}

void Fsm::addParameter() {
  parameter.push_back(Condition(numberOfParameters++, numberOfInputs));
  update();
}

void Fsm::modifyParameter(int id, string name) {
    
  vector <Condition>::iterator iter;
     
    for(iter=parameter.begin(); iter!=parameter.end(); iter++){
      if(iter->cid == id) {
	  iter->cname = name;
	  update();
	return;
       }
    }
}

void Fsm::modifyParameter(int id, int position, char value) {
  
  if(!(value == '1' || value == '0' || value == 'X' || value == 'x')) {
    cerr << "Error illegal use of character" << endl;
    return;
  }

  vector <Condition>::iterator iter;
     
    for(iter=parameter.begin(); iter!=parameter.end(); iter++){
      if(iter->cid == id) {
	if(position < iter->input.size()) {
	  iter->input[position] = value;
	} else {
	  cerr << "Error modification is out of range" << endl;
	}
	update();
	return;
       }
    }
}

void Fsm::deleteParameter(int id) {
  
  vector <Condition>::iterator iter;
     
    for(iter=parameter.begin(); iter!=parameter.end(); iter++){
      if(iter->cid == id) {
	parameterpointer = parameter.end();
      	parameter.erase(iter);
	parameterpointer = parameter.end();
	update();
	return;
      }
    }
}

string Fsm::getOutputName(int pos) {
  if(pos < numberOfOutputs) {
    return outputnames[pos];
  } else {
    cerr << "Error OutPutNamePosition does not exist" << endl;
    return "not found";
  }
}

string Fsm::getInputName(int pos) {
  if(pos < numberOfInputs) {
    return inputnames[pos];
  } else { 
    cerr << "Error InputNamePosition does not exist" << endl;
    return "not found";
  }
}

int Fsm::getFsmID() {
  return fid;
}

int Fsm::getNumberOfInputs() {
  return numberOfInputs;
}

int Fsm::getNumberOfOutputs() {
  return numberOfOutputs;
}

int Fsm::getResetState() {
  return resetstate;
}

string Fsm::getFsmName() {
  return fname;
}


bool Fsm::getFirstParameter() {
  if(parameter.empty()) {
    return false;
  } else {
    parameterpointer = parameter.begin();
    return true;
  }
}

bool Fsm::getNextParameter() {
  if(parameterpointer == parameter.end()) {
    return false;
  } else {
    parameterpointer++;
    if(parameterpointer == parameter.end()) {
      return false;
    } else {
      return true;
    }
  }
}

string Fsm::getParameterName() {
  return parameterpointer->cname;
}

string Fsm::getParameterValue() {
  return parameterpointer->input;
}

int Fsm::getParameterID() {
  return parameterpointer->cid;
}

bool Fsm::getFirstState() {
 if(sv.empty()) {
    return false;
  } else {
    statepointer = sv.begin();
    return true;
  }
}

bool Fsm::getNextState() {
 if(statepointer == sv.end()) {
    return false;
  } else {
    statepointer++;
    if(statepointer == sv.end()) {
      return false;
    } else {
      return true;
    }
  }
}

string Fsm::getStateName() {
  return statepointer->sname;
}

double Fsm::getStatePosx() {
return statepointer->pos.first;
}

double Fsm::getStatePosy() {
return statepointer->pos.second;
}

int Fsm::getStateID() {
return statepointer->sid;
}

unsigned int Fsm::getStateColor() {
return statepointer->color;
}

int Fsm::getStateLevel() {
return statepointer->hlevel;
} 

string Fsm::getStateOutput() {
return statepointer->output;
}

char Fsm::getStateOutputBit(int pos) {
    string st = statepointer->output;
    if (pos<numberOfOutputs)
    {
        char bit = st[pos];
        return bit;}
    else {
        char err;
        err = ' ';
        return err;
    }
}

bool Fsm::getFirstTrans() {
  if(tv.empty()) {
    return false;
  } else {
    transpointer = tv.begin();
    return true;
  }
}

bool Fsm::getNextTrans() {
  if(transpointer == tv.end()) {
    return false;
  } else {
    transpointer++;
    if(transpointer == tv.end()) {
      return false;
    } else {
      return true;
    }
  }
}

int Fsm::getTransStart() {
  return transpointer->start;
}

int Fsm::getTransEnd() {
  return transpointer->end;
}

string Fsm::getTransName() {
  return transpointer->tname;
}

int Fsm::getTransID() {
  return transpointer->tid;
}

int Fsm::getTransColor() {
  return transpointer->color;
}

bool Fsm::getTransDefault() {
  return transpointer->d;
}  

double Fsm::getTransTextPosx() {
  return transpointer->textpos.first;
}

double Fsm::getTransTextPosy() {
  return transpointer->textpos.second;
}

int Fsm::getTransLevel() {
  return transpointer->level;
}

bool Fsm::getTransFirstCondition() {
  return transpointer->getFirstCondition();
}

bool Fsm::getTransNextCondition() {
  return transpointer->getNextCondition();
}

string Fsm::getTransConditionName() {
  return transpointer->getConditionName();
}

string Fsm::getTransConditionValue() {
  return transpointer->getConditionValue();
}

char Fsm::getTransConditionBit(int pos) {
    string st = transpointer->getConditionValue();
    if (pos<numberOfInputs)
    {
        char bit = st[pos];
        return bit;
    }
    else {
        char err;
        err = ' ';
        return err;
    }
}

int Fsm::getTransConditionID() {
  return transpointer->getConditionID();
} 

bool Fsm::getTransFirstTrackpoint() {
  return transpointer->getFirstTrackpoint();
} 

bool Fsm::getTransNextTrackpoint() {
  return transpointer->getNextTrackpoint() ;
} 

double Fsm::getTransTrackpointPosx() {
  return transpointer->getTrackpointPosx();
}

double Fsm::getTransTrackpointPosy() {
  return transpointer->getTrackpointPosy();
}

int Fsm::getTransTrackpointHldirection() {
  return transpointer->getTrackpointHldirection();
}

int Fsm::getTransTrackpointColor() {
  return transpointer->getTrackpointColor();
}

int Fsm::getTransTrackpointLevel() {
  return transpointer->getTrackpointLevel();
}

int Fsm::getTransTrackpointID() {
  return transpointer->getTrackpointID();
}

bool Fsm::getTransTrackpointLink() {
  return transpointer->getTrackpointLink();
}

int Fsm::getTransTrackpointLinkID() {
  return transpointer->getTrackpointLinkID();
}

bool Fsm::getTransTrackpointJoin() {
  return transpointer->getTrackpointJoin();
}

bool Fsm::getFirstLink() {
  if(lv.empty()) {
    return false;
  } else {
    linkpointer = lv.begin();
    return true;
  }
}

bool Fsm::getNextLink() {
  if(linkpointer == lv.end()) {
    return false;
  } else {
    linkpointer++;
    if(linkpointer == lv.end()) {
      return false;
    } else {
      return true;
    }
  }
}

int Fsm::getLinkID() {
  return linkpointer->lid;
}

int Fsm::getLinkGoal() {
  return linkpointer->goal;
}

int Fsm::getLinkColor() {
  return linkpointer->color;
}

int Fsm::getLinkLevel() {
  return linkpointer->hlevel;
}

double Fsm::getLinkPosx() {
  return linkpointer->linkpos.first;
}

double Fsm::getLinkPosy() {
  return linkpointer->linkpos.second;
}

bool Fsm::getFirstHypertrans() {
  if(htv.empty()) {
    return false;
  } else {
    hypertranspointer = htv.begin();
    return true;
  }
}

bool Fsm::getNextHypertrans() {
  if(hypertranspointer == htv.end()) {
    return false;
  } else {
    hypertranspointer++;
    if(hypertranspointer == htv.end()) {
      return false;
    } else {
      return true;
    }
  }
}

int Fsm::getHypertransColor() {
  return hypertranspointer->color;
}

int Fsm::getHypertransID() {
  return hypertranspointer->hid;
}

int Fsm::getHypertransLevel() {
  return hypertranspointer->hlevel;
}

int Fsm::getHypertransGoal() {
  return hypertranspointer->goal;
}

bool Fsm::getHypertransType() {
  return hypertranspointer->c;
}

double Fsm::getHypertransPosx() {
  return hypertranspointer->hpos.first;

}

double Fsm::getHypertransPosy() {
  return hypertranspointer->hpos.second;
}

bool Fsm::getHypertransFirstCondition() {
  return hypertranspointer->getFirstCondition();
}

bool Fsm::getHypertransNextCondition() {
  return hypertranspointer->getNextCondition();
}

string Fsm::getHypertransConditionName() {
  return hypertranspointer->getConditionName();
}

string Fsm::getHypertransConditionValue() {
  return hypertranspointer->getConditionValue();
}

int Fsm::getHypertransConditionID() {
  return hypertranspointer->getConditionID();
}

bool Fsm::getStatebyID(int id) {
  if(getFirstState()) {
    if(getStateID() == id) {
      return true;
    }
    while (getNextState()) {
	if(getStateID() == id) {
	  return true;
	}
    }    
    return false;
  }
}

bool Fsm::getTransbyID(int id) {
  if(getFirstTrans()) {
    if(getTransID() == id) {
      return true;
    }
    while (getNextTrans()) {
      if(getTransID() == id) {
	return true;
      }
    }
    return false;
  }
}

bool Fsm::getTransConditionbyID(int id) {
  if(getTransFirstCondition()) {
    if(getTransConditionID() == id) {
      return true;
    }
    while (getTransNextCondition()) {
      if(getTransConditionID() == id) {
	return true;
      }
    }
    return false;
  }
}

bool Fsm::getTransTrackpointbyID(int id) {
  if(getTransFirstTrackpoint()) {
    if(getTransTrackpointID() == id) {
      return true;
    }
    while (getTransNextTrackpoint()) {
      if(getTransTrackpointID() == id) {
	return true;
      }
    }
    return false;
  }
}

bool Fsm::getLinkbyID(int id) {
 if(getFirstLink()) {
    if(getLinkID() == id) {
      return true;
    }
    while (getNextLink()) {
      if(getLinkID() == id) {
	return true;
      }
    }
    return false;
  }
}

bool Fsm::getHypertransbyID(int id) {
 if(getFirstHypertrans()) {
    if(getHypertransID() == id) {
      return true;
    }
    while (getNextHypertrans()) {
      if(getHypertransID() == id) {
	return true;
      }
    }
    return false;
  }
}

bool Fsm::getHypertransConditionbyID(int id) {
 if(getHypertransFirstCondition()) {
    if(getHypertransConditionID() == id) {
      return true;
    }
    while (getHypertransNextCondition()) {
      if(getHypertransConditionID() == id) {
	return true;
      }
    }
    return false;
  }
}

void Fsm::optimize() {
  int newid = 0;
  int oldid;
  if(getFirstState()) {
    do {
      oldid = getStateID();
      if(oldid != newid) {
	statepointer->sid = newid;
	if(getFirstTrans()) {
	  do {
	    if (getTransStart() == oldid) {
	      int end = getTransEnd();
	      int transid = getTransID();
	      modifyTrans(transid, newid, end);
	    } 
	    if (getTransEnd() == oldid) {
	      int start = getTransStart();
	      int transid = getTransID();
	      modifyTrans(transid, start, newid);
	    }
	    if(getResetState() == oldid) {
	      resetState(newid);
	    }
	  }while(getNextTrans());
	}
	if(getFirstHypertrans()) {
	  do {
	    if (getHypertransGoal() == oldid) {
	      int hyperid = getHypertransID();
	      double posx = getHypertransPosx();
	      double posy = getHypertransPosy();
	      modifyHypertrans(hyperid, posx ,posy, newid);
	    }
	  }while(getNextHypertrans());
	}
	if(getFirstLink()) {
	  do {
	    if (getLinkGoal() == oldid) {
	      int linkid = getLinkID();
	      linkGoal(linkid, newid);
	    }
	  }while(getNextLink());
	}
      }
      newid++;
    } while (getNextState());
    numberOfStates = newid;
  }

  if(getFirstLink()) {
    newid = 0;
    oldid = 0;
    do {
      oldid = getLinkID();
      if(oldid != newid) {
	linkpointer->lid = newid;
	if(getFirstTrans()) {
	  do {
	    if(getTransFirstTrackpoint()) {
	      do {
		if(getTransTrackpointLink()) {
		  if(getTransTrackpointLinkID() == oldid) {
		    int transid = getTransID();
		    int trackid = getTransTrackpointID();
		    modifyTrans(transid, trackid, true, newid);
		  }
		}
	      }while(getTransNextTrackpoint());
	    }
	  }while(getNextTrans());
	}
      }
      newid++;
    } while (getNextLink());
    numberOfLinks = newid;
  }
}

void Fsm::clockName(string s) {
  globals.first = s;
}

void Fsm::resetName(string s) {
  globals.second = s;
}  

string Fsm::getClockName() {
  return globals.first;
}
  
string Fsm::getResetName() {
  return globals.second;
}
