/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <cstdlib>
#include <vector>
#include <iterator>
#include <iostream>

#include "hypertrans.h"

using namespace std;

Hypertrans::Hypertrans(int id, bool b, double posx, double posy, int target, int level) {
  color = 0;
  c = b;
  hid = id;
  hpos.first = posx;
  hpos.second = posy;
  goal = target;
  numberOfConditions = 0;
  hlevel = level;
  conditionpointer = cv.end();
}

Hypertrans& Hypertrans::operator=(const Hypertrans &ht){
  this->color = ht.color;
  this->c = ht.c;
  this->hid = ht.hid;
  this->goal = ht.goal;
  this->hpos.first = ht.hpos.first;
  this->hpos.second = ht.hpos.second;
  this->numberOfConditions = ht.numberOfConditions;
  this->cv = ht.cv;
  this->hlevel = ht.hlevel;
  return *this;
}

int Hypertrans::operator==(const Hypertrans &ht) const{
  if(hid == ht.hid) return 1;
  return 0;

}

int Hypertrans::operator<(const Hypertrans &ht) const{
  if(hid < ht.hid) return 1;
  return 0;

}

void Hypertrans::addCondition(int number) {
   cv.push_back(Condition(numberOfConditions++, number));
   conditionpointer = cv.end();
   conditionpointer--;
}

void Hypertrans::deleteCondition(int id) {
  
  vector <Condition>::iterator iter;
     
    for(iter=cv.begin(); iter!=cv.end(); iter++){
      if(iter->cid == id) { 
	conditionpointer = cv.end();
      	cv.erase(iter);
	conditionpointer = cv.end();
	return;
	//iter--;

	
       }
    }
}

void Hypertrans::modifyCondition(int id, int position, char value) {
  
  if(!(value == '1' || value == '0' || value == 'X' || value == 'x' || value == '-')) {
    cerr << "Error illegal use of character" << endl;
    return;
  }
  
  vector <Condition>::iterator iter;
     
    for(iter=cv.begin(); iter!=cv.end(); iter++){
      if(iter->cid == id) {
	if(position < iter->input.size()) {
	  iter->input[position] = value;
	} else {
	  cerr << "Error modification is out of range" << endl;
	}
	return;
       }
    }
}

void Hypertrans::modifyCondition(int id, string name) {
  
  vector <Condition>::iterator iter;
     
    for(iter=cv.begin(); iter!=cv.end(); iter++){
      if(iter->cid == id) {
	  iter->cname = name;
	return;
       }
    }
}

bool Hypertrans::getFirstCondition() {
  if(cv.empty()) {
    return false;
  } else {
    conditionpointer = cv.begin();
    return true;
  }
}

bool Hypertrans::getNextCondition() {
 if(conditionpointer == cv.end()) {
    return false;
  } else {
    conditionpointer++;
    if(conditionpointer == cv.end()) {
      return false;
    } else {
      return true;
    }
  }
}

string Hypertrans::getConditionName() {
  return conditionpointer->cname;
}

string Hypertrans::getConditionValue() {
  return conditionpointer->input;
}

int Hypertrans::getConditionID() {
  return conditionpointer->cid;
}
