/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

/*!
  \file hypertrans.h
  \brief Hypertrans
  
  \author Frank Lemke
  \date 2006-02-28
*/

#ifndef HYPERTRANS_H_
#define HYPERTRANS_H_

#include <vector>
#include <string>
#include <cstdlib>

#include "condition.h"

using namespace std;

/*!
  \brief Hypertrans-object

  The Hypertrans object
*/

class Hypertrans {

 private:
  vector <Condition>::iterator conditionpointer;
  int numberOfConditions;
 public:
  int color;
  bool c; // all_states is true, rest_states is false
  int hid;
  pair <double, double> hpos;
  vector <Condition> cv;
  int goal;
  int hlevel;

  Hypertrans(int id, bool b, double posx, double posy, int target, int level);
  ~Hypertrans(){};
  Hypertrans &operator=(const Hypertrans &ht);
  int operator==(const Hypertrans &ht) const;
  int operator<(const Hypertrans &ht) const;

  /*!
    \brief addCondition

    Function to add Condition to Hypertrans.
    \param number Number of inputbits.
  */
 
  void addCondition(int number);

  /*!
    \brief deleteCondition

    Delete Condition by ID.
    \param id ID to delete associated %Condition.
  */
 
  void deleteCondition(int id);

  /*!
    \brief modifyCondition

    Change value of a inputbit.
    \param id ID of a %Condition. 
    \param position Position of the bit you want to change.
    \param value New value for choosen bit.
  */
 
  void modifyCondition(int id, int position, char value);

  /*!
    \brief modifyCondition

    Function to change inputparameters of Condition.
    \param id ID of a %Condition.
    \param name New name for %Condition.
  */
 
  void modifyCondition(int id, string name);

  /*!
    \brief getFirstCondition

    This function checks if there is a Condition. In case of success it sets
    the conditonpointer to the first one.
    \return bool True if %Condition-vector is not empty.
  */
 
  bool getFirstCondition();

  /*!
    \brief getNextCondition

    Checks for following Conditions if there is one it makes the conditionpointer
    point to the next one in the vector.
    \return bool True if there is a %Condition after the current one.
  */
 
  bool getNextCondition();

  /*!
    \brief getConditionName
    
    To get the name of current Condition.
    \return string Conditionname 
  */
 
  string getConditionName();

  /*!
    \brief getConditionValue

    To get the value of current Condition.
    \return string Conditionvalue 
  */
 
  string getConditionValue();

  /*!
    \brief getConditionID

    To get the ID of current Condition.
    \return int ID
  */
 
  int getConditionID();
};


#endif
