/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <cstdlib>
#include <iostream>
#include <sstream>

#include "trans.h"


using namespace std;



Trans::Trans(int id, int s, int e) {
  tid = id;
  start = s;
  end = e;
  d = false;
  color = 0;
  level = 0;
  tname = "trans_";
  tname = tname + itos(tid);
  numberOfConditions = 0;
  numberOfTrackpoints = 0;
  conditionpointer = cv.end();
  trackpointpointer = tl.end();
  textpos.first = 10;
  textpos.second = 10;
}

Trans& Trans::operator=(const Trans &tra) {
  this->start = tra.start;
  this->end = tra.end;
  this->tname = tra.tname;
  this->tid = tra.tid;
  this->d = tra.d;
  this->color = tra.color;
  this->numberOfConditions = tra.numberOfConditions;
  this->numberOfTrackpoints = tra.numberOfTrackpoints;
  this->trackpointpointer = tra.trackpointpointer;
  this->conditionpointer = tra.conditionpointer;
  this->cv = tra.cv;
  this->tl = tra.tl; 
  this->level = tra.level;
  this->textpos.first = tra.textpos.first;
  this->textpos.second = tra.textpos.second;
  return *this;
}


int Trans::operator==(const Trans &tra) const {
  if(start == tra.start) return 1;
  return 0;
}


int Trans::operator<(const Trans &tra) const { 
  if(start < tra.start) return 1;
  return 0;
}

string Trans::itos(int i) {
  stringstream s;
  s << i;
  return s.str();

}

void Trans::addCondition(int number) {
   cv.push_back(Condition(numberOfConditions++, number));
   conditionpointer = cv.end();
   conditionpointer--;
}

void Trans::deleteCondition(int id) {
  
  vector <Condition>::iterator iter;
     
    for(iter=cv.begin(); iter!=cv.end(); iter++){
      if(iter->cid == id) { 
	conditionpointer = cv.end();
      	cv.erase(iter);
	conditionpointer = cv.end();
	return;
      }
    }
}

void Trans::modifyCondition(int id, int position, char value) {
  
  if(!(value == '1' || value == '0' || value == 'X' || value == 'x' || value == '-')) {
    cerr << "Error illegal use of character" << endl;
    return;
  }

  vector <Condition>::iterator iter;
     
    for(iter=cv.begin(); iter!=cv.end(); iter++){
      if(iter->cid == id) {
	if(position < iter->input.size()) {
	  iter->input[position] = value;
	} else {
	  cerr << "Error modification is out of range" << endl;
	}
	return;
       }
    }
}

void Trans::modifyCondition(int id, string name) {
  
  vector <Condition>::iterator iter;
     
    for(iter=cv.begin(); iter!=cv.end(); iter++){
      if(iter->cid == id) {
	  iter->cname = name;
	return;
       }
    }
}

/*Function to add Trackpoints to Trans. Trackpoint can be added at the 
  beginning by passing a negativ value to posid parameter or at the end by
  passing a value greater then 999 to posid. To place new Trackpoint before
  a special chosen position set parameter b to true, false will place it
  after it.*/

void Trans::addTrackpoint(int posid, bool b, double posx, double posy, int hl) {
  if(tl.empty()) {
    tl.push_back(Trackpoint(numberOfTrackpoints++, posx, posy, hl));
    trackpointpointer = tl.end();
    trackpointpointer--;
    return;
  } else {
    if (posid > 999) {
      tl.push_back(Trackpoint(numberOfTrackpoints++, posx, posy, hl));
      trackpointpointer = tl.end();
      trackpointpointer--;
      return;
    }
    if(posid < 0) {
      tl.push_front(Trackpoint(numberOfTrackpoints++, posx, posy, hl ));
      trackpointpointer = tl.begin();
      return;
    } else {
      
      list <Trackpoint>::iterator ti;
      
      for(ti = tl.begin(); ti != tl.end(); ti++) {
	if(ti->trid == posid) {
	  if(b) {
	    tl.insert(ti, Trackpoint(numberOfTrackpoints++, posx, posy, hl));
	    trackpointpointer = --ti;
	    ti++;
	  } else {
	    tl.insert(++ti, Trackpoint(numberOfTrackpoints++, posx, posy, hl));
	    trackpointpointer = --ti;
	  }
	  return;
	}
      }
      cout << "ERROR....was not able to insert Trackpoint!" << endl; 
    }
  }
}

void Trans::deleteTrackpoint(int id) {

  list <Trackpoint>::iterator ti;
  
  for(ti = tl.begin(); ti!=tl.end(); ti++) {
    if(ti->trid == id) {
      trackpointpointer = tl.end();
      tl.erase(ti);
      trackpointpointer = tl.end();
      return;
    }
  }

}

void Trans::modifyTrackpoint(int id, double posx, double posy) {

  list <Trackpoint>::iterator ti;
  
  for(ti = tl.begin(); ti!=tl.end(); ti++) {
    if(ti->trid == id) {
      ti->pos.first = posx;
      ti->pos.second = posy;
      return;
    }
  }
} 

void Trans::modifyTrackpoint(int id, bool b, int linkid) {

  list <Trackpoint>::iterator ti;
  
  for(ti = tl.begin(); ti!=tl.end(); ti++) {
    if(ti->trid == id) {
      ti->link = b;
      ti->lid = linkid;
      return;
    }
  }
}

void Trans::modifyTrackpoint(int id, bool b) {

  list <Trackpoint>::iterator ti;
  
  for(ti = tl.begin(); ti!=tl.end(); ti++) {
    if(ti->trid == id) {
      ti->join = b;
      return;
    }
  }
}

void Trans::modifyTrackpoint(int id, int color) {

  list <Trackpoint>::iterator ti;
  
  for(ti = tl.begin(); ti!=tl.end(); ti++) {
    if(ti->trid == id) {
      ti->color = color;
      return;
    }
  }
}

void Trans::trackpointLevel(int id, int hldir) {

  list <Trackpoint>::iterator ti;
  
  for(ti = tl.begin(); ti!=tl.end(); ti++) {
    if(ti->trid == id) {
      ti->hldirection = hldir;
      return;
    }
  }
}

bool Trans::getFirstCondition() {
  if(cv.empty()) {
    return false;
  } else {
    conditionpointer = cv.begin();
    return true;
  }
}

bool Trans::getNextCondition() {
  if(conditionpointer == cv.end()) {
    return false;
  } else {
    conditionpointer++;
    if(conditionpointer == cv.end()) {
      return false;
    } else {
      return true;
    }
  }
}

string Trans::getConditionName() {
  return conditionpointer->cname;
}

string Trans::getConditionValue() {
  return conditionpointer->input;
}

int Trans::getConditionID() {
  return conditionpointer->cid;
}

bool Trans::getFirstTrackpoint() {
 if(tl.empty()) {
    return false;
  } else {
    trackpointpointer = tl.begin();
    return true;
  }
}

bool Trans::getNextTrackpoint() {
if(trackpointpointer == tl.end()) {
    return false;
  } else {
    trackpointpointer++;
    if(trackpointpointer == tl.end()) {
      return false;
    } else {
      return true;
    }
  }
}

double Trans::getTrackpointPosx() {
  return trackpointpointer->pos.first;
}

double Trans::getTrackpointPosy() {
  return trackpointpointer->pos.second;
}

int Trans::getTrackpointHldirection() {
  return trackpointpointer->hldirection;
}

int Trans::getTrackpointColor() {
  return trackpointpointer->color;
}

int Trans::getTrackpointLevel() {
  return trackpointpointer->hlevel;
}

int Trans::getTrackpointID() {
  return trackpointpointer->trid;
}

bool Trans::getTrackpointLink() {
  return trackpointpointer->link;
}

int Trans::getTrackpointLinkID() {
  return trackpointpointer->lid;
}

bool Trans::getTrackpointJoin() {
  return trackpointpointer->join;
}
