/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

/*!
  \file fsm_I.h
  \brief Fsm_I
  
  \author Frank Lemke
  \date 2006-02-28
*/

#ifndef FSM_I_H_
#define FSM_I_H_

#include <string>
#include <cstdlib>

#include "update_I.h"

using namespace std;

/*!
  \brief Fsm_Interface

  Interface which provies all functions to manipulate current used FSMs or get
  informations about the current used FSMs.
*/

class Fsm_I {

public:
    virtual void optimize() = 0;
    virtual void registerView(Update_I* u) = 0;
    virtual void signoffView(Update_I* u) = 0;
    virtual void update() = 0;
    virtual void addInputName()= 0;
    virtual void addOutputName() = 0;
    virtual void deleteInputName() = 0;
    virtual void deleteOutputName() = 0;
    virtual void inputName(int i, string name) = 0;
    virtual void outputName(int i, string name) = 0;
    virtual void deleteInput(int i) = 0;
    virtual void deleteOutput(int i) = 0;
    virtual void moveupInput(int i) = 0;
    virtual void moveupOutput(int i) = 0;
    virtual void movedownInput(int i) = 0;
    virtual void movedownOutput(int i) = 0;
    virtual void numberOfOuts(int i, bool defaultValue) = 0;
    virtual void numberOfIns(int i, int defaultValue) = 0;
    virtual void resetState(int id) = 0;
    virtual void fsmName(string str) = 0;

    virtual void addState() = 0;
    virtual void deleteState(int id) = 0;
    virtual void modifyState(int id, string name) = 0;
    virtual void modifyStateName(int id, string name) {modifyState(id,name);};
    virtual void modifyState(int id, unsigned int colour) = 0;
    virtual void modifyStateColour(int id, int colour) {modifyState(id,colour);};
    virtual void modifyState(int id, double posx, double posy) = 0;
    virtual void modifyStatePos(int id, double posx, double posy) {modifyState(id,posx,posy);};
    virtual void modifyState(int id, int position, char c) = 0;
    virtual void modifyStateOutputBit(int id, int position, char c) {modifyState(id,position,c);};
    virtual void stateOutput(int id, string output) = 0;
    virtual void stateLevel(int id, int hierarchylevel) = 0;

    virtual void addTrans(int start, int end) = 0;
    virtual void deleteTrans(int id) = 0;
    virtual void modifyTrans(int id, int colour) = 0;
    virtual void modifyTransColour(int id, int c) {modifyTrans(id,c);};
    virtual void modifyTransLevel(int id, int newlevel) = 0;
    virtual void modifyTrans(int id, bool b) = 0;
    virtual void modifyTransDefault(int id, bool b) {modifyTrans(id,b);};
    virtual void modifyTrans(int id, string name) = 0;
    virtual void modifyTrans(int id, double posx, double posy, int level) = 0;
    virtual void modifyTrans(int id, int start, int end) = 0;
    virtual void modifyTransEnds(int id, int s,int e) {modifyTrans(id,s,e);};
    virtual void modifyTrans(int id, int cid, int position, char value) = 0;
    virtual void modifyTransCondition(int id,int cid, int pos,char v) {modifyTrans(id,cid,pos,v);};
    virtual void modifyTransConditionValue(int id, int cid, string value) = 0;
    virtual void modifyTrans(int id, int cid, string name) = 0;
    virtual void modifyTransConditionName(int id,int cid, string name) {modifyTrans(id,cid,name);};
    virtual void addConditionToTrans(int id) = 0;
    virtual void deleteConditionOfTrans(int id, int cid) = 0;
    virtual void addTrackpointToTrans(int id, int posid, bool b, double posx, double posy, int hl) = 0;
    virtual void deleteTrackpointOfTrans(int id, int trid) = 0;
    virtual void modifyTrans(int id, int trid, double posx, double posy) = 0;
    virtual void modifyTransTrackpointPos(int id,int trid, double x,double y) {modifyTrans(id,trid,x,y);};
    virtual void modifyTrans(int id,int trid, bool b, int linkid) = 0;
    virtual void modifyTransTrackpointLink(int id,int trid,bool b, int linkid) {modifyTrans(id,trid,b,linkid);};
    virtual void modifyTrans(int id,int trid, bool b) = 0;
    virtual void modifyTransTrackpointJoin(int id,int trid, bool b) {modifyTrans(id,trid,b);};
    virtual void modifyTransTrackpointColor(int id,int trid, int color) = 0;
    virtual void transLevel(int id, int trid, int hldir) = 0;

    virtual void addHypertrans(bool type, double posx, double posy, int target, int level) = 0;
    virtual void deleteHypertrans(int id) = 0;
    virtual void modifyHypertrans(int id, int colour) = 0;
    virtual void modifyHypertransColour(int id, int colour) {modifyHypertrans(id,colour);};
    virtual void modifyHypertrans(int id, double posx, double posy) = 0;
    virtual void modifyHypertransPos(int id, double posx, double posy) {modifyHypertransPos(id,posx,posy);};
    virtual void modifyHypertrans(int id, double posx, double posy, int target) = 0;
    virtual void modifyHypertransPosTarget(int id, double posx, double posy, int target) {modifyHypertrans(id,posx,posy,target);};
    virtual void modifyHypertrans(int id, bool b) = 0;
    virtual void modifyHypertransType(int id, bool b) {modifyHypertrans(id,b);};
    virtual void modifyHypertrans(int id, int cid, int position, char value) = 0;
    virtual void modifyHypertransConditionBit(int id, int cid, int position, char value) {modifyHypertrans(id,cid,position,value);};
    virtual void modifyHypertransConditionValue(int id, int cid, string value) = 0;
    virtual void modifyHypertrans(int id, int cid, string name) = 0;
    virtual void modifyHypertransConditionName(int id, int cid, string name) {modifyHypertrans(id,cid,name);};

    virtual void addConditionToHypertrans(int id) = 0;
    virtual void deleteConditionOfHypertrans(int id, int cid) = 0;
    virtual void hypertransLevel(int id, int hierarchylevel) = 0;

    virtual void addLink(double posx, double posy, int target, int level) = 0;
    virtual void deleteLink(int id) = 0;
    virtual void modifyLink(int id, int colour) = 0;
    virtual void modifyLinkColour(int id, int colour) {modifyLink(id,colour);};
    virtual void modifyLink(int id, double posx, double posy) = 0;
    virtual void modifyLinkPos(int id, double posx, double posy) {modifyLink(id,posx,posy);};
    virtual void linkLevel(int id, int hierarchylevel) = 0;
    virtual void linkGoal(int id, int target) = 0;

    virtual void addParameter() = 0;
    virtual void modifyParameter(int id, int position, char value) = 0;
    virtual void modifyParameterValue(int id, int position, char value) {modifyParameter(id,position,value);};
    virtual void modifyParameter(int id, string name) = 0;
    virtual void modifyParameterName(int id, string name) {modifyParameter(id,name);};
    virtual void deleteParameter(int id) = 0;

    virtual string getOutputName(int pos) = 0;
    virtual string getInputName(int pos) = 0;
    virtual int getFsmID() = 0;
    virtual int getNumberOfInputs() = 0;
    virtual int getNumberOfOutputs() = 0;
    virtual int getResetState() = 0;
    virtual string getFsmName() = 0;

    virtual bool getFirstParameter() = 0;
    virtual bool getNextParameter() = 0;
    virtual string getParameterName() = 0;
    virtual string getParameterValue() = 0;
    virtual int getParameterID() = 0;

    virtual bool getFirstState() = 0;
    virtual bool getNextState() = 0;
    virtual string getStateName() = 0;
    virtual double getStatePosx() = 0;
    virtual double getStatePosy() = 0;
    virtual int getStateID() = 0;
    virtual unsigned int getStateColor() = 0;
    virtual int getStateLevel() = 0;
    virtual string getStateOutput() = 0;
    virtual char getStateOutputBit(int pos) = 0;


    virtual bool getFirstTrans() = 0;
    virtual bool getNextTrans() = 0;
    virtual int getTransStart() = 0;
    virtual int getTransEnd() = 0;
    virtual string getTransName() = 0;
    virtual int getTransID() = 0;
    virtual int getTransColor() = 0;
    virtual bool getTransDefault() = 0;
    virtual double getTransTextPosx() = 0;
    virtual double getTransTextPosy() = 0;
    virtual int getTransLevel() = 0;

    virtual bool getTransFirstCondition() = 0;
    virtual bool getTransNextCondition() = 0;
    virtual string getTransConditionName() = 0;
    virtual string getTransConditionValue() = 0;
    virtual char getTransConditionBit(int pos) = 0;
    virtual int getTransConditionID() = 0;
    virtual bool getTransFirstTrackpoint() = 0;
    virtual bool getTransNextTrackpoint() = 0;
    virtual double getTransTrackpointPosx() = 0;
    virtual double getTransTrackpointPosy() = 0;
    virtual int getTransTrackpointHldirection() = 0;
    virtual int getTransTrackpointColor() = 0;
    virtual int getTransTrackpointLevel() = 0;
    virtual int getTransTrackpointID() = 0;
    virtual bool getTransTrackpointLink() = 0;
    virtual int getTransTrackpointLinkID() = 0;
    virtual bool getTransTrackpointJoin() = 0;

    virtual bool getFirstLink() = 0;
    virtual bool getNextLink() = 0;
    virtual int getLinkID() = 0;
    virtual int getLinkGoal() = 0;
    virtual int getLinkColor() = 0;
    virtual int getLinkLevel() = 0;
    virtual double getLinkPosx() = 0;
    virtual double getLinkPosy() = 0;

    virtual bool getFirstHypertrans() = 0;
    virtual bool getNextHypertrans() = 0;
    virtual int getHypertransColor() = 0;
    virtual int getHypertransID() = 0;
    virtual int getHypertransLevel() = 0;
    virtual int getHypertransGoal() = 0;
    virtual bool getHypertransType() = 0;
    virtual double getHypertransPosx() = 0;
    virtual double getHypertransPosy() = 0;
    virtual bool getHypertransFirstCondition() = 0;
    virtual bool getHypertransNextCondition() = 0;
    virtual string getHypertransConditionName() = 0;
    virtual string getHypertransConditionValue() = 0;
    virtual int getHypertransConditionID() = 0;

    virtual bool getStatebyID(int id) = 0;
    virtual bool getTransbyID(int id) = 0;
    virtual bool getTransConditionbyID(int id) = 0;
    virtual bool getTransTrackpointbyID(int id) = 0;
    virtual bool getLinkbyID(int id) = 0;
    virtual bool getHypertransbyID(int id) = 0;
    virtual bool getHypertransConditionbyID(int id) = 0;

    virtual void clockName(string s) = 0;
    virtual void resetName(string s) = 0;
    virtual string getClockName() = 0;
    virtual string getResetName() = 0;

};



#endif
