/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#define DEBUGGENERATIONOFVERILOG

#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtCore/QString>
#include <fstream>
#include <vector>

#include "generationofverilog.h"
#include "../verification/verify.h"
#include "../verification/logicmin.h"
#include "../verification/invertDNF.h"

#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"
#include "../load_I.h"
#include "../loadfactory.h"

using namespace std;

GenerationOfVerilog::GenerationOfVerilog(string str, string stri, bool gui, bool st, bool sc, bool tc) {
    resetstate = 0;
    reset = 0;
    clock = 0;
    numberofinputs = 0;
    numberofoutputs = 0;
    fsmdata = str;
    optionalpath = stri;
    g=gui;
    Statetrans = st;
    Statecov = sc;
    Transcov = tc;

    if(g==true) {
        Verify v(str);
        v.verifyFsm();
        if(v.getVerified()) {
            v.info();
            createVerilog();
            //MATADD
            //calcHammingDistance();
            //MATADD end
        } else {
            v.info();
            if(wanttogenver()) {
                createVerilog();
                //MATADD
                //calcHammingDistance();
                //MATADD end
            }
        }
    } else {
        cout<<"Not doing verification in command line mode."<<endl;
        Core_I& c = CoreFactory::getCore_I();
        Load_I& l = LoadFactory::getLoad_I();
        l.load(fsmdata);
        createVerilog();
    }
}

bool GenerationOfVerilog::wanttogenver() {
    cout<<"iwashere"<<endl;
    QWidget w;
    int genver = QMessageBox::question(&w, "Generate Verilog",
                                       "Do you still want to generate Verilog?"
                                       , QMessageBox::Yes, QMessageBox::Cancel, QMessageBox::NoButton);

    if (genver ==  QMessageBox::Yes) {
        return true;
    } else {
        return false;
    }
}

string GenerationOfVerilog::getVerilogFilename() {
    string dirname = fsmdata;
    QString name ;
    string verilogname;

    if(g==true) {
        int len = dirname.length();
        if (len > 0) {
            string s;
            s = dirname;
            do {
                len--;
            } while ( (s[len] != '/') && (len != 0));
            if (len == 0) {
                dirname = "";
            } else {
                dirname.assign(s, 0, (len + 1));
            }
        }
        if( optionalpath == "none") {
            QWidget w;
            name = QFileDialog::getSaveFileName(&w, "Choose a filename to save Verilog", QString::fromStdString(dirname), "Verilog (*.v)");
        } else {
            QWidget w;
            name = QFileDialog::getSaveFileName(&w, "Choose a filename to save Verilog",
                                                QString::fromStdString(optionalpath), "Verilog (*.v)");
        }
    } else {
        name=QString::fromStdString(optionalpath);
    }
    if(!name.isNull()) {
        int len = name.length();
        if(name.at(len - 1) == 'v' && name.at(len - 2) == '.') {
            name.remove(len - 2, len - 1);
        }
        verilogname = name.toStdString() + ".v";
    } else {
        verilogname = "";
    }
    return verilogname;
}

void GenerationOfVerilog::setMainWidget(QWidget* w) {
    wid = w;
}

void GenerationOfVerilog::createVerilog() {

    /// Data initialization
    cout<<"creating verilog"<<endl;
    Core_I& c = CoreFactory::getCore_I();
    if(c.getFirstFsm()) {
        //int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        //Fsm_I& f = c.getFsm(currentidMaingui);
        Fsm_I& f = c.getCurrentFsm();
        string filename = getVerilogFilename();
        if(!filename.empty()){
            const char* const dateiname = filename.c_str();
            ofstream out(dateiname, ios_base::out|ios_base::trunc);
            if(out.is_open()) {
                int maxlen = filename.length();
                int len = maxlen;
                if (len > 0) {
                    string s;
                    s = filename;
                    do {
                        len--;
                    } while ( (s[len] != '/') && (len != 0));
                    if (len == 0) {
                        modulename = "";
                    } else {
                        modulename.assign(s, (len + 1), (maxlen - len - 3));
                    }
                }
                numberofinputs = f.getNumberOfInputs();
                numberofoutputs = f.getNumberOfOutputs();
                resetstate = f.getResetState();

                ///errordetection

                if (numberofinputs == 0 || numberofoutputs == 0) {
                    cout << "Error: illegal input or output length" << endl;
                    out.close();
                }else{

                    //calcHammingDistance();
                    //extendStateEncoding();
                    //createStateVariations();

                    //// Write module header

                    out << "module " << modulename << " ( " ;
                    out << f.getClockName() << ", ";
                    out << f.getResetName() << ", ";
                    for(int i = 0; i < numberofinputs; i++) {
                        out << f.getInputName(i) << ", ";
                    }
                    out << endl;
                    for(int i = 0; i < numberofoutputs - 1; i++) {
                        out << f.getOutputName(i) << ", ";
                    }
                    out << f.getOutputName(numberofoutputs - 1);
                    out << " );" << endl;
                    out << endl;
                    out << "input wire ";
                    out << f.getClockName() << ", ";
                    out << f.getResetName() << ", ";
                    for(int i = 0; i < numberofinputs - 1; i++) {
                        out << f.getInputName(i) << ", ";
                    }
                    out << f.getInputName(numberofinputs - 1) << ";" << endl;
                    out << "output wire ";
                    for(int i = 0; i < numberofoutputs - 1; i++) {
                        out << f.getOutputName(i) << ", ";
                    }
                    out << f.getOutputName(numberofoutputs - 1)  << ";" << endl << endl;
                    if(f.getFirstState()) {
                        do {
                            out << "parameter " << f.getStateName() << " = " << numberofoutputs << "'b" << f.getStateOutput() << ";" << endl;
                        } while(f.getNextState());
                    }
                    out << endl << "reg [" << (numberofoutputs - 1) << ":0] current_state, next_state;" << endl;
                    out << "assign {";
                    for(int i = 0; i < numberofoutputs - 1; i++) {
                        out << f.getOutputName(i) << ", ";
                    }
                    out << f.getOutputName(numberofoutputs - 1);
                    out << "} = current_state;" << endl << endl;
                    out << "wire [" << (numberofinputs - 1) << ":0] inputvector;" << endl;
                    out << "assign inputvector = {";
                    for(int i = 0; i < numberofinputs - 1; i++) {
                        out << f.getInputName(i) << ", ";
                    }
                    out << f.getInputName(numberofinputs - 1);
                    out << "};" << endl << endl << endl;

                    //// Write combinational logic always-block

                    out << "always @(*) begin" << endl;
                    out << "  casex({inputvector, current_state})" << endl;
                    if(f.getFirstState()) {
                        int stateid;
                        string statename;
                        do {
                            stateid = f.getStateID();
                            statename = f.getStateName();
                            if(f.getFirstTrans()) {
                                do {
                                    if(f.getTransStart() == stateid) {
                                        int transid = f.getTransID();
                                        f.getStatebyID(f.getTransEnd());
                                        if(f.getTransDefault()) {

                                            // Determine Inputs
                                            vector<string> input;
                                            bool notfound = true;
                                            if(f.getFirstTrans()) {
                                                do {
                                                    if(f.getTransStart() == stateid && !f.getTransDefault()) {
                                                        if(f.getTransFirstCondition()) {
                                                            notfound = false;
                                                            do {
                                                                string condition = f.getTransConditionValue();
                                                                string result = "";
                                                                char X;
                                                                for(int i = 0; i < numberofinputs; i++) {
                                                                    X = condition[i];
                                                                    if( X == 'x' || X == 'X') {
                                                                        result = result + '-';
                                                                    } else {
                                                                        result = result + X;
                                                                    }
                                                                }
                                                                input.push_back(result);
                                                            } while (f.getTransNextCondition());
                                                        }
                                                    }
                                                } while (f.getNextTrans());
                                            }
                                            if(f.getFirstHypertrans()) {
                                                do {
                                                    if(f.getHypertransFirstCondition()) {
                                                        notfound = false;
                                                        do {
                                                            string condition = f.getHypertransConditionValue();
                                                            string result = "";
                                                            char X;
                                                            for(int i = 0; i < numberofinputs; i++) {
                                                                X = condition[i];
                                                                if( X == 'x' || X == 'X') {
                                                                    result = result + '-';
                                                                } else {
                                                                    result = result + X;
                                                                }
                                                            }
                                                            input.push_back(result);
                                                        } while(f.getHypertransNextCondition());
                                                    }
                                                } while (f.getNextHypertrans());
                                            }

                                            // Go on if we have inputs
                                            if(input.size()>0) {
                                                //the old invert functionality did not produce correct results under rare circumstances
                                                //pair<string, string> result;
                                                //logicMin(input,result);
                                                //Testschleife
                                                //for(int i = 0; i < input.size() ; i++) {
                                                //cout << input[i] << endl;
                                                //}



                                                InvertDNF INV;
                                                string defaultvalue = INV.invert(input);
                                                int maxlen = defaultvalue.length();
                                                int len = maxlen;
                                                if(len > 0) {


													out << "    {" << numberofinputs << "'b";

                                                    while (len > numberofinputs) {
                                                        string s = defaultvalue;
                                                        do {
                                                            len--;
                                                        } while ( (s[len] != '+') && (len != 0));
                                                        s.assign(s, (len + 1), (maxlen - len - 1));
                                                        defaultvalue.assign(defaultvalue, 0, len);
                                                        char X;
                                                        for(int i = 0; i < numberofinputs; i++) {
                                                            X = s[i];
                                                            if( X == '-' || X == 'X') {
                                                                out << "x";
                                                            } else {
                                                                out << X;
                                                            }
                                                        }
                                                        out << ", " << statename << "}:   next_state = " << f.getStateName() << ";" << endl;
                                                        out << "    {" << numberofinputs << "'b";
                                                    }

                                                    char X;
                                                    for(int i = 0; i < numberofinputs; i++) {

                                                        X = defaultvalue[i];
                                                        if( X == '-' || X == 'X') {
                                                            out << "x";
                                                        } else {
                                                            out << X;
                                                        }
                                                    }

                                                    out << ", " << statename << "}:   next_state = " << f.getStateName() << ";" << endl;
                                                }
                                            }
                                            if(notfound) {
                                            	out << "    {" << numberofinputs << "'b";
                                                string invec = "";
                                                for(int i = 0; i < numberofinputs; i++) {
                                                    invec = invec + "x";
                                                }
                                                out << invec << ", " << statename << "}:   next_state = " << f.getStateName() << ";" << endl;
                                            }
                                            f.getTransbyID(transid);
                                        } else {
                                            if(f.getTransFirstCondition()) {
                                                string condition;
                                                do {
                                                    out << "    {" << numberofinputs << "'b";
                                                    condition = f.getTransConditionValue();
                                                    char X;
                                                    for(int i = 0; i < numberofinputs; i++) {
                                                        X = condition[i];
                                                        if( X == '-' || X == 'X') {
                                                            out << "x";
                                                        } else {
                                                            out << X;
                                                        }
                                                    }
                                                    out << ", " << statename << "}:   next_state = " << f.getStateName() << ";" << endl;
                                                } while (f.getTransNextCondition());
                                            }
                                        }
                                    }
                                } while (f.getNextTrans());
                            }
                            f.getStatebyID(stateid);
                        } while (f.getNextState());
                    }
                    if(f.getFirstHypertrans()) {
                        do {
                            if(f.getHypertransFirstCondition()) {
                                string condition;
                                do {
                                    out << "    {" << numberofinputs << "'b";
                                    condition = f.getHypertransConditionValue();
                                    char X;
                                    for(int i = 0; i < numberofinputs; i++) {
                                        X = condition[i];
                                        if( X == '-' || X == 'X') {
                                            out << "x";
                                        } else {
                                            out << X;
                                        }
                                    }
                                    out << ", " << numberofoutputs << "'b";
                                    for(int i = 0; i < numberofoutputs; i++) {
                                        out << "x";
                                    }
                                    f.getStatebyID(f.getHypertransGoal());
                                    out << "}:   next_state = " << f.getStateName() << ";" << endl;
                                } while (f.getHypertransNextCondition());
                            }
                        } while (f.getNextHypertrans());
                    }
                    out << "    default:  next_state = ";
                    if(f.getFirstState()) {
                        bool notfound = true;
                        do {
                            if(resetstate == f.getStateID()) {
                                out << f.getStateName();
                                notfound = false;
                            }
                        } while (f.getNextState());
                        if(notfound) {
                            if(f.getFirstState()) {
                                out << f.getStateName();
                            }
                        }
                    }
                    out << ";" << endl;
                    out << "  endcase" << endl;
                    out << "end" << endl << endl;

                    //// Write always-block => assigns next_state to state

                    out << "always @(posedge " << f.getClockName() << " or negedge " << f.getResetName() <<" ) begin" << endl;
                    out << "  if (" << f.getResetName() << "  == 1'b0)" << endl;
                    out << "    current_state <= ";
                    if(f.getFirstState()) {
                        bool notfound = true;
                        do {
                            if(resetstate == f.getStateID()) {
                                out << f.getStateName();
                                notfound = false;
                            }
                        } while (f.getNextState());
                        if(notfound) {
                            if(f.getFirstState()) {
                                out << f.getStateName();
                            }
                        }
                    }
                    out << ";" << endl;
                    out << "  else " << endl;
                    out << "    current_state <= next_state;" << endl;
                    out << "end" << endl;



                    //// Verification Exentions, can be used with Cadence formal verifier

                    if(Statetrans || Statecov || Transcov) {

                        out << endl << endl << "// synopsys translate_off" << endl << endl;

                        ///// declare helper variables

                        if(f.getFirstState()) {
                            do {
                                out << "wire state_" << f.getStateName() << ";" << endl;
                            } while (f.getNextState());
                        }
                        out << endl;
                        if(f.getFirstState()) {
                            do {
                                out << "assign state_" << f.getStateName() << " = (current_state == " << f.getStateName() << ");" << endl;
                            } while (f.getNextState());
                        }
                        out << endl;

                        //// Assertions for legal state transitions

                        if(Statetrans) {
                            if(f.getFirstState()) {
                                int stateid;
                                string statename;
                                do {
                                    stateid = f.getStateID();
                                    statename = f.getStateName();
                                    out << "a_legal_trans_"<< statename << " : assert property (@(posedge " << f.getClockName() << ") disable iff (!" << f.getResetName() << ") state_"<< statename << " |=> (";
                                    if(f.getFirstState()) {
                                        int endstate;
                                        bool firsthit = true;
                                        bool reached = false;
                                        do {
                                            endstate = f.getStateID();
                                            if(f.getFirstTrans()) {
                                                do {
                                                    if(f.getTransStart() == stateid) {
                                                        if(f.getTransEnd() == endstate) {
                                                            reached = true;
                                                        }
                                                    }
                                                } while (f.getNextTrans() && !reached);
                                            }
                                            if(reached) {
                                                reached = false;
                                                if(firsthit) {
                                                    out << "state_"<< f.getStateName();
                                                    firsthit = false;
                                                } else {
                                                    out << " || state_" << f.getStateName();
                                                }
                                            }
                                        } while (f.getNextState());
                                    }

                                    out << "));" << endl;
                                    f.getStatebyID(stateid);
                                } while (f.getNextState());
                            }
                            out << endl;
                        }


                        //// Transition coverage checking

                        if(Transcov) {
                            if(f.getFirstState()) {
                                int stateid;
                                string statename;
                                do {
                                    stateid = f.getStateID();
                                    statename = f.getStateName();
                                    if(f.getFirstState()) {
                                        int endstate;
                                        bool exist = false;
                                        do {
                                            endstate = f.getStateID();
                                            if(f.getFirstTrans()) {
                                                do {
                                                    if(f.getTransStart() == stateid) {
                                                        if(f.getTransEnd() == endstate) {
                                                            exist = true;
                                                        }
                                                    }
                                                } while (f.getNextTrans() && !exist);
                                            }
                                            if(exist) {
                                                out << "c_trans_"<< statename << "_" << f.getStateName() << " : cover property (@(posedge " << f.getClockName() << ") disable iff (!" << f.getResetName() << ") state_"<< statename << " |=> state_" << f.getStateName() << ");" << endl;

                                                exist = false;
                                            }
                                        } while (f.getNextState());
                                    }
                                    f.getStatebyID(stateid);
                                } while (f.getNextState());
                            }

                            out << endl;
                        }

                        //// State coverage checking

                        if(Statecov) {
                            if(f.getFirstState()) {
                                do {
                                    out << "c_state_" << f.getStateName() << " : cover property (@(posedge "<< f.getClockName() <<") disable iff (!"<< f.getResetName() <<") " << f.getStateName() << ");" << endl;
                                } while (f.getNextState());
                            }
                            out << endl;
                        }

                        out << endl << "// synopsys translate_on" << endl << endl;
                    }

                    out << endl << "endmodule" << endl;

                    out.close();
                }
            }
        }
    }
}

void GenerationOfVerilog::calcHammingDistance() {

    /// Data initialization
    cout<<"calculating Hamming Distance of original States"<<endl;
    Core_I& c = CoreFactory::getCore_I();
    originalStateOutputs = QStringList();
    originalStateNames= QStringList();
    originalStateName_index.clear();
    desired_distance = 0;
    min_distance = numberofoutputs;
    int distance[numberofoutputs][numberofoutputs];
    int numberOfStates = 0;

    //USER Input simulation
    desired_distance = 3;

    if(c.getFirstFsm())
    {
        Fsm_I& f = c.getCurrentFsm();
        //Extend and Change the parameter string containing the current_state to a bucket of similar states.
        //Put original State Names and its binary coding into QStringLists
        int numberOfStates=0;
        if(f.getFirstState())
        {
            do {
                originalStateNames << QString(f.getStateName().c_str());
                originalStateOutputs << QString(f.getStateOutput().c_str());
                originalStateName_index.push_back(numberOfStates);
                //cout << "parameter " << f.getStateName() << " = " << numberofoutputs << "'b" << f.getStateOutput() << ";" << endl;
                numberOfStates++;
            } while(f.getNextState());
        }
#ifdef DEBUGGENERATIONOFVERILOG
        qDebug()<<"Number of State Outputs in originalStateOutputs :"<<originalStateOutputs.size();
        qDebug()<<"Number of State Names in originalStateNames :"<< originalStateNames.size();
        string teststr1 ("11010000");
        string teststr2 ("01010011");
        qDebug()<<"Hamming distance of a: "<<QString(teststr1.c_str()) <<"and b: "<<QString(teststr2.c_str())<<"is: "<<calcXOR(teststr1, teststr2);
        //qDebug()<<"Number of States: "<<numberOfStates;
#endif
        //Generate 2D-table with the Hamming Distance of each State to the other States,
        //save in distance[i][j]
        for(int i=0;i<originalStateOutputs.size();i++)
        {
            for(int j=0;j<originalStateOutputs.size();j++)
            {
                distance[i][j]=calcXOR(originalStateOutputs.at(i).toStdString(), originalStateOutputs.at(j).toStdString() );
                if ((distance[i][j] < min_distance) && (i!=j) )
                {
                    min_distance = distance[i][j];
                }
#ifdef DEBUGGENERATIONOFVERILOG
                qDebug()<<"Distance between: "<<i <<"th state = "<<originalStateOutputs.at(i)<<" and " <<j  <<"th state = "<<originalStateOutputs.at(j)<<" is " << distance[i][j];
                qDebug()<<"minimal Distance is :"<<min_distance;
                qDebug()<<"Number of States: "<<numberOfStates;
#endif
            }
        }

    }
}

int GenerationOfVerilog::calcExtendedHammingDistance(QString testedString) {

    /// Data initialization
    cout<<"calculating Hamming Distance of extended States"<<endl;
    Core_I& c = CoreFactory::getCore_I();
    int ext_distance[numberofoutputs][numberofoutputs];
    ext_min_distance = numberofoutputs;
    QStringList cur_extendedStateEncoding = extendedStateEncoding;
    cur_extendedStateEncoding << testedString;

#ifdef DEBUGGENERATIONOFVERILOG
        qDebug()<<"Number of State Outputs in extendedStateEncoding :"<<extendedStateEncoding.size();
#endif
        //Generate 2D-table with the Hamming Distance of each State to the other States,
        //save in distance[i][j]
        for(int i=0;i<cur_extendedStateEncoding.size();i++)
        {
            for(int j=0;j<cur_extendedStateEncoding.size();j++)
            {
                ext_distance[i][j]=calcXOR(cur_extendedStateEncoding.at(i).toStdString(), cur_extendedStateEncoding.at(j).toStdString() );
                if ((ext_distance[i][j] < ext_min_distance) && (i!=j) )
                {
                    ext_min_distance = ext_distance[i][j];
                }
#ifdef DEBUGGENERATIONOFVERILOG
                qDebug()<<"Distance between extended: "<<i <<"th state = "<<cur_extendedStateEncoding.at(i)<<" and " <<j  <<"th state = "<<cur_extendedStateEncoding.at(j)<<" is " << ext_distance[i][j];
                qDebug()<<"minimal Distance is :"<<ext_min_distance;
#endif
            }
        }
return ext_min_distance;
    }


int GenerationOfVerilog::calcXOR(string stringA, string stringB ) {

    int dist = 0;

    if (stringA.length() == stringB.length())
    {
        int i;
        for(i=0; i < stringA.length(); i++)
        {
            if( ((stringA[i] == '0') && (stringB[i] == '0')) || ((stringA[i] == '1') && (stringB[i] == '1')) )
            {
                xorDifference[i] = '0';
            }
            else if( ((stringA[i] == '1') && (stringB[i] == '0')) || ((stringA[i] == '0') && (stringB[i] == '1')) )
            {
                xorDifference[i] = '1';
                dist++;
            }
            else if( (stringA[i] == '-')  ||  (stringB[i] == '-') || (stringA[i] == 'x')  ||  (stringB[i] == 'x') || (stringA[i] == 'X')  ||  (stringB[i] == 'X') )
            {
                xorDifference[i] = '0';
            }
        }

    }

    return dist;

}

void GenerationOfVerilog::extendStateEncoding() {

    /// Data initialization
    cout<<"extending State Output encoding"<<endl;
    if ((desired_distance - min_distance) >0)
    {
        extendedBy = desired_distance - min_distance;
    } else {
        extendedBy =0;
    }
    int cur_extendedBy = extendedBy;
    current_extension_int = 0;
    extendedStateEncoding = QStringList();
    QStringList already_used_extensions = QStringList();

    current_extension_int = 0; //test
    //current_extension_string = QString::number( current_extension_int , 2);
    current_extension_string = QString( "%1" ).arg( current_extension_int, extendedBy, 2);
    current_extension_string.replace(QString(" "), QString("0"));  // replace space character with 0s

#ifdef DEBUGGENERATIONOFVERILOG
    QString str2;
    str2 = QString( "Decimal 10 is %1 in binary" ).arg( 10, 12, 2);
    str2.replace(QString(" "), QString("0"));
    qDebug()<<str2;
    qDebug()<<current_extension_int <<" as String: "<<current_extension_string;
    qDebug()<< "desired_distance: "<<desired_distance <<" extendedBy: "<<extendedBy;
#endif
    //extendedStateEncoding
    //QString originalStateOutputs
    //        apppend

    for(cur_extendedBy=extendedBy; extendedStateEncoding.size()<originalStateOutputs.size(); cur_extendedBy++)
    {
#ifdef DEBUGGENERATIONOFVERILOG
        qDebug()<< "extendedStateEncoding size: "<<extendedStateEncoding.size() <<" originalStateOutputs size: "<<originalStateOutputs.size();
        qDebug()<< "cur_extendedBy: "<<cur_extendedBy;
#endif
        for(current_extension_int=0; current_extension_int<(2^cur_extendedBy); current_extension_int++)
        {
            current_extension_string = QString( "%1" ).arg( current_extension_int, cur_extendedBy, 2);
            current_extension_string.replace(QString(" "), QString("0"));  // replace space character with 0s
            QString complete_string = originalStateOutputs.at(extendedStateEncoding.size()-1); //?
            complete_string.append(current_extension_string);
#ifdef DEBUGGENERATIONOFVERILOG
        qDebug()<< "current_extension_int: "<<current_extension_int <<" current_extension_string: "<<current_extension_string;
        qDebug()<< "2^cur_extendedBy: "<<(2^cur_extendedBy) <<" complete_string: "<<complete_string;
#endif
            bool in_ext_list = false;
            for (int ext_list_counter = 0; ext_list_counter < already_used_extensions.size(); ++ext_list_counter)
            {
                if (already_used_extensions.at(ext_list_counter)==current_extension_string)
                {   in_ext_list = true; }
#ifdef DEBUGGENERATIONOFVERILOG
        qDebug()<< "already_used_extensions.at(i): "<<already_used_extensions.at(ext_list_counter) <<" current_extension_string: "<<current_extension_string;
        qDebug()<< "in_ext_list: "<<in_ext_list;
#endif
            }
            if ((in_ext_list) && (calcExtendedHammingDistance(complete_string)>=desired_distance) )
            {
                extendedStateEncoding << complete_string;
                already_used_extensions << current_extension_string;
            }

        }
    }

    /*
while(extendedStateEncoding.size()<originalStateOutputs.size())
    {
#ifdef DEBUGGENERATIONOFVERILOG
        // qDebug()<< "extendedStateEncoding size: "<<extendedStateEncoding.size() <<" originalStateOutputs size: "<<originalStateOutputs.size();
#endif
        for (current_extension_int=0; current_extension_int<(2^cur_extendedBy); current_extension_int++)
        {
            current_extension_string = QString( "%1" ).arg( current_extension_int, cur_extendedBy, 2);
            current_extension_string.replace(QString(" "), QString("0"));  // replace space character with 0s
            QString complete_string = originalStateOutputs.at(extendedStateEncoding.size());
            complete_string.append(current_extension_string);
            if (calcExtendedHammingDistance(complete_string)>=desired_distance )
            {
                extendedStateEncoding << complete_string;
            }
            else{
                //cur_extendedBy++;
            }

        }

    }
*/
           /* for(int i=0;i<originalStateOutputs.size();i++)
            {
                QString currentExtendedStateEncoding = originalStateOutputs.at(i);
                //current_extension_int = 1;
                current_extension_string = QString( "%1" ).arg( current_extension_int, extendedBy, 2);
                current_extension_string.replace(QString(" "), QString("0"));  // replace space character with 0s
                QString complete_string = currentExtendedStateEncoding;
                complete_string.append(current_extension_string);
                if (calcExtendedHammingDistance(complete_string)>=desired_distance );
                {
                    extendedStateEncoding.at(i) = complete_string;
                }
                current_extension_int++;
            }
            */


        }

        void GenerationOfVerilog::createStateVariations() {

            /// Data initialization
            cout<<"create Variations with SEUs of States"<<endl;



        }
