#include "./xmlread.h"

#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtCore/QString>
#include <fstream>


#include "../verification/verify.h"
#include "../verification/logicmin.h"

#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"
#include "../load_I.h"
#include "../loadfactory.h"



XERCES_CPP_NAMESPACE_USE

void cmdsyntaxerror(int i)
{
	cout<<"Usage: fsmvhdlgeneration [testbench] [gui] infile outfile"<<endl;
	cout<<"	If you add the option 'testbench', a testbench will be generated instead."<<endl;
	cout<<"	If you add the option 'gui', you will have some gui output."<<endl;
	cout<<"		In this case 'outfile' is optional and is interpreted as"<<endl;
	cout<<"		search path for your output file's directory."<<endl;
	cout<<i<<endl;
};

int main (int argc, char* args[]) 
{
	//if(argc<3)
	//{
	//		cmdsyntaxerror(0);
	//		return(1);
	//}
  	//if(strcmp(args[2],"gui")==0 || strcmp(args[1],"gui")==0)QApplication a( argc, args );
	try 
	{
		XMLPlatformUtils::Initialize();
        }
        catch (const XMLException& toCatch) 
	{
            char* message = XMLString::transcode(toCatch.getMessage());
            cout << "Error during initialization! :\n";
            cout << "Exception message is: \n"
                 << message << "\n";
            XMLString::release(&message);
            return 1;
        }

        xmlread* defaultHandler;
	char* xmlFile;
	char* vhdlFile;
	bool gui=false;

	cout<<argc<<endl;
	//read commandline arguments
	if(argc<3)
	{
			cmdsyntaxerror(0);
			return(1);
	}
	else
	{

		if(strcmp(args[1],"gui")==0)
		{
			gui=true;

			// gui testbench infile [outfile]
			if(strcmp(args[2],"testbench")==0)
			{
				if(argc==5)
				{
					vhdlFile=args[4];
					xmlFile=args[3];
				}
				else if(argc==4) 
				{
					vhdlFile="./";
					xmlFile=args[3];
				}
				else
				{
					cmdsyntaxerror(1);
					return 1;
				};

				defaultHandler = new xmlread(vhdlFile,true,true);
			}
			// gui infile [outfile]
			else
			{
				
				if(argc==4)
				{
					vhdlFile=args[3];
					xmlFile=args[2];
				}
				else if(argc==3)
				{
					vhdlFile="./";
					xmlFile=args[2];
				}
				else
				{
					cmdsyntaxerror(2);
					return 1;
				};
				
				defaultHandler = new xmlread(vhdlFile,false,true);
			};
		}
		else if(strcmp(args[1],"testbench")==0)
		{
			//testbench gui infile [outfile]
			if(strcmp(args[2],"gui")==0)
			{
				gui=true;
				if(argc==5)
				{
					vhdlFile=args[4];
					xmlFile=args[3];
				}
				else if(argc==4)
				{
					vhdlFile="./";
					xmlFile=args[3];
				}
				else
				{
					cmdsyntaxerror(3);
					return 1;
				};

				defaultHandler = new xmlread(vhdlFile,true,true);
			}
			//testbench infile outfile
			else
			{
				if(argc==4)
				{
					vhdlFile=args[3];
					xmlFile=args[2];
				}
				else 
				{
					cmdsyntaxerror(4);
					return 1;
				};
				
				defaultHandler = new xmlread(vhdlFile,true,false);
			};
		}
		//infile outfile
		else
		{
			if(argc==3)
			{
				cout<<"infile outfile"<<endl;
				vhdlFile=args[2];
				xmlFile=args[1];
			}
			else
			{
				cmdsyntaxerror(5);
				return 1;
			};

			defaultHandler = new xmlread(vhdlFile,false,false);
		};
	};
	cout<<"Parsing "<<xmlFile<<endl;	
	cout<<"Into "<<vhdlFile<<endl;	

	SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();
        parser->setFeature(XMLUni::fgSAX2CoreValidation, true);   // optional
        parser->setFeature(XMLUni::fgSAX2CoreNameSpaces, false);   // optional

        parser->setContentHandler(defaultHandler);
        parser->setErrorHandler(defaultHandler);

        try {
		if(gui==true)
		{
			QApplication a( argc, args );
	    		Verify V(xmlFile);
			V.verifyFsm();
			if(V.getVerified())
			{
				V.info();
            			parser->parse(xmlFile);
			}
			else 
			{
				cout<<"\nVerification failed. Parsing anyway."<<endl;
				V.info();
            			parser->parse(xmlFile);
			};
		}
		else 
		{
			cout<<"Not doing verification in command line mode."<<endl;
            		parser->parse(xmlFile);
		}
        }
        catch (const XMLException& toCatch) {
            char* message = XMLString::transcode(toCatch.getMessage());
            cout << "Exception message is: \n"
                 << message << "\n";
            XMLString::release(&message);
            return -1;
        }
        catch (const SAXParseException& toCatch) {
            char* message = XMLString::transcode(toCatch.getMessage());
            cout << "Exception message is: \n"
                 << message << "\n";
            XMLString::release(&message);
            return -1;
        }
        catch (...) {
            cout << "Unexpected Exception \n" ;
            return -1;
        }

	delete parser;
        delete defaultHandler;
        return 0;
    }

