#include"xmlread.h"


//Init
xmlread::xmlread(char* vhdlFile, bool testbench, bool gui)
{
	g=gui;
	tb=testbench;
	//result="";
	chars_append=false;
	chars_data="";

	outF=vhdlFile;
};


//Start of an Element
void xmlread::startElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname, const Attributes& att)
{
  	char* tagname = XMLString::transcode(qname);
	
	//start of a transition
	if(strcmp(tagname,"trans")==0)
	{
		current_trans.isHyperTrans=false;
		current_trans.Input.clear();
		current_trans.NState="";
		current_trans.Start="";
		current_trans.Default="";
		chars_append=false;
	}
	
	//start of an input
	else if(strcmp(tagname,"input")==0)
		chars_append=true;
	
	//start of a start tag
	else if(strcmp(tagname,"start")==0)
		chars_append=true;
	
	//start of an end tag
	else if(strcmp(tagname,"end")==0)
		chars_append=true;
	
	//start of a default tag
	else if(strcmp(tagname,"default")==0)
		chars_append=true;
	
 	//start of a state
	else if(strcmp(tagname,"state")==0)
	{
		current_state.StateName="";
		current_state.Output="";
		current_state.Transitions.clear();
		chars_append=false;
	}
	
	//start of a state name. get chars 
	else if(strcmp(tagname,"sname")==0)
		chars_append=true;
	
	//start of state output. get chars
	else if(strcmp(tagname,"output")==0)
		chars_append=true;
	
	//start of a goal tag 
	else if(strcmp(tagname,"goal")==0)
		chars_append=true;
	
	//start of a hypertransition
	if(strcmp(tagname,"hypertrans")==0)
	{
		current_trans.isHyperTrans=true;
		current_trans.Input.clear();
		current_trans.NState="";
		current_trans.Start="";
		current_trans.Default="";
		chars_append=false;
	}

	//start of fsm
	else if(strcmp(tagname,"fsm")==0)
	{
		//read all the fsm's attributes into corresponding class-variables
		fsmname=XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0));
		resetstate=XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 1));
		numinputs=XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2));
		numoutputs=XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 3));
	}

};


//End of an Element
void xmlread::endElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname)
{
  	char* tagname = XMLString::transcode(qname);

	//end of parsing a transition
	if(strcmp(tagname,"trans")==0)
		TransList.push_back(current_trans);
	//end of parsing an (transitions condition) input
	else if(strcmp(tagname,"input")==0)
		current_trans.Input.push_back(chars_data);
	//end of parsing a start tag
	else if(strcmp(tagname,"start")==0)
		current_trans.Start=chars_data;
	//end of parsing an end tag
	else if(strcmp(tagname,"end")==0)
		current_trans.NState=chars_data;
	//end of parsing a default tag
	else if(strcmp(tagname,"default")==0)
		current_trans.Default=chars_data;
	//end of parsing a state
	else if(strcmp(tagname,"state")==0)
		StateTable.push_back(current_state);
	//completion of state name
	else if(strcmp(tagname,"sname")==0)
		current_state.StateName=chars_data;
	//completion of output
	else if(strcmp(tagname,"output")==0)
		current_state.Output=chars_data;
	//end of parsing a hypertransition (treat like normal transition)
	else if(strcmp(tagname,"hypertrans")==0)
		TransList.push_back(current_trans);
	//completion of goal tag (treat like end tag in normal transition)
	else if(strcmp(tagname,"goal")==0)
		current_trans.NState=chars_data;
	//completion of the whole fsm
	else if(strcmp(tagname,"fsm")==0)
	{
		//put each transition into the transition list of
		//the state that is it's "<start>"
		for(int i=0; i<TransList.size(); i++)
		{
			if(TransList[i].isHyperTrans==true)
			{
				for(int j=0; j<StateTable.size(); j++)
					StateTable[j].Transitions.push_back(TransList[i]);
			}
			else
			{
				int pos=atoi(TransList[i].Start.c_str());
				StateTable[pos].Transitions.push_back(TransList[i]);
				if(atoi(TransList[i].Default.c_str())==1)
					StateTable[pos].Default=StateTable[atoi(TransList[i].NState.c_str())].StateName;
			};
		};
	};

	//reset character buffer
	if(chars_append==true)
	{
		chars_data="";
		chars_append=false;
	};
};


//Start of the Document
void xmlread::startDocument()
{
	cout<<"Starting to parse"<<endl;
	//Nothing to do(?)
};

string xmlread::getVHDLFilename() 
{
  string dirname = outF;
  QString name ;

  if(g==true)
  {
  	int len = dirname.length();
  	if (len > 0) {
    	string s;
    	s = dirname;
  	  do {
      	len--;
    	} while ( (s[len] != '/') && (len != 0));
   	 if (len == 0) {
      	dirname = "";
    	} else {
      	dirname.assign(s, 0, (len + 1));
    	}
 	}
  
  	QWidget w;
  	name = QFileDialog::getSaveFileName(&w, "Choose a filename to save VHDL", QString::fromStdString(dirname), "VHDL (*.vhd)");
  }
  else
	name=QString::fromStdString(outF);  

  string vhdlname;
  if(!name.isNull()) 
  {
    int len = name.length();
    if(name.at(len - 1) == 'd' && name.at(len - 2) == 'h' && name.at(len - 3) == 'v' && name.at(len - 4) == '.') 
    	vhdlname=name.toStdString();
    else 
	vhdlname = name.toStdString() + ".vhd";
  } 
  else 
  {
    vhdlname = "";
  }
  return vhdlname;
}


//As the document is ended now
//we definitely have all the needed
//information to generate the vhdl-code
void xmlread::endDocument()
{

	//VHDL generation
	if(tb==false)
	{
		cout<<"Through the parsing, starting vhdl generation"<<endl;

		ofstream fout;
		string vhdlfilename=getVHDLFilename();
		if(vhdlfilename=="")
		{
			cout<<"File selection cancelled."<<endl;
			exit(2);
		}	
		else 
			fout.open(vhdlfilename.c_str());

		//Header with comment with the FSMs name
		//and library includes
		fout<<"-- ";
		fout<<fsmname;
		fout<<"\n-- Libraries";
		fout<<"\n library IEEE;";
		fout<<"\n use IEEE.STD_LOGIC_1164.ALL;";
		fout<<"\n use IEEE.STD_LOGIC_ARITH.ALL;"; 
		fout<<"\n use IEEE.STD_LOGIC_UNSIGNED.ALL;"; 
		fout<<"\n use IEEE.numeric_std.ALL;";
		fout<<"\n --LIBRARY UNISIM;";
		fout<<"\n --USE UNISIM.Vcomponents.ALL;";

		//Entity Description
		fout<<"\n\n-- Entity Description with In and Outputs";
		fout<<"\nentity ";
		fout<<fsmname;
		fout<<" is";
        	fout<<"\nport";
		fout<<"\n(";
        	fout<<"\n 	clk : in std_logic;    	 -- Clock";
        	fout<<"\n 	rst : in std_logic;     	 -- Reset";
        	fout<<"\n 	inp  : in std_logic_vector (";
		fout<<(atoi(numinputs.c_str())-1);
		fout<<" downto 0); -- Input";
        	fout<<"\n 	outp : out std_logic_vector(";
		fout<<(atoi(numoutputs.c_str())-1);
		fout<<" downto 0) -- Output";
        	fout<<"\n);";
		fout<<"\nend;";

		//Behaviour Description
		fout<<"\n\n--Behaviour Description of the FSM"; 
		fout<<"\narchitecture Behaviour of ";
		fout<<fsmname;

		//Definition of Types
		//States:
		fout<<" is";
		fout<<"\n 	type state is (";
		fout<<StateTable[0].StateName;

		//List all States
		for(int i=1; i<StateTable.size(); i++)
		{
			fout<<", ";
			fout<<StateTable[i].StateName;
		};

		//Some Definitions
		// + Sync Process
		fout<<");";
		fout<<"\n 	signal recState, nextState : state;";
		fout<<"\nbegin";
        	fout<<"\n\n	-- Synchronize, set new state";
        	fout<<"\n 	synchronisation : process(clk, rst)";
        	fout<<"\n 	begin";
        	fout<<"\n 	if(rst='0') then";
        	fout<<"\n 		recState <= " + StateTable[atoi(resetstate.c_str())].StateName +";";
        	fout<<"\n 	elsif clk'event and clk='1' then";
        	fout<<"\n 		recState <= nextState;";
		fout<<"\n	end if;";
        	fout<<"\n 	end process;";
     	
		//Transition Process	
		fout<<"\n\n	--Calculate new state from input and current state";
       		fout<<"\n 	transition : process(recState, inp)";
       		fout<<"\n 	begin";
		fout<<"\n 	case recState is";

		bool firstreal;
		//List all the states with possible Inputs and resulting Transitions
		for(int i=0; i<StateTable.size(); i++)
		{
			firstreal=true;
			//fout<<"\n-- State "<<i<<endl;
			fout<<"\n		when ";
			fout<<StateTable[i].StateName;
			fout<<" =>";

			/*Using case and simple comparision in vhdl together with don't-cares will not work*/
			/*Use if-clauses and the std_match function instead*/
			
			//Loop through the transitions starting from the current state
			for(int j=0; j<StateTable[i].Transitions.size(); j++)
			{
				//fout<<"\n-- State "<<i<<" Transition "<<j<<endl;
				//fout<<"\n-- Transition is triggered by "<<StateTable[i].Transitions[j].Input.size()<<" different Inputs."<<endl;
				/*if(StateTable[i].Transitions[j].Input.size()==0)
					fout<<"-- Transition to "<<StateTable[atoi(StateTable[i].Transitions[j].NState.c_str())].StateName<<endl;*/
				//A transition can be triggered by several different inputs
				for(int k=0; k<StateTable[i].Transitions[j].Input.size(); k++)
				{
					//fout<<"\n-- State "<<i<<" Transition "<<j<<"+"<<k<<": ";
					if((j==0 && k==0) || firstreal==true)
						fout<<"\n			if";
					else
						fout<<"\n			elsif";
					fout<<" std_match(inp,\""<<StateTable[i].Transitions[j].Input[k]<<"\") then";
					fout<<"\n                                     nextState <= ";
                               		fout<<StateTable[atoi(StateTable[i].Transitions[j].NState.c_str())].StateName;
					fout<<";";
					firstreal=false;
				};
			};

			//Add the default case (use default transition or just stay here)
			if(StateTable[i].Transitions.size()>0)
				fout<<"\n			else";
                        fout<<"\n                                     nextState <= ";
			if(StateTable[i].Default!="")
                                fout<<StateTable[i].Default;
                        else
                                fout<<"recState";
                        fout<< ";";
			fout<<"\n			end if;";

		};

		//Default case for nonexisting states 		
		if(StateTable.size()>0)
			fout<<"\n		when others =>";
                fout<<"\n                       	nextState <= ";
                fout<<StateTable[atoi(resetstate.c_str())].StateName;
                fout<< ";";

		fout<<"\n 	end case;";
		fout<<"\n 	end process;";
		//Output Process
       		fout<<"\n\n	-- Produce outputs from information about the recent state";
        	fout<<"\n 	output : process(recState)";
       		fout<<"\n 	begin";
		fout<<"\n 	case recState is";
	
		//List all the states and their outputs
		for(int i=0; i<StateTable.size(); i++)
		{
			fout<<"\n		when ";
			fout<<StateTable[i].StateName;
			fout<<"=>\n			outp <= ";
			fout<<"\""+StateTable[i].Output+"\"";
			fout<<";";
		};
	
		//Finish
		fout<<"\n	 end case;";
        	fout<<"\n 	 end process;";
		fout<<"\nend; \n";
	
		fout.close();
		cout<<"Done!"<<endl;
	}

	//Testbench generation
	else
	{
		cout<<"Parsing done, generating testbench."<<endl;

		ofstream fout;
		string vhdlfilename=getVHDLFilename();
		if(vhdlfilename=="")
		{
			cout<<"File selection cancelled."<<endl;
			exit(2);
		}	
		else 
			fout.open(vhdlfilename.c_str());	
		
		fout<<"library IEEE;\n";
		fout<<"use IEEE.std_logic_1164.all;\n";
		fout<<"use IEEE.std_logic_textio.all;\n";
		fout<<"use STD.textio.all;"<<endl<<endl;

		fout<<"entity "<<fsmname<<"_test is\n";
		fout<<"end "<<fsmname<<"_test;"<<endl<<endl;

		//Output of the portlist of the DUT
		fout<<"architecture test of "<<fsmname<<"_test is"<<endl;
		fout<<"\n	component "<<fsmname<<endl;
		fout<<"			port"<<endl;
		fout<<"			("<<endl;
	        fout<<"				clk : in std_logic;      -- Clock";
                fout<<"\n       				rst : in std_logic;              -- Reset";
                fout<<"\n       				inp  : in std_logic_vector (";
                fout<<(atoi(numinputs.c_str())-1);
                fout<<" downto 0); -- Input";
                fout<<"\n       				outp : out std_logic_vector(";
                fout<<(atoi(numoutputs.c_str())-1);
                fout<<" downto 0) -- Output";
		fout<<"\n			);"<<endl;
		fout<<"	end component;"<<endl<<endl;

		//Definitions
		fout<<"	signal SimDone : std_logic := '0';"<<endl;
		fout<<"	signal ip : std_logic_vector("<<(atoi(numinputs.c_str())-1)<<" downto 0);"<<endl;
		fout<<"	signal op : std_logic_vector("<<(atoi(numoutputs.c_str())-1)<<" downto 0);"<<endl;
		fout<<"	signal clock,reset : std_logic;"<<endl<<endl;
		fout<<"begin"<<endl<<endl;
        	fout<<"	fsm1_I0:"<<fsmname<<" port map(clock,reset,ip,op);"<<endl<<endl;

		//Clock process
		fout<<"	clk : process\n";
		fout<<"	-- Please set your clock period\n";
		fout<<"	constant period : time := 10 ns;\n";
		fout<<"	constant waitt : time := 20 ns;\n";
        	fout<<"	begin\n";
                fout<<"		clock<='0','1' after period;\n";
                fout<<"		wait for waitt;\n";
                fout<<"		if SimDone = '1' then\n";
                fout<<"   		wait;\n";
                fout<<"		end if;\n";
        	fout<<"	end process clk;"<<endl<<endl;

		//Process for reading testvectors and feeding them to the FSM
		fout<<"--Please enter your testvectors in this process \n--or comment in the code for reading testvectors from a file.\n";
		fout<<"	io_process: process\n";
        	fout<<"--	file infile : text is in \"in.txt\";\n";
        	fout<<"--	variable buf : line;\n";
        	fout<<"	variable ip1 : std_logic_vector ("<<(atoi(numinputs.c_str())-1)<<" downto 0);\n";
        	fout<<"	begin\n";
		fout<<"		reset <='1';"<<endl;
                fout<<"		--while not (endfile(infile)) loop\n";
                fout<<"       	--	readline(infile,buf);\n";
                fout<<"       	--	read (buf,ip1);\n";
                fout<<"       	--	ip<=ip1;\n";
                fout<<"       	--	wait until falling_edge(clock);\n";
                fout<<"		--end loop;\n";
		fout<<"		SimDone<= '1';"<<endl;
                fout<<"		wait;\n";
        	fout<<"	end process io_process;"<<endl<<endl;
		
		fout<<"end test;"<<endl;

		cout<<"Done!"<<endl;
		cout<<"Testbench template created."<<endl;
		fout.close();
	};
};


//Characters enclosed by a tag
void xmlread::characters (const XMLCh* const chars, const unsigned long length)
{
	char* help = XMLString::transcode(chars);

  	if(chars_append) 
    		chars_data.append(help);
};


//Error Messages
void xmlread::fatalError(const SAXParseException& exc) 
{
    char* message = XMLString::transcode(exc.getMessage());
    cout << "Fatal Error: " << message
         << " at line: " << exc.getLineNumber()
         << endl;
}

void xmlread::warning(const SAXParseException& exc) 
{
 char* message = XMLString::transcode(exc.getMessage());
    cout << "Warning: " << message
         << " at line: " << exc.getLineNumber()
         << endl;
}

void xmlread::error(const SAXParseException& exc) 
{
 char* message = XMLString::transcode(exc.getMessage());
    cout << "Error: " << message
         << " at line: " << exc.getLineNumber()
         << endl;
}

