

#ifndef LOADHANDLER_H_
#define LOADHANDLER_H_

//standard includes
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtCore/QString>
#include <iostream>
#include <string>
#include <cstdlib>
#include <vector>	
#include <fstream>
//sax includes
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/util/XMLString.hpp>

#include "../verification/verify.h"
#include "../verification/logicmin.h"

#include "../XercesDefinitions.hpp"

XERCES_CPP_NAMESPACE_USE
using namespace std;

//Struct for a Transition
typedef struct MyTransition
{	
	vector<string> Input;
	string NState;
	//needed during the parsing process only:
	bool isHyperTrans;
	string Start; 
	string Default;
}MyTransition;


//Struct for a State
typedef struct MyState
{
	string StateName;
	string Output;
	string Default;
	vector<MyTransition> Transitions;
}MyState;


//The Handler for reading and converting the FSM
class xmlread : public DefaultHandler 
{
  
 private:
	//variables for data about fsm
	string fsmname;	
	string resetstate;
	string numinputs;
	string numoutputs;
	vector<MyState> StateTable;
	char* inF;
	char* outF;
	bool tb;
	bool g;

	//variables needed when parsing
	vector<MyTransition> TransList;	//stores all Transitions before end of parsing
	string chars_data;		//the chars enclosed by tags
	bool chars_append;		//are chars appended?
	MyState current_state;		//tmp for the state we are parsing atm
	MyTransition current_trans;	//tmp for transition that is parsed atm

	string getVHDLFilename();	
 
 public:
	xmlread(char * vhdlFile, bool testbench, bool gui);
	void startElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname, const Attributes& att);
 	void endElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname);
  	void startDocument();
 	void endDocument();
  	void characters (const XMLCh* const chars, const unsigned long length);
  	void fatalError(const SAXParseException& exc);
  	void warning(const SAXParseException& exc);
  	void error(const SAXParseException& exc);

};


#endif
