/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <QtGui/QPushButton>
#include <QtGui/QColorDialog>

#include <string>
#include <iostream>

using namespace std;
 
class ColorButton: public QPushButton {
  Q_OBJECT;
public:

  ColorButton(QWidget* parent, QColor c): QPushButton(parent),color(c),wasClicked(false)
  {
    connect(this,SIGNAL(clicked()),this,SLOT(buttonClicked()));
  }

  ~ColorButton()
    {
      disconnect(this,SIGNAL(clicked()),this,SLOT(buttonClicked()));
    }

  void setColor(unsigned int rgb)
  {
    QColor a(rgb);
    color = a; 
    this->setText(color.name());
  }

  QColor getColor()
    {
      if (wasClicked)
	{
	  QColor c = QColorDialog::getColor(color);
	  if(!c.isValid()) {
	    c = color;
	  }
	  return c; //color;
	}
      else
	return color;
    }

 public slots:
   void buttonClicked()
   {
     wasClicked=true;
     this->close();     
   }

protected:
  QColor color;
  bool wasClicked;
};

