/***********************************************************************
*                                                                      *
* (C) 2009, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#define DEBUGINPUTMODEL

#include <QtGui/QComboBox>
#include <sstream>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>



#include "inputmodel.h"
#include "inputwidget.h"
#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"

using namespace std;


InputModel::InputModel(QObject* parent) : QAbstractItemModel(parent) {
    type = 0;
    propid = 0;
}

InputModel::~InputModel() {

}

void InputModel::setMainWidget(QWidget* w) {
    wid = w;
}

int InputModel::columnCount(const QModelIndex& index) const {

    Q_UNUSED(index);

    return 2;
}

void InputModel::setParams(int status, int id) {
    type = status;
    propid = id;
}

int InputModel::rowCount(const QModelIndex& parent) const {

    if(parent != QModelIndex()) {
        return 0;
    }

    int rows;
    Core_I& co = CoreFactory::getCore_I();
    if (co.isFsm()) // a FSM Project exists
    {
        Fsm_I& fsm = co.getCurrentFsm();
        rows = fsm.getNumberOfInputs(); //State
    } else { rows = 1; }
    return rows;
}

QVariant InputModel::data(const QModelIndex& index, int role) const {


    Core_I& co = CoreFactory::getCore_I();
    if (co.isFsm()) // a FSM Project exists
    {
        Fsm_I& ff = co.getCurrentFsm();
        string s;
        QString qs;
        int ia;
        int i;
        int cnameid = 0;
        string cnames;
        QString cnameqs;
        int cnamerow = 0;
        QString qsind;
        stringstream ss;
        int ifsmid;
        int nInputs;
        nInputs = ff.getNumberOfInputs();
        if (type==2) // Transition
        {
            ifsmid = ff.getFsmID();
            nInputs = ff.getNumberOfInputs();
#ifdef DEBUGINPUTMODEL
            //qDebug()<<"Inputmodel has Number of Inputs = "<<nInputs;
#endif
            // FIRST COLUMN
            if ((index.column() == 0) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole)))
            {

                if ((index.row()< nInputs) )
                {
                    QString qs;
                    qs = QString::fromStdString(ff.getInputName(index.row()));
                    return qs;
                }
                else {
                    return QString("");
                }
            }
            // SECOND COLUMN
            if ((index.column() == 1) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole)))
            {

                if (ff.getTransFirstCondition())
                {
                    //dynamic_cast<MainguiWindow*>(wid)->cNameComboBox->clear();
//                    int ii = 0;
//                    dynamic_cast<MainguiWindow*>(wid)->numberOfConditions = 0;
//                    while (ff.getTransConditionbyID(ii) )
//                    {
//                        ii++;
//                        dynamic_cast<MainguiWindow*>(wid)->numberOfConditions = ii;
//#ifdef DEBUGINPUTMODEL
//                   //     qDebug()<<"InputEditor has numberOfConditions: "<<dynamic_cast<MainguiWindow*>(wid)->numberOfConditions;
//#endif
//                    }
//                    int ii= 0;
//                    while (ff.getTransConditionbyID(ii) )
//                    {
//                        cnames = ff.getTransConditionName();
//                        cnameqs = QString::fromStdString(cnames);
//                        cnamerow = ii;
//                        qsind.setNum(cnamerow);
//                        qsind.append(" - ");
//                        qsind.append(cnameqs);
//                        dynamic_cast<MainguiWindow*>(wid)->cNameComboBox->addItem(qsind);
//                        ii++;
//                    }

                    //dynamic_cast<MainguiWindow*>(wid)->cNameComboBox->setMaxCount(dynamic_cast<MainguiWindow*>(wid)->numberOfConditions );
                    if (dynamic_cast<MainguiWindow*>(wid)->cnameComboIndex >0)
                    {
                        cnameid = dynamic_cast<MainguiWindow*>(wid)->cnameComboIndex;
                    }
                    //dynamic_cast<MainguiWindow*>(wid)->cNameComboBox->clear();
                    //dynamic_cast<MainguiWindow*>(wid)->cnameComboIndex->setCurrentIndex(cnameid);

#ifdef DEBUGINPUTMODEL
                 //   qDebug()<<"InputEditor has cNameComboBox id: "<<cnameid;
#endif
                    ff.getTransConditionbyID(cnameid);

                    if (index.row()< nInputs)
                    {
                        QString qs;
                        qs = QString( QChar( ff.getTransConditionBit(index.row())));
                        return qs;
                    }

                }

                else {
                    return QString("");
                }
            }


        }
        else  // All other objects and no selected objects
        {
#ifdef DEBUGINPUTMODEL
           // qDebug()<<"Inputmodel has found FSM id: "<<ifsmid;
#endif
            dynamic_cast<MainguiWindow*>(wid)->cNameComboBox->clear();
            nInputs = ff.getNumberOfInputs();
            // FIRST COLUMN
            if ( (index.column() == 0) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole) ) )
            {
                if ((index.row()< nInputs) )
                {
                    QString qs;
                    qs = QString::fromStdString(ff.getInputName(index.row()));
                    //qs.append("- Nr.");
                    //qs.append(QString("%1").arg(index.row()));
                    return qs;
                }
            }

        }
    }
    //No FSM present
    else {
        if ( (index.column() == 0) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole) ) )
        {
            return QString("No FSM ");
        }
        else if ( (index.column() == 1) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole) ) )
        {
            return QString("active ");
        }
    }


    return QVariant();

}

QModelIndex InputModel::index(int row, int column, const QModelIndex& parent) const {

    int id = row*2 + column;

    if (!parent.isValid()) {
        return createIndex(row, column, id);
    }

    return createIndex(row, column, 100);
}

QModelIndex InputModel::parent(const QModelIndex& index) const {

    if (!index.isValid() || index.internalId() < 1000) {
        return QModelIndex();
    }
    return createIndex(index.row(), index.column(), 1000);
}

bool InputModel::setData(const QModelIndex& index, const QVariant& value, int role) {

    Core_I& co = CoreFactory::getCore_I();
    if (co.isFsm()) // a FSM Project exists
    {
        Fsm_I& ff = co.getCurrentFsm();
        char cv;
        char cs [255];
        int cnameid = 0;
        cnameid = dynamic_cast<MainguiWindow*>(wid)->cnameComboIndex;

        if( (index.column()== 1) && (type==2))  // SECOND COLUMN + Transition
        {
            ff.getTransbyID(propid);

            dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
            //ff.modifyStateOutputBit(propid, index.row(), value.Char);
            QVariant vv = value;
            //            int val = vv.toInt();
            //            if ( (val==0) || (val==1) )
            //            {
            //                sprintf(cs, "%d", val);
            //                cv = cs[0];
            //                ff.modifyTransCondition(propid, cnameid, index.row(), cv);
            //            }
            QString qs2 = vv.toString();
            QByteArray ba = qs2.toLatin1();
            const char *c_str2 = ba.data();
            char cv2 = c_str2[0];
#ifdef DEBUGINPUTMODEL
            qDebug()<<"Value in Input Editor as Qstring: "<<qs2<<"char: "<< cv2;
#endif
            if ( (cv2=='0') || (cv2=='1') || (cv2=='-') || (cv2=='x') || (cv2=='X'))
            {
                ff.modifyTransCondition(propid, cnameid, index.row(), cv2);
#ifdef DEBUGINPUTMODEL
                qDebug()<<"Value in Input Editor is 1,0 or -: "<<cv2 ;
#endif
                dynamic_cast<MainguiWindow*>(wid)->propertywidget->doItemsLayout();
            } else {
#ifdef DEBUGINPUTMODEL
                qDebug()<<"Value in Input Editor is not legal: "<<cv2 ;
#endif
		cv2 = '-';
		ff.modifyTransCondition(propid, cnameid, index.row(), cv2);
#ifdef DEBUGINPUTMODEL
                qDebug()<<"Value changed to: "<<cv2 ;
#endif
                dynamic_cast<MainguiWindow*>(wid)->propertywidget->doItemsLayout();
	    }
            //dynamic_cast<MainguiWindow*>(wid)->refresh();
        }



    }
}






Qt::ItemFlags InputModel::flags(const QModelIndex& index) const {
    Qt::ItemFlags flag = QAbstractItemModel::flags(index);
    if (isEditable(index)) {
        flag |= Qt::ItemIsEditable;
    }
    return flag;
}

bool InputModel::isEditable(const QModelIndex& index) const {
    //    if(type == 3 || type == 4) { // Link + Hypertransition
    //        if(index.column() == 1 && index.row() == 1) {
    //            return false;
    //        }
    //    }
    //    if(type == 4 && index.column() == 1 && index.row() == 3) { // Hypertransition
    //        return false;
    //    }
    if (index.row() == -1)
    {
        return false;
    }
    else {return true;}
}

QModelIndex InputModel::buddy(const QModelIndex& index) const {
    if (index.column() == 0) {
        return createIndex(index.row(), 1, index.internalPointer());
    }
    return index;
}

QVariant InputModel::headerData(int section, Qt::Orientation orientation, int role) const {
    if (orientation == Qt::Horizontal) {
        if (role != Qt::DisplayRole) {
            return QVariant();
        }
        switch (section) {
        case 0: return QString("Inputs");
        case 1: return QString("Condition Bits");
        default: return QString();
        }
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}

void InputModel::rowsAdded( int first, int last) {

    //dynamic_cast<MainguiWindow*>(w)->outputeditorwidget->model()->
    beginInsertRows(QModelIndex(), first, last);
    endInsertRows();
    layoutChanged();
}

void InputModel::rowsChanged()
{    layoutChanged(); }


void InputModel::comboBoxAddItem(QString text)
{    dynamic_cast<MainguiWindow*>(wid)->cNameComboBox->addItem(text); }

