/***********************************************************************
 *                                                                      *
 * (C) 2007, Frank Lemke, Computer Architecture Group,                  *
 * University of Mannheim, Germany                                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
 * USA                                                                  *
 *                                                                      *
 * For informations regarding this file contact                         *
 *			      office@mufasa.informatik.uni-mannheim.de *
 *                                                                      *
 ***********************************************************************/

#define DEBUGITEMS

#include <iostream>
#include <QtGui/QPainter>
#include <QGraphicsScene>
#include <QtCore/QRect>
#include <cmath>
#include <typeinfo>

#include "../corefactory.h"
#include "../fsm_I.h"
#include "../core_I.h"
#include "items.h"
#include "../fsmdesigner.h"
#include "view.h"
#include "mainguiwindow.h"

using namespace std;

const qreal Pi = 3.14;


NewStateItem::NewStateItem(const QRectF &rect) :
	QGraphicsEllipseItem(rect) {
	name = "";
	id = 0;
	setFlags(ItemIsMovable | ItemIsSelectable | ItemSendsGeometryChanges);
	//        string s;
	//        QString qs;
	//        QString si;
	//        Core_I& co = CoreFactory::getCore_I();
	//        Fsm_I& ff = co.getCurrentFsm();
	//        si = "NewState Item with Output :";
	//        s = ff.getStateOutput();
	//        qs = QString::fromStdString(s);
	//        si.append(qs);
	//        setToolTip(si);
	//setToolTip("NewState Item");
	stateItemColor = Qt::red;
	resetState = false;


}

NewStateItem::~NewStateItem() {
	//hide();
}

QString NewStateItem::text() {
	return name;
}

void NewStateItem::setText(QString s) {
	int len = s.length();
	this->name = s;
}

void NewStateItem::setStateItemColor(QColor color) {

	// Set Color
	this->stateItemColor = color;



}

QVariant NewStateItem::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedChange) {

		if (value.toBool() == true) {
			// Do nothing at the moment
		} else {
			// Unselected, assure opacity to 1
			this->setOpacity(1);
		}
	} else if (change == QGraphicsItem::ItemTransformHasChanged || change == QGraphicsItem::ItemVisibleHasChanged) {

		// Geometry has changed
		// Update Gradient

		// Ball Like Gradient
//		QRadialGradient gradient(-this->rect().width()/4, -this->rect().height()/4, (this->rect().width() / 2) + 6.5);
//		gradient.setColorAt(0, Qt::white);
//		gradient.setColorAt(0.5, this->stateItemColor);
		//gradient.setColorAt(1, Qt::white);

		// Button relief effect
		QRadialGradient gradient(5, 5, (this->rect().width() / 2) + 6.5);
		gradient.setColorAt(0, this->stateItemColor);
		gradient.setColorAt(0.8, this->stateItemColor);
		gradient.setColorAt(1, Qt::white);

		QBrush gradientBrush(gradient);
		//painter->setBrush(gradientBrush);
		this->setBrush(gradientBrush);

	} else if (change==QGraphicsItem::ItemPositionHasChanged) {

		// Just for the record

	}

	return QGraphicsItem::itemChange(change, value);

}

void NewStateItem::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {


	// Let parent do its stuff (placement and so on)
	QGraphicsEllipseItem::mouseReleaseEvent(event);


	// Record position modification, only if there is one then rebuild
	if (this->recordPosition()==true) {
		// Rebuild
		((View*) this->scene()->views()[0])->redraw();
	}





}

void NewStateItem::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {

	// Let parent do its work
	QGraphicsEllipseItem::mouseMoveEvent(event);
	scene()->update();
}

bool NewStateItem::recordPosition() {

	// Get FSM
	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();

	// Test if we have to record
	if(f.getStatebyID(this->id) && (
			f.getStatePosx()!=this->pos().x() ||
			f.getStatePosy()!=this->pos().y())) {

		f.modifyState(this->id, this->pos().x(), this->pos().y());

		// Modified
		return true;
	} else
		// If not return false
		return false;

}


void NewStateItem::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	painter->setRenderHint(QPainter::Antialiasing);

	bool futureDisplay = fsmDesignerSettings->value(SETTINGS_STATE_FUTURE, QVariant(
						SETTINGS_STATE_FUTURE_DEFAULT)).toBool();

	// Apply opacity here so that text always stays independent from global item opacity
	painter->setOpacity(this->opacity());

	//-- handle the reset state double ellipse
	if (resetState == true) {
		painter->setPen(Qt::black);
		//painter->setBrush(QColor(Qt::transparent));//Transparent, white
		painter->drawEllipse(QPoint(0, 0), (int) ((rect().width() / 2) + 7),
				(int) ((rect().height() / 2) + 7));
	}

	if (!futureDisplay)  {

		// Paint ellipse
		//------------------
		painter->setBrush(QBrush(this->stateItemColor));
		painter->setBackgroundColor(this->stateItemColor);
		painter->setPen(Qt::black);
		painter->drawEllipse(this->rect());

		// Paint Text
		//----------------
		painter->setPen(Qt::black);
		QRectF textRect((-this->rect().width()/2)-10,(-this->rect().height()/2)-10,this->rect().width()+10,this->rect().height()+10);
		// MAX 8 characters
		QString textTodraw = this->text().length()<=8?this->text():this->text().left(8);
		//painter->drawText(textRect, Qt::AlignCenter, textTodraw);
		int aboveBorder = 20 ;
    	painter->drawText(QRect(-(rect().width()/2)-(aboveBorder/2) , -(rect().height()/2)-(aboveBorder/2), rect().width()+aboveBorder, rect().height()+aboveBorder) , Qt::AlignCenter, textTodraw);

	} else {

		// Paint Ellipse
		//------------------
		painter->save();



		//--  Let Parent paint
		QGraphicsEllipseItem::paint(painter, option, widget);

		painter->restore();

		// Draw text background
		//---------------------
		painter->save();
		painter->setOpacity(0.5);
		int aboveBorder = 20;

		// Background to text depending on text size
		// Estimate text size
		QFontMetrics fontMetrics = painter->fontMetrics();
		int textWidth = fontMetrics.width(text());
		textWidth += 5;
		int textHeight = fontMetrics.height();
		textHeight += 5;

		// Draw Rect
		QColor settingsTextColor =
				fsmDesignerSettings->value(SETTINGS_STATE_TEXT_BG, QVariant(
						SETTINGS_STATE_TEXT_BG_DEFAULT)).value<QColor> ();
		if (settingsTextColor != painter->brush().color())
			painter->setBrush(QBrush(settingsTextColor));
//		painter->setPen(QPen(Qt::DotLine));
		painter->drawRoundRect(QRect(-textWidth / 2, -textHeight / 2, textWidth,
				textHeight), 10, 10);

		// Points pattern
		painter->setBrush(QBrush(Qt::lightGray, Qt::Dense4Pattern));
		painter->drawRoundRect(QRect(-textWidth / 2, -textHeight / 2, textWidth,
				textHeight), 10, 10);

		painter->restore();

		// Text
		//-----------

		// Text rectangle: position in middle of ellipse, width of the estiimated text width
		QRect textRect(-textWidth / 2, -textHeight / 2, textWidth, textHeight);
		painter->drawText(textRect, Qt::AlignCenter, text());
		//setToolTip("NewState Item");
	}

}

//Transline::Transline(QGraphicsScene* can) : QGraphicsLineItem(can) {
Transline::Transline() :
	QGraphicsLineItem() {
	id = 0;
	//setFlags(ItemIsMovable | ItemIsSelectable);
	setFlags(ItemIsSelectable);
	//setToolTip("Transline");
	selectedTransline = false;
	this->setPen(QPen(Qt::black));
	this->lastSelected = false;
	this->drawArrow = false;
}

Transline::~Transline() {

}

void Transline::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {

	painter->setRenderHint(QPainter::Antialiasing);
	painter->setOpacity(this->opacity());

	// Set thickness if necessary
	int
			width =
					fsmDesignerSettings->value("ui/TransLineWidth", QVariant(4)).toInt();
	if (!this->isSelected() && this->pen().width() != width) {
		QPen newpen(this->pen());
		newpen.setWidth(width);
		this->setPen(newpen);
	}

	// Paint  ourselves
	//------------------
	painter->setPen(this->pen());

	//-- Paint Line
	painter->drawLine(line());

	//-- Arrow if necessary
	if (this->drawArrow) {

		QPolygonF arrowHead;
		qreal arrowSize = 15;
		qreal arrowAngle = 20;

		// Duplicate line,shorten it, and invert p1 and 2
		QLineF traceLine(this->line());
		QPointF p1 = traceLine.p1();
		traceLine.setP1(traceLine.p2());
		traceLine.setP2(p1);
		traceLine.setLength(arrowSize);

		// Start point is always the same and is top of arrow polygon
		arrowHead << traceLine.p1();

		// Deviate of 10° on both directions and  register ending point as new basis of polygon
		traceLine.setAngle(traceLine.angle() + arrowAngle); // |"\"
		arrowHead << traceLine.p2();
		traceLine.setAngle(traceLine.angle() - arrowAngle); // back to normal: |
		traceLine.setAngle(traceLine.angle() - arrowAngle);// "/"|
		arrowHead << traceLine.p2();

		// And paint :)
		painter->setBrush(QBrush(this->pen().color()));
		//		painter->setBackgroundColor();
		//		painter->setBackgroundMode(Qt::OpaqueMode);
		painter->drawPolygon(arrowHead);
	}

}

QVariant Transline::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		// Propagate selection change
		//-----------
		QList<QGraphicsItem*> itemsAtLineEnd = this->scene()->items(this->line().p1());
		itemsAtLineEnd << this->scene()->items(this->line().p2());

		for (int i = 0; i < itemsAtLineEnd.length(); i++) {

			QGraphicsItem * item = itemsAtLineEnd.at(i);
			if (item == this || item->isSelected() == value.toBool()||(item->type()!=Transline::Type && item->type()!=TrackpointItem::Type)) // Don't repeat on itself
				continue;

			// Propagation to trackpoints should not allow them to be moved...thus make them unmoveable
			// And deselection propagation should make them moveable again
			if (item->type()==TrackpointItem::Type) {
				item->setFlag(ItemIsMovable,!value.toBool());
			}
			//cout << "At line end, there is: " << item->Type << endl;
			item->setSelected(value.toBool());

		}

		//-- Do it also for the text

		// Get FSM
		Core_I& core = CoreFactory::getCore_I();
		Fsm_I& fsm = core.getCurrentFsm();

		// Position on transition
		if (fsm.getTransbyID(this->id)) {
			// Get text item using position
			QGraphicsItem* textItem = this->scene()->itemAt(
					fsm.getTransTextPosx(), fsm.getTransTextPosy());
			if (textItem != NULL && textItem->isSelected()!= value.toBool()) {
				// If we have the text, set it to new selection state
				textItem->setSelected(value.toBool());
			}
		}

		// Local painting change && Property box show
		//-------------
		int width = fsmDesignerSettings->value("ui/TransLineWidth",
							QVariant(4)).toInt();
		if (value.toBool() == true) {

			// Show selection in property widget
			(dynamic_cast<MainguiWindow*>(((View*)this->scene()->views().at(0))->wid))->setPropertyStatus(2, this->id);
			(dynamic_cast<MainguiWindow*>(((View*)this->scene()->views().at(0))->wid))->fillcNameCombobox();

			// Pen modifications
			QPen newPen(Qt::red);
			newPen.setWidth(width + 1);
			this->setPen(newPen);
		} else {


			this->setPen(QPen(Qt::black));
			this->pen().setWidth(width);
			this->setOpacity(1.0);
		}

		this->scene()->update();

	}

	return QGraphicsItem::itemChange(change, value);

}

void Transline::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {

	// Propagate selection change
	//-----------
	QList<QGraphicsItem*> itemsAtLineEnd = this->scene()->items(this->line().p1());
	itemsAtLineEnd << this->scene()->items(this->line().p2());

	for (int i = 0; i < itemsAtLineEnd.length(); i++) {

		QGraphicsItem * item = itemsAtLineEnd.at(i);
		// Don't repeat on itself, selectedm, or on not transline or trackpoint
		if (item == this || item->isSelected() == true ||(item->type()!=Transline::Type && item->type()!=TrackpointItem::Type))
			continue;

		// Propagation to trackpoints should not allow them to be moved...thus make them unmoveable
		// And deselection propagation should make them moveable again
		if (item->type()==TrackpointItem::Type) {
			item->setFlag(ItemIsMovable,false);
		}

		//cout << "At line end, there is: " << item->Type << endl;
		item->setSelected(true);

	}

	//-- Do it also for the text

	// Get FSM
	Core_I& core = CoreFactory::getCore_I();
	Fsm_I& fsm = core.getCurrentFsm();

	// Position on transition
	if (fsm.getTransbyID(this->id)) {
		// Get text item using position
		QGraphicsItem* textItem = this->scene()->itemAt(fsm.getTransTextPosx(),
				fsm.getTransTextPosy());
		if (textItem != NULL && textItem->isSelected()!= true) {
			// If we have the text, set it to new selection state
			textItem->setSelected(true);
		}
	}

	this->scene()->update();
}

Leveltransfer::Leveltransfer(const QRectF &rect) :
	QGraphicsEllipseItem(rect) {
	transid = 0;
	id = 0;
	setFlags(ItemIsMovable | ItemIsSelectable);
	setToolTip("Leveltransfer");
}

Leveltransfer::~Leveltransfer() {
	//hide();
}

void Leveltransfer::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	// Q3CanvasEllipse::drawShape(painter);
	//  if(isActive()) {
	//    painter.setPen(Qt::black);
	//    painter.setBrush(Qt::black);
	//    painter.drawRect((int)x() - (int)(width()/2) - 1,(int)y() - 1,2,2);
	//    painter.drawRect((int)x() + (int)(width()/2) - 1,(int)y() - 1,2,2);
	//    painter.drawRect((int)x() - 1,(int)y() - (int)(height()/2) - 1,2,2);
	//    painter.drawRect((int)x() - 1,(int)y() + (int)(height()/2) - 1,2,2);
	//  }
	painter->setRenderHint(QPainter::Antialiasing);
	QGraphicsEllipseItem::paint(painter, option, widget);
	//QPainter::drawEllipse ( (int)x() , (int)y(), 2, 2 );
	if (isSelected()) {
		painter->setPen(Qt::black);
		painter->setBrush(Qt::black);
		painter->drawRect(-(int) (rect().width() / 2) - 1, -1, 2, 2);
		painter->drawRect((int) (rect().width() / 2) - 1, -1, 2, 2);
		painter->drawRect(-1, -(int) (rect().height() / 2) - 1, 2, 2);
		painter->drawRect(-1, (int) (rect().height() / 2) - 1, 2, 2);
	}

}

void Leveltransfer::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event ) {

	// Delegate to parent
	QGraphicsEllipseItem::mouseReleaseEvent(event);

	// Record position
	this->recordPosition();

	// Redraw
	scene()->update();
}

QVariant Leveltransfer::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		if (value.toBool() == true) {


		} else {

			this->setOpacity(1.0);
		}
	}

	return QGraphicsItem::itemChange(change, value);

}


bool Leveltransfer::recordPosition() {
	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();
	f.modifyTrans(transid, id, this->pos().x(), this->pos().y());

	return true;
}

LinkItem::LinkItem(const QRectF &rect) :
	QGraphicsEllipseItem(rect) {
	id = 0;
	setFlags(ItemIsMovable | ItemIsSelectable);
	setToolTip("LinkItem");
	linkItemColor = Qt::green;
}

LinkItem::~LinkItem() {
	//hide();
}

void LinkItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
		QWidget *widget) {
	//  Q3CanvasEllipse::drawShape(painter);
	// QPainter::drawEllipse ( (int)x() , (int)y(), 2, 2 );
	painter->setRenderHint(QPainter::Antialiasing);
	QGraphicsEllipseItem::paint(painter, option, widget);
	if (isSelected()) {
		painter->setPen(Qt::black);
		painter->setBrush(Qt::black);
		//        painter->drawRect((int)x() - (int)(rect().width()/2) - 1,(int)y() - 1,2,2);
		//        painter->drawRect((int)x() + (int)(rect().width()/2) - 1,(int)y() - 1,2,2);
		//        painter->drawRect((int)x() - 1,(int)y() - (int)(rect().height()/2) - 1,2,2);
		//        painter->drawRect((int)x() - 1,(int)y() + (int)(rect().height()/2) - 1,2,2);
		painter->drawRect(-(int) (rect().width() / 2) - 1, -1, 2, 2);
		painter->drawRect((int) (rect().width() / 2) - 1, -1, 2, 2);
		painter->drawRect(-1, -(int) (rect().height() / 2) - 1, 2, 2);
		painter->drawRect(-1, (int) (rect().height() / 2) - 1, 2, 2);
	}

}

void LinkItem::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event ) {

	// Delegate to parent
	QGraphicsEllipseItem::mouseReleaseEvent(event);

	// Record position
	if (this->recordPosition()==true) {
		// Redraw
		scene()->update();
		((View*) this->scene()->views()[0])->redraw();
	}



}

QVariant LinkItem::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		if (value.toBool() == true) {


		} else {

			this->setOpacity(1.0);
		}
	}

	return QGraphicsItem::itemChange(change, value);

}

bool LinkItem::recordPosition() {

	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();

	// Modify if necessary
	if (f.getLinkbyID(this->id) && (f.getLinkPosx()!=this->pos().x() || f.getLinkPosy()!=this->pos().y())) {
		f.modifyLink(id, this->pos().x(), this->pos().y());
		return true;
	} else {
		return false;
	}

}



Linkentrance::Linkentrance() :
	QGraphicsRectItem() {
	transid = 0;
	id = 0;
	setFlags(ItemIsMovable | ItemIsSelectable);
	setToolTip("Linkentrance");
	this->linkMoved = false;
}

Linkentrance::~Linkentrance() {
	//hide();
}

void Linkentrance::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {

	painter->setRenderHint(QPainter::Antialiasing);
	if (isSelected()) {
		painter->setPen(Qt::red);
	}
	//  Q3CanvasRectangle::drawShape(painter);
	QGraphicsRectItem::paint(painter, option, widget);
}

void Linkentrance::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {


	// Let parent do its stuff (placement and so on)
	QGraphicsRectItem::mouseReleaseEvent(event);


	// Record position modification
	if(this->recordPosition()==true) {
		// Rebuild
		scene()->update();
		((View*) this->scene()->views()[0])->redraw();
	}


}

QVariant Linkentrance::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		if (value.toBool() == true) {


		} else {

			this->setOpacity(1.0);
		}
	}

	return QGraphicsItem::itemChange(change, value);

}


void Linkentrance::mouseMoveEvent(QGraphicsSceneMouseEvent * event) {

	// Always call parent
	QGraphicsRectItem::mouseMoveEvent(event);

	this->linkMoved = true;
}

bool Linkentrance::recordPosition() {

	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();
	if (this->linkMoved) {
		this->linkMoved=false;
		f.modifyTrans(transid, id, this->pos().x(), this->pos().y());
		return true;
	}

	// Default is not Modified
	return false;


}

Joinpoint::Joinpoint() : QGraphicsRectItem() {
	transid = 0;
	id = 0;
	setFlags(ItemIsMovable | ItemIsSelectable);
	setToolTip("Joinpoint");
}

Joinpoint::~Joinpoint() {
}

void Joinpoint::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	painter->setRenderHint(QPainter::Antialiasing);
	if (isSelected()) {
		painter->setPen(Qt::red);
		setToolTip("Joinpoint Selected");
	}
	//Q3CanvasRectangle::drawShape(painter);
	QGraphicsRectItem::paint(painter, option, widget);
}


void Joinpoint::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {


	// Let parent do its stuff (placement and so on)
	QGraphicsRectItem::mouseReleaseEvent(event);


	// Record position modification
	if(this->recordPosition()==true) {
		// Rebuild
		((View*) this->scene()->views()[0])->redraw();
	}
//	scene()->update();



}

QVariant Joinpoint::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		if (value.toBool() == true) {


		} else {

			this->setOpacity(1.0);
		}
	}

	return QGraphicsItem::itemChange(change, value);

}

bool Joinpoint::recordPosition() {

	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();
	f.getTransbyID(transid);
	f.getTransTrackpointbyID(id);
	int goal = f.getTransEnd();
	double jposx = f.getTransTrackpointPosx();
	double jposy = f.getTransTrackpointPosy();
	bool modified = false;

	if (jposx==this->pos().x() && jposy == this->pos().y())
		return modified;

	if (f.getFirstTrans()) {
		do {
			if (f.getTransEnd() == goal) {
				if (f.getTransFirstTrackpoint()) {
					do {
						if (f.getTransTrackpointJoin()) {
							if ((f.getTransTrackpointPosx() == jposx)
									&& (f.getTransTrackpointPosy() == jposy)) {
								int curtransid = f.getTransID();
								int curid = f.getTransTrackpointID();

								modified=true;

								f.modifyTrans(curtransid, curid, this->pos().x(), this->pos().y());

								while (f.getTransNextTrackpoint()) {
								}
							}
						}
					} while (f.getTransNextTrackpoint());
				}
			}
		} while (f.getNextTrans());
	}

	return modified;

}


TrackpointItem::TrackpointItem() :
	QGraphicsRectItem() {
	transid = 0;
	id = 0;
	setFlags(ItemIsMovable | ItemIsSelectable);
	setToolTip("TrackpointItem");
}

TrackpointItem::~TrackpointItem() {
	//hide();
}

void TrackpointItem::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	painter->setRenderHint(QPainter::Antialiasing);
	//    if(isSelected()) {
	//        painter->setPen(Qt::red);
	//        painter->setBrush(Qt::red);
	//    }
	//Q3CanvasRectangle::drawShape(painter);
	QGraphicsRectItem::paint(painter, option, widget);
}

QVariant TrackpointItem::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		if (value.toBool() == true) {

			QPen newPen(Qt::red);
			newPen.setWidth(this->pen().width() + 1);
			this->setPen(newPen);
			this->setBrush(QBrush(Qt::red));

		} else {
			int width = fsmDesignerSettings->value("ui/TransLineWidth",
					QVariant(4)).toInt();
			this->setPen(QPen(Qt::black));
			this->setBrush(QBrush(Qt::black));
			this->pen().setWidth(width);
			this->setOpacity(1.0);
		}
	}

	return QGraphicsItem::itemChange(change, value);

}

void TrackpointItem::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {

	// Don't forget to call parent
	QGraphicsRectItem::mouseReleaseEvent(event);

	// Validate new position
	if (this->recordPosition()==true) {

		// Rebuild scene
		((View*) this->scene()->views()[0])->redraw();

	}

}

bool TrackpointItem::recordPosition() {

	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();

	// Test if we have to record

	if(f.getTransbyID(this->transid) && f.getTransTrackpointbyID(this->id) && (
			f.getTransTrackpointPosx()!=this->pos().x() ||
			f.getTransTrackpointPosy()!=this->pos().y())) {

		f.modifyTrans(transid, id, this->pos().x(), this->pos().y());

		// Modified
		return true;
	} else
		// If not return false
		return false;
}

Hypertransition::Hypertransition(const QRectF &rect) :
	QGraphicsEllipseItem(rect) {
	id = 0;
	setFlags(ItemIsMovable | ItemIsSelectable);
	setToolTip("Hypertransition");
}

Hypertransition::~Hypertransition() {
	//hide();
}

void Hypertransition::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	//Q3CanvasEllipse::drawShape(painter);
	painter->setRenderHint(QPainter::Antialiasing);
	QGraphicsEllipseItem::paint(painter, option, widget);
	if (isSelected()) {
		painter->setPen(Qt::black);
		painter->setBrush(Qt::black);
		painter->drawRect(-(int) (rect().width() / 2) - 1, -1, 2, 2);
		painter->drawRect((int) (rect().width() / 2) - 1, -1, 2, 2);
		painter->drawRect(-1, -(int) (rect().height() / 2) - 1, 2, 2);
		painter->drawRect(-1, (int) (rect().height() / 2) - 1, 2, 2);
	}
#ifdef DEBUGITEMS
	// cout << "Hypertransition at x: " << (int)x() << " y: " << (int)y() << " id " << id <<endl;
#endif
}

void Hypertransition::mouseReleaseEvent(QGraphicsSceneMouseEvent * event) {


	// Let parent do its stuff (placement and so on)
	QGraphicsEllipseItem::mouseReleaseEvent(event);


	// Record position modification
	if (this->recordPosition()==true) {

		// Rebuild
		//scene()->update();
		((View*) this->scene()->views()[0])->redraw();

	}

}

QVariant Hypertransition::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		if (value.toBool() == true) {


		} else {

			this->setOpacity(1.0);
		}
	}

	return QGraphicsItem::itemChange(change, value);

}

bool Hypertransition::recordPosition() {

	// Record position modification
	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();

	// Test if we have to record
	if(f.getHypertransbyID(this->id) && (
			f.getHypertransPosx()!=this->pos().x() ||
			f.getHypertransPosy()!=this->pos().y())) {

		f.modifyHypertrans(id, this->pos().x(), this->pos().y());

		// Modified
		return true;
	} else
		// If not return false
		return false;
}


Hierarchyexit::Hierarchyexit() :
	QGraphicsRectItem() {
	transid = 0;
	id = 0;
	setFlags(ItemIsMovable | ItemIsSelectable);
	setToolTip("Hierarchyexit");
}

Hierarchyexit::~Hierarchyexit() {
}

void Hierarchyexit::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	painter->setRenderHint(QPainter::Antialiasing);
	if (isSelected()) {
		painter->setPen(Qt::red);
	}
	// Q3CanvasRectangle::drawShape(painter);
	QGraphicsRectItem::paint(painter, option, widget);
}

void Hierarchyexit::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event ) {

	// Call parent
	QGraphicsRectItem::mouseReleaseEvent(event);

	// Record position
	this->recordPosition();

	// Update view
	this->scene()->update();

}

QVariant Hierarchyexit::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		if (value.toBool() == true) {


		} else {

			this->setOpacity(1.0);
		}
	}

	return QGraphicsItem::itemChange(change, value);

}


bool Hierarchyexit::recordPosition() {

	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();
	f.modifyTrans(transid, id, this->pos().x(), this->pos().y());

	return true;
}

//TranslineText::TranslineText(const QString & qs, QGraphicsItem *parent) : QGraphicsSimpleTextItem(qs, parent) {
TranslineText::TranslineText(const QString &qs, QGraphicsItem* parent = 0) :
	QGraphicsTextItem(qs, parent) {
	id = 0;
	name = qs;
	setFlags(ItemIsMovable | ItemIsSelectable);
	setToolTip("TranslineText");
	this->lastTransline=NULL;
}

QString TranslineText::text() {
	return name;
}

void TranslineText::setText(QString s) {

}

void TranslineText::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	//Q3CanvasText::draw(painter);
	painter->setRenderHint(QPainter::Antialiasing);

	QGraphicsTextItem::paint(painter, option, widget);

	//painter->drawText(QRect(-(boundingRect().width()/2) , -(boundingRect().height()/2), boundingRect().width(), boundingRect().height() ) ,Qt::AlignCenter,text());

#ifdef DEBUGITEMS
	// qDebug() << "TranslineText with Name : "  << text();
#endif
}

QVariant TranslineText::itemChange(GraphicsItemChange change,
		const QVariant & value) {

	// Selection Change
	//---------------------
	if (change == QGraphicsItem::ItemSelectedHasChanged) {

		if (value.toBool() == true) {

			this->setDefaultTextColor(Qt::red);

			// Propagate to transition
			if (this->lastTransline!=NULL && !this->lastTransline->isSelected()) {
				this->lastTransline->setSelected(true);
			}

		} else {

			this->setDefaultTextColor(Qt::black);
			this->setOpacity(1.0);

			// Propagate to transition
			if (this->lastTransline!=NULL && this->lastTransline->isSelected()) {
				this->lastTransline->setSelected(false);
			}

		}
	}

	return QGraphicsItem::itemChange(change, value);

}


void TranslineText::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event ) {

	// Call Parent
	QGraphicsTextItem::mouseReleaseEvent(event);

	// Record position
	this->recordPosition();

	// Propagate selection if necessary
	// Propagate to transition
	if (this->lastTransline!=NULL && !this->lastTransline->isSelected()) {
		this->lastTransline->setSelected(true);
	}

	// Update view
	//scene()->update();
}


bool TranslineText::recordPosition() {
	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();
	f.getTransbyID(id);
	int level = f.getTransLevel();
	f.modifyTrans(id, this->pos().x(), this->pos().y(), level);


	return true;
}

BoundingBox::BoundingBox() :
	QGraphicsRectItem() {
	id = 0;
	//setFlags(ItemIsMovable | ItemIsSelectable);
	//setToolTip("BoundingBox");
}

BoundingBox::~BoundingBox() {
	//hide();
}

void BoundingBox::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	painter->setRenderHint(QPainter::Antialiasing);
	if (isSelected()) {
		painter->setPen(Qt::red);
		//setToolTip("BoundingBox Selected");
	}
	//QGraphicsRectItem::paint(painter, option, widget);
}

/*void BoundingBox::recordPosition() {
	//    Core_I& c = CoreFactory::getCore_I();
	//    Fsm_I& f = c.getCurrentFsm();
	//    f.getBoundingBoxbyID(id);
	//    int level = f.getBoundingBoxLevel();
	//    f.modifyBoundingBox(id, posx, posy, level);
	scene()->update();

	return false;
}
*/

FixedGrid::FixedGrid() :
	QGraphicsLineItem() {
	id = 0;
	//setFlags(ItemIsMovable | ItemIsSelectable);
	//setToolTip("FixedGrid");
}

FixedGrid::~FixedGrid() {
}

void FixedGrid::paint(QPainter *painter,
		const QStyleOptionGraphicsItem *option, QWidget *widget) {
	painter->setRenderHint(QPainter::Antialiasing);
	if (isSelected()) {
		painter->setPen(Qt::red);
		//setToolTip("FixedGrid Selected");
	}
	QPen pen(Qt::black, 0, Qt::DotLine);
	painter->setPen(pen);
	//painter->drawLine(20, 160, 250, 160);
	//painter->drawLine(0, 20, 2500, 20);
	QGraphicsLineItem::paint(painter, option, widget);
	//painter->drawLine(20, 40, 250, 400);

}

void FixedGrid::modifyPos(double posx, double posy) {
	//    Core_I& c = CoreFactory::getCore_I();
	//    Fsm_I& f = c.getCurrentFsm();
	//    f.getFixedGridbyID(id);
	//    int level = f.getFixedGridLevel();
	//    f.modifyFixedGrid(id, posx, posy, level);
	scene()->update();
}
