/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#ifndef ITEMS_H_
#define ITEMS_H_

//#include <Qt3Support/Q3Canvas>
//#include <Qt3Support/Q3CanvasEllipse>
//#include <Qt3Support/Q3CanvasLine>
//#include <Qt3Support/Q3CanvasRectangle>
#include <QGraphicsScene>
#include <QGraphicsEllipseItem>
#include <QGraphicsLineItem>
#include <QGraphicsRectItem>
#include <QGraphicsTextItem>
#include <QGraphicsItem>
#include <QStyleOptionGraphicsItem>
#include <QPainter>
#include <QtCore/QString>
#include <QGraphicsTextItem>
#include <QtCore/QDebug>


/**
 * Common Class for FSM Graphics Items
 */
class FSMGraphicsItem {


public:

	/// Validate new position in FSM core
	/// \return true if a position was recorded, false if not
	virtual bool recordPosition() = 0;

};



/**
 * State Item ( new version)
 */
class NewStateItem : public QGraphicsEllipseItem , public FSMGraphicsItem {


public:
    //enum { RTTI = 1001 };
    enum { Type = UserType + 1 };

    int id;

    NewStateItem(const QRectF &rect);
    ~NewStateItem();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}

    bool resetState;

    QString text();
    void setText(QString s);

    void setStateItemColor(QColor color);
    QColor getStateItemColor() {
    	return this->stateItemColor;
    }

    virtual bool recordPosition();


private:
    QString name;

protected:

    /// defines the background color of the ellipse
    QColor stateItemColor;

	/**
	 * Overriden to detect selection changes and thus painting parameters
	 */
	virtual QVariant 	itemChange ( GraphicsItemChange change, const QVariant & value );

	/// When mouse is released, activate selection, or update position
	virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

	/// Position modification
	virtual void 		mouseMoveEvent ( QGraphicsSceneMouseEvent * event );

};

class Transline : public QGraphicsLineItem , public FSMGraphicsItem {



public:
    //enum { RTTI = 1002 };
    enum { Type = UserType + 2 };

    int id;

    bool lastSelected;
    bool drawArrow;

    Transline();
    ~Transline();


    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}
    bool selectedTransline;

    /// No Implementation
    bool recordPosition() {}

protected:

	/**
	 * Overriden to detect selection changes and thus painting parameters
	 */
	virtual QVariant 	itemChange ( GraphicsItemChange change, const QVariant & value );

	/// When mouse is released, activate selection
	virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

};

class Leveltransfer : public QGraphicsEllipseItem , public FSMGraphicsItem{

public:
    //enum { RTTI = 1003 };
    enum { Type = UserType + 3 };

    int transid;
    int id;

    Leveltransfer(const QRectF &rect);
    ~Leveltransfer();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}
    bool recordPosition();

protected:

    /// When mouse is released, activate selection, or update position
	virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

	/**
	 * Overriden to detect selection changes and thus painting parameters
	 */
	virtual QVariant 	itemChange ( GraphicsItemChange change, const QVariant & value );

};

class LinkItem : public QGraphicsEllipseItem{

public:
    //enum { RTTI = 1004 };
    enum { Type = UserType + 4 };

    int id;

    LinkItem(const QRectF &rect);
    ~LinkItem();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}
    QColor linkItemColor; // defines the color for each LinkItem

    bool recordPosition();

protected:
        /// When mouse is released, activate selection, or update position
		virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

		/**
		 * Overriden to detect selection changes and thus painting parameters
		 */
		virtual QVariant 	itemChange ( GraphicsItemChange change, const QVariant & value );

};

class Linkentrance : public QGraphicsRectItem , public FSMGraphicsItem {

private :

	// To know if the link was moved because we can't deduce it from background
	bool linkMoved;

public:
    //enum { RTTI = 1005 };
    enum { Type = UserType + 5 };

    int transid;
    int id;

    Linkentrance();
    ~Linkentrance();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}

    bool recordPosition();

protected:
    /// When mouse is released, activate selection, and validate new position
	virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

	/// Sets the linkmoved variable to true
	virtual void 		mouseMoveEvent(QGraphicsSceneMouseEvent * event) ;

	/**
	 * Overriden to detect selection changes and thus painting parameters
	 */
	virtual QVariant 	itemChange ( GraphicsItemChange change, const QVariant & value );

};

class Joinpoint : public QGraphicsRectItem , public FSMGraphicsItem{

public:
    //enum { RTTI = 1006 };
    enum { Type = UserType + 6 };

    int transid;
    int id;

    Joinpoint();
    ~Joinpoint();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}

    virtual bool recordPosition();

protected:
        /// When mouse is released, activate selection, or update position
		virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

		/**
		 * Overriden to detect selection changes and thus painting parameters
		 */
		virtual QVariant 	itemChange ( GraphicsItemChange change, const QVariant & value );
};

class TrackpointItem : public QGraphicsRectItem, public FSMGraphicsItem {

public:
    //enum { RTTI = 1007 };
    enum { Type = UserType + 7 };

    int transid;
    int id;

    TrackpointItem();
    ~TrackpointItem();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}

    bool recordPosition();


protected:

	/**
	 * Overriden to detect selection changes and thus painting parameters
	 */
	virtual QVariant itemChange ( GraphicsItemChange change, const QVariant & value );

	/// When mouse is released, activate selection, and validate new position
	virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );



};


class Hypertransition : public QGraphicsEllipseItem, public FSMGraphicsItem {

public:
    //enum { RTTI = 1008 };
    enum { Type = UserType + 8 };

    int id;

    Hypertransition(const QRectF &rect);
    ~Hypertransition();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}

    virtual bool recordPosition() ;


protected:
    /// When mouse is released, activate selection, and validate new position
	virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

	/**
	 * Overriden to detect selection changes and thus painting parameters
	 */
	virtual QVariant 	itemChange ( GraphicsItemChange change, const QVariant & value );
};

class Hierarchyexit : public QGraphicsRectItem{

public:
    //enum { RTTI = 1009 };
    enum { Type = UserType + 9 };

    int transid;
    int id;

    Hierarchyexit();
    ~Hierarchyexit();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}
    bool recordPosition();

protected:
        /// When mouse is released, activate selection, or update position
		virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

		/**
		 * Overriden to detect selection changes and thus painting parameters
		 */
		virtual QVariant 	itemChange ( GraphicsItemChange change, const QVariant & value );
};

class TranslineText : public QGraphicsTextItem, public FSMGraphicsItem {

public:
    //enum { RTTI = 1010 };
    enum { Type = UserType + 10 };

    int id;

    TranslineText(const QString &qs, QGraphicsItem* parent);
    ~TranslineText(){
    };

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    //int rtti() const { return RTTI; }
    int type() const { return Type;}
    QString text();
    void setText(QString s);

    void setLastTransline(Transline * transline) {
    	if (this->lastTransline!=NULL)
    		this->lastTransline=NULL;
    	if (transline!=NULL)
    		this->lastTransline=transline;
    }

    bool recordPosition();

protected:

    /// The Last transline that was added to form the transition
    /// Allows to propagate selected state to transition line when only selecting text
    Transline * lastTransline;

	/**
	 * Overriden to detect selection changes and thus painting parameters
	 */
	virtual QVariant itemChange ( GraphicsItemChange change, const QVariant & value );

	/// When mouse is released, activate selection, or update position
	virtual void		mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );

private:
    QString name;
};

class BoundingBox : public QGraphicsRectItem{

public:
    enum { Type = UserType + 11 };

    int id;

    BoundingBox();
    ~BoundingBox();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    int type() const { return Type;}
};

class FixedGrid : public QGraphicsLineItem{

public:
    enum { Type = UserType + 12 };

    int id;

    FixedGrid();
    ~FixedGrid();

    void paint(QPainter *painter,
               const QStyleOptionGraphicsItem *option, QWidget *widget);
    int type() const { return Type;}
    void modifyPos(double posx, double posy);
};

#endif
