/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <string>
#include <iostream>

#include "maingui.h"
#include "inputwidget.h"
#include "outputtreewidget.h"
#include "outputwidget.h"
#include "inputtreewidget.h"
#include "projecttreewidget.h"
#include "propertywidget.h"
#include "../fsmdesigner.h"
#include <QtGui/QHeaderView>


using namespace std;

void Maingui::setupMaingui(QMainWindow *QMW) {
  addonactions(QMW);
  initOutputWidget();
  initInputWidget();
  initProjectTreeWidget();
  initPropertyWidget();
  initOutputEditorWidget();


  setupSimDock(QMW);
  retranslateSim(QMW);



}

void Maingui::setupSimDock(QMainWindow *MW) {
  simWidget = new QDockWidget(MW);
  simWidget->setObjectName(QString::fromUtf8("Simulation"));
  simWidgetContents = new QWidget(simWidget);
  simWidgetContents->setObjectName(QString::fromUtf8("dockWidgetContents"));
  label = new QLabel(simWidgetContents);
  label->setObjectName(QString::fromUtf8("label"));
  label->setGeometry(QRect(50, 20, 61, 18));
  curtime = new QLineEdit(simWidgetContents);
  curtime->setObjectName(QString::fromUtf8("curtime"));
  curtime->setGeometry(QRect(120, 20, 113, 26));
  simmode = new QGroupBox(simWidgetContents);
  simmode->setObjectName(QString::fromUtf8("simulationmode"));
  simmode->setGeometry(QRect(10, 70, 251, 501));
  inputvectorvalue = new QLineEdit(simmode);
  inputvectorvalue->setObjectName(QString::fromUtf8("inputvectorvalue"));
  inputvectorvalue->setEnabled(true);
  inputvectorvalue->setGeometry(QRect(40, 140, 121, 26));
  currentstatevalue = new QLineEdit(simmode);
  currentstatevalue->setObjectName(QString::fromUtf8("currentstatevalue"));
  currentstatevalue->setEnabled(true);
  currentstatevalue->setGeometry(QRect(40, 300, 121, 26));
  nextstatevalue = new QLineEdit(simmode);
  nextstatevalue->setObjectName(QString::fromUtf8("nextstatevalue"));
  nextstatevalue->setEnabled(true);
  nextstatevalue->setGeometry(QRect(40, 460, 121, 26));
  curstatemode = new QCheckBox(simmode);
  curstatemode->setObjectName(QString::fromUtf8("curstatemode"));
  curstatemode->setGeometry(QRect(30, 200, 241, 23));
  curstatemode->setAutoExclusive(true);
  nextstatemode = new QCheckBox(simmode);
  nextstatemode->setObjectName(QString::fromUtf8("nextstatemode"));
  nextstatemode->setGeometry(QRect(30, 360, 231, 23));
  nextstatemode->setAutoExclusive(true);
  transmode = new QCheckBox(simmode);
  transmode->setObjectName(QString::fromUtf8("transmode"));
  transmode->setGeometry(QRect(30, 40, 231, 23));
  transmode->setChecked(true);
  transmode->setAutoExclusive(true);
  transmode->setTristate(false);
  inputvector = new QLabel(simmode);
  inputvector->setObjectName(QString::fromUtf8("inputvector"));
  inputvector->setGeometry(QRect(40, 90, 81, 18));
  currentstate = new QLabel(simmode);
  currentstate->setObjectName(QString::fromUtf8("currentstate"));
  currentstate->setGeometry(QRect(40, 250, 111, 18));
  nextstate = new QLabel(simmode);
  nextstate->setObjectName(QString::fromUtf8("nextstate"));
  nextstate->setGeometry(QRect(40, 410, 81, 18));
  line_4 = new QFrame(simmode);
  line_4->setObjectName(QString::fromUtf8("line_4"));
  line_4->setGeometry(QRect(10, 110, 231, 20));
  line_4->setFrameShape(QFrame::HLine);
  line = new QFrame(simmode);
  line->setObjectName(QString::fromUtf8("line"));
  line->setGeometry(QRect(10, 170, 231, 16));
  line->setFrameShape(QFrame::HLine);
  line_2 = new QFrame(simmode);
  line_2->setObjectName(QString::fromUtf8("line_2"));
  line_2->setGeometry(QRect(10, 270, 231, 20));
  line_2->setFrameShape(QFrame::HLine);
  line_3 = new QFrame(simmode);
  line_3->setObjectName(QString::fromUtf8("line_3"));
  line_3->setGeometry(QRect(10, 330, 231, 20));
  line_3->setFrameShape(QFrame::HLine);
  line_5 = new QFrame(simmode);
  line_5->setObjectName(QString::fromUtf8("line_5"));
  line_5->setGeometry(QRect(10, 430, 231, 20));
  line_5->setFrameShape(QFrame::HLine);
  simWidget->setWidget(simWidgetContents);
  MW->addDockWidget(static_cast<Qt::DockWidgetArea>(1), simWidget);  
}

void Maingui::retranslateSim(QMainWindow *MaW){
  label->setText(QApplication::translate("MainWindow", "Time : ", 0, QApplication::UnicodeUTF8));
  simmode->setTitle(QApplication::translate("MainWindow", "Simulation Mode", 0, QApplication::UnicodeUTF8));
  nextstatevalue->setText(QApplication::translate("MainWindow", "", 0, QApplication::UnicodeUTF8));
  curstatemode->setText(QApplication::translate("MainWindow", "Show current selected state ", 0, QApplication::UnicodeUTF8));
  nextstatemode->setText(QApplication::translate("MainWindow", "Show predicted next state", 0, QApplication::UnicodeUTF8));
  transmode->setText(QApplication::translate("MainWindow", "Show possible next transition", 0, QApplication::UnicodeUTF8));
    inputvector->setText(QApplication::translate("MainWindow", "Inputvector :", 0, QApplication::UnicodeUTF8));
    currentstate->setText(QApplication::translate("MainWindow", "Current State :", 0, QApplication::UnicodeUTF8));
    nextstate->setText(QApplication::translate("MainWindow", "Next State :", 0, QApplication::UnicodeUTF8));
    Q_UNUSED(MaW);
} // retranslateUi

void Maingui::addonactions(QMainWindow *QMW) {
  QAction* QWhatsThis;
  QWhatsThis = new QAction(QMW);
  QWhatsThis = QWhatsThis::createAction(QMW);
  menu_Help->addAction(QWhatsThis);
  QWhatsThis->setShortcut(QApplication::translate("Maingui", "Ctrl+?", 0, QApplication::UnicodeUTF8));
}


void Maingui::initOutputWidget() {
  //listofoutputs->headerItem()->setText(0, QApplication::translate("Maingui", "Outputs", 0, QApplication::UnicodeUTF8));
  //listofoutputs->clear();
  //listofoutputs->setAlternatingRowColors(true);
}

void Maingui::initInputWidget() {
  //listofinputs->headerItem()->setText(0, QApplication::translate("Maingui", "Inputs", 0, QApplication::UnicodeUTF8));
  //listofinputs->clear();
  //listofinputs->setAlternatingRowColors(true);

  inpmodel = new InputModel;
  inpdelegate = new InputDelegate;
  inputeditorwidget->setModel(inpmodel);
  inputeditorwidget->setItemDelegate(inpdelegate);
  inputeditorwidget->setAlternatingRowColors(true);
  inputeditorwidget->header()->setDefaultSectionSize(150);
}


void Maingui::initProjectTreeWidget() {
  projecttree->headerItem()->setText(0, QApplication::translate("Maingui", "Project", 0, QApplication::UnicodeUTF8));
  projecttree->clear();
}

void Maingui::initPropertyWidget() {
  propmodel = new PropertiesModel;
  propdelegate = new PropertiesDelegate; 
  propertywidget->setModel(propmodel);
  propertywidget->setItemDelegate(propdelegate);
  propertywidget->setAlternatingRowColors(true);
}

void Maingui::initOutputEditorWidget() {
  outpmodel = new OutputModel;
  outpdelegate = new OutputDelegate;
  outputeditorwidget->setModel(outpmodel);
  outputeditorwidget->setItemDelegate(outpdelegate);
  outputeditorwidget->setAlternatingRowColors(true);
  outputeditorwidget->header()->setDefaultSectionSize(150);
  //this->header()->setStretchLastSection(true);
}

