/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#define DEBUGMAINGUIWINDOW

#include <iostream>
#include <sstream>
#include <QtGui/QStatusBar>
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>
#include <QtGui/QPrintDialog>
#include <QtGui/QColorDialog>
#include <QtGui/QPainter>
#include <QtGui/QKeyEvent>
#include <QtCore/QSettings>
#include <QtCore/QRect>
#include <QtGui/QImage>
//#include <qglobal.h>

// #include <QtGui/QPicture>
// #include <Qt3Support/Q3Picture>
//#include <Qt3Support/Q3CanvasItemList>
#include <QtCore/QList>
#include <QtGui/QGraphicsItem>
#include <QtGui/QGraphicsView>
#include <QtGui/QGraphicsScene>



#if QT_VERSION>0x040300
#include <QSvgGenerator>
#endif

#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"
#include "../load_I.h"
#include "../save_I.h"
#include "../loadfactory.h"
#include "../savefactory.h"
#include "../fsmdesigner.h"

#include "printfsm.h"


using namespace std;


MainguiWindow::MainguiWindow() {
    setupUi(this);
    setupMaingui(this);
    projecttree->setQWidget(this);
    propertywidget->setMainWidget(this);
    outputeditorwidget->setMainWidget(this);
    inputeditorwidget->setMainWidget(this);
    //GenerationOfVerilog genver = new GenerationOfVerilog;
    //genver->setMainWidget(this);
    //GenerationOfVerilog::setMainWidget(this);
    (dynamic_cast<PropertiesModel*>(propertywidget->model()))->setMainWidget(this);
    (dynamic_cast<OutputModel*>(outputeditorwidget->model()))->setMainWidget(this);
    (dynamic_cast<InputModel*>(inputeditorwidget->model()))->setMainWidget(this);
    createConnections();
    initvariables();
    initStatusbarlabels();
    workbench();
    enableActions(false);
    initRecentFilesActions();
    readSettings();
    showRecentFiles();
    //simulation....
    simWidget->setVisible(false);
    //setup update for console
    textEdit->mw=this;


    printer.setOutputFormat(QPrinter::PostScriptFormat);
    printer.setOrientation(QPrinter::Landscape);
    printer.setPageSize(QPrinter::A4);
}

void MainguiWindow::initvariables() {
    presaved = false;
    savemode = true;
    simulationmode = false;
    lastfilepath = "/home";
    projectname = "";
    savename = "";
    filename = "";
    currentfsmid = 0;
    currenthierarchylevel = 0;
    currentnumberofhlevels = 0;
    simcounter = 0;
    propertiesid = 0;
    propertiestype = 0;
    maxpointer = 30;
    resetUndoRedo();
    activeProject = false;
    saveaccept = false;
    issaved = false;
    action_Undo->setEnabled(false);
    action_Redo->setEnabled(false);
    logo = new QPixmap(":/icons/FSMDesignerlogosmall.png",0,0);
    verificationstatetrans = false;
    verificationstatecoverage = false;
    verificationtranscoverage = false;
    sceneWidth = 1680; //see MainguiWindow::workbench
    sceneHeight = 2376;
    snapToGridInterval = 50;
    cnameComboIndex = 0;
    numberOfConditions = 0;
}

void MainguiWindow::activeProjectWarning() {

    int next = QMessageBox::warning(this, "A project already exists!",
                                    "Do you really want open an other project?\n"
                                    "Yes will close the old project without saving."
                                    , QMessageBox::Yes, QMessageBox::Abort, QMessageBox::NoButton);

    if (next ==  QMessageBox::Yes) {
        Core_I& c = CoreFactory::getCore_I();
        c.reset();
        setActiveProject(false);
    }
}

void MainguiWindow::about() {

    QMessageBox  qmb("About FSMDesigner4",
                     "<b>About FSMDesigner4</b><br /><br />"
                     "<p>The current version of this program is 4.2.</p>"
                     "<p>The FSMDesigner4 is a advanced tool to create finite<br />"
                     "state machines(FSM) in a interactive way and translate<br />"
                     "them into hardware description language(HDL).</p>"
                     "<p>FSMDesigner is a product of the Institute of Computer<br />"
                     "Engineering at the University of Heidelberg.<br />"
                     "See http://ra.ziti.uni-heidelberg.de "
                     "for more information.</p>"
                     , QMessageBox::Critical, QMessageBox::Ok,
                     QMessageBox::NoButton, QMessageBox::NoButton,this,
                     (Qt::WFlags) Qt::Dialog );
    qmb.setIconPixmap(*logo);
    qmb.adjustSize();
    qmb.exec();

}

void MainguiWindow::aboutQt() {

    QMessageBox::aboutQt(this);

}

void MainguiWindow::keyPressEvent(QKeyEvent* ke) {
    if(ke->key() == Qt::Key_Control) {
        v->setlocked();
    }
}

void MainguiWindow::keyReleaseEvent(QKeyEvent* ke) {
    if(ke->key() == Qt::Key_Control) {
        v->setunlocked();
    }
}
void MainguiWindow::createConnections() { 
    connect(actionAbout_Qt, SIGNAL(triggered()),this,SLOT(aboutQt()));
    connect(action_About,SIGNAL(triggered()),this,SLOT(about()));

    connect(actionChoose,SIGNAL(activated()),this,SLOT(setmodeChoose()));
    connect(action_State,SIGNAL(activated()),this,SLOT(setmodeState()));
    connect(action_Transition,SIGNAL(triggered()),this,SLOT(setmodeTrans()));
    connect(action_Hypertrans,SIGNAL(triggered()),this,SLOT(setmodeHtrans()));
    connect(action_Link,SIGNAL(triggered()),this,SLOT(setmodeLink()));
    connect(action_Join,SIGNAL(triggered()),this,SLOT(setmodeJoin()));
    connect(actionRemove,SIGNAL(triggered()),this,SLOT(setmodeRemove()));
    connect(action_New_Hierarchy,SIGNAL(triggered()),this,SLOT(setmodeHierarchy()));
    connect(actionconnect_to_Link,SIGNAL(triggered()),this,SLOT(setmodeconnecttoLink()));

    connect(actionCu_t,SIGNAL(triggered()),this,SLOT(activateCut()));
    connect(action_Copy,SIGNAL(triggered()),this,SLOT(activateCopy()));
    connect(action_Paste,SIGNAL(triggered()),this,SLOT(activatePaste()));
    connect(action_Delete,SIGNAL(triggered()),this,SLOT(activateRemove()));
//    connect(actionMove_to_Back,SIGNAL(triggered()),this,SLOT(movetoBack()));
//    connect(actionMove_to_Front,SIGNAL(triggered()),this,SLOT(movetoFront()));
    connect(actionMove_to_Back,SIGNAL(triggered()),this,SLOT(movetoBack()));
    connect(actionMove_to_Front,SIGNAL(triggered()),this,SLOT(movetoFront()));
    connect(action_Refresh,SIGNAL(triggered()),this,SLOT(refresh()));
    connect(action_Grid,SIGNAL(triggered()),this,SLOT(changeGrid()));
    connect(action_Undo,SIGNAL(triggered()),this,SLOT(undo()));
    connect(action_Redo,SIGNAL(triggered()),this,SLOT(redo()));
    connect(actionZoom_in,SIGNAL(triggered()),this,SLOT(zoomIn()));
    connect(actionZoom_out,SIGNAL(triggered()),this,SLOT(zoomOut()));
    connect(action_States,SIGNAL(triggered()),this,SLOT(statetable()));
    connect(action_Transitions,SIGNAL(triggered()),this,SLOT(transtable()));


    connect(action_New,SIGNAL(triggered()),this,SLOT(initProject()));
    connect(action_Save,SIGNAL(triggered()),this,SLOT(save()));
    connect(actionSave_As,SIGNAL(triggered()),this,SLOT(saveas()));
    connect(action_Open,SIGNAL(triggered()),this,SLOT(load()));
    connect(action_Print,SIGNAL(triggered()),this,SLOT(printFsm()));
    connect(actionExport_Image,SIGNAL(triggered()),this,SLOT(exportImage()));
    connect(action_NewFSM,SIGNAL(triggered()),this,SLOT(newFsm()));
    connect(menuTools,SIGNAL(aboutToShow()),this,SLOT(adaptStatus()));

//    connect(listofoutputs->rename, SIGNAL(triggered()),this,SLOT(renameoutput()));
//    connect(listofoutputs->newoutput, SIGNAL(triggered()),this,SLOT(output()));
//    connect(listofoutputs->deleteoutput, SIGNAL(triggered()),this,SLOT(deleteoutput()));
//    connect(listofoutputs->deleteselectedoutput, SIGNAL(triggered()),this,SLOT(deleteselectedoutput()));
//    connect(listofoutputs->outputup, SIGNAL(triggered()),this,SLOT(moveupoutput()));
//    connect(listofoutputs->outputdown, SIGNAL(triggered()),this,SLOT(movedownoutput()));

    connect(outputeditorwidget->rename, SIGNAL(triggered()),this,SLOT(renameoutput()));
    connect(outputeditorwidget->newoutput, SIGNAL(triggered()),this,SLOT(output()));
    connect(outputeditorwidget->deleteoutput, SIGNAL(triggered()),this,SLOT(deleteoutput()));
    connect(outputeditorwidget->deleteselectedoutput, SIGNAL(triggered()),this,SLOT(deleteselectedoutput()));
    connect(outputeditorwidget->outputup, SIGNAL(triggered()),this,SLOT(moveupoutput()));
    connect(outputeditorwidget->outputdown, SIGNAL(triggered()),this,SLOT(movedownoutput()));

    connect(inputeditorwidget->rename, SIGNAL(triggered()),this,SLOT(renameinput()));
    connect(inputeditorwidget->newinput, SIGNAL(triggered()),this,SLOT(input()));
    connect(inputeditorwidget->deleteinput, SIGNAL(triggered()),this,SLOT(deleteinput()));
    connect(inputeditorwidget->deleteselectedinput, SIGNAL(triggered()),this,SLOT(deleteselectedinput()));
    connect(inputeditorwidget->inputup, SIGNAL(triggered()),this,SLOT(moveupinput()));
    connect(inputeditorwidget->inputdown, SIGNAL(triggered()),this,SLOT(movedowninput()));

    connect(cNameComboBox , SIGNAL(currentIndexChanged(int)),this, SLOT(cNameComboChanged(int)));

//    connect(listofinputs->rename, SIGNAL(triggered()),this,SLOT(renameinput()));
//    connect(listofinputs->newinput, SIGNAL(triggered()),this,SLOT(input()));
//    connect(listofinputs->deleteinput, SIGNAL(triggered()),this,SLOT(deleteinput()));
//    connect(listofinputs->deleteselectedinput, SIGNAL(triggered()),this,SLOT(deleteselectedinput()));
//    connect(listofinputs->inputup, SIGNAL(triggered()),this,SLOT(moveupinput()));
//    connect(listofinputs->inputdown, SIGNAL(triggered()),this,SLOT(movedowninput()));
    connect(projecttree->addfsm, SIGNAL(triggered()),this,SLOT(newFsm()));
    connect(projecttree->newhierarchylevel, SIGNAL(triggered()),this,SLOT(insertLevel()));
    connect(projecttree->deletelevel, SIGNAL(triggered()),this,SLOT(deleteLevel()));
    connect(projecttree->deletefsm, SIGNAL(triggered()),this,SLOT(deleteFsm()));
    connect(projecttree->deleteproject, SIGNAL(triggered()),this,SLOT(deleteProject()));
    connect(projecttree->renameproject, SIGNAL(triggered()),this,SLOT(renameProject()));
    connect(projecttree->renamefsm, SIGNAL(triggered()),this,SLOT(renameFsm()));

    connect(propertywidget->addCondiToTrans, SIGNAL(triggered()),this,SLOT(addConditionToTrans()));
    connect(propertywidget->deleteCondiOfTrans, SIGNAL(triggered()),this,SLOT(deleteConditionOfTrans()));
    connect(propertywidget->addCondiToHtrans, SIGNAL(triggered()),this,SLOT(addConditionToHypertrans()));
    connect(propertywidget->deleteCondiOfHtrans, SIGNAL(triggered()),this,SLOT(deleteConditionOfHypertrans()));
    connect(action_Verification, SIGNAL(triggered()),this,SLOT(verification()));
    connect(action_Minimize, SIGNAL(triggered()),this,SLOT(minimize()));
    connect(actionGenerate_Mnemonic_Map, SIGNAL(triggered()),this,SLOT(createMMap()));
    connect(actionGenerate_SV_Project_Mnemonic_Map, SIGNAL(triggered()),this,SLOT(createMMapProject()));
    connect(actionTestbench_generator, SIGNAL(triggered()),this,SLOT(generateTestbench()));
    connect(action_Generate_Testbench_VHDL, SIGNAL(triggered()),this,SLOT(generateVHDLTestbench()));
    connect(action_GenVerilog, SIGNAL(triggered()),this,SLOT(genVerilog()));
    connect(actionGenV_HDL, SIGNAL(triggered()),this,SLOT(genVHDL()));
    connect(action_Verification_2, SIGNAL(triggered()),this,SLOT(verificationextern()));
    connect(actionGlobal, SIGNAL(triggered()),this,SLOT(globals()));
    connect(actionPreferences, SIGNAL(triggered()),this,SLOT(preferences()));


}

void MainguiWindow::adaptStatus() {
    if(ProjectFsmTree->isVisible()) {
        actionTree_view->setChecked(true);
    } else {
        actionTree_view->setChecked(false);
    }
    if(Console->isVisible()) {
        action_Console->setChecked(true);
    } else {
        action_Console->setChecked(false);
    }
    if(InputList->isVisible()) {
        action_Inputs->setChecked(true);
    } else {
        action_Inputs->setChecked(false);
    }
    if(OutputList->isVisible()) {
        action_Outputs->setChecked(true);
    } else {
        action_Outputs->setChecked(false);
    }
    if(Properties->isVisible()) {
        action_Properties->setChecked(true);
    } else {
        action_Properties->setChecked(false);
    }
    if(ObjecttoolBar->isVisible()) {
        action_Objecttoolbar->setChecked(true);
    } else {
        action_Objecttoolbar->setChecked(false);
    }
    if(MaintoolBar->isVisible()) {
        action_Maintoolbar->setChecked(true);
    } else {
        action_Maintoolbar->setChecked(false);
    }
    if(ZoomBar->isVisible()) {
        action_Zoombar->setChecked(true);
    } else {
        action_Zoombar->setChecked(false);
    }
}

void MainguiWindow::workbench() {
    //QGraphicsScene* can = new QGraphicsScene(this);
    Scene* can = new Scene(this);
    //can->setSceneRect(0, 0, 1680, 2376);
    can->setSceneRect(0, 0, sceneWidth, sceneHeight); // No visible effect, see MainguiWindow::initvariables
    v = new View(can,this);
    v->setFocus();
    v->setGeometry(0, 0, (sceneWidth), (sceneHeight));
    v->setSceneRect(0, 0, (sceneWidth), (sceneHeight)); // restricts the viewable size
    this->setCentralWidget(v);

    v->setBackgroundBrush(QColor(255, 255, 255));
    //setBgColor( QColor(255, 255, 255));

    //v->setViewportUpdateMode(QGraphicsView::FullViewportUpdate); // Repaints the whole graphic when something is changed or scrolled
    v->setViewportUpdateMode(QGraphicsView::SmartViewportUpdate); // Repaints the whole graphic when something is changed or scrolled
    MainguiWindow::propertywidget->scrollToTop();
    v->ensureVisible(QRectF(0, 0, 0, 0)); //Scrolls so that point (0,0) is visible
    v->show();


}

void MainguiWindow::setmodeChoose() {
    statusBar()->showMessage("Choose-Mode activated",3000);
    v->setmode(CHOOSE);
}

void MainguiWindow::setmodeState() {
    statusBar()->showMessage("State-Mode activated",3000);
    v->setmode(STATE);
}

void MainguiWindow::setmodeTrans() {
    statusBar()->showMessage("Transition-Mode activated",3000);
    v->setmode(TRANS);
}

void MainguiWindow::setmodeHtrans() {
    statusBar()->showMessage("Hypertransition-Mode activated",3000);
    v->setmode(HTRANS);
}

void MainguiWindow::setmodeLink() {
    statusBar()->showMessage("Link-Mode activated",3000);
    v->setmode(LINK);
}

void MainguiWindow::setmodeJoin() {
    statusBar()->showMessage("Join-Mode activated",3000);
    v->setmode(JOIN);
}

void MainguiWindow::setmodeRemove() {
    statusBar()->showMessage("Remove-Mode activated",3000);
    v->setmode(REMOVE);
}

void MainguiWindow::setmodeHierarchy() {
    statusBar()->showMessage("Hierarchy-Mode activated",3000);
    v->setmode(HIERARCHY);
}

void MainguiWindow::setmodeconnecttoLink() {
    statusBar()->showMessage("Linkconnect-Mode activated",3000);
    v->setmode(LINKCONNECT);
}

void MainguiWindow::activateCut() {
    v->cut();
}

void MainguiWindow::activateCopy() {
    v->copy();
}

void MainguiWindow::activatePaste() {
    v->paste();
}

void MainguiWindow::activateRemove() {
    v->remove();
}

void MainguiWindow::movetoBack() {
    v->movetoback();
}

void MainguiWindow::movetoFront() {
    v->movetofront();
}

void MainguiWindow::refreshWorkbench() {
    v->redraw();
}


void MainguiWindow::refresh() {
    refreshWorkbench();
    refreshInputs();
    refreshOutputs();
    refreshProjectTree();
    refreshProperties();
}

void MainguiWindow::changeGrid() {

	// Change background
    v->grid();
}

void MainguiWindow::zoomIn() {

    v->zoomIn();

    //  switch(simcounter) {
    //   case 0:
    //     simulation(1, 0);
    //     break;
    //   case 1:
    //     simulation(2, 0);
    //     break;
    //   case 2:
    //     simulation(1, 1);
    //     break;
    //   case 3:
    //     simulation(2, 1);
    //     break;
    //   case 4:
    //     simulation(1, 2);
    //     break;
    //   case 5:
    //     simulation(2, 2);
    //     break;
    //   case 6:
    //     simulation(1, 0);
    //     break;
    //   case 7:
    //     simulation(2, 0);
    //     break;
    //   case 8:
    //     simulation(1, 1);
    //     break;
    //   case 9:
    //     simulation(3, 0);
    //     break;
    //   case 10:
    //     simulation(1, 0);
    //     break;
    //   }
    //   simcounter++;
}

void MainguiWindow::zoomOut() {
    v->zoomOut();
    //  simcounter = 0;
    //   simulationEnd();
}

void MainguiWindow::activatepmode() {
    v->setPaintmode(true);
    enableActions(true);
    action_New->setEnabled(true);
    action_Open->setEnabled(true);
    action_Save->setEnabled(true);
    actionSave_As->setEnabled(true);
    action_NewFSM->setEnabled(true);
    menu_Recent_Files->setEnabled(true);
}

void MainguiWindow::deactivatepmode() {
    v->setPaintmode(false);
    enableActions(false);
    action_New->setEnabled(false);
    action_Open->setEnabled(false);
    action_Save->setEnabled(false);
    actionSave_As->setEnabled(false);
    action_NewFSM->setEnabled(false);
    menu_Recent_Files->setEnabled(false);
}

void MainguiWindow::noprojectmode() {
    v->setPaintmode(false);
    enableActions(false);
    //action_New->setEnabled(false);
    //action_Open->setEnabled(false);
    action_Save->setEnabled(false);
    actionSave_As->setEnabled(false);
    action_NewFSM->setEnabled(false);
    //menu_Recent_Files->setEnabled(false);
}

void MainguiWindow::statetable() {
    saveforunredo();
    Table* t;
    t = new Table(this);
    t->Tabletab->setCurrentIndex(0);
    t->exec();
    redopointer = endpointer;
    action_Redo->setEnabled(false);
}

void MainguiWindow::transtable() {
    saveforunredo();
    Table* t;
    t = new Table(this);
    t->Tabletab->setCurrentIndex(1);
    t->exec();
    redopointer = endpointer;
    action_Redo->setEnabled(false);
}

void MainguiWindow::initProject() {
    presaved=false;
    NewPro* Project;
    Project = new NewPro(this);
    Project->exec();
}

void MainguiWindow::newFsm() {
    saveforunredo();
    FsmName* FSM;
    FSM = new FsmName(this);
    FSM->exec();
}

void MainguiWindow::initFsm() {
    NewFsm* Fsm;
    Fsm = new NewFsm(this,(Qt::WFlags) 0);
    Fsm->show();
}

void MainguiWindow::output() {
    NewOutput* Output;
    Output = new NewOutput(this);
    Output->exec();
}

void MainguiWindow::input() {
    NewInput* Input;
    Input = new NewInput(this);
    Input->exec();
}



void MainguiWindow::save() {
    setPropertyStatus(0, 0); //disable propertywidget
    if (presaved) {
        v->activateItem(0);
        QApplication::setOverrideCursor(Qt::WaitCursor);
        Save_I& s = SaveFactory::getSave_I();
        if(savemode) {
            s.saveProject(filename);
            resetUndoRedo();
            refreshRecentFiles();
            statusBar()->showMessage("Project saved",3000);
            alreadysaved(true);
        } else {
            s.save(filename, savename);
            resetUndoRedo();
            refreshRecentFiles();
            statusBar()->showMessage("Fsm saved",3000);
            alreadysaved(true);
        }
    } else {
        saveas();
    }
    QApplication::restoreOverrideCursor();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getFsm(getcurrentfsmid());
}

void MainguiWindow::saveas() {
    setPropertyStatus(0, 0); //disable propertywidget
    v->activateItem(0);
    Save* FileSave;
    FileSave = new Save(this);
    FileSave->exec();
    if(saveaccept) {
        QString file;
        file = QFileDialog::getSaveFileName(this, "Choose a filename to save", lastfilepath, "XML files (*.xml)");
        QApplication::setOverrideCursor(Qt::WaitCursor);
        if(!file.isNull()) {
            presaved = true;
            int len = file.length();
            if(file.at(len - 1) == 'l' && file.at(len - 2) == 'm' && file.at(len - 3) == 'x' && file.at(len - 4) == '.') {
                file.remove(len - 4, len - 1);
            }
            len = file.length();
            if(file.at(len - 1) == 'o' && file.at(len - 2) == 'r' && file.at(len - 3) == 'p' && file.at(len - 4) == '.') {
                file.remove(len - 4, len - 1);
            }
            filename = file.toStdString();
            len = file.length();
            if(len > 0) {
                do {
                    len--;
                } while ( (file.at(len) != '/') && (len != 0));
                file.remove(len+1, file.length() - 1);
            }
            lastfilepath = file;
            Save_I& s = SaveFactory::getSave_I();
            if(savemode) {
                filename = filename + ".pro.xml";
                s.saveProject(filename);
                resetUndoRedo();
                refreshRecentFiles();
                statusBar()->showMessage("Project saved",3000);
                alreadysaved(true);
            } else {
                filename = filename + ".xml";
                s.save(filename, savename);
                resetUndoRedo();
                refreshRecentFiles();
                statusBar()->showMessage("Fsm saved",3000);
                alreadysaved(true) ;
            }
        }
        setSaveAccept(false);
    }
    QApplication::restoreOverrideCursor();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getFsm(getcurrentfsmid());
}

void MainguiWindow::load() {

	resetUndoRedo();

	// First ask the project to open
	//------------
    // (Add a .pro.xml file filter)
    QString file = QFileDialog::getOpenFileName(this, "Choose a filename to load", lastfilepath, "FSMDesigner Project XML files (*.pro.xml)");

    //QApplication::setOverrideCursor(Qt::WaitCursor);

    // If nothing loaded then exit
    if (file.isNull())
    	return;

    // Ask if we are sure
   	activeProjectWarning();

   	// After aksip, there should be no active project anymore
   	if (!getActiveProject()) {

   		// Set new file names
   		filename = file.toStdString();
		lastfilepath = file;

		// Load
		Load_I& l = LoadFactory::getLoad_I();
		activatepmode();
		l.loadProject(filename);
		refreshRecentFiles();
		statusBar()->showMessage("Project loaded",3000);
		setActiveProject(true);
		Core_I& c = CoreFactory::getCore_I();
		if(c.isFsm()) {
			c.getFirstFsm();
			Fsm_I& f = c.getCurrentFsm();
			setcurrentfsmid(f.getFsmID());
			setcurrenthierarchylevel(0);
		}
		refresh();
   	}



  //  QApplication::restoreOverrideCursor();
}

void MainguiWindow::setSavemode(bool b) {
    savemode = b;
}

void MainguiWindow::setSavename(string name) {
    savename = name;
}

void MainguiWindow::setcurrentfsmid(int i) {

    this-> currentfsmid = i;
    QString qs;
    if(i > -1) {
        Core_I& c = CoreFactory::getCore_I();
        c.setCurrentFsm(currentfsmid);
        Fsm_I& f = c.getCurrentFsm();

#ifdef DEBUGMAINGUIWINDOW
        qDebug() << "MainguiWindow: setcurrentfsmid has set id to: "<<f.getFsmID() ;
#endif

        qs = " " + QString::fromStdString(f.getFsmName()) + " ";
    } else {
        qs = " none ";
    }
    curfsmname->setText(qs);
}


void MainguiWindow::setcurrenthierarchylevel(int l) {
    currenthierarchylevel = l;
    level->setText(QString::fromStdString(itos(currenthierarchylevel)));
} 

void MainguiWindow::setcurrentnumberofhlevels(int cur) {
    currentnumberofhlevels = cur;
}

int MainguiWindow::getcurrenthierarchylevel() {
    return currenthierarchylevel;
}

int MainguiWindow::getcurrentfsmid() {
    return currentfsmid;
}

void MainguiWindow::setActiveProject(bool a) {
    activeProject = a;
}

bool MainguiWindow::getActiveProject() {
    return activeProject;
}

void MainguiWindow::setSaveAccept(bool b) {
    saveaccept = b;
}

void MainguiWindow::enableActions(bool b) {
    actionCu_t->setEnabled(b);
    action_Copy->setEnabled(b);
    action_Paste->setEnabled(b);
    action_Delete->setEnabled(b);
    //  actionZoom_in->setEnabled(b);
    //   actionZoom_out->setEnabled(b);
    actionMove_to_Back->setEnabled(b);
    actionMove_to_Front->setEnabled(b);
    action_State->setEnabled(b);
    action_Transition->setEnabled(b);
    action_Hypertrans->setEnabled(b);
    action_Link->setEnabled(b);
    action_Join->setEnabled(b);
    actionChoose->setEnabled(b);
    actionRemove->setEnabled(b);
    actionconnect_to_Link->setEnabled(b);
    action_New_Hierarchy->setEnabled(b);
    action_Refresh->setEnabled(b);
    //listofinputs->setEnabled(b);
    //listofoutputs->setEnabled(b);
    projecttree->setEnabled(b);
    propertywidget->setEnabled(b);
    inputeditorwidget->setEnabled(b);
    outputeditorwidget->setEnabled(b);
    action_States->setEnabled(b);
    action_Transitions->setEnabled(b);
    action_Verification->setEnabled(b);
    action_Minimize->setEnabled(b);
    action_GenVerilog->setEnabled(b);
    actionTestbench_generator->setEnabled(b);
    actionGenerate_Mnemonic_Map->setEnabled(b);
    actionGenerate_SV_Project_Mnemonic_Map->setEnabled(b);
    actionGenV_HDL->setEnabled(b);
    action_Generate_Testbench_VHDL->setEnabled(b);
    actionGlobal->setEnabled(b);
}

void MainguiWindow::refreshInputs() {
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    //listofinputs->clear();
    int numberofinputs = f.getNumberOfInputs();
    for(int i = 0; i < numberofinputs; i++) {
        string name = f.getInputName(i);
        QString qs = QString::fromStdString(name);
        //QTreeWidgetItem *it = new QTreeWidgetItem(listofinputs);
        //it->setText(0,qs);
    }
}

void MainguiWindow::refreshOutputs() {
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    //listofoutputs->clear();
    int numberofoutputs = f.getNumberOfOutputs();
    for(int i = 0; i < numberofoutputs; i++) {
        string name = f.getOutputName(i);
        QString qs = QString::fromStdString(name);
        //QTreeWidgetItem *it = new QTreeWidgetItem(listofoutputs);
        //it->setText(0,qs);
    }


}

void MainguiWindow::refreshProjectTree() {
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& fi = c.getCurrentFsm();
    projecttree->clear();
    int id = fi.getFsmID();
    int numberoflevels;
    string name = c.getProjectName();
    QString qs = QString::fromStdString(name);
    QTreeWidgetItem *twit = new QTreeWidgetItem(projecttree);
    twit->setText(0,qs);
    projecttree->setItemExpanded(twit,true);

    // Loop on all FSMs that we have
    if(c.getFirstFsm()) {
        do {
            numberoflevels = 0;
            Fsm_I& f = c.getCurrentFsm();
            string name = f.getFsmName();
            QString qs = QString::fromStdString(name);
            QTreeWidgetItem *it = new QTreeWidgetItem(twit);
            it->setText(0,qs);
            if(f.getFirstState()) {
                do {
                    if(f.getStateLevel() > numberoflevels) {
                        numberoflevels = f.getStateLevel();
                    }
                } while (f.getNextState());
            }
            if(f.getFirstTrans()) {
                do {
                    if(f.getTransFirstTrackpoint()) {
                        do {
                            if(f.getTransTrackpointLevel() > numberoflevels) {
                                numberoflevels = f.getTransTrackpointLevel();
                            }
                        } while(f.getTransNextTrackpoint());
                    }
                } while (f.getNextTrans());
            }
            for(int i = 0; i < numberoflevels; i++) {
                string name = "level ";
                name = name + itos(i + 1);
                QString qs = QString::fromStdString(name);
                QTreeWidgetItem *levelit = new QTreeWidgetItem(it);
                levelit->setText(0,qs);
            }
            if(f.getFsmID() == id) {
                if(numberoflevels < currentnumberofhlevels) {
                    for(int i = numberoflevels; i < currentnumberofhlevels; i++) {
                        string name = "level ";
                        name = name + itos(i + 1);
                        QString qs = QString::fromStdString(name);
                        QTreeWidgetItem *levelit = new QTreeWidgetItem(it);
                        levelit->setText(0,qs);
                    }
                } else {
                    currentnumberofhlevels = numberoflevels;
                }
            }
        } while(c.getNextFsm()) ;
    }
    c.getFsm(fi.getFsmID());
}

string MainguiWindow::itos(int i) {
    stringstream s;
    s << i;
    return s.str();
}

void MainguiWindow::renameoutput() {
    saveforunredo();
    RenameOutput* ReOut;
    ReOut = new RenameOutput(this);
    ReOut->exec();
}

void MainguiWindow::deleteoutput() {
    saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    int o = f.getNumberOfOutputs();
    if(o > 0) {
        o--;
        f.numberOfOuts(o, false);
    }
    refresh();
}

void MainguiWindow::deleteselectedoutput() {
    saveforunredo();
    //QTreeWidgetItem* it = this->listofoutputs->currentItem();
    //int i = this->listofoutputs->indexOfTopLevelItem(it);
    int i = this->outputeditorwidget->currentIndex().row();
    //QModelIndex mi = this->outputeditorwidget->indexAt(e->pos() );
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    f.deleteOutput(i);
    refresh();
}

void MainguiWindow::moveupoutput() {
    saveforunredo();
    //QTreeWidgetItem* it = this->listofoutputs->currentItem();
    //int i = this->listofoutputs->indexOfTopLevelItem(it);
    int i = this->outputeditorwidget->currentIndex().row();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    f.moveupOutput(i);
    refresh();
}

void MainguiWindow::movedownoutput() {
    saveforunredo();
    //QTreeWidgetItem* it = this->listofoutputs->currentItem();
    //int i = this->listofoutputs->indexOfTopLevelItem(it);
    int i = this->outputeditorwidget->currentIndex().row();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    f.movedownOutput(i);
    refresh();
}

void MainguiWindow::renameinput() {
    saveforunredo();
    RenameInput* ReIn;
    ReIn = new RenameInput(this);
    ReIn->exec();
}

void MainguiWindow::deleteinput() {
    saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    int i = f.getNumberOfInputs();
    if(i > 0) {
        i --;
        f.numberOfIns(i, -1000);
    }
    refresh();
}

void MainguiWindow::deleteselectedinput() {
    saveforunredo();
    //QTreeWidgetItem* it = this->listofinputs->currentItem();
    //int i = this->listofinputs->indexOfTopLevelItem(it);
    int i = this->inputeditorwidget->currentIndex().row();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    f.deleteInput(i);
    refresh();
}

void MainguiWindow::moveupinput() {
    saveforunredo();
    //QTreeWidgetItem* it = this->listofinputs->currentItem();
    //int i = this->listofinputs->indexOfTopLevelItem(it);
    int i = this->inputeditorwidget->currentIndex().row();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    f.moveupInput(i);
    refresh();
}

void MainguiWindow::movedowninput() {
    saveforunredo();
    //QTreeWidgetItem* it = this->listofinputs->currentItem();
    //int i = this->listofinputs->indexOfTopLevelItem(it);
    int i = this->inputeditorwidget->currentIndex().row();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    f.movedownInput(i);
    refresh();
}

void MainguiWindow::renameProject() {
    saveforunredo();
    RenameProject* RePro;
    RePro = new RenameProject(this);
    RePro->exec();
}

void MainguiWindow::renameFsm() {
    saveforunredo();
    RenameFsm* ReFsm;
    ReFsm = new RenameFsm(this);
    ReFsm->exec();
}

void MainguiWindow::deleteFsm() {
    saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();

    string name = f.getFsmName();
    QString qs = QString::fromStdString(name);

    int next = QMessageBox::warning(this, "Delete FSM",
                                    "Selected Fsm: " + qs + "       <br />"
                                    "Do you really want to delete this FSM?"
                                    , QMessageBox::Yes, QMessageBox::Abort, QMessageBox::NoButton);

    if (next ==  QMessageBox::Yes) {
        int i = f.getFsmID();
        c.deleteFsm(i);
        if(c.isFsm()){
            setcurrentfsmid(f.getFsmID());
            setcurrenthierarchylevel(0);
            refresh();
        } else {
            noprojectmode();
            string ep;
           c.projectName(ep);
        }
        refreshProjectTree() ;
    }

}

void MainguiWindow::deleteFsmWithoutAsking() {
    saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();

        int i = f.getFsmID();
        c.deleteFsm(i);
        if(c.isFsm()){
            setcurrentfsmid(f.getFsmID());
            setcurrenthierarchylevel(0);
            refresh();
        } else {
            noprojectmode();
            string ep;
           c.projectName(ep);
        }
        refreshProjectTree() ;
}

void MainguiWindow::deleteProject() {
    saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    string name = c.getProjectName();
    QString qs = QString::fromStdString(name);
    int next = QMessageBox::warning(this, "Close FSM",
                                    "This Project: " + qs + "       <br />"
                                    "Do you really want to close this Project without saving FSMs?"
                                    , QMessageBox::Yes, QMessageBox::Abort, QMessageBox::NoButton);
    if (next ==  QMessageBox::Yes)
    {

        if(c.isFsm())
        {
            while (c.getFirstFsm())
            {
                deleteFsmWithoutAsking();
                //deleteFsm();
            }
        }
    }
}

void MainguiWindow::insertLevel() {

    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& fs = c.getCurrentFsm();
    int idbefore = fs.getFsmID();
    int idafter = fs.getFsmID();
    QString qs;
    string str;
    if(projecttree->currentIndex().isValid()) {
        QString qs = projecttree->currentItem()->text(0);
        string str = qs.toStdString();
        if((strncmp(str.c_str(),"level",5)) == 0) {
            qs = projecttree->currentItem()->parent()->text(0);
            str = qs.toStdString();
        }
        bool found = false;
        if(!(strcmp(str.c_str(),c.getProjectName().c_str()) == 0)) {
            if(c.getFirstFsm()) {
                do {
                    Fsm_I& f = c.getCurrentFsm();
                    if(strcmp(str.c_str(),f.getFsmName().c_str()) == 0) {
                        found = true;
                        idafter = f.getFsmID();
                    }
                } while (c.getNextFsm() && !found);
            }
            if(!found) {
                QTreeWidgetItem* wit = projecttree->currentItem()->parent();
                QString qstr = wit->text(0);
                string stri = qstr.toStdString();
                found = false;
                if(c.getFirstFsm()) {
                    do {
                        Fsm_I& f = c.getCurrentFsm();
                        if(strcmp(stri.c_str(),f.getFsmName().c_str()) == 0) {
                            found = true;
                            idafter = f.getFsmID();
                        }
                    } while (c.getNextFsm() && !found);
                }
            }
        }
        if(idafter != idbefore) {
            setcurrentnumberofhlevels(0);
        }
        c.getFsm(idafter);
        refreshProjectTree();
    }
    currentnumberofhlevels++;
    refresh();
}

void MainguiWindow::deleteLevel() {

    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& fs = c.getCurrentFsm();
    int id = fs.getFsmID();
    QString qs;
    string str;
    int searchedlevel;

    QString qs2 = projecttree->currentItem()->text(0);
    int next = QMessageBox::warning(this, "Close FSM",
                                    "This Level: " + qs2 + "       <br />"
                                    "Do you really want to delete this Level without saving ?"
                                    , QMessageBox::Yes, QMessageBox::Abort, QMessageBox::NoButton);
    if (next ==  QMessageBox::Yes)
    {
        if(projecttree->currentIndex().isValid())
        {
            //QString qs = projecttree->currentItem()->text(0);
            string str = qs2.toStdString();
#ifdef DEBUGMAINGUIWINDOW
            QString qs3 = QString(str.c_str() );
            qDebug() << "deleteLevel: projecttree Item is valid. "<< qs3;
            //cout << str;
#endif
            if((strncmp(str.c_str(),"level",5)) == 0)
            {
#ifdef DEBUGMAINGUIWINDOW
                qDebug() << "deleteLevel: found level in current Selection. ";
#endif
                searchedlevel = 0;
                for(int i= 1; i<(currentnumberofhlevels+1); i++ )
                {
                    string levelnrstr = itos(i);
                    QString qlevelnrstr = QString(levelnrstr.c_str() );
                    //qDebug() << "deleteLevel: loop for searching nr in current selection: " << QString(levelnrstr.c_str() );
                    //if((strncmp(str.c_str(),levelnrstr.c_str(),10)) == 0)
                    if ( qs2.contains(qlevelnrstr)  )
                    {
                        searchedlevel = i;
#ifdef DEBUGMAINGUIWINDOW
                        qDebug() << "deleteLevel: found number in current Selection: " << i;
#endif
                    }


                }

                if (searchedlevel == currentnumberofhlevels)
                {
                    deleteAllOnLevel(searchedlevel);
                }
                else
                {
                    for(int i2= searchedlevel; i2<(currentnumberofhlevels+1); i2++ )
                    {
                        int ie = 3;
#ifdef DEBUGMAINGUIWINDOW
                        qDebug() << "deleteLevel: moveElementsFromLevel2Level old: " << i2<<"new: "<< (i2-1);
#endif
                        //moveElementsFromLevel2Level(ii, (ii-1) );
                        moveElementsFromLevel2Level(i2, (i2-1) );
                        //moveElementsFromLevel2Level(3, 2 );

                    }
                }


                //setcurrentnumberofhlevels(currentnumberofhlevels-1);
                currentnumberofhlevels--;
            }


        }
        c.getFsm(id);
        refreshProjectTree();
        //currentnumberofhlevels++;
        refresh();
    }
}

void MainguiWindow::deleteAllOnLevel(int level) //Delete all Elements on a specified Level
{
    Core_I& co = CoreFactory::getCore_I();
    Fsm_I& fs = co.getCurrentFsm();

    for (int i=0;i<2; i++ ) //delete all remaining States
    {
        if(fs.getFirstState()) { //delete States
            do {
                if(fs.getStateLevel() == level)
                {
                    int did = fs.getStateID();
                    fs.deleteState(did);
                }
            } while (fs.getNextState());
            if(fs.getStateLevel() == level) // Last State too
            {
                int did = fs.getStateID();
                fs.deleteState(did);
            }
        }
    }


    if(fs.getFirstTrans()) { // Delete TransitionTrackpoints
        do {
            if(fs.getTransFirstTrackpoint()) {
                do {
                    if(fs.getTransTrackpointLevel() == level) {
                        int did = fs.getTransTrackpointID();
                        int did2 = fs.getTransID();
                        fs.deleteTrackpointOfTrans( did2, did);
                    }
                } while(fs.getTransNextTrackpoint());
                if(fs.getTransTrackpointLevel() == level) {
                    int did = fs.getTransTrackpointID();
                    int did2 = fs.getTransID();
                    fs.deleteTrackpointOfTrans( did2, did);
                }
            }
        } while (fs.getNextTrans());
    }
    if(fs.getFirstTrans()) { // Delete Transitions
        do {
            if(fs.getTransLevel() == level)
            {
                int did = fs.getTransID();
                fs.deleteTrans( did);
            }
        }
        while (fs.getNextTrans());
        if(fs.getTransLevel() == level) // For last Tranistion
        {
            int did = fs.getTransID();
            fs.deleteTrans( did);
        }
    }

    if(fs.getFirstHypertrans()) { // Delete HyperTransitions
        do {
            if(fs.getHypertransLevel() == level)
            {
                int did = fs.getHypertransID();
                fs.deleteHypertrans( did);
            }
        }
        while (fs.getNextHypertrans());
        if(fs.getHypertransLevel() == level) // For last HyperTranistion
        {
            int did = fs.getHypertransID();
            fs.deleteHypertrans( did);
        }
    }

    if(fs.getFirstLink()) { // Delete Links
        do {
            if(fs.getLinkLevel() == level)
            {
                int did = fs.getLinkID();
                fs.deleteLink( did);
            }
        }
        while (fs.getNextLink());
        if(fs.getLinkLevel() == level) // For last Link
        {
            int did = fs.getLinkID();
            fs.deleteLink( did);
        }
    }

}


void MainguiWindow::moveElementsFromLevel2Level(int prevlevel, int newlevel) //Move all Elements from prevlevel to newlevel
{
    Core_I& co = CoreFactory::getCore_I();
    Fsm_I& fs = co.getCurrentFsm();

    for (int i=0;i<2; i++ ) //Move all remaining States
    {
        if(fs.getFirstState()) { //Move States
            do {
                if(fs.getStateLevel() == prevlevel)
                {
                    int did = fs.getStateID();
#ifdef DEBUGMAINGUIWINDOW
                    qDebug() << "deleteLevel: moveElementsFromLevel2Level calls fs.stateLevel(did, newlevel), did: " << did<<"newlevel: "<< newlevel;
#endif
                    fs.stateLevel(did, newlevel);
                }
            } while (fs.getNextState()); // Last State too
            if(fs.getStateLevel() == prevlevel)
                {
                    int did = fs.getStateID();
                    fs.stateLevel(did, newlevel);
                }
        }
    }


    if(fs.getFirstTrans()) { // Move TransitionTrackpoints
        do {
            if(fs.getTransFirstTrackpoint()) {
                do {
                    if(fs.getTransTrackpointLevel() == prevlevel) {
                        int did = fs.getTransTrackpointID();
                        int did2 = fs.getTransID();
                        fs.transLevel( did2, did, newlevel);
                    }
                } while(fs.getTransNextTrackpoint());
                if(fs.getTransTrackpointLevel() == prevlevel) {
                        int did = fs.getTransTrackpointID();
                        int did2 = fs.getTransID();
                        fs.transLevel( did2, did, newlevel);
                    }
            }
        } while (fs.getNextTrans());
    }
    if(fs.getFirstTrans()) { // Move Transitions
        do {
            if(fs.getTransLevel() == prevlevel)
            {
                int did = fs.getTransID();
                fs.modifyTransLevel(did, newlevel);
            }
        }
        while (fs.getNextTrans());// For last Tranistion
        if(fs.getTransLevel() == prevlevel)
            {
                int did = fs.getTransID();
                fs.modifyTransLevel(did, newlevel);
            }
    }

    if(fs.getFirstHypertrans()) { // Move HyperTransitions
        do {
            if(fs.getHypertransLevel() == prevlevel)
            {
                int did = fs.getHypertransID();
                fs.hypertransLevel( did, newlevel);
            }
        }
        while (fs.getNextHypertrans());// For last HyperTranistion
        if(fs.getHypertransLevel() == prevlevel)
            {
                int did = fs.getHypertransID();
                fs.hypertransLevel( did, newlevel);
            }
    }

    if(fs.getFirstLink()) { // Move Links
        do {
            if(fs.getLinkLevel() == prevlevel)
            {
                int did = fs.getLinkID();
                fs.linkLevel( did, newlevel);
            }
        }
        while (fs.getNextLink());// For last Link
        if(fs.getLinkLevel() == prevlevel)
            {
                int did = fs.getLinkID();
                fs.linkLevel( did, newlevel);
            }
    }

}


int MainguiWindow::getPropertiesType() {
    return propertiestype;
}

int MainguiWindow::getPropertiesID() {
    return propertiesid;
}

int MainguiWindow::getsceneWidth() {
    return sceneWidth;
}

int MainguiWindow::getsceneHeight() {
    return sceneHeight;
}

int MainguiWindow::getsnapToGridInterval() {
    return snapToGridInterval;
}


void MainguiWindow::setPropertyStatus(int status, int id) {
    if(id > -1) {
        if(status != propertiestype || id != propertiesid) {
            dynamic_cast<PropertiesModel*>(propertywidget->model())->setParams(status, id);
            dynamic_cast<PropertiesDelegate*>(propertywidget->itemDelegate())->setParams(status, id);
            dynamic_cast<OutputModel*>(outputeditorwidget->model())->setParams(status, id);
            dynamic_cast<OutputDelegate*>(outputeditorwidget->itemDelegate())->setParams(status, id);
            dynamic_cast<InputModel*>(inputeditorwidget->model())->setParams(status, id);
            dynamic_cast<InputDelegate*>(inputeditorwidget->itemDelegate())->setParams(status, id);

        }
        propertiestype = status;
        propertiesid = id;
        propertywidget->doItemsLayout();
        outputeditorwidget->doItemsLayout();
        inputeditorwidget->doItemsLayout();
    }
}

void MainguiWindow::refreshProperties() {
    propertywidget->doItemsLayout();
}

void MainguiWindow::addConditionToTrans() { 
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& fsm = core.getCurrentFsm();
    fsm.addConditionToTrans(propertiesid);
    propertywidget->doItemsLayout();
    (dynamic_cast<InputModel*>(inputeditorwidget->model()))->rowsChanged();
}

void MainguiWindow::deleteConditionOfTrans() {
    DeleteConditionTrans* DCT;
    DCT = new DeleteConditionTrans(this);
    DCT->exec();
    (dynamic_cast<InputModel*>(inputeditorwidget->model()))->rowsChanged();
}

void MainguiWindow::fillcNameCombobox() {
    Core_I& co = CoreFactory::getCore_I();
    if (co.isFsm()) // a FSM Project exists
    {
        Fsm_I& ff = co.getCurrentFsm();
        //qDebug()<<"fillcNameCombobox called : ";
        string cnames;
        QString cnameqs;
        int cnamerow = 0;
        QString qsind;
        if ( propertiestype ==2)
        {

            if (ff.getTransFirstCondition())
            {
                //cNameComboBox->clear();
                int ii = 0;
                numberOfConditions = 0;
                while (ff.getTransConditionbyID(ii) )
                {
                    ii++;
                    numberOfConditions = ii;
#ifdef DEBUGMAINGUIWINDOW
                    //qDebug()<<"fillcNameCombobox of Input Editor has numberOfConditions: "<< numberOfConditions;
#endif
                }

                ii= 0;
                while (ff.getTransConditionbyID(ii) )
                {
                    cnames = ff.getTransConditionName();
                    cnameqs = QString::fromStdString(cnames);
                    cnamerow = ii;
                    qsind.setNum(cnamerow);
                    qsind.append(" - ");
                    qsind.append(cnameqs);
#ifdef DEBUGMAINGUIWINDOW
                   // qDebug()<<"fillcNameCombobox Item: "<< qsind;
#endif
                    cNameComboBox->addItem(qsind);
                    //(dynamic_cast<InputModel*>(inputeditorwidget->model()))->comboBoxAddItem(qsind);
                    ii++;
                }
#ifdef DEBUGMAINGUIWINDOW
                   // qDebug()<<"numberOfConditions : "<< numberOfConditions;
#endif
                //cNameComboBox->setMaxCount(numberOfConditions );
                //cnameComboIndex->setCurrentIndex(cnameid);
            }
        }
    }
}

void MainguiWindow::addConditionToHypertrans() {
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& fsm = core.getCurrentFsm();
    fsm.addConditionToHypertrans(propertiesid);
    propertywidget->doItemsLayout();
}

void MainguiWindow::deleteConditionOfHypertrans() {
    DeleteConditionHypertrans* DCH;
    DCH = new DeleteConditionHypertrans(this);
    DCH->exec();
}

void MainguiWindow::initStatusbarlabels() { 
    curfsmname = new QLabel(" no fsm ", statusBar());
    curfsmname->setAlignment(Qt::AlignHCenter);
    curfsmname->setMinimumSize(curfsmname->sizeHint());
    statusBar()->addPermanentWidget(curfsmname);
    level =  new QLabel("ssss", statusBar());
    level->setAlignment(Qt::AlignHCenter);
    level->setMinimumSize(level->sizeHint());
    statusBar()->addPermanentWidget(level);
    level->setText("non");
}

void MainguiWindow::writeSettings() {
    QSettings settings("Universitaet Mannheim", "FSMDesigner4");
    settings.beginGroup("mainguiwindow");
    settings.setValue("size", size());
    settings.setValue("fullScreen", isFullScreen());
    settings.setValue("pos", pos());
    settings.setValue("lastFilePath", lastfilepath);
    settings.setValue("treeview", ProjectFsmTree->isVisible());
    settings.setValue("properties", Properties->isVisible());
    settings.setValue("inputs", InputList->isVisible());
    settings.setValue("outputs", OutputList->isVisible());
    settings.setValue("console", Console->isVisible());
    settings.setValue("objecttoolbar", ObjecttoolBar->isVisible());
    settings.setValue("maintoolbar", MaintoolBar->isVisible());
    settings.setValue("zoombar", ZoomBar->isVisible());
    settings.setValue("recentfiles", recentFiles);
    settings.setValue("maxundos", maxpointer);
    settings.setValue("stateinitcolor", stateColor().rgb());
    settings.setValue("bgcolor", bgColor().rgb());
    settings.setValue("snapToGridInterval", snapToGridInterval);
    settings.setValue("geometry", saveGeometry());
    settings.setValue("windowState", saveState());
    settings.endGroup();
}

void MainguiWindow::readSettings() {
    QSettings settings("Universitaet Mannheim", "FSMDesigner4");
    settings.beginGroup("mainguiwindow");
    resize(settings.value("size", QSize(1000,1000)).toSize());
    move(settings.value("pos", QPoint(100,100)).toPoint());
    if(settings.value("fullScreen", false).toBool()) {
        showFullScreen();
    }
    actionTree_view->setChecked(settings.value("treeview", true).toBool());
    ProjectFsmTree->setVisible(settings.value("treeview", true).toBool());
    action_Properties->setChecked(settings.value("properties", true).toBool());
    Properties->setVisible(settings.value("properties", true).toBool());
    action_Inputs->setChecked(settings.value("inputs", true).toBool());
    InputList->setVisible(settings.value("inputs", true).toBool());
    action_Outputs->setChecked(settings.value("outputs", true).toBool());
    OutputList->setVisible(settings.value("outputs", true).toBool());
    action_Console->setChecked(settings.value("console", true).toBool());
    Console->setVisible(settings.value("console", true).toBool());
    action_Objecttoolbar->setChecked(settings.value("objecttoolbar", true).toBool());
    ObjecttoolBar->setVisible(settings.value("objecttoolbar", true).toBool());
    action_Maintoolbar->setChecked(settings.value("maintoolbar", true).toBool());
    MaintoolBar->setVisible(settings.value("maintoolbar", true).toBool());
    action_Zoombar->setChecked(settings.value("zoombar", true).toBool());
    ZoomBar->setVisible(settings.value("zoombar", true).toBool());
    lastfilepath = settings.value("lastFilePath", "/home").toString();
    recentFiles = settings.value("recentfiles").toStringList();
    maxpointer = settings.value("maxundos", 30).toInt();
    unsigned int defaultsc = 4294901760;
    QColor sc = settings.value("stateinitcolor", QVariant(defaultsc)).toUInt();
    unsigned int defaultbgc = 4294967295;
    QColor bgc = settings.value("bgcolor", QVariant(defaultbgc)).toUInt();
    snapToGridInterval = settings.value("snapToGridInterval", 30).toInt();
    settings.endGroup();
    setStateColor(sc);
    setBgColor(bgc);
    //MainguiWindow->restoreGeometry(settings.value("MainguiWindow/geometry").toByteArray());
    restoreGeometry(settings.value("geometry").toByteArray());
    restoreState(settings.value("windowState").toByteArray());
}

void MainguiWindow::closeEvent(QCloseEvent* event) {
    if(doyouwanttoquit()) {
        writeSettings();
        event->accept();
    } else {
        event->ignore();
    }
}

bool MainguiWindow::doyouwanttoquit() {
    if(!issaved){
        int savebefore = QMessageBox::question(this, "Quit",
                                               "Do you want to save before you exit FSMDesigner4?"
                                               , QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel);
        if (savebefore ==  QMessageBox::Yes) {
            save();
            return true;
        } else if(savebefore ==  QMessageBox::No) {
            return true;
        } else {
            return false;
        }
    } else {
        int quit = QMessageBox::question(this, "Quit",
                                         "Do you really want to quit?"
                                         , QMessageBox::Yes, QMessageBox::Abort, QMessageBox::NoButton);

        if (quit ==  QMessageBox::Yes) {
            if (simulationmode) {
                ST->quit();
            }
            return true;
        } else {
            return false;
        }
    }
}

void MainguiWindow::initRecentFilesActions() {
    recentFile1 = new QAction(this);
    recentFile1->setObjectName("recentFile1");
    recentFile1->setToolTip(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile1->setWhatsThis(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile2 = new QAction(this);
    recentFile2->setObjectName("recentFile2");
    recentFile2->setToolTip(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile2->setWhatsThis(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile3 = new QAction(this);
    recentFile3->setObjectName("recentFile3");
    recentFile3->setToolTip(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile3->setWhatsThis(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile4 = new QAction(this);
    recentFile4->setObjectName("recentFile4");
    recentFile4->setToolTip(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile4->setWhatsThis(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile5 = new QAction(this);
    recentFile5->setObjectName("recentFile5");
    recentFile5->setToolTip(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    recentFile5->setWhatsThis(QApplication::translate("MainguiWindow", "Recent File", 0, QApplication::UnicodeUTF8));
    connect(recentFile1,SIGNAL(triggered()),this,SLOT(recentfile1()));
    connect(recentFile2,SIGNAL(triggered()),this,SLOT(recentfile2()));
    connect(recentFile3,SIGNAL(triggered()),this,SLOT(recentfile3()));
    connect(recentFile4,SIGNAL(triggered()),this,SLOT(recentfile4()));
    connect(recentFile5,SIGNAL(triggered()),this,SLOT(recentfile5()));
    connect(action_ClearMenu,SIGNAL(triggered()),this,SLOT(clearRecentFiles()));
}

void MainguiWindow::openRecentFile(int pos) {
    resetUndoRedo();
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QString file = recentFiles[pos];
    if(!file.isNull()) {
        filename = file.toStdString();
        bool pro;
        int len = file.length();
        if(file.at(len - 1) == 'l' && file.at(len - 2) == 'm' && file.at(len - 3) == 'x' && file.at(len - 4) == '.') {
            if(file.at(len - 1) == 'l' && file.at(len - 2) == 'm' && file.at(len - 3) == 'x' && file.at(len - 4) == '.' && file.at(len - 5) == 'o' && file.at(len - 6) == 'r' && file.at(len - 7) == 'p' && file.at(len - 8) == '.') {
                pro = true;
            } else {
                pro = false;
            }
            if(len > 0) {
                do {
                    len--;
                } while ( (file.at(len) != '/') && (len != 0));
                file.remove(len+1, file.length() - 1);
            }
            lastfilepath = file;
            Load_I& l = LoadFactory::getLoad_I();
            if(pro) {
                if(getActiveProject()) {
                    QApplication::restoreOverrideCursor();
                    activeProjectWarning();
                    QApplication::setOverrideCursor(Qt::WaitCursor);
                }
                if(!getActiveProject()) {
                    activatepmode();
                    l.loadProject(filename);
                    refreshRecentFiles();
                    statusBar()->showMessage("Project loaded",3000);
                    setActiveProject(true);
                    Core_I& c = CoreFactory::getCore_I();
                    if(c.getFirstFsm()) {
                        Fsm_I& f = c.getCurrentFsm();
                        setcurrentfsmid(f.getFsmID());
                        setcurrenthierarchylevel(0);
                    }
                }
            } else {
                l.load(filename);
                refreshRecentFiles();
                statusBar()->showMessage("Fsm loaded",3000);
                Core_I& c = CoreFactory::getCore_I();
                Fsm_I& f = c.getCurrentFsm();
                setcurrentfsmid(f.getFsmID());
                setcurrenthierarchylevel(0);
                setActiveProject(true);
                activatepmode();
            }
            refresh();
        }
    }
    QApplication::restoreOverrideCursor();
}

void MainguiWindow::clearRecentFiles() {
    menu_Recent_Files->clear();
    menu_Recent_Files->addAction(action_ClearMenu);
    menu_Recent_Files->addSeparator();
    while (!(recentFiles.isEmpty())) {
        recentFiles.pop_back();
    }
}

void MainguiWindow::refreshRecentFiles() {
    QString file = QString::fromStdString(filename);
    if(!recentFiles.isEmpty()) {
        for(int i = 0; i < recentFiles.size(); i++) {
            if(strcmp((recentFiles[i]).toStdString().c_str(),filename.c_str()) == 0) {
                recentFiles.removeAt(i);
                i = recentFiles.size();
            }
        }
    }
    recentFiles.push_front(file);
    showRecentFiles();
}

void MainguiWindow::showRecentFiles() {
    while ((int)recentFiles.size() > 5) {
        recentFiles.pop_back();
    }
    for(int i = 0; i < (int)recentFiles.size(); i++) {
        QString file = recentFiles[i];
        int len = file.length();
        if(len > 0) {
            do {
                len--;
            } while ( (file.at(len) != '/') && (len != 0));
            file.remove(0, len + 1);
        }
        QString qs =  QString::fromStdString(itos(i)) + "   " + file;
        switch(i) {
        case 0:
            recentFile1->setText(qs);
            menu_Recent_Files->addAction(recentFile1);
            break;
        case 1:
            recentFile2->setText(qs);
            menu_Recent_Files->addAction(recentFile2);
            break;
        case 2:
            recentFile3->setText(qs);
            menu_Recent_Files->addAction(recentFile3);
            break;
        case 3:
            recentFile4->setText(qs);
            menu_Recent_Files->addAction(recentFile4);
            break;
        case 4:
            recentFile5->setText(qs);
            menu_Recent_Files->addAction(recentFile5);
            break;
        }
    }
}

void MainguiWindow::recentfile1() {
    openRecentFile(0);
}

void MainguiWindow::recentfile2() {
    openRecentFile(1);
}

void MainguiWindow::recentfile3() {
    openRecentFile(2);
}

void MainguiWindow::recentfile4() {
    openRecentFile(3);
}

void MainguiWindow::recentfile5() {
    openRecentFile(4);
}

void MainguiWindow::resetUndoRedo() {
    startpointer = 1;
    endpointer = 1;
    redopointer = 1;
    action_Undo->setEnabled(false);
    action_Redo->setEnabled(false);
}

void MainguiWindow::saveforunredo() { 
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        if(dir.mkdir(".fsmdesigner4")) {
            dir.cd(".fsmdesigner4");
            tempfilepath = dir.path();
        } else {
            tempfilepath = lastfilepath;
        }
    }
    string filename = tempfilepath.toStdString() + "/temp" + (QString::number(endpointer, 10)).toStdString() + ".pro.xml";
    s.saveProject(filename);
    if(endpointer == maxpointer) {
        endpointer = 1;
    } else {
        endpointer++;
    }
    if(startpointer == endpointer) {
        if(startpointer == maxpointer) {
            startpointer = 1;
        } else {
            startpointer++;
        }
    }
    redopointer = endpointer;
    action_Undo->setEnabled(true);
    action_Redo->setEnabled(false);
    Core_I& core = CoreFactory::getCore_I();
    core.getFsm(getcurrentfsmid());
    alreadysaved(false);
}

void MainguiWindow::nomoreredo() { 
    redopointer = endpointer;
    action_Redo->setEnabled(false);
}

void MainguiWindow::redo() { 
    Core_I& core = CoreFactory::getCore_I();
    if(endpointer == maxpointer) {
        endpointer = 1;
    } else {
        endpointer++;
    }
    Load_I& l = LoadFactory::getLoad_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        tempfilepath = lastfilepath;
    }
    string filename = tempfilepath.toStdString() + "/temp" + (QString::number(endpointer, 10)).toStdString() + ".pro.xml";
    core.reset();
    l.loadProject(filename);
    action_Undo->setEnabled(true);
    if(endpointer == redopointer) {
        action_Redo->setEnabled(false);
    }
    core.getFsm(getcurrentfsmid());
    refresh();
}

void MainguiWindow::undo() {  
    Core_I& core = CoreFactory::getCore_I();
    endpointer--;
    if(endpointer == 0){
        endpointer = maxpointer;
    }
    Load_I& l = LoadFactory::getLoad_I();
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        tempfilepath = lastfilepath;
    }
    if(endpointer + 1 == redopointer) {
        string filename = tempfilepath.toStdString() + "/temp" + (QString::number(endpointer + 1, 10)).toStdString() + ".pro.xml";
        s.saveProject(filename);
    }
    string filename = tempfilepath.toStdString() + "/temp" + (QString::number(endpointer, 10)).toStdString() + ".pro.xml";
    core.reset();
    l.loadProject(filename);
    action_Redo->setEnabled(true);
    if(endpointer == startpointer) {
        action_Undo->setEnabled(false);
    }
    core.getFsm(getcurrentfsmid());
    refresh();
} 

void MainguiWindow::exportImage()
{
    QStringList filenames;
    QString filename;
    QString format;
    int x,y;

    x=1024;
    y=768;
    format="png";

    // show save as dialog
    QFileDialog *dialog=new QFileDialog(this);
    dialog->setFileMode(QFileDialog::AnyFile);
    QStringList filters;
    filters<<"PNG-Image (*.png)"<<"JPG-Images (*.jpg)";
#if QT_VERSION>0x040300  
    filters<<"SVG-Image (*.svg)";
#endif
    filters<<"Acrobat-PDF (*.pdf)";
    dialog->setFilters(filters);
    dialog->setWindowTitle("Choose a filename to export to");
    dialog->setDirectory(lastfilepath);
    dialog->setAcceptMode(QFileDialog::AcceptSave);
    if (!dialog->exec())
        return;

    format=dialog->selectedFilter();
    filenames=dialog->selectedFiles();
    delete dialog;
    if (filenames.empty())
    {
        std::cout<<"Filenames is empty!"<<std::endl;
        return;
    }
    filename=filenames[0];
    // filename=getSaveFileName(this, "Choose a filename to export to", lastfilepath, "PNG-Image (*.png);; JPG-Images (*.jpg)");

    if (filename.isEmpty())
        return;


    int len = filename.length();
    if ( (filename.at(len - 1).toUpper() == format.at(15).toUpper() && filename.at(len - 2).toUpper() == format.at(14).toUpper() &&
          filename.at(len - 3).toUpper() == format.at(13).toUpper() && filename.at(len - 4).toUpper() == format.at(12).toUpper() ) )
    {
        filename.remove(len - 4, len - 1);
        std::cout<<"removing ending"<<std::endl;
    } 

    std::cout<<"Filename has been chosen: "<<filename.toStdString()<<std::endl;
    std::cout<<"Format is: "<<format.toStdString()<<std::endl;
    filename=filename + format.right(5).left(4);
    std::cout<<"Save to file: "<<filename.toStdString()<<std::endl;
    double x1=-1,y1=-1,x2=0,y2=0;
    // what about size etc???
    //QList <QGraphicsItem *> list= v->scene()->items();
    QList <QGraphicsItem *> list= v->scene()->items();
    QList <QGraphicsItem *>::iterator it;
    for ( it = list.begin(); it != list.end(); ++it )
    {
        if (x1==-1)
        {
            x1=(*it)->x();
        }
        if (y1==-1)
        {
            y1=(*it)->y();
        }

        if ((*it)->x() > x2 )
            x2=(*it)->x();
        if ((*it)->y() > y2 )
            y2=(*it)->y();


        if ((*it)->x() < x1 )
            x1=(*it)->x();
        if ((*it)->y() < y1 )
            y1=(*it)->y();

    }
    x=(int) (x2-x1);
    y=(int) (y2-y1);

    std::cout<<"Koords are: x1: "<<x1<<" y1:"<<y1<<" x2: "<<x2<<" y2:"<<y2<<std::endl;
    //PDF & svg export do not work with earlier QT4 versions...
#if QT_VERSION>0x040300 
    if (format=="Acrobat-PDF (*.pdf)")
    {
        QPrinter pdfprinter;
        pdfprinter.setOutputFormat(QPrinter::PdfFormat);
        pdfprinter.setFullPage(false);
        pdfprinter.setPageSize(QPrinter::A4);
        pdfprinter.setOrientation(QPrinter::Landscape);
        pdfprinter.setColorMode(QPrinter::Color);
        pdfprinter.setOutputFileName(filename);
        QPainter painter;
        painter.begin(&pdfprinter);
        painter.translate(x1,y1);
        painter.scale((double)printer.pageRect().height()/(double)y,(double)printer.pageRect().width()/(double)x);
        //v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        v->scene()->render(&painter);
        painter.end();
        return;
    }

    else if (format=="SVG-Image (*.svg)")
    {
        std::cout<<"SVG export chosen..."<<std::endl;
        QSvgGenerator svggen;
        svggen.setFileName (filename);
        if ( (-1.0 * x1 +100) > 0.0 )
            x1=0.0;
        else
            x1=-1.0 * x1 +100;
        if ( (-1.0 * y1 +100) > 0.0 )
            y1=0.0;
        else
            y1=-1.0 * y1 +100;

        svggen.setSize(QSize(x+100,y+100));
        svggen.setResolution(150);
        QPainter painter;
        painter.begin(&svggen);           // paint in picture
        painter.translate(x1,y1);
        //v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        v->scene()->render(&painter);
        painter.end();                     // painting done
        std::cout<<"SVG painting done"<<std::endl;
        return;
    }
#endif
    // paint the image
    QImage picture((int)x+200,(int)y+200,QImage::Format_RGB32);
    //QImage picture(1024,768,QImage::Format_RGB32);
    QPainter painter;
    painter.begin(&picture);           // paint in picture
    if ( (-1.0 * x1 +100) > 0.0 )
        x1=0.0;
    else
        x1=-1.0 * x1 +100;

    if ( (-1.0 * y1 +100) > 0.0 )
        y1=0.0;
    else
        y1=-1.0 * y1 +100;

    painter.translate(x1,y1);
    //v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    v->scene()->render(&painter);
    painter.end();                     // painting done

    // save the image to file
    QFile file(filename);
    file.open(QIODevice::ReadWrite);
    std::cout<<"format to save is : "<<format.right(4).left(3).toUpper().toStdString().c_str()<<std::endl;
    //picture.save(&file,format.right(5).left(4).toStdString().c_str());       // save picture
    if (format=="PNG-Image (*.png)")
        picture.save(&file,"PNG");
    else if (format=="JPG-Images (*.jpg)")
        picture.save(&file,"JPG");
    else
        std::cerr<<"Unknown file format. Ignoring."<<std::endl;

    file.close();
}

void MainguiWindow::printFsm() {


    double x1=-1,y1=-1,x2=0,y2=0;
    QList <QGraphicsItem *> list= v->items();
    QList <QGraphicsItem *>::iterator it;
    for ( it = list.begin(); it != list.end(); ++it )
    {
        if (x1==-1)
        {
            x1=(*it)->x();
        }
        if (y1==-1)
        {
            y1=(*it)->y();
        }

        if ((*it)->x() > x2 )
            x2=(*it)->x();
        if ((*it)->y() > y2 )
            y2=(*it)->y();


        if ((*it)->x() < x1 )
            x1=(*it)->x();
        if ((*it)->y() < y1 )
            y1=(*it)->y();

    }

    x1 = x1 - 40;
    if(x1 < 0) {
        x1 = 0;
    }
    y1 = y1 - 40;
    if(y1 < 0) {
        y1 = 0;
    }

    x2 = x2 + 40;
    y2 = y2 + 40;


    // int pagenum=0;
    //   bool landscape=true;
    //   bool portrait=true;
    //   bool a4=false;
    //   bool a3=false;
    //   bool letter=false;

    //   int x,y;
    //   double x1=-1,y1=-1,x2=0,y2=0;

    //   int x_end1,y_end1;

    //   int page_x, page_y;
    //   int print_x, print_y;

    //   double max_xscale, max_yscale;
    //   double scale;

    //   PrintFSM printFSM(&printer,this);
    //   cout<<"printFSM called"<<endl;
    //   //QPrintDialog printDialog(&printer, this);
    //   //  if(printDialog.exec() == QDialog::Accepted) {
    //   if(printFSM.exec() == QDialog::Accepted) {
    //     cout<<"printFSM returned"<<endl;
    //     if (printFSM.singlePage->isChecked())
    //       pagenum=1;
    //     else if (printFSM.doublePage->isChecked())
    //       pagenum=2;
    //     else
    //       pagenum=4;
    
    //     if(printer.orientation() == QPrinter::Portrait)
    //       {
    // 	landscape=false;
    // 	portrait=true;
    //       }
    //     if (printer.pageSize()==QPrinter::QPrinter::A4)
    //       a4=true;
    //     if (printer.pageSize()==QPrinter::QPrinter::A3)
    //       a3=true;
    //     if (printer.pageSize()==QPrinter::QPrinter::Letter)
    //       letter=true;

    //     cout<<"Number of pages: "<<pagenum<<endl;
    //     cout<<"Landscape is: "<<landscape<<endl;
    //     cout<<"A4 is: "<<a4<<endl;
    //     cout<<"A3 is: "<<a3<<endl;
    //     cout<<"Letter is: "<<letter<<endl;
    //     cout<<"Scaling is: "<<false<<endl;
    
    //     //calculate the bounding box of the FSM...
    //     Q3CanvasItemList list= v->canvas()->allItems();
    //     Q3CanvasItemList::iterator it;
    //     for ( it = list.begin(); it != list.end(); ++it )
    //       {
    // 	if (x1==-1)
    // 	  {
    // 	    x1=(*it)->x();
    // 	}
    // 	if (y1==-1)
    // 	  {
    // 	    y1=(*it)->y();
    // 	  }

    // 	if ((*it)->x() > x2 )
    // 	  x2=(*it)->x();
    //       if ((*it)->y() > y2 )
    // 	y2=(*it)->y();


    //       if ((*it)->x() < x1 )
    // 	x1=(*it)->x();
    //       if ((*it)->y() < y1 )
    // 	y1=(*it)->y();

    //       }
    //     x=(int) (x2-x1);
    //     y=(int) (y2-y1);

    //     //find out the coordinates of the page(s) & all the printing area
    //     page_x=printer.pageRect().width();
    //     page_y=printer.pageRect().height();
    //     if (landscape) {
    //       switch (pagenum) {
    //       case 1:{
    // 	print_x=page_x;
    // 	print_y=page_y;
    //       break; }
    //       case 2:{
    // 	print_x=page_x;
    // 	print_y=2*page_y;
    //       break; }
    //       case 4:{
    // 	print_x=2*page_x;
    // 	print_y=2*page_y;
    //       break; }
    //       }
    //     } else {
    //       switch (pagenum) {
    //       case 1:{
    // 	print_x=page_x;
    // 	print_y=page_y;
    //       break; }
    //       case 2:{
    // 	print_x=2*page_x;
    // 	print_y=page_y;
    //       break; }
    //       case 4:{
    // 	print_x=2*page_x;
    // 	print_y=2*page_y;
    //       break; }
    //       }
    //     }

    //     //set up painter
    //     printer.setFullPage(true);
    //     QPainter painter;
    //     painter.begin(&printer);

    //     //set the scaling... & translation
    //     max_xscale=(double)print_x/(double)x;
    //     max_yscale=(double)print_y/(double)y;
    //     if (max_xscale>=max_yscale) scale=max_yscale;
    //     else scale=max_xscale;
    //     painter.translate(x1,y1);
    //     painter.scale(scale,scale); //keep aspect ratio!

    //     //find out the corner coordinates of the print-drawing... & print page(s) ??
    //      if (landscape) {
    //       switch (pagenum) {
    //       case 1:{
    // 	//print all
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    //       break; }
    //       case 2:{

    //       break; }
    //       case 4:{
    // 	painter.setWindow(x1,y1,x1+x/2,y1+y/2);
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    // 	printer.newPage();
    // 	painter.setWindow(x1+x/2,y1,x1+x/2,y1+y/2);
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    // 	printer.newPage();
    // 	painter.setWindow(x1,y1+y/2,x1+x/2,y1+y/2);
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    // 	printer.newPage();
    // 	painter.setWindow(x1+x/2,y1+y/2,x1+x/2,y1+y/2);
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    //       break; }
    //       }
    //     } else {
    //       switch (pagenum) {
    //       case 1:{
    // 	//print all
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    //       break; }
    //       case 2:{
    // 	print_x=2*page_x;
    // 	print_y=page_y;
    //       break; }
    //       case 4:{
    // 	painter.setWindow(x1,y1,x1+x/2,y1+y/2);
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    // 	printer.newPage();
    // 	painter.setWindow(x1+x/2,y1,x1+x/2,y1+y/2);
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    // 	printer.newPage();
    // 	painter.setWindow(x1,y1+y/2,x1+x/2,y1+y/2);
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    // 	printer.newPage();
    // 	painter.setWindow(x1+x/2,y1+y/2,x1+x/2,y1+y/2);
    // 	v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
    // 	break; }
    //       }
    //     }
    //     //print end
    //      painter.end();

    //   }
    //   return;
    QPrintDialog printDialog(&printer, this);
    if(printDialog.exec() == QDialog::Accepted)
    {

        cout<<"PageRect: y:"<<printer.pageRect().height()<<" x:"<<printer.pageRect().width()<<endl;

        /*if(printer.pageSize() == QPrinter::A4) {
      printer.setFullPage(true);
      QPainter painter;
      painter.begin(&printer);
      if(printer.orientation() == QPrinter::Portrait) {
        if(printer.printRange() == QPrinter::AllPages) {
          //works only with all pages right now
          painter.setWindow(0,0,840,1188);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
          printer.newPage();
          painter.setWindow(841,0,840,1188);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
          printer.newPage();
          painter.setWindow(0,1189,840,1188);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
          printer.newPage();
          painter.setWindow(841,1189,840,1188);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        } else {
          painter.setWindow(0,0,840,1188);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        }
      }else {
        if(printer.printRange() == QPrinter::AllPages) {
          painter.setWindow(0,0,1188,840);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
          printer.newPage();
          painter.setWindow(0,840,1188,840);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        } else {
          painter.setWindow(0,0,1188,840);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        }
      }
      painter.end();
    } else if(printer.pageSize() == QPrinter::A3) {
      printer.setFullPage(true);
      QPainter painter;
      painter.begin(&printer);
      if(printer.orientation() == QPrinter::Portrait) {
        QRect rect = painter.viewport();
        QSize size = v->canvas()->rect().size();
        size.scale(rect.size(), Qt::KeepAspectRatio);
        painter.setViewport(rect.x(),rect.y(),size.width(),size.height());
        painter.setWindow(v->canvas()->rect());
        v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
      }else {
        if(printer.printRange() == QPrinter::AllPages) {
          painter.setWindow(0,0,1680,1188);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
          printer.newPage();
          painter.setWindow(0,1188,1680,1188);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        } else {
          painter.setWindow(0,0,1680,1188);
          v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        }
      }
      painter.end();
      } else {*/



        //       printer.setFullPage(false);
        //       QPainter painter;
        //       painter.begin(&printer);
        //       QRect rect = painter.viewport();
        //       QRect extrect = painter.viewport();
        //       int rectwid = rect.width();
        //       int extwid = rectwid;
        //       extrect.setWidth(extwid);
        //       QSize size = v->canvas()->rect().size();
        //       size.scale(extrect.size(), Qt::KeepAspectRatio);
        //       painter.setViewport(rect.x(),rect.y(),size.width(),size.height());

        //       QRect firstrec= v->canvas()->rect();
        //       QRect secrec= v->canvas()->rect();
        //       int wid = firstrec.width();
        //       wid = wid / 2;
        //       firstrec.setWidth(wid);
        //       int x0=secrec.x();
        //       x0=x0+wid;
        //       secrec.setX(x0);
        //       painter.setWindow(firstrec);
        //       v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        //       printer.newPage();
        //       painter.setViewport(rect.x(),rect.y(),size.width(),size.height());
        //       painter.setWindow(secrec);
        //       v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        //       painter.end();



        //      printer.setFullPage(false);
        //       QPainter painter;
        //       painter.begin(&printer);
        //       QRect rect = painter.viewport();
        //       QRect obj = QRect(x1,y1,x2-x1,y2-y1);
        //       QSize size = obj.size();
        //       size.scale(rect.size(), Qt::KeepAspectRatio);
        //       painter.setViewport(rect.x(),rect.y(),size.width(),size.height());
        //       painter.setWindow(v->canvas()->rect());
        //       v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        //       painter.end();



        // map all to single page
        printer.setFullPage(false);
        QPainter painter;
        painter.begin(&printer);
        QRect rect = painter.viewport();
        //QSize size = v->canvas()->rect().size();
        QSizeF size = v->scene()->sceneRect().size();
        size.scale(rect.size(), Qt::KeepAspectRatio);
        painter.setViewport(rect.x(),rect.y(),size.width(),size.height());
        //painter.setWindow(v->canvas()->rect());
        //v->canvas()->drawArea(v->canvas()->rect(), &painter, false);
        int wwidth = (v->scene()->sceneRect().bottomRight().x() - v->scene()->sceneRect().topLeft().x() );
        int hheight = (v->scene()->sceneRect().bottomRight().y() - v->scene()->sceneRect().topLeft().y() );
        painter.setWindow(v->scene()->sceneRect().topLeft().x(), v->scene()->sceneRect().topLeft().y(), wwidth, hheight );
        v->scene()->render(&painter);
        painter.end();
        //}
    }
}

void MainguiWindow::verification() {
    int result;
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& f = core.getCurrentFsm();
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        if(dir.mkdir(".fsmdesigner4")) {
            dir.cd(".fsmdesigner4");
            tempfilepath = dir.path();
        } else {
            tempfilepath = lastfilepath;
        }
    }
    string filename = tempfilepath.toStdString() + "/temp.xml";
    savename = f.getFsmName();
    s.save(filename, savename);
    string open = "fsmverification " + filename;
    result=system(open.c_str());
    if (result==-1)
    {
        fprintf(stderr,"Verification command could not be started.\n");
    }
    else if (result!=0)
        fprintf(stderr,"Verification returned with error code %d\n",result);
}

void MainguiWindow::verificationextern() {
    VerificationDialog* pd;
    pd = new VerificationDialog(this);
    pd->exec();
}  

void MainguiWindow::minimize() {
    Min* min;
    min = new Min(this);
    min->exec();
}

void MainguiWindow::genVerilog() {
    int result;
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& f = core.getCurrentFsm();
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        if(dir.mkdir(".fsmdesigner4")) {
            dir.cd(".fsmdesigner4");
            tempfilepath = dir.path();
        } else {
            tempfilepath = lastfilepath;
        }
    }
    string filename = tempfilepath.toStdString() + "/temp.xml";
    savename = f.getFsmName();
    string c1;
    string c2;
    string c3;
    s.save(filename, savename);
    if(verificationstatetrans) {
        c1 = " true";
    } else {
        c1 = " false";
    }
    if(verificationstatecoverage) {
        c2 = " true";
    } else {
        c2 = " false";
    }
    if(verificationtranscoverage) {
        c3 = " true";
    } else {
        c3 = " false";
    }
    string open = "fsmveriloggeneration gui " + filename + " " + lastfilepath.toStdString() + c1 + c2 + c3;
    result = system(open.c_str());
    if (result==-1)
        fprintf(stderr,"Verilog generation command could not be started.\n");
    else if (result!=0)
        fprintf(stderr,"Verilog generation returned with error code %d\n",result);

}

void MainguiWindow::genVHDL() {
    int result;
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& f = core.getCurrentFsm();
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        if(dir.mkdir(".fsmdesigner4")) {
            dir.cd(".fsmdesigner4");
            tempfilepath = dir.path();
        } else {
            tempfilepath = lastfilepath;
        }
    }
    string filename = tempfilepath.toStdString() + "/temp.xml";
    savename = f.getFsmName();
    s.save(filename, savename);
    string open = "fsmvhdlgeneration gui " + filename + " " + lastfilepath.toStdString();
    result = system(open.c_str());
    if (result==-1)
        fprintf(stderr,"VHDL generation command could not be started.\n");
    else if (result!=0)
        fprintf(stderr,"VHDL generation returned with error code %d\n",result);

}

void MainguiWindow::generateVHDLTestbench() 
{
    int result;
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& f = core.getCurrentFsm();
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        if(dir.mkdir(".fsmdesigner4")) {
            dir.cd(".fsmdesigner4");
            tempfilepath = dir.path();
        } else {
            tempfilepath = lastfilepath;
        }
    }
    string filename = tempfilepath.toStdString() + "/temp.xml";
    savename = f.getFsmName();
    s.save(filename, savename);
    string open = "fsmvhdlgeneration gui testbench " + filename + " " + lastfilepath.toStdString();
    result = system(open.c_str());
    if (result==-1)
        fprintf(stderr,"VHDL generation command could not be started.\n");
    else if (result!=0)
        fprintf(stderr,"VHDL generation returned with error code %d\n",result);

}

void MainguiWindow::globals() {
    SelectInputs* si;
    si = new SelectInputs();
    si->exec();
}

void MainguiWindow::simulationEnd() {
    activatepmode();
    simulationmode = false;
}

void MainguiWindow::simulation(int type, int id) {
    if(!simulationmode) {
        simWidget->setVisible(true);
        ST = new SimThread(this);
        ST->start();
        deactivatepmode();
        simulationmode = true;
        issaved = true;
        connectsimwindow();
    }
    v->simulationMark(type, id);
}

void MainguiWindow::connectsimwindow() {
    connect(curstatemode,SIGNAL(stateChanged(int)),this,SLOT(updatemarking()));
    connect(nextstatemode,SIGNAL(stateChanged(int)),this,SLOT(updatemarking()));
    connect(transmode,SIGNAL(stateChanged(int)),this,SLOT(updatemarking()));
}


void MainguiWindow::setValues() {
    QString qs;
    qs = QString::fromStdString(invector);
    inputvectorvalue->setText(qs);
    qs = QString::fromStdString(nstate);
    nextstatevalue->setText(qs);
    qs = QString::fromStdString(cstate);
    currentstatevalue->setText(qs);
    qs = QString::fromStdString(cursortime);
    curtime->setText(qs);
}

void MainguiWindow::simstep() {
    invector = getInputvector();
    cstate = getState();
    nstate = getNextState();
    setValues();
    calcID();
    updatemarking();
}

void MainguiWindow::updatemarking() {
    if(transmode->isChecked() && !(inputID < 0)) {
        simulation(2,inputID);
    } else if (curstatemode->isChecked() && !(cstateID < 0)) {
        simulation(1,cstateID);
    } else if (nextstatemode->isChecked() && !(nstateID < 0)) {
        simulation(1,nstateID);
    } else {
        cout << "Marking is not possible input is not valid!" << endl;
    }
}

void MainguiWindow::calcID() {

    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    int count = 0;
    bool go = true;
    if(f.getFirstState()) {
        do {
            string s = f.getStateOutput();
            if(strcmp(s.c_str(),cstate.c_str()) == 0) {
                cstateID = count;
                go = false;
            }
            count++;
        } while (f.getNextState() && go);
    }
    if(go) {
        cstateID = -1;
    }
    count = 0;
    go = true;
    if(f.getFirstState()) {
        do {
            string s = f.getStateOutput();
            if(strcmp(s.c_str(),nstate.c_str()) == 0) {
                nstateID = count;
                go = false;
            }
            count++;
        } while (f.getNextState() && go);
    }
    if(go) {
        nstateID = -1;
    }
    go = true;
    bool defaultfound = false;
    int defaultID = 0;
    int numberofinputs = f.getNumberOfInputs();
    if(invector.length() != numberofinputs) {
        go = false;
    }
    if(f.getFirstTrans()){
        do {
            if(f.getTransStart() == cstateID) {
                if(f.getTransFirstCondition()) {
                    do {
                        if(f.getTransDefault()) {
                            defaultID = f.getTransID();
                        } else {
                            string s = f.getTransConditionValue();
                            if(strcmp(s.c_str(),invector.c_str()) == 0){
                                inputID = f.getTransID();
                                go = false;
                            }
                        }
                    } while (f.getTransNextCondition());
                }
            }
        } while (f.getNextTrans() && go);
    }
    if(go) {
        if(f.getFirstTrans()){
            do {
                if(f.getTransStart() == cstateID) {
                    if(f.getTransFirstCondition()) {
                        do {
                            bool found = true;
                            string condition = f.getTransConditionValue();
                            char X;
                            for(int i = 0; i < numberofinputs; i++) {
                                X = condition[i];
                                if(X != 'x' || X != 'X' || X != '-') {
                                    if(condition[i] != invector[i]) {
                                        i = numberofinputs;
                                        found = false;
                                    }
                                }
                            }
                            if(found) {
                                go = false;
                                inputID = f.getTransID();
                            }
                        } while (f.getTransNextCondition());
                    }
                }
            } while (f.getNextTrans() && go);
        }
    }
    if(go) {
        if(defaultfound) {
            inputID = defaultID;
        }else {
            inputID = -1;
        }
    }
}

void MainguiWindow::preferences() {
    PreferencesDialog* pd;
    pd = new PreferencesDialog(this);
    pd->exec();
}

void MainguiWindow::maxUndo(int i) {
    maxpointer = i;
}

int MainguiWindow::getMaxUndo() {
    return maxpointer;
}

QColor MainguiWindow::bgColor() {
    //return v->canvas()->backgroundColor();
    return v->scene()->backgroundBrush().color();
}

void MainguiWindow::setBgColor(QColor c) {
    //v->canvas()->setBackgroundColor(c);
    v->scene()->setBackgroundBrush( c );
}

QColor MainguiWindow::stateColor() {
    return v->getInitStateColor();
}

void MainguiWindow::setStateColor(QColor c) {
    v->initStateColor(c);
}

void MainguiWindow::setsnapToGridInterval(int i) {
    snapToGridInterval = i;
}

void MainguiWindow::alreadysaved(bool b) {
    issaved = b;
}

string MainguiWindow::getInputvector() {
    return inputvec;
}

string MainguiWindow::getState() {
    return currentstate;
}

string MainguiWindow::getNextState() {
    return nextstate;
}

string MainguiWindow::getCurrentTime() {
    return cursortime;
}

void MainguiWindow::inputvector(string s) {
    inputvec = s;
}

void MainguiWindow::state(string s) {
    currentstate = s;
}

void MainguiWindow::nextState(string s) {
    nextstate = s;
}

void MainguiWindow::currentTime(string s) {
    cursortime = s;
}

void MainguiWindow::simulationstep() {
    simstep();
}


void MainguiWindow::createMMap() {
    int result;
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& f = core.getCurrentFsm();
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        if(dir.mkdir(".fsmdesigner4")) {
            dir.cd(".fsmdesigner4");
            tempfilepath = dir.path();
        } else {
            tempfilepath = lastfilepath;
        }
    }
    string filename = tempfilepath.toStdString() + "/temp.xml";
    savename = f.getFsmName();
    s.save(filename, savename);
    string open = "createmmap " + filename;
    result = system(open.c_str());
    if (result==-1)
        fprintf(stderr,"Mnemonic Map creation command could not be started.\n");
    else if (result!=0)
        fprintf(stderr,"Mnemonic Map creation returned with error code %d\n",result);
}


void MainguiWindow::createMMapProject() {
    int result;
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& f = core.getCurrentFsm();
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        if(dir.mkdir(".fsmdesigner4")) {
            dir.cd(".fsmdesigner4");
            tempfilepath = dir.path();
        } else {
            tempfilepath = lastfilepath;
        }
    }
    string filename = tempfilepath.toStdString() + "/temp.pro.xml";
    s.saveProject(filename);
    string open = "createmmap " + filename;
    result = system(open.c_str());
    if (result==-1)
        fprintf(stderr,"Mnemonic Map creation command could not be started.\n");
    else if (result!=0)
        fprintf(stderr,"Mnemonic Map creation returned with error code %d\n",result);
}

void MainguiWindow::generateTestbench() {
    int result;
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& f = core.getCurrentFsm();
    Save_I& s = SaveFactory::getSave_I();
    QString tempfilepath;
    QDir dir = QDir::home();
    if(dir.cd(".fsmdesigner4")) {
        tempfilepath = dir.path();
    } else {
        if(dir.mkdir(".fsmdesigner4")) {
            dir.cd(".fsmdesigner4");
            tempfilepath = dir.path();
        } else {
            tempfilepath = lastfilepath;
        }
    }
    string filename = tempfilepath.toStdString() + "/temp.xml";
    savename = f.getFsmName();
    s.save(filename, savename);
    string open = "createtestbench gui " + filename;
    result = system(open.c_str());
    if (result==-1)
        fprintf(stderr,"Testbench generation command could not be started.\n");
    else if (result!=0)
        fprintf(stderr,"Testbench generation returned with error code %d\n",result);
}

bool MainguiWindow::getverificationstatetrans() {
    return verificationstatetrans;
}

bool MainguiWindow::getverificationstatecoverage() {
    return verificationstatecoverage;
}

bool MainguiWindow::getverificationtranscoverage() {
    return verificationtranscoverage;
}

void MainguiWindow::setverificationstatetrans(bool b) {
    verificationstatetrans = b;
}

void MainguiWindow::setverificationstatecoverage(bool b) {
    verificationstatecoverage = b;
} 

void MainguiWindow::setverificationtranscoverage(bool b) {
    verificationtranscoverage = b;
}

void MainguiWindow::cNameComboChanged(int index)
{
    cnameComboIndex = index;
    (dynamic_cast<InputModel*>(inputeditorwidget->model()))->rowsChanged();
#ifdef DEBUGMAINGUIWINDOW
    // qDebug()<<"MainguiWindow has a received from cNameComboBox the index change to: "<<cnameComboIndex;
#endif

}

void MainguiWindow::expLineThicknessChanged(int i) {
//	cout << "New Thickness: " << i <<endl;

	// Change value
	fsmDesignerSettings->setValue("ui/TransLineWidth",QVariant(i));

	// Redraw
	refresh();
}

void MainguiWindow::expStateTextBackgroundColor() {

	// Show Color Dialog
	QColorDialog colorDialog;
	colorDialog.setCurrentColor(fsmDesignerSettings->value(SETTINGS_STATE_TEXT_BG,QVariant(SETTINGS_STATE_TEXT_BG_DEFAULT)).value<QColor>());
	if (colorDialog.exec()==QDialog::Accepted) {

		// Get Color and register
		fsmDesignerSettings->setValue(SETTINGS_STATE_TEXT_BG,QVariant(colorDialog.selectedColor()));

		// Redraw
		refresh();

	}
}

void MainguiWindow::expStateFutureDisplay(int value) {

	// Change value
	bool newval = value==Qt::Checked ? true: false;
	fsmDesignerSettings->setValue(SETTINGS_STATE_FUTURE,QVariant(newval));

	// Redraw
	refresh();

}
