/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/
#ifndef MAINGUIWINDOW_H_
#define MAINGUIWINDOW_H_

#include <QtGui/QMainWindow>
#include <QtGui/QAction>
#include <QtGui/QPrinter>
#include <QtCore/QStringList>
#include <string>
#include <iostream>

#include "newproject.h"
#include "newfsm.h"
#include "newoutput.h"
#include "newinput.h"
#include "view.h"
#include "scene.h"
#include "fsmgui.h"
#include "maingui.h"
#include "save.h"
#include "newfsmname.h"
#include "renameoutput.h"
#include "renameinput.h"
#include "renameproject.h"
#include "renamefsm.h"
#include "deleteconditiontrans.h"
#include "deleteconditionhypertrans.h"
#include "propertiesmodel.h"
#include "table.h"
#include "selectinputs.h"
#include "min.h"
#include "preferencesdialog.h"
#include "verificationdialog.h"
#include "../genverilog/generationofverilog.h"
#include "../simvision/simthread.h"
//#include "../simvision/simwindow.h"

using namespace std;

class MainguiWindow : public QMainWindow, public Maingui {

  Q_OBJECT
  
 public:
  QLabel* level;
  QLabel* curfsmname;
  QPrinter printer;
  SimThread* ST;
  View* v;
 
  MainguiWindow();

  void setSavemode(bool b); 
  void setSaveAccept(bool b);
  void setSavename(string name);
  void setcurrentfsmid(int i);
  void setcurrenthierarchylevel(int l);
  void setcurrentnumberofhlevels(int cur);
  void setActiveProject(bool a); 
  void setPropertyStatus(int status, int id); // Sets the property status and id to propertywidget and outputeditorwidget
  int getcurrenthierarchylevel();
  int getcurrentfsmid();
  bool getActiveProject();
  int getPropertiesType();
  int getPropertiesID();
  int getsceneWidth();
  int getsceneHeight();
  int getsnapToGridInterval();
  void saveforunredo();
  void simulation(int type, int id);
  void simulationEnd();
  void maxUndo(int i);
  int getMaxUndo();
  QColor bgColor();
  void setBgColor(QColor c); 
  QColor stateColor();
  void setStateColor(QColor c);
  void setsnapToGridInterval(int i);
  void nomoreredo();
  void alreadysaved(bool b);
  string getInputvector();
  string getState();
  string getNextState();
  string getCurrentTime();
  void inputvector(string s);
  void state(string s);
  void nextState(string s);
  void currentTime(string s);
  void simulationstep();  
  bool getverificationstatetrans();
  bool getverificationstatecoverage();
  bool getverificationtranscoverage(); 
  void setverificationstatetrans(bool b);
  void setverificationstatecoverage(bool b);
  void setverificationtranscoverage(bool b);
  int cnameComboIndex;
  int numberOfConditions;
  void fillcNameCombobox();

 public slots:
   void setmodeChoose();
   void setmodeState();
   void setmodeTrans();
   void setmodeHtrans();
   void setmodeLink();
   void setmodeJoin();
   void setmodeRemove(); 
   void setmodeHierarchy();
   void setmodeconnecttoLink();
   void activateCut();
   void activateCopy();
   void activatePaste();
   void activateRemove();
   void movetoBack();
   void movetoFront();
   void refreshWorkbench();
   void refresh();
   void changeGrid();
   void zoomIn();
   void zoomOut();
   void activatepmode();
   void deactivatepmode();
   void noprojectmode();
   void initProject();
   void newFsm();
   void initFsm();
   void output();
   void input();
   void save();
   void saveas();
   void load();
   void activeProjectWarning();
   void refreshInputs();
   void refreshOutputs();
   void refreshProjectTree();
   void refreshProperties();
   void renameoutput();
   void deleteoutput(); 
   void deleteselectedoutput();
   void moveupoutput();
   void movedownoutput();
   void renameinput();
   void deleteinput();
   void deleteselectedinput();
   void moveupinput();
   void movedowninput();
   void renameProject();
   void renameFsm();
   void deleteFsm();
   void deleteFsmWithoutAsking();
   void deleteProject();
   void insertLevel();
   void deleteLevel();
   void deleteAllOnLevel(int level);
   void moveElementsFromLevel2Level(int prevlevel, int newlevel);
   void redo();
   void undo();
   void printFsm(); 
   void addConditionToTrans();
   void deleteConditionOfTrans();
   void addConditionToHypertrans();
   void deleteConditionOfHypertrans();
   void statetable();
   void transtable();
   void verification();
   void verificationextern();
   void minimize();
   void genVerilog();
   void genVHDL();
   void globals();
   void preferences();
   void exportImage();
   void generateTestbench();
   void generateVHDLTestbench();
   void createMMap();
   void createMMapProject();
   void cNameComboChanged(int index);
   
   /// Experimental parameter
   void expLineThicknessChanged(int i);

   /// Experimental: Called to choose background color of state text
   void expStateTextBackgroundColor();

   /// Experimental: Use new design for state display ??
   void expStateFutureDisplay(int);

 private slots:
   void about();
   void aboutQt();
   void adaptStatus();
   void recentfile1(); 
   void recentfile2();
   void recentfile3();
   void recentfile4(); 
   void recentfile5();
   void clearRecentFiles();
  
 private:
   //View* v;
   QPixmap* logo;
   QString lastfilepath;
   QStringList recentFiles;
   QAction* recentFile1;
   QAction* recentFile2;
   QAction* recentFile3;
   QAction* recentFile4;
   QAction* recentFile5;
   string projectname;
   string filename;
   string savename;
   bool savemode;
   bool saveaccept;
   bool issaved;
   bool presaved;
   bool activeProject;
   bool simulationmode;
   bool verificationstatetrans;
   bool verificationstatecoverage;
   bool verificationtranscoverage;
   int simcounter;
   int currentnumberofhlevels;
   int currentfsmid;
   int currenthierarchylevel;
   int propertiestype;
   int propertiesid;
   int redopointer;
   int maxpointer;
   int startpointer;
   int endpointer;
   int sceneWidth;
   int sceneHeight;
   int snapToGridInterval; // Size of the grid to perform Snap To Grid function
   string inputvec;
   string currentstate;
   string nextstate;  
   string cursortime;


   void createConnections();
   void workbench();
   void initvariables();
   void initStatusbarlabels();
   void initRecentFilesActions();
   void writeSettings();
   void readSettings();
   void openRecentFile(int pos);
   void refreshRecentFiles();
   void showRecentFiles();
   void resetUndoRedo();
   void enableActions(bool b);
   void keyPressEvent(QKeyEvent* ke);
   void keyReleaseEvent(QKeyEvent* ke);
   void closeEvent(QCloseEvent* event);
   void connectsimwindow();
 

   bool doyouwanttoquit(); 

 public:
   string itos(int i);

   /// simulation /////
   
 private:
   string invector;
   string cstate;
   string nstate;
   int inputID;
   int cstateID;
   int nstateID;
   
   void calcID(); 
   void setValues();
   
 public slots:
   void updatemarking();
   void simstep();
   

};

#endif

