/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include "min.h"

#include "../verification/logicmin.h"
#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"

Min::Min(QWidget* w) {
  qw = w;
  setupUi(this);
  label->setText(QApplication::translate("Min", "<u>Choose Trans:</u>", 0, QApplication::UnicodeUTF8));
  this->setWindowTitle(QApplication::translate("Min", "Minimization", 0, QApplication::UnicodeUTF8));
  int count = 0;
  int i = 0;
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  if(f.getFirstTrans()) {
    do {
      if(f.getTransFirstCondition()) {
	if(f.getTransNextCondition()) {
	  count++;
	  string name = f.getTransName();
	  QString qs = QString::fromStdString(name);
	  idlist->insertItem(i++,qs);
	}
      }
    } while (f.getNextTrans());
    idlist->insertItem(i++,"all");
  }
  if(count == 0) {
    okButton->setEnabled(false);
  }
}


void Min::accept() {
  Core_I& c = CoreFactory::getCore_I(); 
  Fsm_I& f = c.getCurrentFsm();
  vector<string> input;
  pair<string, string> result;
  string condition;
  string name;
  int id;
  int i = -1;
  string transname =(idlist->currentText()).toStdString();
  if(strcmp(transname.c_str(), "all") == 0) {
    if(f.getFirstTrans()) {
      do {
	if(f.getTransFirstCondition()) {
	  if(f.getTransNextCondition()) {
	    i = f.getTransID();
	    f.getTransbyID(i);
	    if(f.getTransFirstCondition()){
	      do {
		condition = f.getTransConditionValue();
		string result = "";
		char X;
		for(int i = 0; i < f.getNumberOfInputs(); i++) { 
		  X = condition[i];
		  if( X == 'x' || X == 'X') {
		    result = result + '-';
		  } else {
		    result = result + X;
		  }
		}
		input.push_back(result); 
	      } while (f.getTransNextCondition());
	    }
	    while(f.getTransFirstCondition()){
	      id = f.getTransConditionID();
	      f.deleteConditionOfTrans(i, id);
	    }
	    logicMin(input,result);
	    int count = 1;
	    condition = result.first;
	    int mlen = condition.length();
	    int len = mlen;	 
	    if(len > 0) {
	      while (len > f.getNumberOfInputs()) {
		string s = condition;
		do {
		  len--;
		} while ( (s[len] != '+') && (len != 0));
		s.assign(s, (len + 1), (mlen - len - 1));
		condition.assign(condition, 0, len);      
		f.addConditionToTrans(i);
		id = f.getTransConditionID();
		f.modifyTransConditionValue(i, id, s);
		name = "min_" + (QString::number(count, 10)).toStdString();
		count++;
		f.modifyTrans(i, id, name);
	      }
	    }
	    f.addConditionToTrans(i);
	    id = f.getTransConditionID();
	    f.modifyTransConditionValue(i, id, condition);
	    name = "min_" + (QString::number(count, 10)).toStdString();
	    f.modifyTrans(i, id, name);
	  }
	}
      } while (f.getNextTrans());
    }
  } else {
    if(f.getFirstTrans()) {
      do {
	if(strcmp(transname.c_str(), f.getTransName().c_str()) == 0) {
	  i = f.getTransID();
	}
      } while (f.getNextTrans());
    }
    if(i > -1) {
      f.getTransbyID(i);
      if(f.getTransFirstCondition()){
	do {
	  condition = f.getTransConditionValue();
	  string result = "";
	  char X;
	  for(int i = 0; i < f.getNumberOfInputs(); i++) { 
	    X = condition[i];
	    if( X == 'x' || X == 'X') {
	      result = result + '-';
	    } else {
	      result = result + X;
	    }
	  }
	  input.push_back(result); 
	} while (f.getTransNextCondition());
      }
      while(f.getTransFirstCondition()){
	id = f.getTransConditionID();
	f.deleteConditionOfTrans(i, id);
      }
      logicMin(input,result);
      int count = 1;
      condition = result.first;
      int mlen = condition.length();
      int len = mlen;	 
      if(len > 0) {
	while (len > f.getNumberOfInputs()) {
	  string s = condition;
	  do {
	    len--;
	  } while ( (s[len] != '+') && (len != 0));
	  s.assign(s, (len + 1), (mlen - len - 1));
	  condition.assign(condition, 0, len);      
	  f.addConditionToTrans(i);
	  id = f.getTransConditionID();
	  f.modifyTransConditionValue(i, id, s);
	  name = "min_" + (QString::number(count, 10)).toStdString();
	  count++;
	  f.modifyTrans(i, id, name);
	}
      }
      f.addConditionToTrans(i);
      id = f.getTransConditionID();
      f.modifyTransConditionValue(i, id, condition);
      name = "min_" + (QString::number(count, 10)).toStdString();
      f.modifyTrans(i, id, name);
    }
  }
  dynamic_cast<MainguiWindow*>(qw)->refresh(); 
  this->done(0); 
}
  
  
