/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include "newfsm.h"
#include "mainguiwindow.h"
#include "newoutputinit.h"
#include "newinputinit.h"

#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"

NewFsm::NewFsm(QWidget* w, Qt::WFlags f) : QDialog(w,f) {
  qw = w;
  setupUi(this);
  connections();
  incount = 0;
  outcount = 0;
}

void NewFsm::connections() {
  connect(newinput, SIGNAL(clicked()), this, SLOT(newInput()));
  connect(newoutput, SIGNAL(clicked()), this, SLOT(newOutput()));
  connect(deleteinput, SIGNAL(clicked()), this, SLOT(deleteInput()));
  connect(deleteoutput, SIGNAL(clicked()), this, SLOT(deleteOutput()));
  connect(inputup, SIGNAL(clicked()), this, SLOT(moveInputUp()));
  connect(outputup, SIGNAL(clicked()), this, SLOT(moveOutputUp()));
  connect(inputdown, SIGNAL(clicked()), this, SLOT(moveInputDown()));
  connect(outputdown, SIGNAL(clicked()), this, SLOT(moveOutputDown()));
}


void NewFsm::deleteOutput() {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  QTreeWidgetItem *it = outputlist->currentItem();
  int i = outputlist->indexOfTopLevelItem(it);
  if (i > -1) {
    while (i < (outcount - 1)) {
      f.outputName(i, f.getOutputName(i + 1));
      i++;
    }
  delete it;
  //f.deleteOutputName();
  outcount--;
  f.numberOfOuts(outcount, false);
  }
}

void NewFsm::deleteInput() {
  QTreeWidgetItem *it = inputlist->currentItem();
  int i = inputlist->indexOfTopLevelItem(it);
  if (i > -1) { 
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm(); 
    while (i < (incount - 1)) {
      f.inputName(i, f.getInputName(i + 1));
      i++;
    }
    delete it; 
    //f.deleteInputName();
    incount--;
    f.numberOfIns(incount, -1000);
  }
}

void NewFsm::newInput() {
  NewInputInit* nii;
  nii = new NewInputInit(this);
  nii->exec();
}
 
void NewFsm::newOutput() {
  NewOutputInit* noi;
  noi = new NewOutputInit(this);
  noi->exec();
}

void NewFsm::inText(QString qs) {
  QTreeWidgetItem *it = new QTreeWidgetItem(inputlist);
  it->setText(0,qs);
  incount++;
}

void NewFsm::outText(QString qs) {
  QTreeWidgetItem *it = new QTreeWidgetItem(outputlist);
  it->setText(0,qs);
  outcount++;
}

void NewFsm::reject() {
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getCurrentFsm();
  // for (int i = f.getNumberOfInputs(); i > 0; i--) {
//     f.deleteInputName();
//   }
//   for(int i = f.getNumberOfOutputs(); i > 0; i--) {
//     f.deleteOutputName();
//   }
  f.numberOfIns(0, -1000);
  f.numberOfOuts(0, false);
  //this->close();
  this->done(0);
}

void NewFsm::moveInputUp() {
 QTreeWidgetItem *it = inputlist->currentItem();
  int i = inputlist->indexOfTopLevelItem(it);
  if (i > 0) { 
     Core_I& c = CoreFactory::getCore_I();
     Fsm_I& f = c.getCurrentFsm();   
     string s;
     s = f.getInputName(i);
     f.inputName(i, f.getInputName(i - 1));
     f.inputName(i - 1, s);
     QTreeWidgetItem *tit = inputlist->takeTopLevelItem(i);
     inputlist->insertTopLevelItem(i - 1, tit); 
     inputlist->setCurrentItem(tit);
  }
}

void NewFsm::moveInputDown() {
 QTreeWidgetItem *it = inputlist->currentItem();
  int i = inputlist->indexOfTopLevelItem(it);
  if (i > -1 && i < (incount - 1)) { 
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm(); 
    string s;
    s = f.getInputName(i); 
    f.inputName(i, f.getInputName(i + 1)); 
    f.inputName(i + 1, s);
    QTreeWidgetItem *tit = inputlist->takeTopLevelItem(i); 
    inputlist->insertTopLevelItem(i + 1, tit); 
    inputlist->setCurrentItem(tit);
  }
}

void NewFsm::moveOutputUp() {
 QTreeWidgetItem *it = outputlist->currentItem();
  int i = outputlist->indexOfTopLevelItem(it);
  if (i > 0) { 
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();  
    string s;
    s = f.getOutputName(i);   
    f.outputName(i, f.getOutputName(i - 1)); 
    f.outputName(i - 1, s);
    QTreeWidgetItem *tit = outputlist->takeTopLevelItem(i); 
    outputlist->insertTopLevelItem(i - 1, tit); 
    outputlist->setCurrentItem(tit);
  }
}

void NewFsm::moveOutputDown() {
 QTreeWidgetItem *it = outputlist->currentItem();
  int i = outputlist->indexOfTopLevelItem(it);
  if (i > -1 && i < (outcount - 1)) { 
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();  
    string s;
    s = f.getOutputName(i);  
    f.outputName(i, f.getOutputName(i + 1)); 
    f.outputName(i + 1, s);
    QTreeWidgetItem *tit = outputlist->takeTopLevelItem(i);
    outputlist->insertTopLevelItem(i + 1, tit);
    outputlist->setCurrentItem(tit);
  }
}

void NewFsm::accept() {
  (dynamic_cast<MainguiWindow*>(qw))->refresh();
  this->done(0);
}


