/***********************************************************************
*                                                                      *
* (C) 2009, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#define DEBUGOUTPUTMODEL

#include <QtGui/QComboBox>
#include <sstream>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>



#include "outputmodel.h"
#include "outputwidget.h"
#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"

using namespace std;


OutputModel::OutputModel(QObject* parent) : QAbstractItemModel(parent) {
    type = 0;
    propid = 0;
}

OutputModel::~OutputModel() {

}

void OutputModel::setMainWidget(QWidget* w) {
    wid = w;
}

int OutputModel::columnCount(const QModelIndex& index) const {

    Q_UNUSED(index);

    return 2;
}

void OutputModel::setParams(int status, int id) {
    type = status;
    propid = id;
}

int OutputModel::rowCount(const QModelIndex& parent) const {

    if(parent != QModelIndex()) {
        return 0;
    }

    int rows;
    Core_I& co = CoreFactory::getCore_I();
    if (co.isFsm()) // a FSM Project exists
    {
        Fsm_I& fsm = co.getCurrentFsm();
        rows = fsm.getNumberOfOutputs(); //State
    } else { rows = 1; }
    return rows;
}

QVariant OutputModel::data(const QModelIndex& index, int role) const {


    Core_I& co = CoreFactory::getCore_I();
    if (co.isFsm()) // a FSM Project exists
    {
        Fsm_I& ff = co.getCurrentFsm();

        /*if (&ff==NULL || &ff==0) {
        	co.getFirstFsm();
        	ff = co.getCurrentFsm();
        }*/
        string s;
        QString qs;
        int ia;
        int i;
        stringstream ss;
        int ifsmid;
        int nOutputs;
        nOutputs = ff.getNumberOfOutputs();
        if (type==1) // State
        {
            ifsmid = ff.getFsmID();
            nOutputs = ff.getNumberOfOutputs();
#ifdef DEBUGOUTPUTMODEL
            //qDebug()<<"Outputmodel has Number of Outputs = "<<nOutputs;
#endif
            // FIRST COLUMN
            if ((index.column() == 0) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole)))
            {

                if ((index.row()< nOutputs) )
                {
                    QString qs;
                    qs = QString::fromStdString(ff.getOutputName(index.row()));
                    //qs.append("- Nr.");
                    //qs.append(QString("%1").arg(index.row()));
                    return qs;
                }
                else {
                    return QString("");
                }
            }
            // SECOND COLUMN
            if ((index.column() == 1) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole)))
            {
            ff.getStatebyID(propid);

                if (index.row()< nOutputs)
                {
                    QString qs;
                    qs = QString( QChar( ff.getStateOutputBit(index.row())));
                    return qs;
                }
                else {
                    return QString("");
                }
            }


        }
        else  // All other objects and no selected objects
        {
#ifdef DEBUGOUTPUTMODEL
            //qDebug()<<"Outputmodel has found FSM id: "<<ifsmid;
#endif
            //ifsmid =0;
            nOutputs = ff.getNumberOfOutputs();
            // FIRST COLUMN
            if ( (index.column() == 0) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole) ) )
            {
                if ((index.row()< nOutputs) )
                {
                    QString qs;
                    qs = QString::fromStdString(ff.getOutputName(index.row()));
                    //qs.append("- Nr.");
                    //qs.append(QString("%1").arg(index.row()));
                    return qs;
                }
            }

        }
    }  //No FSM present
    else{
        if ( (index.column() == 0) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole) ) )
        {
            return QString("No FSM ");
        }
        else if ( (index.column() == 1) && ( (role == Qt::EditRole) || (role == Qt::DisplayRole) ) )
        {
            return QString("active ");
        }
    }


    return QVariant();

}

QModelIndex OutputModel::index(int row, int column, const QModelIndex& parent) const {

    int id = row*2 + column;

    if (!parent.isValid()) {
        return createIndex(row, column, id);
    }

    return createIndex(row, column, 100);
}

QModelIndex OutputModel::parent(const QModelIndex& index) const {

    if (!index.isValid() || index.internalId() < 1000) {
        return QModelIndex();
    }
    return createIndex(index.row(), index.column(), 1000);
}

bool OutputModel::setData(const QModelIndex& index, const QVariant& value, int role) {

    Core_I& co = CoreFactory::getCore_I();
    if (co.isFsm()) // a FSM Project exists
    {
        Fsm_I& ff = co.getCurrentFsm();
        char cv;
        char cs [255];

        if( (index.column()== 1) && (type==1))  // SECOND COLUMN + State
        {
            ff.getStatebyID(propid);

            dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
            //ff.modifyStateOutputBit(propid, index.row(), value.Char);
            QVariant vv = value;
#ifdef DEBUGOUTPUTMODEL
            //qDebug()<<"MainGui Widget in Output Editor: "<<wid;
            /*if (vv.canConvert<int>())
            { qDebug()<<"Edited Value as QVariant Value in Output Editor: "<<vv<<"int: "<< vv.toInt();}

            if (vv.canConvert<QString>() )
            { qDebug()<<"Edited Value as QVariant Value in Output Editor: "<<vv<<"string: "<< vv.toString(); } */
#endif
            int val = vv.toInt();
            if ( (val==0) || (val==1) )
            {
                sprintf(cs, "%d", val);
                cv = cs[0];
                ff.modifyStateOutputBit(propid, index.row(), cv);
            }
#ifdef DEBUGOUTPUTMODEL
            qDebug()<<"Edited Value in Output Editor: "<<val<<"char: "<< cv; //<<"string: "<<sv;
            cout <<"Cout Char of Edited Value in Output Editor: "<<cv << endl;;
#endif
            //dynamic_cast<MainguiWindow*>(wid)->outputeditorwidget->scrollToTop();
            //dynamic_cast<MainguiWindow*>(wid)->refresh();
        }



    }
}






Qt::ItemFlags OutputModel::flags(const QModelIndex& index) const {
    Qt::ItemFlags flag = QAbstractItemModel::flags(index);
    if (isEditable(index)) {
        flag |= Qt::ItemIsEditable;
    }
    return flag;
}

bool OutputModel::isEditable(const QModelIndex& index) const {
//    if(type == 3 || type == 4) { // Link + Hypertransition
//        if(index.column() == 1 && index.row() == 1) {
//            return false;
//        }
//    }
//    if(type == 4 && index.column() == 1 && index.row() == 3) { // Hypertransition
//        return false;
//    }
    if (index.row() == -1)
    {
        return false;
    }
    else {return true;}
}

QModelIndex OutputModel::buddy(const QModelIndex& index) const {
    if (index.column() == 0) {
        return createIndex(index.row(), 1, index.internalPointer());
    }
    return index;
}

QVariant OutputModel::headerData(int section, Qt::Orientation orientation, int role) const {
    if (orientation == Qt::Horizontal) {
        if (role != Qt::DisplayRole) {
            return QVariant();
        }
        switch (section) {
        case 0: return QString("Outputs");
        case 1: return QString("Output Bits");
        default: return QString();
        }
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}

void OutputModel::rowsAdded( int first, int last) {

    //dynamic_cast<MainguiWindow*>(w)->outputeditorwidget->model()->
            beginInsertRows(QModelIndex(), first, last);
            endInsertRows();
            layoutChanged();
}

void OutputModel::rowsChanged()
{            layoutChanged(); }
