/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#define DEBUGPROJECTTREEWIDGET

#include <QtGui/QMenu>
#include <QtGui/QContextMenuEvent>

#include "projecttreewidget.h"

#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"


ProjectTreeWidget::ProjectTreeWidget(QWidget* w) : QTreeWidget(w) {
    createActions();
}

void ProjectTreeWidget::setQWidget(QWidget* w){
    qw = w;
}

void ProjectTreeWidget::contextMenuEvent(QContextMenuEvent* e) {
    QMenu contextMenu(this);
    contextMenu.addAction(renameproject);
    contextMenu.addAction(deleteproject);
    contextMenu.addAction(addfsm);
    contextMenu.addSeparator();
    contextMenu.addAction(renamefsm);
    contextMenu.addAction(newhierarchylevel);
    contextMenu.addAction(deletelevel);
    contextMenu.addAction(deletefsm);
    contextMenu.exec(e->globalPos());

}

void ProjectTreeWidget::mouseDoubleClickEvent(QMouseEvent* e) {

	// React if leftclick on an item
	if(e->button() == Qt::LeftButton && this->currentItem()) {

		// Name of clicked on FSM
		Core_I& c = CoreFactory::getCore_I();
		string  searchedFSMName = this->currentItem()->text(0).toStdString();

		// If clicked on project name, don't go any further
		if(this->currentItem()->childCount()>0)
			return;

		// Get current FSM
		int idbefore = c.getCurrentFsm().getFsmID();

		// Search
		//---------------
		bool found = false;
		Fsm_I * searchCurrentFSM = &(c.getCurrentFsm());
		c.getFirstFsm(); // move on first FSM

		// loop while there is an FSM and the right FSM was not found
		while( (searchCurrentFSM=&(c.getCurrentFsm()))!=NULL) {

			// Searched Name == current search Name, match
			if(strcmp(searchedFSMName.c_str(),searchCurrentFSM->getFsmName().c_str()) == 0) {
				found = true;

				dynamic_cast<MainguiWindow*>(qw)->setcurrentfsmid(searchCurrentFSM->getFsmID());


				// If changed FSM
				if(searchCurrentFSM->getFsmID() != idbefore) {
					dynamic_cast<MainguiWindow*>(qw)->setcurrentnumberofhlevels(0);
				}
				dynamic_cast<MainguiWindow*>(qw)->setcurrenthierarchylevel(0);
				c.setCurrentFsm(searchCurrentFSM->getFsmID());

#ifdef DEBUGPROJECTTREEWIDGET
				qDebug()<<"ProjectTreeWidget foundFSM, FSMID before : "<<idbefore <<" , FSMID new : "<< searchCurrentFSM->getFsmID() <<" , currentfsmidMaingui : "<<dynamic_cast<MainguiWindow*>(qw)->getcurrentfsmid();
#endif

				// Refresh display
				dynamic_cast<MainguiWindow*>(qw)->refreshWorkbench();

				break;
			} else
				// Go to next
				c.getNextFsm();
		}
    }
}

void ProjectTreeWidget::createActions(){
    addfsm = new QAction(this);
    addfsm->setObjectName(QString::fromUtf8("addFsm"));
    addfsm->setEnabled(true);
    addfsm->setText(QApplication::translate("ProjectTreeWidget", "New FSM", 0, QApplication::UnicodeUTF8));
    newhierarchylevel = new QAction(this);
    newhierarchylevel->setObjectName(QString::fromUtf8("newHierarchyLevel"));
    newhierarchylevel->setEnabled(true);
    newhierarchylevel->setText(QApplication::translate("ProjectTreeWidget", "Insert new level", 0, QApplication::UnicodeUTF8));
    deletelevel = new QAction(this);
    deletelevel->setObjectName(QString::fromUtf8("deleteLevel"));
    deletelevel->setEnabled(true);
    deletelevel->setText(QApplication::translate("ProjectTreeWidget", "Delete current level", 0, QApplication::UnicodeUTF8));
    deletefsm = new QAction(this);
    deletefsm->setObjectName(QString::fromUtf8("deleteFsm"));
    deletefsm->setEnabled(true);
    deletefsm->setText(QApplication::translate("ProjectTreeWidget", "Delete FSM", 0, QApplication::UnicodeUTF8));
    deleteproject = new QAction(this);
    deleteproject->setObjectName(QString::fromUtf8("deleteProject"));
    deleteproject->setEnabled(true);
    deleteproject->setText(QApplication::translate("ProjectTreeWidget", "Close Project", 0, QApplication::UnicodeUTF8));
    renameproject = new QAction(this);
    renameproject->setObjectName(QString::fromUtf8("renameProject"));
    renameproject->setEnabled(true);
    renameproject->setText(QApplication::translate("ProjectTreeWidget", "Rename project", 0, QApplication::UnicodeUTF8));
    renamefsm = new QAction(this);
    renamefsm->setObjectName(QString::fromUtf8("renameFsm"));
    renamefsm->setEnabled(true);
    renamefsm->setText(QApplication::translate("ProjectTreeWidget", "Rename FSM", 0, QApplication::UnicodeUTF8));
}

string ProjectTreeWidget::itos(int i) {
    stringstream s;
    s << i;
    return s.str();
}
