/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <QtGui/QComboBox>
#include <sstream>

#include "propertiesmodel.h"
#include "propertywidget.h"
#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"


PropertiesModel::PropertiesModel(QObject* parent) : QAbstractItemModel(parent) {
    type = 0;
    propid = 0;
}  

PropertiesModel::~PropertiesModel() {

}

void PropertiesModel::setMainWidget(QWidget* w) {
    wid = w;
}

int PropertiesModel::columnCount(const QModelIndex& index) const {

    Q_UNUSED(index);

    return 2;
}

void PropertiesModel::setParams(int status, int id) {
    type = status;
    propid = id;
}

int PropertiesModel::rowCount(const QModelIndex& parent) const { 

    if(parent != QModelIndex()) {
        return 0;
    }

    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& fsm = core.getCurrentFsm();
    int rows;

    switch(type) {
    case 0:
        rows = 1;
        break;
    case 1:
        rows = 5; //State
        break;
    case 2:
        rows = 5; //Transition Transline
        fsm.getTransbyID(propid);
        if(fsm.getTransFirstCondition()) {
            do {
                rows = rows + 2;
            } while (fsm.getTransNextCondition());
        }
        break;
    case 3:
        //rows = 3; //Link without color
        rows = 4; //Link with color
        break;
    case 4:
        rows = 4; //Hypertransition
        fsm.getHypertransbyID(propid);
        if(fsm.getHypertransFirstCondition()) {
            do {
                rows = rows + 2;
            } while (fsm.getHypertransNextCondition());
        }
        break;
    default:
        rows = 0;
    }

    return rows;
}

QVariant PropertiesModel::data(const QModelIndex& index, int role) const {

    Core_I& co = CoreFactory::getCore_I();
    Fsm_I& ff = co.getCurrentFsm();
    string s;
    QString qs;
    int ia;
    int i;
    stringstream ss;

    switch (index.column()) {
    case 0: // FIRST COLUMN
        switch (role) {
        case Qt::EditRole:
        case Qt::DisplayRole:
            switch(type) {

            case 0:     // default empty
                return QString("Item");
                break;
            case 1:     // State
                switch(index.row()) {
                case 0:
                    return QString("Item");
                    break;
                case 1:
                    return QString("Name");
                    break;
                case 2:
                    return QString("Resetstate");
                    break;
                case 3:
                    return QString("Output");
                    break;
                case 4:
                    return QString("Color");
                    break;
                default:
                    break;
                }
            case 2:     // Transline Transition
                switch(index.row()) {
                case 0:
                    return QString("Item");
                    break;
                case 1:
                    return QString("Name");
                    break;
                case 2:
                    return QString("Start");
                    break;
                case 3:
                    return QString("End");
                    break;
                case 4:
                    return QString("Default");
                    break;
                default:
                    if(index.row() < rowCount(QModelIndex())) {
                        if((index.row()%2) == 0) {
                            return QString("CValue");
                        } else {
                            return QString("CName");
                        }
                    }
                    break;
                }
            case 3:     // NOT Leveltransfer, should be Link
                switch(index.row()) {
                case 0:
                    return QString("Item");
                    break;
                case 1:
                    return QString("ID");
                    break;
                case 2:
                    return QString("Target");
                    break;
                case 3:
                    return QString("Color");
                    break;
                default:
                    break;
                }
            case 4:     // NOT Link, should be Hypertransition
                switch(index.row()) {
                case 0:
                    return QString("Item");
                    break;
                case 1:
                    return QString("ID");
                    break;
                case 2:
                    return QString("Target");
                    break;
                case 3:
                    return QString("Type");
                    break;
                default:
                    if(index.row() < rowCount(QModelIndex())) {
                        if((index.row()%2) == 0) {
                            return QString("CName");
                        } else {
                            return QString("CValue");
                        }
                    }
                    break;
                }

            }
        default:
            break;
        }
        break;

    case 1: { // SECOND COLUMN
            switch (role) {
            case Qt::EditRole:
            case Qt::DisplayRole:
                switch(type) {
                case 0:     // default empty
                    return QString("not selected");
                    break;
                case 1:     // State
                    ff.getStatebyID(propid);
                    switch(index.row()) {
                    case 0:
                        return QString("State");
                        break;
                    case 1:
                        s = ff.getStateName();
                        qs = QString::fromStdString(s);
                        return qs;
                        break;
                    case 2:
                        i = ff.getResetState();
                        if(i == propid) {
                            return true;
                        } else {
                            return false;
                        }
                        break;
                    case 3:
                        s = ff.getStateOutput();
                        qs = QString::fromStdString(s);
                        return qs;
                        break;
                    case 4:{
                            i = (int) ff.getStateColor();
                            QColor c=QColor::fromRgb((unsigned int) i);
                            QVariant v=c;
                            return v;}
                        break;
                    default:
                        break;
                    }
                case 2:     // Transline Transition
                    ff.getTransbyID(propid);
                    switch(index.row()) {
                    case 0:
                        return QString("Transition");
                        break;
                    case 1:
                        s = ff.getTransName();
                        qs = QString::fromStdString(s);
                        return qs;
                        break;
                    case 2:
                        switch (role) {
                        case Qt::EditRole:
                            i = ff.getTransStart();
                            return i;
                            break;
                        case Qt::DisplayRole:
                            i = ff.getTransStart();
                            if(i < 0) {
                                qs = "none";
                            } else {
                                ff.getStatebyID(i);
                                s = ff.getStateName();
                                qs = QString::fromStdString(s);
                            }
                            return qs;
                            break;
                        }
                    case 3:
                        switch (role) {
                        case Qt::EditRole:
                            i = ff.getTransEnd();
                            return i;
                            break;
                        case Qt::DisplayRole:
                            i = ff.getTransEnd();
                            if (i < 0) {
                                qs = "none";
                            } else {
                                ff.getStatebyID(i);
                                s = ff.getStateName();
                                qs = QString::fromStdString(s);
                            }
                            return qs;
                            break;
                        }
                    case 4:
                        if(ff.getTransDefault()) {
                            return true;
                        } else {
                            return false;
                        }
                        break;
                    default:
                        if(index.row() < rowCount(QModelIndex())) {
                            if(ff.getTransFirstCondition()){
                                int c = index.row();
                                c = c - 6;
                                while(c > 0) {
                                    ff.getTransNextCondition();
                                    c = c - 2;

                                }
                            }
                            if((index.row()%2) == 0) {
                                s = ff.getTransConditionValue();
                                qs = QString::fromStdString(s);
                                return qs;
                            } else {
                                s = ff.getTransConditionName();
                                qs = QString::fromStdString(s);
                                return qs;
                            }
                        }
                        break;
                    }
                case 3: // Link
                    ff.getLinkbyID(propid);
                    switch(index.row()) {
                    case 0:
                        return QString("Link");
                        break;
                    case 1:
                        i = ff.getLinkID();
                        ss << i;
                        s =  ss.str();
                        qs = QString::fromStdString(s);
                        return qs;
                        break;
                    case 2:
                        switch (role) {
                        case Qt::EditRole:
                            i = ff.getLinkGoal();
                            return i;
                            break;
                        case Qt::DisplayRole:
                            i = ff.getLinkGoal();
                            ff.getStatebyID(i);
                            s = ff.getStateName();
                            qs = QString::fromStdString(s);
                            return qs;
                            break;
                        }
                    case 3:{
                            i = (int) ff.getLinkColor();
                            QColor c=QColor::fromRgb((unsigned int) i);
                            QVariant v=c;
                            return v;}
                        break;
                    default:
                        break;
                    }
                case 4: // Hypertransition
                    ff.getHypertransbyID(propid);
                    switch(index.row()) {
                    case 0:
                        return QString("Hypertransition");
                        break;
                    case 1:
                        i = ff.getHypertransID();
                        ss << i;
                        s =  ss.str();
                        qs = QString::fromStdString(s);
                        return qs;
                        break;
                    case 2:
                        switch (role) {
                        case Qt::EditRole:
                            i = ff.getHypertransGoal();
                            return i;
                            break;
                        case Qt::DisplayRole:
                            i = ff.getHypertransGoal();
                            ff.getStatebyID(i);
                            s = ff.getStateName();
                            qs = QString::fromStdString(s);
                            return qs;
                            break;
                        }
                    case 3:
                        if(ff.getHypertransType()) {
                            qs = "all_states";
                        } else {
                            qs = "rest_states";
                        }
                        return qs;
                        break;
                    default:
                        if(index.row() < rowCount(QModelIndex())) {
                            if(ff.getHypertransFirstCondition()){
                                int c = index.row();
                                c = c - 5;
                                while(c > 0) {
                                    ff.getHypertransNextCondition();
                                    c = c - 2;
                                }
                            }
                            if((index.row()%2) == 0) {
                                s = ff.getHypertransConditionName();
                                qs = QString::fromStdString(s);
                                return qs;
                            } else {
                                s = ff.getHypertransConditionValue();
                                qs = QString::fromStdString(s);
                                return qs;
                            }
                        }
                        break;
                    }

                }
            default:
                break;
            }
            break;
        }

    default:
        break;
    }

    return QVariant();

}

QModelIndex PropertiesModel::index(int row, int column, const QModelIndex& parent) const {

    int id = row*2 + column;

    if (!parent.isValid()) {
        return createIndex(row, column, id);
    }

    return createIndex(row, column, 100);
}

QModelIndex PropertiesModel::parent(const QModelIndex& index) const { 

    if (!index.isValid() || index.internalId() < 100) {
        return QModelIndex();
    }
    return createIndex(index.row(), index.column(), 100);
}

bool PropertiesModel::setData(const QModelIndex& index, const QVariant& value, int role) { 
    Core_I& co = CoreFactory::getCore_I();
    Fsm_I& ff = co.getCurrentFsm();
    string s;
    int i;
    int level;
    int posi;
    int rowhelp;
    int rowcnt;

    if(index.column() == 1) {
        switch(type) {
        case 1: // State
            {
                ff.getStatebyID(propid);
                switch(index.row()) {
                case 1: //Name
                    {
                        i = propid;
                        dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
                        QString qs = value.toString().trimmed();
                        FilterKeywords* fk;
                        fk = new FilterKeywords();
                        qs = fk->RemoveBlanks(qs);
                        qs = fk->FilterSpecialChars(qs);
                        qs = fk->FilterHDLKeywords(qs);
                        qs = fk->FilterBusChars(qs);
                        string name = fk->QStrToStr(qs);
                        ff.modifyState(propid, name);
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                        dynamic_cast<MainguiWindow*>(wid)->refresh();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
                    }
                    break;
                case 2: //Reset State
                    {
                        bool b=value.toBool();
                        if (b==true) {
                            i = propid;
                            dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                            dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
                            ff.resetState(propid);
                            dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                            dynamic_cast<MainguiWindow*>(wid)->refresh();
                            dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
                        }
                    }
                    break;
                case 3: //State Output
                    {
                        i = propid;
                        dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
                        ff.stateOutput(propid, value.toString().trimmed().toStdString());
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                        dynamic_cast<MainguiWindow*>(wid)->refresh();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
                    }
                    break;
                case 4: // State color
                    {
                        i = propid;
                        dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
                        unsigned int color = value.value<QColor>().rgb();
                        ff.modifyState(propid, color);
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                        dynamic_cast<MainguiWindow*>(wid)->refresh();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
                    }
                    break;
                default:
                    break;
                }
            }
            break;
        case 2: // Transition Transline
            {
                ff.getTransbyID(propid);
                switch(index.row()) {
                case 0:
                    break;
                case 1: // Transistion Name
                    {
                        i = propid;
                        dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                        QString qs = value.toString().trimmed();
                        FilterKeywords* fk;
                        fk = new FilterKeywords();
                        qs = fk->RemoveBlanks(qs);
                        qs = fk->FilterSpecialChars(qs);
                        qs = fk->FilterHDLKeywords(qs);
                        qs = fk->FilterBusChars(qs);
                        string name = fk->QStrToStr(qs);
                        ff.modifyTrans(propid, name);
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                        dynamic_cast<MainguiWindow*>(wid)->refresh();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                    }
                    break;
                case 2:
                    {
                        i = propid;
                        dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                        ff.getTransbyID(propid);
                        s = value.toString().trimmed().toStdString();
                        if(strcmp(s.c_str(), "none") == 0) {
                            ff.modifyTrans(propid, -1, ff.getTransEnd());
                        } else {
                            i = ff.getTransStart();
                            if(i > -1) {
                                ff.getStatebyID(i);
                                level = ff.getStateLevel();
                            } else {
                                i = ff.getTransEnd();
                                if(i > -1) {
                                    ff.getStatebyID(i);
                                    level = ff.getStateLevel();
                                } else {
                                    level = -1;
                                }
                            }
                            if(level == -1) {
                                if(ff.getFirstState()) {
                                    do {
                                        if(strcmp(s.c_str(), ff.getStateName().c_str()) == 0) {
                                            ff.modifyTrans(propid, ff.getStateID(), ff.getTransEnd());
                                        }
                                    } while (ff.getNextState());
                                }
                            } else {
                                if(ff.getFirstState()) {
                                    do {
                                        if(strcmp(s.c_str(), ff.getStateName().c_str()) == 0) {
                                            if(ff.getStateLevel() == level) {
                                                ff.modifyTrans(propid, ff.getStateID(), ff.getTransEnd());
                                            }
                                        }
                                    } while (ff.getNextState());
                                }
                            }
                        }
                        i = propid;
                        posi = dynamic_cast<MainguiWindow*>(wid)->propertywidget->verticalScrollBar()->sliderPosition();
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                        dynamic_cast<MainguiWindow*>(wid)->refreshWorkbench();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->verticalScrollBar()->setSliderPosition(posi);
                    }
                    break;
                case 3:
                    {
                        i = propid;
                        dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                        ff.getTransbyID(propid);
                        s = value.toString().trimmed().toStdString();
                        if(strcmp(s.c_str(), "none") == 0) {
                            ff.modifyTrans(propid, ff.getTransStart(), -1);
                        } else {
                            i = ff.getTransEnd();
                            if(i > -1) {
                                ff.getStatebyID(i);
                                level = ff.getStateLevel();
                            } else {
                                i = ff.getTransStart();
                                if(i > -1) {
                                    ff.getStatebyID(i);
                                    level = ff.getStateLevel();
                                } else {
                                    level = -1;
                                }
                            }
                            if(level == -1) {
                                if(ff.getFirstState()) {
                                    do {
                                        if(strcmp(s.c_str(), ff.getStateName().c_str()) == 0) {
                                            ff.modifyTrans(propid, ff.getTransStart(), ff.getStateID());
                                        }
                                    } while (ff.getNextState());
                                }
                            } else {
                                if(ff.getFirstState()) {
                                    do {
                                        if(strcmp(s.c_str(), ff.getStateName().c_str()) == 0) {
                                            if(ff.getStateLevel() == level) {
                                                ff.modifyTrans(propid, ff.getTransStart(), ff.getStateID());
                                            }
                                        }
                                    } while (ff.getNextState());
                                }
                            }
                        }
                        i = propid;
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                        dynamic_cast<MainguiWindow*>(wid)->refreshWorkbench();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                    }
                    break;
                case 4:
                    {
                        i = propid;
                        dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                        bool b=value.toBool();
                        ff.modifyTrans(propid, b);
                        posi = dynamic_cast<MainguiWindow*>(wid)->propertywidget->verticalScrollBar()->sliderPosition();
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                        dynamic_cast<MainguiWindow*>(wid)->refreshWorkbench();
                        dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                        dynamic_cast<MainguiWindow*>(wid)->propertywidget->verticalScrollBar()->setSliderPosition(posi);
                    }
                    break;
                default:
                    i = propid;
                    rowhelp = index.row();
                    rowcnt = rowCount(QModelIndex());
                    dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                    dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                    ff.getTransbyID(propid);
                    if(rowhelp < rowcnt) {
                        if(ff.getTransFirstCondition()){
                            int c = rowhelp;
                            c = c - 6;
                            while(c > 0) {
                                ff.getTransNextCondition();
                                c = c - 2;
                            }
                        }
                        int cid = ff.getTransConditionID();
                        if((rowhelp%2) == 0) {
                            ff.modifyTransConditionValue(propid, cid, value.toString().trimmed().toStdString());
                        } else {
                            ff.modifyTrans(propid, cid, value.toString().trimmed().toStdString());
                        }
                    }
                    i = propid;
                    posi = dynamic_cast<MainguiWindow*>(wid)->propertywidget->verticalScrollBar()->sliderPosition();
                    dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                    dynamic_cast<MainguiWindow*>(wid)->refreshWorkbench();
                    dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(2, i);
                    dynamic_cast<MainguiWindow*>(wid)->propertywidget->verticalScrollBar()->setSliderPosition(posi);
                    break;
                }
            }

            break;
        case 3: // Link
            ff.getLinkbyID(propid);
            switch(index.row()) {
            case 2:
                i = propid;
                dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(3, i);
                ff.getLinkbyID(propid);
                level = ff.getLinkLevel();
                s = value.toString().trimmed().toStdString();
                if(ff.getFirstState()) {
                    do {
                        if(strcmp(s.c_str(), ff.getStateName().c_str()) == 0) {
                            if(ff.getStateLevel() == level) {
                                ff.linkGoal(propid, ff.getStateID());
                            }
                        }
                    } while (ff.getNextState());
                }
                dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                dynamic_cast<MainguiWindow*>(wid)->refreshWorkbench();
                dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(3, i);
                break;

            case 3:
                {
//                    i = propid;
//                    dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
//                    dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);
//                    unsigned int color = value.value<QColor>().rgb();
//                    ff.modifyState(propid, color);
//                    dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
//                    dynamic_cast<MainguiWindow*>(wid)->refresh();
//                    dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(1, i);

                    i = propid;
                    dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                    dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(3, i);
                    unsigned int color = value.value<QColor>().rgb();
                    ff.modifyLinkColour(propid, color);
                    dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                    dynamic_cast<MainguiWindow*>(wid)->refresh();
                    dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(3, i);
                }
                break;
            }
            break;
        case 4: // Hypertransition
            ff.getHypertransbyID(propid);
            switch(index.row()) {
            case 0:
                break;
            case 1:
                break;
            case 2:
                i = propid;
                dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(4, i);
                ff.getHypertransbyID(propid);
                level = ff.getHypertransLevel();
                s = value.toString().trimmed().toStdString();
                if(ff.getFirstState()) {
                    do {
                        if(strcmp(s.c_str(), ff.getStateName().c_str()) == 0) {
                            if(ff.getStateLevel() == level) {
                                ff.modifyHypertrans(propid, ff.getHypertransPosx(), ff.getHypertransPosy(), ff.getStateID());
                            }
                        }
                    } while (ff.getNextState());
                }
                dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                dynamic_cast<MainguiWindow*>(wid)->refreshWorkbench();
                dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(4, i);
                break;
            case 3:
                break;
            default:
                i = propid;
                dynamic_cast<MainguiWindow*>(wid)->saveforunredo();
                dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(4, i);
                ff.getHypertransbyID(propid);
                if(index.row() < rowCount(QModelIndex())) {
                    if(ff.getHypertransFirstCondition()){
                        int c = index.row();
                        c = c - 5;
                        while(c > 0) {
                            ff.getHypertransNextCondition();
                            c = c - 2;
                        }
                    }
                    int cid = ff.getHypertransConditionID();
                    if((index.row()%2) == 0) {
                        ff.modifyHypertrans(propid, cid, value.toString().trimmed().toStdString());
                    } else {
                        ff.modifyHypertransConditionValue(propid, cid, value.toString().trimmed().toStdString());
                    }
                }
                posi = dynamic_cast<MainguiWindow*>(wid)->propertywidget->verticalScrollBar()->sliderPosition();
                dynamic_cast<MainguiWindow*>(wid)->propertywidget->scrollToTop();
                dynamic_cast<MainguiWindow*>(wid)->refreshWorkbench();
                dynamic_cast<MainguiWindow*>(wid)->setPropertyStatus(4, i);
                dynamic_cast<MainguiWindow*>(wid)->propertywidget->verticalScrollBar()->setSliderPosition(posi);
                break;
            }
            break;
        default:
            break;
        }
        return true;
    }
    return false;
}

Qt::ItemFlags PropertiesModel::flags(const QModelIndex& index) const {
    Qt::ItemFlags flag = QAbstractItemModel::flags(index);
    if (isEditable(index)) {
        flag |= Qt::ItemIsEditable;
    }
    return flag;
}

bool PropertiesModel::isEditable(const QModelIndex& index) const {
    if(type == 3 || type == 4) { // Link + Hypertransition
        if(index.column() == 1 && index.row() == 1) {
            return false;
        }
    }
    if(type == 4 && index.column() == 1 && index.row() == 3) { // Hypertransition
        return false;
    }
    return index.column() == 1 && !(index.row() == 0);
}

QModelIndex PropertiesModel::buddy(const QModelIndex& index) const {
    if (index.column() == 0) {
        return createIndex(index.row(), 1, index.internalPointer());
    }
    return index;
}

QVariant PropertiesModel::headerData(int section, Qt::Orientation orientation, int role) const {
    if (orientation == Qt::Horizontal) {
        if (role != Qt::DisplayRole) {
            return QVariant();
        }
        switch (section) {
        case 0: return QString("Property");
        case 1: return QString("Value");
        default: return QString();
        }
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}

