/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <QtGui/QMenu>
#include <QtGui/QContextMenuEvent>

#include "propertywidget.h"

#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"

#include <iostream>

PropertyWidget::PropertyWidget(QWidget* w) : QTreeView(w) {
    createActions();
}

void PropertyWidget::setMainWidget(QWidget* w) {
    wid = w;
}

QWidget* PropertyWidget::getMainWidget() {
    return wid;
}



void PropertyWidget::contextMenuEvent(QContextMenuEvent* e) {
    Core_I& core = CoreFactory::getCore_I();
    Fsm_I& fsm = core.getCurrentFsm();
    if((dynamic_cast<MainguiWindow*>(wid)->getPropertiesType()) == 2 || (dynamic_cast<MainguiWindow*>(wid)->getPropertiesType()) == 4) {
        QMenu contextMenu(this);
        if((dynamic_cast<MainguiWindow*>(wid)->getPropertiesType()) == 2) {
            contextMenu.addAction(addCondiToTrans);
            contextMenu.addAction(deleteCondiOfTrans);
            fsm.getTransbyID((dynamic_cast<MainguiWindow*>(wid))->getPropertiesID());
            if(fsm.getTransFirstCondition()) {
                deleteCondiOfTrans->setEnabled(true);
            } else {
                deleteCondiOfTrans->setEnabled(false);
            }
            contextMenu.exec(e->globalPos());
        } else {
            contextMenu.addAction(addCondiToHtrans);
            contextMenu.addAction(deleteCondiOfHtrans);
            fsm.getHypertransbyID((dynamic_cast<MainguiWindow*>(wid))->getPropertiesID());
            if(fsm.getHypertransFirstCondition()) {
                deleteCondiOfHtrans->setEnabled(true);
            } else {
                deleteCondiOfHtrans->setEnabled(false);
            }
            contextMenu.exec(e->globalPos());
        }
    }
}

void PropertyWidget::createActions() {
    addCondiToTrans = new QAction(this);
    addCondiToTrans->setObjectName(QString::fromUtf8("CondiToTrans"));
    addCondiToTrans->setEnabled(true);
    addCondiToTrans->setText(QApplication::translate("PropertyWidget", "Add Condition", 0, QApplication::UnicodeUTF8));
    deleteCondiOfTrans = new QAction(this);
    deleteCondiOfTrans->setObjectName(QString::fromUtf8("deleteCondiOfTrans"));
    deleteCondiOfTrans->setEnabled(true);
    deleteCondiOfTrans->setText(QApplication::translate("PropertyWidget", "Delete Condition", 0, QApplication::UnicodeUTF8));
    addCondiToHtrans = new QAction(this);
    addCondiToHtrans->setObjectName(QString::fromUtf8("CondiToHtrans"));
    addCondiToHtrans->setEnabled(true);
    addCondiToHtrans->setText(QApplication::translate("PropertyWidget", "Add Condition", 0, QApplication::UnicodeUTF8));
    deleteCondiOfHtrans = new QAction(this);
    deleteCondiOfHtrans->setObjectName(QString::fromUtf8("deleteCondiOfHtrans"));
    deleteCondiOfHtrans->setEnabled(true);
    deleteCondiOfHtrans->setText(QApplication::translate("PropertyWidget", "Delete Condition", 0, QApplication::UnicodeUTF8));
}
