/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"
#include <QtGui/QComboBox>

class StateComboBox: public QComboBox {
  Q_OBJECT;
public:
  // the closeClicked parameter can be used to automatically close the combobox when a selection
  // has been made. This is usefull for the property editor!
  StateComboBox(QWidget * parent, bool closeClicked=false): QComboBox(parent),wasClicked(false),toClose(closeClicked)
    {
      //printf("statecombox\n");
      QStringList statenamelist;
      Core_I& c = CoreFactory::getCore_I(); 
      Fsm_I& f = c.getCurrentFsm();
      int i = -1;
      //  (dynamic_cast<View*>(qw))->sethelpvar(i);
      insertItem(i++,"none",-1);  
      int count = 0;
      if(f.getFirstState()) {
	//    if (f.getStateLevel() == (dynamic_cast<View*>(qw))->getCurrentLevel()) {
	count++;
	string s = f.getStateName();
	QString qs = QString::fromStdString(s);
	//printf("add state to box %s\n",qs.toStdString().c_str());
	insertItem(i++,qs,f.getStateID());
	//}
	while (f.getNextState()) { 
	  //if (f.getStateLevel() == (dynamic_cast<View*>(qw))->getCurrentLevel()) {
	  count++;
	  string s = f.getStateName();
	  QString qs = QString::fromStdString(s);
	  //printf("add state to box %s\n",qs.toStdString().c_str());
	  insertItem(i++,qs,f.getStateID());
	  //     }
	}
      }  
      
      if (toClose)
	 connect(this,SIGNAL(currentIndexChanged(int)),this,SLOT(changed(int)));
    }
  ~StateComboBox()
    {
      if (toClose)
	disconnect(this,SIGNAL(currentIndexChanged(int)),this,SLOT(changed(int)));
    }
 
 public slots:
   void changed(int idx)
   {
     wasClicked=true;
     this->close();     
   }

 protected:
  bool wasClicked;
  bool toClose;
  
};

