/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include "table.h"

#include "mainguiwindow.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"

Table::Table(QWidget* w) : QDialog(w){
  wid = w;
  setupUi(this);
  initStateView();
  initTransitionView();
  createconnections();
}

void Table::initStateView() {
  statemodel = new TableStateModel;
  statedelegate = new TableStateDelegate; 
  StateView->setModel(statemodel);
  dynamic_cast<TableStateModel*>(StateView->model())->setMainWidget(StateView);
  StateView->setItemDelegate(statedelegate);
  StateView->setAlternatingRowColors(true);
  StateView->setSortingEnabled(true);
  StateView->sortByColumn (2, Qt::AscendingOrder);
}

void Table::initTransitionView() {
  transmodel = new TableTransModel;
  transdelegate = new TableTransDelegate; 
  TransitionView->setModel(transmodel);
  TransitionView->setItemDelegate(transdelegate);
  TransitionView->setAlternatingRowColors(true);
  TransitionView->setSortingEnabled(true);
}

void Table::createconnections() {
  connect(StateView->addState, SIGNAL(triggered()),this,SLOT(addState()));
  connect(StateView->deleteState, SIGNAL(triggered()),this,SLOT(deleteState()));
  connect(TransitionView->addTrans, SIGNAL(triggered()),this,SLOT(addTrans()));
  connect(TransitionView->deleteTrans, SIGNAL(triggered()),this,SLOT(deleteTrans()));
  connect(TransitionView->addCondition, SIGNAL(triggered()),this,SLOT(addCondition()));
  connect(TransitionView->deleteCondition, SIGNAL(triggered()),this,SLOT(deleteCondition()));
}

void Table::addState() {
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  f.addState();
  int id = f.getStateID();
  f.modifyState(id, (double) 50, (double) 50);
  unsigned int color =  dynamic_cast<MainguiWindow*>(wid)->stateColor().rgb();
  f.modifyState(id, color);
  StateView->doItemsLayout();
  StateView->sortByColumn (2, Qt::AscendingOrder);
  dynamic_cast<TableStateModel*>(StateView->model())->sort(1, Qt::AscendingOrder);
}

void Table::deleteState() {
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  int counter = 0;
  int deleteid;
  if(f.getFirstState()) {
    do {
      if(counter == (StateView->currentIndex()).row()) {
	deleteid = f.getStateID();
      }
      counter++;
    } while (f.getNextState());
    f.deleteState(deleteid);
  }
  StateView->doItemsLayout(); 
  TransitionView->doItemsLayout();
}

void Table::addTrans() {
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  f.addTrans(-1,-1);
  TransitionView->doItemsLayout();
}

void Table::deleteTrans() {
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  int counter = 0;
  int deleteid;
  if(f.getFirstTrans()) {
    do {
      if(counter == (TransitionView->currentIndex()).row()) {
	deleteid = f.getTransID();
      }
      counter++;
    } while (f.getNextTrans());
    f.deleteTrans(deleteid);
  }
  TransitionView->doItemsLayout();
}

void Table::addCondition() {
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  int counter = 0;
  int id;
  if(f.getFirstTrans()) {
    do {
      if(counter == (TransitionView->currentIndex()).row()) {
	id = f.getTransID();
      }
      counter++;
    } while (f.getNextTrans());
    f.addConditionToTrans(id);
  }
  TransitionView->doItemsLayout();
}

void Table::deleteCondition() {
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  int counter = 0;
  int id;
  if(f.getFirstTrans()) {
    do {
      if(counter == (TransitionView->currentIndex()).parent().row()) {
	id = f.getTransID();
      }
      counter++;
    } while (f.getNextTrans());    
  }
  f.getTransbyID(id);
  counter = 0;
  int cid;
  if(f.getTransFirstCondition()) {
    do {
      if(counter == (TransitionView->currentIndex()).row()) {
	cid = f.getTransConditionID();
      }
      counter++;
    } while (f.getTransNextCondition());    
    f.deleteConditionOfTrans(id, cid);
  }
  TransitionView->doItemsLayout();
}

void Table::accept() {
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  int id = -1;
  bool found = true;
  while(found) {
    found = false;
    if(f.getFirstTrans()) {
      do {
	if(!f.getTransFirstTrackpoint()) {
	  if(f.getTransStart() == -1) {
	    id = f.getTransID();
	    found = true;
	  }
	  if(f.getTransEnd() == -1){
	    id = f.getTransID();
	    found = true;
	  }
	}
      } while (f.getNextTrans() && !found);    
    }
    if(found) {
      f.deleteTrans(id);
      id = -1;
    }
  }
  dynamic_cast<MainguiWindow*>(wid)->refresh();
  this->done(0);
}

void Table::reject() {
  dynamic_cast<MainguiWindow*>(wid)->undo();
  dynamic_cast<MainguiWindow*>(wid)->refresh();
  this->done(0);
}
