/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <QtGui/QComboBox>

#include "colorbutton.h"
#include "tablestatedelegate.h"
#include "table.h"


TableStateDelegate::TableStateDelegate(QObject* parent) : QItemDelegate(parent) {

}

void TableStateDelegate::paint(QPainter* painter,const QStyleOptionViewItem& option, const QModelIndex& index) const {
 QItemDelegate::paint(painter, option, index);
}

QSize TableStateDelegate::sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const {  
  return QItemDelegate::sizeHint(option, index);
}

void TableStateDelegate::setEditorData(QWidget* editor,const QModelIndex& index) const {
  if(index.column() == 2) {
    unsigned int rgb = (index.model()->data(index,Qt::EditRole)).value<QColor>().rgb();
    ColorButton* b=static_cast<ColorButton*>(editor);
    b->setColor(rgb);
  } else {
    QItemDelegate::setEditorData(editor, index);
  }
} 

QWidget* TableStateDelegate::createEditor(QWidget* parent, const QStyleOptionViewItem& option, const QModelIndex& index) const {
  if(index.column() == 2) {
    ColorButton *b=new ColorButton(parent,QColor());
    return b;
  } else {
    if((index.row()%2) == 0) {
      return QItemDelegate::createEditor(parent, option, index);
    } else {
      QWidget* w = QItemDelegate::createEditor(parent, option, index);
      QPalette p(w->palette());
      p.setBrush(QPalette::Base, QBrush(QColor(221,223,228,255) ,Qt::SolidPattern));
      w->setPalette(p);
      return w;
    }
  }
}

void TableStateDelegate::setModelData(QWidget* editor, QAbstractItemModel* model, const QModelIndex& index) const {
  if(index.column() == 2) {
    ColorButton *b = static_cast<ColorButton*>(editor);
    const_cast<QAbstractItemModel*>(index.model())->setData(index, b->getColor(), Qt::EditRole);
  } else if ( index.column() == 3) {
    QComboBox *box=static_cast<QComboBox*>(editor);
    if (box->currentIndex()==0)
      const_cast<QAbstractItemModel*>(index.model())->setData(index,false,Qt::EditRole);
    else
      const_cast<QAbstractItemModel*>(index.model())->setData(index,true,Qt::EditRole);  
  } else {
    QItemDelegate::setModelData(editor, model, index);
  }
}
