/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <sstream>
#include <iostream>
#include <QtGui/QColor>

#include "tablestatemodel.h"
#include "tablestate.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"


TableStateModel::TableStateModel(QObject* parent) : QAbstractItemModel(parent) {
}  
   
int TableStateModel::columnCount(const QModelIndex& index) const {
  
  Q_UNUSED(index);

  return 4;
}

void TableStateModel::setMainWidget(QWidget* w) {
  wid = w;
}

int TableStateModel::rowCount(const QModelIndex& parent) const { 
  
  if(parent != QModelIndex()) {
    return 0;
  }
  
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  int rows = 0;
  if(f.getFirstState()){
    do {
      rows++;
    } while (f.getNextState());
  }
  return rows;
}

QVariant TableStateModel::data(const QModelIndex& index, int role) const {

  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  string s;
  QString qs;
  int ir;
  int i;
  int id;
  stringstream ss;

  switch (index.column()) {
  case 0:
    switch (role) {
    case Qt::EditRole:
    case Qt::DisplayRole:
      ir = index.row();
      i = 0;
      id = -1;
      if(f.getFirstState()) {
	do{
	  if(ir == i) {
	    id = f.getStateID();
	  }
	  i++;
	} while (f.getNextState());
      }
      if(id > -1) {
	f.getStatebyID(id);
	s = f.getStateName();
	qs = QString::fromStdString(s);
	return qs;
	break;
      }
    }
  case 1:
    switch (role) {
    case Qt::EditRole:
    case Qt::DisplayRole:
      ir = index.row();
      i = 0;
      id = -1;
      if(f.getFirstState()) {
	do{
	  if(ir == i) {
	    id = f.getStateID();
	  }
	  i++;
	} while (f.getNextState());
      }
      if(id > -1) {
	f.getStatebyID(id);   
	s = f.getStateOutput();
	qs = QString::fromStdString(s);
	return qs;
	break;
      }
    }
  case 2:
    switch (role) {
    case Qt::EditRole:
    case Qt::DisplayRole:
      ir = index.row();
      i = 0;
      id = -1;
      if(f.getFirstState()) {
	do{
	  if(ir == i) {
	    id = f.getStateID();
	  }
	  i++;
	} while (f.getNextState());
      }
      if(id > -1) {
	f.getStatebyID(id);
	i = (int) f.getStateColor();
	QColor c=QColor::fromRgb((unsigned int) i);
	QVariant v=c;
	return v;
	break;
      }
    }
  case 3:
    switch (role) {
    case Qt::EditRole:
      ir = index.row();
      i = 0;
      id = -1;
      if(f.getFirstState()) {
	do{
	  if(ir == i) {
	    id = f.getStateID();
	  }
	  i++;
	} while (f.getNextState());
      }
      if(id > -1) {
	i = f.getResetState();
	if(i == id) {
	  return true;
	} else {
	  return false;
	}
	break;
      } 
    case Qt::DisplayRole:
      ir = index.row();
      i = 0;
      id = -1;
      if(f.getFirstState()) {
	do{
	  if(ir == i) {
	    id = f.getStateID();
	  }
	  i++;
	} while (f.getNextState());
      }
      if(id > -1) {
	i = f.getResetState();
	if(i == id) {
	  return "True";
	} else {
	  return "False";
	}
	break;
      }
    }
  }
    
  return QVariant(); 
  
}

QModelIndex TableStateModel::index(int row, int column, const QModelIndex& parent) const {
  
  int id = row*4 + column;

  if (!parent.isValid()) {
    return createIndex(row, column, id);
  }
  
  return createIndex(row, column, 1000);
}

QModelIndex TableStateModel::parent(const QModelIndex& index) const { 
  
  if (!index.isValid() || index.internalId() < 1000) {
    return QModelIndex();
  }
  return createIndex(index.row(), index.column(), 1000);
}

bool TableStateModel::setData(const QModelIndex& index, const QVariant& value, int role) { 
  Core_I& co = CoreFactory::getCore_I();  
  Fsm_I& f = co.getCurrentFsm();
  string s;
  int i;
  int ir;
  int id;

  switch (index.column()) {
  case 0:
    ir = index.row();
    i = 0;
    id = -1;
    if(f.getFirstState()) {
      do{
	if(ir == i) {
	  id = f.getStateID();
	}
	i++;
      } while (f.getNextState());
    }
    if(id > -1) {
        QString qs = value.toString().trimmed();
        FilterKeywords* fk;
        fk = new FilterKeywords();
        qs = fk->RemoveBlanks(qs);
        qs = fk->FilterSpecialChars(qs);
        qs = fk->FilterHDLKeywords(qs);
        qs = fk->FilterBusChars(qs);
        string name = fk->QStrToStr(qs);
        //f.modifyState(id, value.toString().trimmed().toStdString());
        f.modifyState(id, name);
    }      
    break;
  case 1:
    ir = index.row();
    i = 0;
    id = -1;
    if(f.getFirstState()) {
      do{
	if(ir == i) {
	  id = f.getStateID();
	}
	i++;
      } while (f.getNextState());
    }
    if(id > -1) {
      f.stateOutput(id, value.toString().trimmed().toStdString());
    }     
    break;
  case 2:
    ir = index.row();
    i = 0;
    id = -1;
    if(f.getFirstState()) {
      do{
	if(ir == i)  {
	  id = f.getStateID();
	}
	i++;
      } while (f.getNextState());
    }
    if(id > -1) {
      unsigned int color = value.value<QColor>().rgb(); //value.toString().trimmed().toInt();	
      f.modifyState(id, color);
    }
    break;
  case 3:
    ir = index.row();
    i = 0;
    id = -1;
    if(f.getFirstState()) {
      do{
	if(ir == i) {
	  id = f.getStateID();
	}
	i++;
      } while (f.getNextState());
    }
    if(id > -1) {
      bool b = value.toBool();
      if (b == true) {
	f.resetState(id);
	dynamic_cast<TableState*>(wid)->doItemsLayout();
      }
    }
    break;
  default:
    break;
    return true;
}  
  return false;
}

Qt::ItemFlags TableStateModel::flags(const QModelIndex& index) const {
  Qt::ItemFlags flag = QAbstractItemModel::flags(index);
  if (isEditable(index)) {
    flag |= Qt::ItemIsEditable;
  }
  return flag;
}

bool TableStateModel::isEditable(const QModelIndex& index) const {
  return true;
}


QVariant TableStateModel::headerData(int section, Qt::Orientation orientation, int role) const {
  if (orientation == Qt::Horizontal) {
    if (role != Qt::DisplayRole) {
      return QVariant();
    }
    switch (section) {
    case 0: return QString("Name");
    case 1: return QString("Output");
    case 2: return QString("Color");
    case 3: return QString("Resetstate");
    default: return QString();
    }
  }
  return QAbstractItemModel::headerData(section, orientation, role);
}

void TableStateModel::sort ( int column, Qt::SortOrder order = Qt::AscendingOrder )
{
 QAbstractItemModel::sort (  column, Qt::AscendingOrder );
}
