/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <QtGui/QMenu>
#include <QtGui/QContextMenuEvent>

#include "tabletrans.h"

#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"

TableTrans::TableTrans(QWidget* w) : QTreeView(w) {
 createActions();
}

void TableTrans::contextMenuEvent(QContextMenuEvent* e) {
  Core_I& core = CoreFactory::getCore_I();  
  Fsm_I& f = core.getCurrentFsm();
  QMenu contextMenu(this);
  contextMenu.addAction(addTrans);
  if(currentIndex().parent() == QModelIndex()) {
    int row = 0;
    if(f.getFirstTrans()) {
      do {
	if(row == currentIndex().row()) { 
	  contextMenu.addAction(deleteTrans);
	  contextMenu.addAction(addCondition);
	}
	row++;
      } while (f.getNextTrans());
    } 
  } else {
    if(currentIndex().isValid()) {      
      contextMenu.addAction(deleteCondition);
    }
  }
  contextMenu.exec(e->globalPos());
}

void TableTrans::createActions() {
  addTrans = new QAction(this);
  addTrans->setObjectName(QString::fromUtf8("addtrans"));
  addTrans->setEnabled(true);    
  addTrans->setText(QApplication::translate("TableTrans", "Add Transition", 0, QApplication::UnicodeUTF8));
  deleteTrans = new QAction(this);
  deleteTrans->setObjectName(QString::fromUtf8("deletetrans"));
  deleteTrans->setEnabled(true);    
  deleteTrans->setText(QApplication::translate("TableTrans", "Delete Transition", 0, QApplication::UnicodeUTF8));
  addCondition = new QAction(this);
  addCondition->setObjectName(QString::fromUtf8("addcondition"));
  addCondition->setEnabled(true);    
  addCondition->setText(QApplication::translate("TableTrans", "Add Condition", 0, QApplication::UnicodeUTF8));
  deleteCondition = new QAction(this);
  deleteCondition->setObjectName(QString::fromUtf8("deleteCondition"));
  deleteCondition->setEnabled(true);    
  deleteCondition->setText(QApplication::translate("TableTrans", "Delete Condition", 0, QApplication::UnicodeUTF8));
}
