/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include "tabletransdelegate.h"
#include "table.h"
#include "statecombobox.h"


TableTransDelegate::TableTransDelegate(QObject* parent) : QItemDelegate(parent) {

}



void TableTransDelegate::paint(QPainter* painter,const QStyleOptionViewItem& option, const QModelIndex& index) const {
 QItemDelegate::paint(painter, option, index);
}

QSize TableTransDelegate::sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const {  
  return QItemDelegate::sizeHint(option, index);
}

void TableTransDelegate::setEditorData(QWidget* editor,const QModelIndex& index) const {
  if (index.parent() == QModelIndex() && (index.column()==3 || index.column()==4) )//start state
    {
      StateComboBox *box=static_cast<StateComboBox*>(editor);
      int id = index.model()->data(index,Qt::EditRole).toInt();
      int idx=box->findData ( id);
      box->setCurrentIndex(idx);
    }
  else if (index.parent() == QModelIndex() && index.column()==2  )//start state
    {
      int idx = index.model()->data(index,Qt::EditRole).toInt();
      QComboBox* box=static_cast<QComboBox*>(editor);
      box->setCurrentIndex(idx);
    }
   else
    QItemDelegate::setEditorData(editor, index);
} 

QWidget* TableTransDelegate::createEditor(QWidget* parent, const QStyleOptionViewItem& option, const QModelIndex& index) const {
  if(index.parent() == QModelIndex()) { //top model item -> transition
      if (index.column()==3 || index.column()==4) //start state or end state
	{
	  StateComboBox* w=new StateComboBox(parent);
	  w->installEventFilter(const_cast<TableTransDelegate*>(this));
	  return w;
	}
      else  if((index.row()%2) == 0) {
	return QItemDelegate::createEditor(parent, option, index);
      } else {
	  QWidget* w = QItemDelegate::createEditor(parent, option, index);
	  QPalette p(w->palette());
	  p.setBrush(QPalette::Base, QBrush(QColor(221,223,228,255) ,Qt::SolidPattern));
	  w->setPalette(p);
	  return w;
	}
    }
   else { // not top level -> condition
     if((index.row()%2) == 0) {
       QWidget* w = QItemDelegate::createEditor(parent, option, index);
       QPalette p(w->palette());
       p.setBrush(QPalette::Base, QBrush(QColor(221,223,228,255) ,Qt::SolidPattern));
       w->setPalette(p);
       return w; 
     } else {
       return QItemDelegate::createEditor(parent, option, index);
     }
   }
}

void TableTransDelegate::setModelData(QWidget* editor, QAbstractItemModel* model, const QModelIndex& index) const {
  if (index.parent() == QModelIndex() && ( index.column()==3 || index.column()==4 ) ) //start state
    {      
      StateComboBox *box=static_cast<StateComboBox*>(editor);
      QString qs = box->currentText();
      FilterKeywords* fk;
      fk = new FilterKeywords();
      qs = fk->RemoveBlanks(qs);
      qs = fk->FilterSpecialChars(qs);
      qs = fk->FilterHDLKeywords(qs);
      qs = fk->FilterBusChars(qs);
      const_cast<QAbstractItemModel*>(index.model())->setData(index,qs,Qt::EditRole);
  }
  else if (index.parent() == QModelIndex() && index.column()==2  )//start state
    {
     QComboBox *box=static_cast<QComboBox*>(editor);
     if (box->currentIndex()==0)
       const_cast<QAbstractItemModel*>(index.model())->setData(index,false,Qt::EditRole);
     else
       const_cast<QAbstractItemModel*>(index.model())->setData(index,true,Qt::EditRole);
    }
  else
    QItemDelegate::setModelData(editor, model, index);
}
