/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <sstream>
#include <stdio.h>

#include "tabletransmodel.h"
#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"


TableTransModel::TableTransModel(QObject* parent) : QAbstractItemModel(parent) {
}  
   
int TableTransModel::columnCount(const QModelIndex& index) const {
  
  Q_UNUSED(index);

  return 5;
}

int TableTransModel::rowCount(const QModelIndex& parent) const {
  
  Core_I& core = CoreFactory::getCore_I();  
  Fsm_I& f = core.getCurrentFsm();
  int rows;
  int id;
  if(parent != QModelIndex()) {
    if(parent.internalId() < 1000) {
      rows = 0;
      if(f.getFirstTrans()){
	do {
	  if(rows == parent.row()) {
	    id = f.getTransID();
	  }
	  rows++;
	} while (f.getNextTrans());
      }
      f.getTransbyID(id);      
      rows = 0;
      if(f.getTransFirstCondition()) {
	do {
	  rows++;
	} while (f.getTransNextCondition());
      }
      return rows;
    } else {
      return 0;
    }
  }  

  rows = 0;
  if(f.getFirstTrans()){
    do {
      rows++;
    } while (f.getNextTrans());
  }
  return rows;
}

QVariant TableTransModel::data(const QModelIndex& index, int role) const {

  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  string s;
  QString qs;
  int id;
  int i;
  int rows;
  int rows2;
  stringstream ss;

  if(index.parent() == QModelIndex()) {
    rows = 0;
    if(f.getFirstTrans()){
      do {
	if(rows == index.row()) {
	  id = f.getTransID();
	}
	rows++;
      } while (f.getNextTrans());
    }
    f.getTransbyID(id);
    switch (index.column()) {
    case 0:
      switch (role) {
      case Qt::EditRole:
      case Qt::DisplayRole:
// 	i = f.getTransID();
// 	ss << i;
// 	s =  ss.str();
	s = f.getTransName();
	qs = QString::fromStdString(s);
	return qs;
	break;
      }
    case 1: 
      switch (role) {
      case Qt::EditRole:
      case Qt::DisplayRole:  
	return QString("Default");
	break;
      } 
    case 2: 
      switch (role) {
      case Qt::EditRole:
      case Qt::DisplayRole: 
	if(f.getTransDefault()) {
	  return true;
	} else {
	  return false;
	}
	//return qs;
	break;
      }
    case 3: 
      switch (role) {
      case Qt::EditRole:
	i = f.getTransStart();
	return i;
	break;
      case Qt::DisplayRole: 
	i = f.getTransStart();
	if(i < 0) {
	  qs = "none";
	} else {
	  f.getStatebyID(i);
	  s = f.getStateName();
	  qs = QString::fromStdString(s);
	}
	return qs;
	break;
      }
    case 4: 
      switch (role) {
      case Qt::EditRole:
	i = f.getTransEnd();
	return i;
	break;
      case Qt::DisplayRole:
	i = f.getTransEnd();
	if (i < 0) {
	  qs = "none";
	} else {
	  f.getStatebyID(i);
	  s = f.getStateName();
	  qs = QString::fromStdString(s);
	}
	return qs;
	break;
      }
    default:
      break;
    }
  } else {
    rows = 0;
    if(f.getFirstTrans()){
      do {
	if(rows == index.parent().row()) {
	  id = f.getTransID();
	}
	rows++;
      } while (f.getNextTrans());
    }
    f.getTransbyID(id);
    id = -1;
    rows2 = 0;
    if(f.getTransFirstCondition()){
      do {
	if(rows2 == index.row()) {
	  id = f.getTransConditionID();
	}
	rows2++;
      } while (f.getTransNextCondition());
      f.getTransConditionbyID(id);
    }
    switch (index.column()) {
    case 0:
      switch (role) {
      case Qt::EditRole:
      case Qt::DisplayRole:
	i = id;
	ss << i;
	s =  ss.str();
	qs = QString::fromStdString(s);
	return qs;
	break;
      }
    case 1: 
      switch (role) {
      case Qt::EditRole:
      case Qt::DisplayRole:  
	if(id > -1) {
	  s = f.getTransConditionName();
	  qs = QString::fromStdString(s);
	  return qs;
	}
	break;
      } 
    case 2: 
      switch (role) {
      case Qt::EditRole:
      case Qt::DisplayRole: 
	if(id > -1) {
	  s = f.getTransConditionValue();
	  qs = QString::fromStdString(s);
	  return qs;
	}
	break;
      } 
    default:
      break;
    }
    
  }
  return QVariant();   
}  

QModelIndex TableTransModel::index(int row, int column, const QModelIndex& parent) const {
    
  int id = row*5 + column;

  if (!parent.isValid()) {
    return createIndex(row, column, id);
  } 
  
  return createIndex(row, column, (id + ((parent.row() + 1)*1000)));
}

QModelIndex TableTransModel::parent(const QModelIndex& index) const { 
  
  if (!index.isValid() || index.internalId() < 1000) {
    return QModelIndex();
  } else {
    double d = index.internalId()/1000;
    int r = (int) d - 1;
    return index.model()->index(r, index.column(), QModelIndex());
  }
}

bool TableTransModel::setData(const QModelIndex& index, const QVariant& value, int role) { 
  Core_I& c = CoreFactory::getCore_I();  
  Fsm_I& f = c.getCurrentFsm();
  string s;
  int i;
  int rows;
  int rows2;
  int id;
  int ids;
  int level;

  if(index.parent() == QModelIndex()) {
    rows = 0;
    if(f.getFirstTrans()){
      do {
	if(rows == index.row()) {
	  id = f.getTransID();
	}
	rows++;
      } while (f.getNextTrans());
    }
    f.getTransbyID(id);
    switch (index.column()) {
    case 0: // Transition Name
      {
            QString qs = value.toString().trimmed();
            FilterKeywords* fk;
            fk = new FilterKeywords();
            qs = fk->RemoveBlanks(qs);
            qs = fk->FilterSpecialChars(qs);
            //qs = fk->FilterHDLKeywords(qs);
            qs = fk->FilterBusChars(qs);
            string name = fk->QStrToStr(qs);
            f.modifyTrans(id, name);
      }
#warning added the following break on 13.03.2007 MN
      break;
    case 2: 
      {
	//s = value.toString().toLower().trimmed().toStdString();
	bool b=value.toBool();
	printf("set value to %d\n",b);
	printf("type: %d\n",value.type());
	int i=value.toInt();
	printf("set value to %d\n",i);
	if (b==true)
	  {
	    printf("true\n");
	    f.modifyTrans(id, true);
	  }
	else
	  {
	    printf("false\n");	  
	    f.modifyTrans(id, false);
	  }

	//       if((strcmp(s.c_str(),"true") == 0) || (strcmp(s.c_str(),"1") == 0)) {
	// 	f.modifyTrans(id, true);
	//       }
	//       if((strcmp(s.c_str(),"false") == 0) || (strcmp(s.c_str(),"0") == 0)) {
	// 	f.modifyTrans(id, false);
	//       }
      }
      break;
    case 3: 
      {
	s = value.toString().trimmed().toStdString();
	if(strcmp(s.c_str(), "none") == 0) {
	  f.modifyTrans(id, -1, f.getTransEnd());
	} else {
	  i = f.getTransStart();
	  if(i > -1) {
	    f.getStatebyID(i);
	    level = f.getStateLevel();	  
	  } else { 
	    i = f.getTransEnd();
	    if(i > -1) {
	      f.getStatebyID(i);
	      level = f.getStateLevel();	  
	    } else {
	      level = -1;
	    }
	  }
	  if(level == -1) {
	    if(f.getFirstState()) {
	      do {
		if(strcmp(s.c_str(), f.getStateName().c_str()) == 0) {	
		  f.modifyTrans(id, f.getStateID(), f.getTransEnd());
		}
	      } while (f.getNextState());
	    }
	  } else {
	    if(f.getFirstState()) {
	      do {
		if(strcmp(s.c_str(), f.getStateName().c_str()) == 0) {
		  if(f.getStateLevel() == level) {
		    f.modifyTrans(id, f.getStateID(), f.getTransEnd());
		  }
		}
	      } while (f.getNextState());
	    }
	  }
	}
      }
      break;
    case 4: 
      {
	s = value.toString().trimmed().toStdString();
	if(strcmp(s.c_str(), "none") == 0) {
	  f.modifyTrans(id, f.getTransStart(), -1);
	} else {
	  i = f.getTransEnd();
	  if(i > -1) {
	    f.getStatebyID(i);
	    level = f.getStateLevel();	  
	  } else {
	    i = f.getTransStart();
	    if(i > -1) {
	      f.getStatebyID(i);
	      level = f.getStateLevel();	  
	    } else { 
	      level = -1;
	    }
	  }
	  if(level == -1) {
	    if(f.getFirstState()) {
	      do {
		if(strcmp(s.c_str(), f.getStateName().c_str()) == 0) {
		  f.modifyTrans(id, f.getTransStart(), f.getStateID());
		}
	      } while (f.getNextState());
	    }
	  } else {
	    if(f.getFirstState()) {
	      do {
		if(strcmp(s.c_str(), f.getStateName().c_str()) == 0) {
		  if(f.getStateLevel() == level) {
		    f.modifyTrans(id, f.getTransStart(), f.getStateID());
		  }
		}
	      } while (f.getNextState());
	    }	  
	  }
	}
      }
      break;      
    default:
      break;
    }
  } else {
    rows = 0;
    if(f.getFirstTrans()){
      do {
	if(rows == index.parent().row()) {
	  id = f.getTransID();
	}
	rows++;
      } while (f.getNextTrans());
    }
    f.getTransbyID(id);
    ids = -1;
    rows2 = 0;
    if(f.getTransFirstCondition()){
      do {
	if(rows2 == index.row()) {
	  ids = f.getTransConditionID();
	}
	rows2++;
      } while (f.getTransNextCondition());
      f.getTransConditionbyID(ids);
    }
    switch (index.column()) {
    case 1:      
      f.modifyTrans(id, ids, value.toString().trimmed().toStdString());
	break;
    case 2: 
      f.modifyTransConditionValue(id, ids, value.toString().trimmed().toStdString());
	break;
    default:
      break;
    }
  }
}

Qt::ItemFlags TableTransModel::flags(const QModelIndex& index) const {
  Qt::ItemFlags flag = QAbstractItemModel::flags(index);
  if (isEditable(index)) {
    flag |= Qt::ItemIsEditable;
  }
  return flag;
}

bool TableTransModel::isEditable(const QModelIndex& index) const {
  if(index.parent() != QModelIndex()) {
    if(index.column() == 1) {
      return true;
    } else {
      return index.column() > 1;
    }
  }
  return true;;
}

QModelIndex TableTransModel::buddy(const QModelIndex& index) const {
  if(index.parent() != QModelIndex()) {
    if(index.column() == 0) {
      return createIndex(index.row(), 2, index.internalPointer());
    }
  }
  if(index.parent() == QModelIndex()) {
    if(index.parent() == QModelIndex() && index.column() == 1) {
      return createIndex(index.row(), 2, index.internalPointer());
    } else {
      return index;
    }
  }
  return createIndex(index.row(), index.column(), index.parent().child(index.row(),0).internalPointer());
}

QVariant TableTransModel::headerData(int section, Qt::Orientation orientation, int role) const {
  if (orientation == Qt::Horizontal) {
    if (role != Qt::DisplayRole) {
      return QVariant();
    }
    switch (section) {
    case 0: return QString("Trans");
    case 1: return QString("Name");
    case 2: return QString("Value");
    case 3: return QString("Start");
    case 4: return QString("End");
    default: return QString();
    }
  }
  return QAbstractItemModel::headerData(section, orientation, role);
}

