/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

//#define DEBUGVIEW
#define DEBUGVIEW2

//#include <Qt3Support/Q3Canvas>
//#include <Qt3Support/Q3CanvasView>
#include <QtGui>
#include <QtGui/QGraphicsScene>
#include <QtGui/QGraphicsView>
#include <QtGui/QMenu>
#include <QtGui/QContextMenuEvent>
#include <QtGui/QGraphicsSceneMouseEvent>
#include <QtGui/QGraphicsSceneContextMenuEvent>
#include <QtGui/QStyleOptionGraphicsItem>
#include <QtGui/QClipboard>
#include <QtCore/QTextStream>
#include <QtCore/QPoint>
#include <QtCore/QPointF>
#include <QtGui/QMatrix>
#include <QtGui/QToolTip>
#include <QtGui/QPen>
 #include <QtOpenGL/QGLWidget>
 #include <QtOpenGL/QGLFormat>

#include <cstdlib>
#include <iostream>
#include <cmath>


#include "view.h"
#include "mainguiwindow.h"
#include "items.h"
#include "newlink.h"
#include "newhyper.h"
#include "changestart.h"
#include "changeend.h"
#include "linkconnector.h"
#include "hierarchyconnector.h"
#include "../fsm_I.h"
#include "../core_I.h"
#include "../corefactory.h"
#include "selectionanimator.h"


using namespace std;

View::View (Scene* c, QWidget* w) : QGraphicsView (c,w){
    wid = w;
    can = c;
    initvariables();
    bgGrid = new QPixmap(":/icons/grid1.png",0,0);
    createActionsforContextMenu();
    connectContextMenuActions();
    //setDragMode(ScrollHandDrag); //Sets dragging of the dokument


    scene()->setBackgroundBrush(QPixmap());
    can->update();



}

//View::~View () {
//    delete can;
//}

void View::initvariables() {
    paintmode = false;
    objectmode = CHOOSE;
    lock = UNLOCKED;    // Used for drawing of multiple objects when locked, currently set by A key
    ctrlSet = false;       //Control key not pressed down
    altSet = false;     //Alt key not pressed down
    gSet = false;     //G key not pressed down
    xSet = false;     //X key not pressed down
    activatedItem = 0;
    newItem = 0;
    front = 0;
    back = 0;
    helpvar = 0;
    posx = 0.1;
    posy = 0.1;
    startx = 0.1;
    starty = 0.1;
    endx = 0.1;
    endy = 0.1;
    transbegin = -1;
    transend = -1;
    mousemoved = false;
    usegrid = false;
    transbegin = true;
    //transinserted = false;
    foundstate = false;
    statecolor = Qt::red;
    linkcolor = Qt::green;
    transConditionString = "Transition Item without Condition";
    showFixedGrid = false;
    snapToGrid = false;
    translineWidth = 1;
    stateSize = 60;

    // Instanciate the SelectionAnimator
    //---------------
    this->selectionAnimator = new SelectionAnimator(this->scene());

    // Area selection
    this->areaSelectionInitial = NULL;

}

void View::drawForeground ( QPainter * painter, const QRectF & rect ) {

	// Parent Job
	//QGraphicsView::drawForeground (painter, rect ) ;
	if (this->areaSelectionInitial==NULL)
		return;
	// Paint selection area if necessary
	QPainterPath path = this->scene()->selectionArea();
	QPolygonF polygon = path.toFillPolygon();

	// Do paint
	painter->save();
	painter->setOpacity(0.15);
	QBrush brush(Qt::Dense1Pattern);
	brush.setColor(Qt::gray);
	painter->setBrush(brush);
	painter->setPen(Qt::DashLine);
	painter->drawPath(path);
	painter->restore();


}


void View::mouseMoveEvent(QMouseEvent* e) {

	// Parent job
	QGraphicsView::mouseMoveEvent(e);

	// Update Selection area
	//----------------
	if (this->areaSelectionInitial==NULL)
		return;

	// Convert position to scene coordinates
	QPointF initialOnScene = this->mapToScene(this->areaSelectionInitial->toPoint());
	QPointF currentOnScene = this->mapToScene(e->pos());

	// Painter path to selection
	QPainterPath* selectionPath = new QPainterPath(initialOnScene);
	selectionPath->addRoundRect(initialOnScene.x(),initialOnScene.y(),currentOnScene.x()-initialOnScene.x(),currentOnScene.y()-initialOnScene.y(),10);
	this->scene()->setSelectionArea(*selectionPath);

	this->scene()->update();

}

void View::mouseReleaseEvent(QMouseEvent* e) {

	// Parent job
	QGraphicsView::mouseReleaseEvent(e);

	// Deactivate areaSelection
	if (this->areaSelectionInitial!=NULL)
		delete this->areaSelectionInitial;
	this->areaSelectionInitial=NULL;

	this->scene()->update();
}

void View::mousePressEvent(QMouseEvent* e) {

	// Must call parent
	QGraphicsView::mousePressEvent(e);




    //make sure the view window gets the focus, so property changes etc are saved
    //setFocus(Qt::MouseFocusReason);


    if (e->button() == Qt::MidButton) {
    	resetTransform();
        ensureVisible(QRectF(0, 0, 0, 0)); //Scrolls so that point (0,0) is visible
    }

#ifdef DEBUGVIEW
    qDebug()<<"MousePressEvent with Pos x:"<< e->pos().x()<<"y:"<<e->pos().y()<<"globalPos:"<<e->globalPos()<< "paintmode="<<paintmode<<" activatedItem= "<<activatedItem<<" drag= "<<drag;
    //qDebug() << "MousePressEvent with Pos x: "  << e->pos().x()<< "y: " <<  e->pos().y() ;
#endif

    if (paintmode) {
        QPointF qpf = mapToScene(e->pos());
        QPoint qp = qpf.toPoint();
        posx = qpf.x();
        posy = qpf.y();
        //scenePosX =  qpf.x();
        //scenePosY =  qpf.y();
#if QT_VERSION >= 0x040400
        mousePosX = e->posF().x();
        mousePosY = e->posF().y();
#else
        mousePosX = e->pos().x();
        mousePosY = e->pos().y();
#endif
        mousemoved = false;

        //Q3CanvasItemList cil = canvas()->collisions(e->pos());
        //QList <QGraphicsItem *> cil = can->collidingItems (  e->pos(), mode = Qt::IntersectsItemShape );
        //QList <QGraphicsItem *> cil = can->collidingItems ( itemAt(qp) , mode = Qt::IntersectsItemShape );

        //-- List with Items under the mouse
        QList <QGraphicsItem *> itemsUnderMouse;
        itemsUnderMouse.clear();

        //-- Add the items under the nmouse
        if (itemAt(e->pos()) != NULL ) {
        	itemsUnderMouse << itemAt(e->pos());
        }
        itemsUnderMouse << can->collidingItems ( itemAt(e->pos()) , mode = Qt::IntersectsItemShape );
		#ifdef DEBUGVIEW
				qDebug() <<"MousePressEvent with item at(x,y):"<<itemAt(qp)<<" collidingItems: "<<itemsUnderMouse<<" at e->pos(): "<<e->pos() <<" mapToScene(e->pos()): "<<qp ;
		#endif

		//-- Right Button only activates items
        if(e->button() == Qt::RightButton) {
            if(itemsUnderMouse.empty()) {
               	activateItem(NULL);
            } else {
                activateItem(*itemsUnderMouse.begin());
            }
        }
        // Update the scene after selection
        //scene()->update();

        //-- With left button, check we are not in a special object add mode
        if(e->button() == Qt::LeftButton) {


        	// Init area selection if nothing else selected
        	if (this->scene()->selectedItems().size()==0) {
        		this->areaSelectionInitial = new QPointF(e->posF());
        		this->scene()->setSelectionArea(QPainterPath(*this->areaSelectionInitial));
        	}

        	// get current FSM
            Core_I& c = CoreFactory::getCore_I();
            int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
            Fsm_I& f = c.getCurrentFsm();

            //-- Insert Object Mode ---------------------
            switch(objectmode) {
            case STATE:
                {
                    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
                    transbegin = true;
                    addState();
//                    addNewState();
                    //newItem->setPos(e->pos().x(),e->pos().y());
                    newItem->setPos(qpf);     //set the position of the object on the Scene

                    // Record position
                    newItem->setPos(qpf);
                    (dynamic_cast<FSMGraphicsItem*>(newItem))->recordPosition();

                    // Show Item
                    placeItem(newItem);
                    newItem = 0;

					#ifdef DEBUGVIEW
						qDebug() << "Draw NewState with Pos x: " << e->pos().x()<< "y: " <<  e->pos().y()<<"alias mapToScene(e->pos()):"<<qpf;
					#endif

                    if(lock == UNLOCKED) {
                        objectmode = CHOOSE;
                    }
                }
                break;
            case TRANS:
                {
                    activateItem(0);

                    // Do nothing if nothing clicked
                    if(itemsUnderMouse.isEmpty())
                    	break;

                    // If The transition setting is  beginning
                    //----------------------------
                    if(transbegin) {

                        (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();

						// Not begining transition anymore
						transbegin = false;

						// Get Item under mouse
						QGraphicsItem* itemUnderMouse = itemsUnderMouse.takeFirst();

						// Beginning transition on a state
						if (itemUnderMouse->type() == NewStateItem::Type) {

							// Transition start on clicked state
							transstart = (dynamic_cast<NewStateItem*>(itemUnderMouse))->id;
							transend = -1;
							foundstate = true;

							// Position FSM on state
							f.getStatebyID(transstart);
							startx = f.getStatePosx();
							starty = f.getStatePosy();

						}
						// Beginning transition on a JoinPoint
						else if(itemUnderMouse->type() == Joinpoint::Type) {

							helpvar = (dynamic_cast<Joinpoint*>(itemUnderMouse))->transid;
							f.getTransbyID(helpvar);
							transstart = f.getTransStart();
							int z;
							z = (dynamic_cast<Joinpoint*>(itemUnderMouse))->id;
							f.getTransTrackpointbyID(z);
							int y;
							y = helpvar;
							while(f.getTransNextTrackpoint()) {
								f.deleteTrackpointOfTrans(f.getTransID(), f.getTransTrackpointID());
								f.getTransTrackpointbyID(z);
							}

							if(testJoin(helpvar,z)) {
								refreshOtherJoins(helpvar,z);
								testjoin = true;
							}

							if(testjoin){
								changeJoinTarget(helpvar,1000,-1);
							} else {
								f.modifyTrans(helpvar, f.getTransStart(), -1);
							}

							redraw();
							//update();

							f.getTransbyID(y);
							f.getTransTrackpointbyID(z);
							startx = f.getTransTrackpointPosx();
							starty = f.getTransTrackpointPosy();

						}
						// Beginning transition on a Trackpoint
						else if (itemUnderMouse->type() == TrackpointItem::Type) {

							helpvar = (dynamic_cast<TrackpointItem*>(itemUnderMouse))->transid;
							f.getTransbyID(helpvar);
							transstart = f.getTransStart();
							int z;
							z = (dynamic_cast<TrackpointItem*>(itemUnderMouse))->id;
							f.getTransTrackpointbyID(z);
							int y;
							y = helpvar;
							while(f.getTransNextTrackpoint()) {
								f.deleteTrackpointOfTrans(f.getTransID(), f.getTransTrackpointID());
								f.getTransTrackpointbyID(z);
							}
							if(testJoin(helpvar,z)) {
								refreshOtherJoins(helpvar,z);
								testjoin = true;
							}
							if(testjoin) {
								changeJoinTarget(helpvar, 1000, -1);
							} else {
								f.modifyTrans(helpvar, f.getTransStart(), -1);
							}
							redraw();
							//update();
							f.getTransbyID(y);
							f.getTransTrackpointbyID(z);
							startx = f.getTransTrackpointPosx();
							starty = f.getTransTrackpointPosy();
						}
						// Beginning transition on something else
						else {

						   transstart = -1;
							transend = -1;
							addTrans();
							f.modifyTrans(helpvar , (double) qpf.x() + 5, (double) qpf.y() + 5, getCurrentLevel());
							addTrackpoint();
							newItem->setPos(qpf.x() - 2.5,qpf.y() - 2.5);
							placeItem(newItem);
							newItem = 0;
							activateItem(0);
							startx = qpf.x();
							starty = qpf.y();
						}

                    }
                    // Not begining the transition
                    //----------------------------
                    else {
                        if(itemsUnderMouse.isEmpty()) {

                        	if(foundstate) {
                                addTrans();
                                foundstate = false;
                                int distance = 29;
                                double cx = (qpf.x() - startx);
                                double cy = (qpf.y() - starty);
                                double length = sqrt(cx*cx+cy*cy);
                                if(length != 0) {
                                    startx = (int)((cx/(length/distance)) + startx);
                                    starty = (int)((cy/(length/distance)) + starty);
                                }
                            }
                            addTransline((int)startx, (int)starty, (int)qpf.x(), (int)qpf.y());
                            addTrackpoint();
                            newItem->setPos(qpf.x() - 2.5,qpf.y() - 2.5);
                            placeItem(newItem);
                            newItem = 0;
                            activateItem(0);
                            startx = qpf.x();
                            starty = qpf.y();
                            f.modifyTrans(helpvar , (double) startx + 5, (double) starty + 5, getCurrentLevel());

                        } else {

                        	// Get the item we clicked
                            QGraphicsItem* itemUnderMouse = itemsUnderMouse.takeFirst();

                            // Clicked on state
                            if(itemUnderMouse->type() == NewStateItem::Type) {

								// This will be our transition end...
                                transend = (dynamic_cast<NewStateItem*>(itemUnderMouse))->id;

                                //...For a transition that begun on a state
                                if(foundstate) {

                                	// Transition not on self
                                    if(transend != transstart) { // avoid Transition draw error!!!!
                                        addTrans();
                                        int distance = 70;
                                        double cx = (qpf.x() - startx);
                                        double cy = (qpf.y() - starty);
                                        double length = sqrt(cx*cx+cy*cy);
                                        double txtposx;
                                        double txtposy;
                                        if(length != 0) {
                                            txtposx = ((cx/(length/distance)) + startx);
                                            txtposy = ((cy/(length/distance)) + starty);
                                        } else {
                                            txtposx = startx + 20;
                                            txtposy = starty + 20;
                                        }
                                        if(length > 80) {
                                            f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
                                        } else {
                                            f.modifyTrans(helpvar , (double) startx + 20, (double) starty + 20, getCurrentLevel());
                                        }
                                        foundstate = false;
                                    }

                                }
                                // ???
                                if(!foundstate) {
                                    if(testJoin(helpvar, 1000)) {
                                        changeJoinTarget(helpvar, 1000, transend);
                                        refreshOtherJoins(helpvar, 1000);
                                    } else {
                                        f.modifyTrans(helpvar, transstart, transend);
                                    }

                                    f.getStatebyID(transend);
                                    double x = f.getStatePosx();
                                    double y = f.getStatePosy();
                                    addTransline((int)startx, (int)starty, (int)x, (int)y);
                                    transbegin = true;
                                    if(lock == UNLOCKED) {
                                        objectmode = CHOOSE;
                                    }
                                    redraw();
                                    //update();
                                }

                            } else if(itemUnderMouse->type() == Joinpoint::Type) {
                                if(foundstate) {
                                    addTrans();
                                    {
                                        int distancetxt = 70;
                                        double cx = (qpf.x() - startx);
                                        double cy = (qpf.y() - starty);
                                        double length = sqrt(cx*cx+cy*cy);
                                        double txtposx;
                                        double txtposy;
                                        if(length != 0) {
                                            txtposx = ((cx/(length/distancetxt)) + startx);
                                            txtposy = ((cy/(length/distancetxt)) + starty);
                                        } else {
                                            txtposx = startx + 20;
                                            txtposy = starty + 20;
                                        }
                                        if(length > 80) {
                                            f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
                                        } else {
                                            f.modifyTrans(helpvar , (double) qpf.x() + 5, (double) qpf.y() + 5, getCurrentLevel());
                                        }
                                    }
                                    foundstate = false;
                                    int distance = 29;
                                    double cx = (qpf.x() - startx);
                                    double cy = (qpf.y() - starty);
                                    double length = sqrt(cx*cx+cy*cy);
                                    if(length != 0) {
                                        startx = (int)((cx/(length/distance)) + startx);
                                        starty = (int)((cy/(length/distance)) + starty);
                                    }
                                }
                                f.getTransbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
                                f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->id);
                                posx = f.getTransTrackpointPosx();
                                posy = f.getTransTrackpointPosy();
                                addTransline((int)startx, (int)starty, (int)posx, (int)posy);
                                activateItem(0);
                                f.getTransbyID(helpvar);
                                transstart = f.getTransStart();
                                f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());
                                int z;
                                z = f.getTransTrackpointID();
                                f.modifyTrans(helpvar, z, true);
                                f.getTransbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
                                f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->id);
                                while (f.getTransNextTrackpoint()) {
                                    posx = f.getTransTrackpointPosx();
                                    posy = f.getTransTrackpointPosy();
                                    z = f.getTransTrackpointID();
                                    int hld;
                                    hld = f.getTransTrackpointHldirection();
                                    bool j;
                                    j =  f.getTransTrackpointJoin();
                                    bool l;
                                    l = f.getTransTrackpointLink();
                                    int lid;
                                    lid = f.getTransTrackpointLinkID();
                                    f.getTransbyID(helpvar);
                                    f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());
                                    f.modifyTrans(helpvar, f.getTransTrackpointID(), j);
                                    f.modifyTrans(helpvar, f.getTransTrackpointID(), l, lid);
                                    f.transLevel(helpvar, f.getTransTrackpointID(), hld);
                                    f.getTransbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
                                    f.getTransTrackpointbyID(z);
                                }
                                transend = f.getTransEnd();
                                f.modifyTrans(helpvar, transstart, transend);
                                transbegin = true;
                                if(lock == UNLOCKED) {
                                    objectmode = CHOOSE;
                                }
                                f.getTransbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
                                if((f.getTransStart() < 0)) {
                                    f.deleteTrans((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
                                }
                                redraw();
                                //update();

                            } else if(itemUnderMouse->type() == Linkentrance::Type) {
                                if(foundstate) {
                                    addTrans();
                                    int distancetxt = 70;
                                    double cx = (qpf.x() - startx);
                                    double cy = (qpf.y() - starty);
                                    double length = sqrt(cx*cx+cy*cy);
                                    double txtposx;
                                    double txtposy;
                                    if(length != 0) {
                                        txtposx = ((cx/(length/distancetxt)) + startx);
                                        txtposy = ((cy/(length/distancetxt)) + starty);
                                    } else {
                                        txtposx = startx + 20;
                                        txtposy = starty + 20;
                                    }
                                    if(length > 80) {
                                        f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
                                    } else {
                                        f.modifyTrans(helpvar , (double) qpf.x() + 5, (double) qpf.y() + 5, getCurrentLevel());
                                    }
                                    foundstate = false;
                                }
                                addTransline((int)startx, (int)starty, (int)qpf.x(), (int)qpf.y());
                                activateItem(0);
                                f.getTransbyID((dynamic_cast<Linkentrance*>(itemUnderMouse))->transid);
                                f.getTransTrackpointbyID((dynamic_cast<Linkentrance*>(itemUnderMouse))->id);
                                posx = f.getTransTrackpointPosx();
                                posy = f.getTransTrackpointPosy();
                                int w;
                                w = f.getTransTrackpointLinkID();
                                transend = f.getTransEnd();
                                f.modifyTrans(helpvar, transstart, transend);
                                transbegin = true;
                                f.getTransbyID(helpvar);
                                f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());
                                f.modifyTrans(helpvar, f.getTransTrackpointID(), true, w);
                                if(lock == UNLOCKED) {
                                    objectmode = CHOOSE;
                                }
                                f.getTransbyID((dynamic_cast<Linkentrance*>(itemUnderMouse))->transid);
                                if(f.getTransStart() < 0) {
                                    f.deleteTrans((dynamic_cast<Linkentrance*>(itemUnderMouse))->transid);
                                }
                                redraw();
                                //update();

                            } else if(itemUnderMouse->type() == Leveltransfer::Type) {
                                if(foundstate) {
                                    addTrans();
                                    int distancetxt = 70;
                                    double cx = (qpf.x() - startx);
                                    double cy = (qpf.y() - starty);
                                    double length = sqrt(cx*cx+cy*cy);
                                    double txtposx;
                                    double txtposy;
                                    if(length != 0) {
                                        txtposx = ((cx/(length/distancetxt)) + startx);
                                        txtposy = ((cy/(length/distancetxt)) + starty);
                                    } else {
                                        txtposx = startx + 20;
                                        txtposy = starty + 20;
                                    }
                                    if(length > 80) {
                                        f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
                                    } else {
                                        f.modifyTrans(helpvar , (double) qpf.x() + 5, (double) qpf.y() + 5, getCurrentLevel());
                                    }
                                    foundstate = false;
                                }
                                addTransline((int)startx, (int)starty, (int)(dynamic_cast<Leveltransfer*>(itemUnderMouse))->x(), (int)(dynamic_cast<Leveltransfer*>(itemUnderMouse))->y());
                                activateItem(0);
                                f.getTransbyID((dynamic_cast<Leveltransfer*>(itemUnderMouse))->transid);
                                f.getTransTrackpointbyID((dynamic_cast<Leveltransfer*>(itemUnderMouse))->id);
                                posx = f.getTransTrackpointPosx();
                                posy = f.getTransTrackpointPosy();
                                transend = f.getTransEnd();
                                int w;
                                w = f.getTransTrackpointHldirection();
                                f.getTransbyID(helpvar);
                                f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());
                                f.transLevel(helpvar, f.getTransTrackpointID(), w);
                                f.modifyTrans(helpvar, transstart, transend);
                                transbegin = true;
                                if(lock == UNLOCKED) {
                                    objectmode = CHOOSE;
                                }
                                f.getTransbyID((dynamic_cast<Leveltransfer*>(itemUnderMouse))->transid);
                                if((f.getTransStart() < 0) && (f.getTransEnd() < 0)) {
                                    f.deleteTrans((dynamic_cast<Leveltransfer*>(itemUnderMouse))->transid);
                                }
                                redraw();
                                //update();
                            }
                            // Clicked in empty
                            //--------
                            else {
                            	// We were previously on a state -> create transition
                                if(foundstate) {
                                    addTrans();
                                    foundstate = false;
                                    int distance = 29;
                                    double cx = (qpf.x() - startx);
                                    double cy = (qpf.y() - starty);
                                    double length = sqrt(cx*cx+cy*cy);
                                    if(length != 0) {
                                        startx = (int)((cx/(length/distance)) + startx);
                                        starty = (int)((cy/(length/distance)) + starty);
                                    }
                                }
                                // Draw the line
                                addTransline((int)startx, (int)starty, (int)qpf.x(), (int)qpf.y());

                                // Add a tranckpoint
                                addTrackpoint();
                                newItem->setPos(qpf.x() - 2.5,qpf.y() - 2.5);
                                placeItem(newItem);
                                newItem = 0;
                                activateItem(0);

                                // New start position is at trackpoint position
                                startx = qpf.x();
                                starty = qpf.y();
                                f.modifyTrans(helpvar , (double) startx + 5, (double) starty + 5, getCurrentLevel());
                            }
                        }
                    }
                }
                break;
            case HTRANS:
                {
                    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
                    transbegin = true;
                    addHypertrans();
                    if(helpvar > -1) {
                        newItem->setPos(qpf.x(),qpf.y());
                        placeItem(newItem);
                        newItem = 0;
                    }
                    //cout << "Draw Hypertransition" << endl;
                    if(lock == UNLOCKED) {
                        objectmode = CHOOSE;
                    }
                }
                break;
            case LINK:
                {
                    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
                    transbegin = true;
                    addLink();
                    if(helpvar > -1) {
                        newItem->setPos(qpf );
                        placeItem(newItem);
                        newItem = 0;
                    }
                    //cout << "Draw Link" << endl;
                    if(lock == UNLOCKED) {
                        objectmode = CHOOSE;
                    }
                }
                break;
            case LINKCONNECT:
                {
                    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
                    transbegin = true;
                    addLinkentrance();
                    if(helpvar > -1) {
                        newItem->setPos(qpf );
                        placeItem(newItem);
                        newItem = 0;
                    }
                    //cout << "Draw Link" << endl;
                    if(lock == UNLOCKED) {
                        objectmode = CHOOSE;
                    }
                }
                break;
            case JOIN:
                {
                    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
                    transbegin = true;
                    addJoin();
                    newItem->setPos(qpf );
                    placeItem(newItem);
                    newItem = 0;
                    //cout << "Draw Join" << endl;
                    if(lock == UNLOCKED) {
                        objectmode = CHOOSE;
                    }
                }
                break;
            case REMOVE:
                {
                    transbegin = true;
                    if(itemsUnderMouse.empty()) {
                        activateItem(0);
                    } else {
                        activateItem(*itemsUnderMouse.begin());
                    }
                    remove();
                    //cout << "Remove activated item" << endl;
                    if(lock == UNLOCKED) {
                        objectmode = CHOOSE;
                    }
                }
                break;
            case HIERARCHY:
                {
                    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
                    transbegin = true;
                    addHierarchy();
                    if(helpvar = -1) {
                        newItem->setPos(qpf );
                        placeItem(newItem);
                        newItem = 0;
                    }
                    //cout << "Draw Hierarchy" << endl;
                    if(lock == UNLOCKED) {
                        objectmode = CHOOSE;
                    }
                }
                break;
            case TRACKPOINT:
                {
                    transbegin = true;
                    int id = f.getTransID();
                    int trid = f.getTransTrackpointID();
                    f.modifyTrans(id, trid, (double)qpf.x(), (double)qpf.y());
                    if(testjoin) {
                        refreshOtherJoins(id, trid);
                    }
                    //cout << "Draw Trackpoint" << endl;
                    objectmode = CHOOSE;
                    redraw();
                    //update();
                    break;
                }
            default:
                {
                	// Reinit transbegin
                    transbegin = true;

                    // Then just activate item under mouse
                    activateItem(*itemsUnderMouse.begin());

                    /*
                    //transinserted = false

                    // Nothing under the mouse (never happens)
                    if(itemsUnderMouse.empty()) {
                        activateItem(0);
                    } else {
                        drag = true;
                        QApplication::setOverrideCursor(Qt::PointingHandCursor);
                        activateItem(*itemsUnderMouse.begin());
                        //QString tooltipString("Dbg:MousePressEvent default mode");
                        //QToolTip::showText(e->globalPos(), tooltipString);
                        //activatedItem->setPos(e->posF().x()+10, e->posF().y()+10);
                        scene()->update();
                    }
                    */
                }
                //cout << "CHOOSE" << endl;
            }

            //-- END INSERT OBJECT MODE --------//
        }
    }

}

/*void View::mouseReleaseEvent(QMouseEvent* e) {

	cout << "Mouse Released in view" << endl;

	//-- With left button, check we are not in a special object add mode
	if(e->button() == Qt::LeftButton) {

		 //-- List with Items under the mouse
		QList <QGraphicsItem *> itemsUnderMouse;
		itemsUnderMouse << itemAt(e->pos());
		itemsUnderMouse << this->scene()->collidingItems ( this->itemAt(e->pos()) , mode = Qt::IntersectsItemShape );

		//-- Position of mouse on the scene
		QPointF sceneMousePosition = mapToScene(e->pos());

		// get current FSM
		Core_I& c = CoreFactory::getCore_I();
		int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
		Fsm_I& f = c.getCurrentFsm();

		//-- Insert Object Mode ---------------------
		switch(objectmode) {
		case STATE:
			{
				(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
				transbegin = true;
				addState();
				//newItem->setPos(e->pos().x(),e->pos().y());
				newItem->setPos(sceneMousePosition);     //set the position of the object on the Scene
				placeItem(newItem);  // Led to a crash, debug please, works now
				newItem = 0;

				#ifdef DEBUGVIEW
					qDebug() << "Draw NewState with Pos x: " << e->pos().x()<< "y: " <<  e->pos().y()<<"alias mapToScene(e->pos()):"<<sceneMousePosition;
				#endif

				if(lock == UNLOCKED) {
					objectmode = CHOOSE;
				}
			}
			break;
		case TRANS:
			{
				activateItem(0);


				// If The transition setting is  beginning
				//----------------------------
				if(transbegin) {

					(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();

					// Not begining transition anymore
					transbegin = false;

					// Get Item under mouse
					QGraphicsItem* itemUnderMouse = itemsUnderMouse.takeFirst();

					//if(i->rtti() == StateItem::RTTI) {
					if (itemUnderMouse->type() == NewStateItem::Type) {

						// Transition start on clicked state
						transstart = (dynamic_cast<NewStateItem*>(itemUnderMouse))->id;
						transend = -1;
						foundstate = true;

						// Position FSM on state
						f.getStatebyID(transstart);
						startx = f.getStatePosx();
						starty = f.getStatePosy();

					} else if(itemUnderMouse->type() == Joinpoint::Type) {

						helpvar = (dynamic_cast<Joinpoint*>(itemUnderMouse))->transid;
						f.getTransbyID(helpvar);
						transstart = f.getTransStart();
						int z;
						z = (dynamic_cast<Joinpoint*>(itemUnderMouse))->id;
						f.getTransTrackpointbyID(z);
						int y;
						y = helpvar;
						while(f.getTransNextTrackpoint()) {
							f.deleteTrackpointOfTrans(f.getTransID(), f.getTransTrackpointID());
							f.getTransTrackpointbyID(z);
						}

						if(testJoin(helpvar,z)) {
							refreshOtherJoins(helpvar,z);
							testjoin = true;
						}

						if(testjoin){
							changeJoinTarget(helpvar,1000,-1);
						} else {
							f.modifyTrans(helpvar, f.getTransStart(), -1);
						}

						this->redraw();
						//update();

						f.getTransbyID(y);
						f.getTransTrackpointbyID(z);
						startx = f.getTransTrackpointPosx();
						starty = f.getTransTrackpointPosy();

					} else if (itemUnderMouse->type() == TrackpointItem::Type) {

						helpvar = (dynamic_cast<TrackpointItem*>(itemUnderMouse))->transid;
						f.getTransbyID(helpvar);
						transstart = f.getTransStart();
						int z;
						z = (dynamic_cast<TrackpointItem*>(itemUnderMouse))->id;
						f.getTransTrackpointbyID(z);
						int y;
						y = helpvar;
						while(f.getTransNextTrackpoint()) {
							f.deleteTrackpointOfTrans(f.getTransID(), f.getTransTrackpointID());
							f.getTransTrackpointbyID(z);
						}
						if(testJoin(helpvar,z)) {
							refreshOtherJoins(helpvar,z);
							testjoin = true;
						}
						if(testjoin) {
							changeJoinTarget(helpvar, 1000, -1);
						} else {
							f.modifyTrans(helpvar, f.getTransStart(), -1);
						}
						redraw();
						//update();
						f.getTransbyID(y);
						f.getTransTrackpointbyID(z);
						startx = f.getTransTrackpointPosx();
						starty = f.getTransTrackpointPosy();
					}
					// Clicked on something else
					else {
						// Keep begining
						transbegin = true;

//					    transstart = -1;
//						transend = -1;
//						addTrans();
//						f.modifyTrans(helpvar , (double) qpf.x() + 5, (double) qpf.y() + 5, getCurrentLevel());
//						addTrackpoint();
//						newItem->setPos(qpf.x() - 2.5,qpf.y() - 2.5);
//						placeItem(newItem);
//						newItem = 0;
//						activateItem(0);
//						startx = qpf.x();
//						starty = qpf.y();
					}

				}
				// Not begining the transition
				//----------------------------
				else {

					// Get the item we clicked
					QGraphicsItem* itemUnderMouse = itemsUnderMouse.takeFirst();

					// Clicked on state
					if(itemUnderMouse->type() == NewStateItem::Type) {

						// This will be our transition end...
						transend = (dynamic_cast<NewStateItem*>(itemUnderMouse))->id;

						//...For a transition that begun on a state
						if(foundstate) {

							// Transition not on self
							if(transend != transstart) { // avoid Transition draw error!!!!
								addTrans();
								int distance = 70;
								double cx = (sceneMousePosition.x() - startx);
								double cy = (sceneMousePosition.y() - starty);
								double length = sqrt(cx*cx+cy*cy);
								double txtposx;
								double txtposy;
								if(length != 0) {
									txtposx = ((cx/(length/distance)) + startx);
									txtposy = ((cy/(length/distance)) + starty);
								} else {
									txtposx = startx + 20;
									txtposy = starty + 20;
								}
								if(length > 80) {
									f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
								} else {
									f.modifyTrans(helpvar , (double) startx + 20, (double) starty + 20, getCurrentLevel());
								}
								foundstate = false;
							}

						}
						// ???
						if(!foundstate) {
							if(testJoin(helpvar, 1000)) {
								changeJoinTarget(helpvar, 1000, transend);
								refreshOtherJoins(helpvar, 1000);
							} else {
								f.modifyTrans(helpvar, transstart, transend);
							}

							f.getStatebyID(transend);
							double x = f.getStatePosx();
							double y = f.getStatePosy();
							addTransline((int)startx, (int)starty, (int)x, (int)y);
							transbegin = true;
							if(lock == UNLOCKED) {
								objectmode = CHOOSE;
							}
							this->redraw();
							//update();
						}

					} else if(itemUnderMouse->type() == Joinpoint::Type) {
						if(foundstate) {
							addTrans();
							{
								int distancetxt = 70;
								double cx = (sceneMousePosition.x() - startx);
								double cy = (sceneMousePosition.y() - starty);
								double length = sqrt(cx*cx+cy*cy);
								double txtposx;
								double txtposy;
								if(length != 0) {
									txtposx = ((cx/(length/distancetxt)) + startx);
									txtposy = ((cy/(length/distancetxt)) + starty);
								} else {
									txtposx = startx + 20;
									txtposy = starty + 20;
								}
								if(length > 80) {
									f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
								} else {
									f.modifyTrans(helpvar , (double) sceneMousePosition.x() + 5, (double) sceneMousePosition.y() + 5, getCurrentLevel());
								}
							}
							foundstate = false;
							int distance = 29;
							double cx = (sceneMousePosition.x() - startx);
							double cy = (sceneMousePosition.y() - starty);
							double length = sqrt(cx*cx+cy*cy);
							if(length != 0) {
								startx = (int)((cx/(length/distance)) + startx);
								starty = (int)((cy/(length/distance)) + starty);
							}
						}
						f.getTransbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
						f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->id);
						posx = f.getTransTrackpointPosx();
						posy = f.getTransTrackpointPosy();
						addTransline((int)startx, (int)starty, (int)posx, (int)posy);
						activateItem(0);
						f.getTransbyID(helpvar);
						transstart = f.getTransStart();
						f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());
						int z;
						z = f.getTransTrackpointID();
						f.modifyTrans(helpvar, z, true);
						f.getTransbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
						f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->id);
						while (f.getTransNextTrackpoint()) {
							posx = f.getTransTrackpointPosx();
							posy = f.getTransTrackpointPosy();
							z = f.getTransTrackpointID();
							int hld;
							hld = f.getTransTrackpointHldirection();
							bool j;
							j =  f.getTransTrackpointJoin();
							bool l;
							l = f.getTransTrackpointLink();
							int lid;
							lid = f.getTransTrackpointLinkID();
							f.getTransbyID(helpvar);
							f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());
							f.modifyTrans(helpvar, f.getTransTrackpointID(), j);
							f.modifyTrans(helpvar, f.getTransTrackpointID(), l, lid);
							f.transLevel(helpvar, f.getTransTrackpointID(), hld);
							f.getTransbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
							f.getTransTrackpointbyID(z);
						}
						transend = f.getTransEnd();
						f.modifyTrans(helpvar, transstart, transend);
						transbegin = true;
						if(lock == UNLOCKED) {
							objectmode = CHOOSE;
						}
						f.getTransbyID((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
						if((f.getTransStart() < 0)) {
							f.deleteTrans((dynamic_cast<Joinpoint*>(itemUnderMouse))->transid);
						}
						redraw();
						//update();

					} else if(itemUnderMouse->type() == Linkentrance::Type) {
						if(foundstate) {
							addTrans();
							int distancetxt = 70;
							double cx = (sceneMousePosition.x() - startx);
							double cy = (sceneMousePosition.y() - starty);
							double length = sqrt(cx*cx+cy*cy);
							double txtposx;
							double txtposy;
							if(length != 0) {
								txtposx = ((cx/(length/distancetxt)) + startx);
								txtposy = ((cy/(length/distancetxt)) + starty);
							} else {
								txtposx = startx + 20;
								txtposy = starty + 20;
							}
							if(length > 80) {
								f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
							} else {
								f.modifyTrans(helpvar , (double) sceneMousePosition.x() + 5, (double) sceneMousePosition.y() + 5, getCurrentLevel());
							}
							foundstate = false;
						}
						addTransline((int)startx, (int)starty, (int)sceneMousePosition.x(), (int)sceneMousePosition.y());
						activateItem(0);
						f.getTransbyID((dynamic_cast<Linkentrance*>(itemUnderMouse))->transid);
						f.getTransTrackpointbyID((dynamic_cast<Linkentrance*>(itemUnderMouse))->id);
						posx = f.getTransTrackpointPosx();
						posy = f.getTransTrackpointPosy();
						int w;
						w = f.getTransTrackpointLinkID();
						transend = f.getTransEnd();
						f.modifyTrans(helpvar, transstart, transend);
						transbegin = true;
						f.getTransbyID(helpvar);
						f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());
						f.modifyTrans(helpvar, f.getTransTrackpointID(), true, w);
						if(lock == UNLOCKED) {
							objectmode = CHOOSE;
						}
						f.getTransbyID((dynamic_cast<Linkentrance*>(itemUnderMouse))->transid);
						if(f.getTransStart() < 0) {
							f.deleteTrans((dynamic_cast<Linkentrance*>(itemUnderMouse))->transid);
						}
						redraw();
						//update();

					} else if(itemUnderMouse->type() == Leveltransfer::Type) {
						if(foundstate) {
							addTrans();
							int distancetxt = 70;
							double cx = (sceneMousePosition.x() - startx);
							double cy = (sceneMousePosition.y() - starty);
							double length = sqrt(cx*cx+cy*cy);
							double txtposx;
							double txtposy;
							if(length != 0) {
								txtposx = ((cx/(length/distancetxt)) + startx);
								txtposy = ((cy/(length/distancetxt)) + starty);
							} else {
								txtposx = startx + 20;
								txtposy = starty + 20;
							}
							if(length > 80) {
								f.modifyTrans(helpvar , (double) txtposx + 5, (double) txtposy + 5, getCurrentLevel());
							} else {
								f.modifyTrans(helpvar , (double) sceneMousePosition.x() + 5, (double) sceneMousePosition.y() + 5, getCurrentLevel());
							}
							foundstate = false;
						}
						addTransline((int)startx, (int)starty, (int)(dynamic_cast<Leveltransfer*>(itemUnderMouse))->x(), (int)(dynamic_cast<Leveltransfer*>(itemUnderMouse))->y());
						activateItem(0);
						f.getTransbyID((dynamic_cast<Leveltransfer*>(itemUnderMouse))->transid);
						f.getTransTrackpointbyID((dynamic_cast<Leveltransfer*>(itemUnderMouse))->id);
						posx = f.getTransTrackpointPosx();
						posy = f.getTransTrackpointPosy();
						transend = f.getTransEnd();
						int w;
						w = f.getTransTrackpointHldirection();
						f.getTransbyID(helpvar);
						f.addTrackpointToTrans(helpvar, 1000, true, posx, posy, getCurrentLevel());
						f.transLevel(helpvar, f.getTransTrackpointID(), w);
						f.modifyTrans(helpvar, transstart, transend);
						transbegin = true;
						if(lock == UNLOCKED) {
							objectmode = CHOOSE;
						}
						f.getTransbyID((dynamic_cast<Leveltransfer*>(itemUnderMouse))->transid);
						if((f.getTransStart() < 0) && (f.getTransEnd() < 0)) {
							f.deleteTrans((dynamic_cast<Leveltransfer*>(itemUnderMouse))->transid);
						}
						redraw();
						//update();
					}
					// Clicked in empty
					//--------
					else {
						// We were previously on a state -> create transition
						if(foundstate) {
							addTrans();
							foundstate = false;
							int distance = 29;
							double cx = (sceneMousePosition.x() - startx);
							double cy = (sceneMousePosition.y() - starty);
							double length = sqrt(cx*cx+cy*cy);
							if(length != 0) {
								startx = (int)((cx/(length/distance)) + startx);
								starty = (int)((cy/(length/distance)) + starty);
							}
						}
						// Draw the line
						addTransline((int)startx, (int)starty, (int)sceneMousePosition.x(), (int)sceneMousePosition.y());

						// Add a tranckpoint
						addTrackpoint();
						newItem->setPos(sceneMousePosition.x() - 2.5,sceneMousePosition.y() - 2.5);
						placeItem(newItem);
						newItem = 0;
						activateItem(0);

						// New start position is at trackpoint position
						startx = sceneMousePosition.x();
						starty = sceneMousePosition.y();
						f.modifyTrans(helpvar , (double) startx + 5, (double) starty + 5, getCurrentLevel());
					}

				}
			}
			break;
		case HTRANS:
			{
				(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
				transbegin = true;
				addHypertrans();
				if(helpvar > -1) {
					newItem->setPos(sceneMousePosition.x(),sceneMousePosition.y());
					placeItem(newItem);
					newItem = 0;
				}
				//cout << "Draw Hypertransition" << endl;
				if(lock == UNLOCKED) {
					objectmode = CHOOSE;
				}
			}
			break;
		case LINK:
			{
				(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
				transbegin = true;
				addLink();
				if(helpvar > -1) {
					newItem->setPos(sceneMousePosition );
					placeItem(newItem);
					newItem = 0;
				}
				//cout << "Draw Link" << endl;
				if(lock == UNLOCKED) {
					objectmode = CHOOSE;
				}
			}
			break;
		case LINKCONNECT:
			{
				(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
				transbegin = true;
				addLinkentrance();
				if(helpvar > -1) {
					newItem->setPos(sceneMousePosition );
					placeItem(newItem);
					newItem = 0;
				}
				//cout << "Draw Link" << endl;
				if(lock == UNLOCKED) {
					objectmode = CHOOSE;
				}
			}
			break;
		case JOIN:
			{
				(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
				transbegin = true;
				addJoin();
				newItem->setPos(sceneMousePosition );
				placeItem(newItem);
				newItem = 0;
				//cout << "Draw Join" << endl;
				if(lock == UNLOCKED) {
					objectmode = CHOOSE;
				}
			}
			break;
		case REMOVE:
			{
				transbegin = true;
				if(itemsUnderMouse.empty()) {
					activateItem(0);
				} else {
					activateItem(*itemsUnderMouse.begin());
				}
				remove();
				//cout << "Remove activated item" << endl;
				if(lock == UNLOCKED) {
					objectmode = CHOOSE;
				}
			}
			break;
		case HIERARCHY:
			{
				(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
				transbegin = true;
				addHierarchy();
				if(helpvar = -1) {
					newItem->setPos(sceneMousePosition );
					placeItem(newItem);
					newItem = 0;
				}
				//cout << "Draw Hierarchy" << endl;
				if(lock == UNLOCKED) {
					objectmode = CHOOSE;
				}
			}
			break;
		case TRACKPOINT:
			{
				transbegin = true;
				int id = f.getTransID();
				int trid = f.getTransTrackpointID();
				f.modifyTrans(id, trid, (double)sceneMousePosition.x(), (double)sceneMousePosition.y());
				if(testjoin) {
					refreshOtherJoins(id, trid);
				}
				//cout << "Draw Trackpoint" << endl;
				objectmode = CHOOSE;
				redraw();
				//update();
				break;
			}
		default:
			{
				transbegin = true;
				//transinserted = false
				// Nothing under the mouse
				if(itemsUnderMouse.empty()) {
					activateItem(0);
				} else {
					drag = true;
					QApplication::setOverrideCursor(Qt::PointingHandCursor);
					activateItem(*itemsUnderMouse.begin());
					//QString tooltipString("Dbg:MousePressEvent default mode");
					//QToolTip::showText(e->globalPos(), tooltipString);
					//activatedItem->setPos(e->posF().x()+10, e->posF().y()+10);
					scene()->update();
				}
			}
			//cout << "CHOOSE" << endl;
		}

		//-- END INSERT OBJECT MODE --------//
	}

	// Call parent
	QGraphicsView::mouseReleaseEvent(e);

}*/

//Zoom in and out with the  mouse Wheel
void View::wheelEvent(QWheelEvent *event)
{
    if(ctrlSet) {
        //QGraphicsView::prepareGeometryChange();

    	// Calculate the scaling factor depending on wheeled degrees
    	double numDegrees = std::abs(event->delta()) / 8.0;
        double numSteps = numDegrees / 15.0;
        double factor = std::pow(1.125, numSteps);



        // Scale
        // To scale backward, use 1/factor, not -factor
        factor = event->delta()<0 ? 1/factor : factor;
        scale(factor, factor);

        // Center on the position
		centerOn(event->pos());

        //int sceneWidth = 1680;
        //int sceneHeight = 2376;
        //setSceneRect(0, 0, sceneWidth*factor, sceneHeight*factor);
#ifdef DEBUGVIEW
        qDebug()<<"wheelEvent with Factor :"<<factor; //<<"ScreenWidth :"<<sceneWidth*factor ;
#endif
    } else
    	// Default behaviour is to scroll
    	QGraphicsView::wheelEvent( event);
}






void View::contextMenuEvent(QContextMenuEvent* e) {
    
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    QMenu contextMenu(this);
    objectmode = CHOOSE;
    
    if(activatedItem) {
        if(activatedItem->type() == Transline::Type) {
            if((dynamic_cast<Transline*>(activatedItem))->id > -1) {
                contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
                contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
                contextMenu.addSeparator();
                int helpervar;
                f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
                if(f.getTransStart() != f.getTransEnd()) {
                    helpervar = f.getTransStart();
                    if(helpervar > -1) {
                        f.getStatebyID(helpervar);
                        if(f.getStateLevel() == getCurrentLevel()) {
                            helpervar = f.getTransEnd();
                            if(helpervar > -1) {
                                f.getStatebyID(helpervar);
                                if(f.getStateLevel() == getCurrentLevel()) {
                                    contextMenu.addAction(Straight);
                                }
                            }
                        }
                    }
                }
                contextMenu.addAction(setDefault);
                bool trackpointpossible = true;
                f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
                if(f.getTransFirstTrackpoint()) {
                    do {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                            if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
                                trackpointpossible = false;
                            }
                        }
                    } while (f.getTransNextTrackpoint());
                    if(trackpointpossible) {
                        contextMenu.addAction(insertTrackpoint);
                    }
                } else {
                    contextMenu.addAction(insertTrackpoint);
                }
                f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
                if(f.getTransFirstTrackpoint()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                        contextMenu.addAction(change_Start);
                    }
                } else {
                    contextMenu.addAction(change_Start);
                }
                bool targetpossible = true;
                bool linked = false;
                if(f.getTransFirstTrackpoint()) {
                    do {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                            targetpossible = true;
                        } else {
                            targetpossible = false;
                        }
                        if(f.getTransTrackpointLink()) {
                            linked = true;
                        }
                    } while (f.getTransNextTrackpoint());
                    if(targetpossible && !linked) {
                        contextMenu.addAction(change_Target);
                    }
                } else {
                    contextMenu.addAction(change_Target);
                }
                contextMenu.addSeparator();
                contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
            }
        } else if (activatedItem->type() == TrackpointItem::Type) {
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
            contextMenu.addSeparator();
            contextMenu.addAction(insertTrackpointbefore);
            contextMenu.addAction(insertTrackpointafter);
            contextMenu.addAction(connecttoLink);
            f.getTransbyID((dynamic_cast<TrackpointItem*>(activatedItem))->transid);
            int nostart = f.getTransStart();
            if(nostart == -1) {
                contextMenu.addAction(connecttoHierarchy);
            }
            contextMenu.addAction(changetoHierarchy);
            contextMenu.addAction(createJoin);
            contextMenu.addSeparator();
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
        } else if (activatedItem->type() == Joinpoint::Type) {
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
            contextMenu.addSeparator();
            contextMenu.addAction(transformToTrackpoint);
            contextMenu.addSeparator();
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
        } else if (activatedItem->type() == Linkentrance::Type) {
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
            contextMenu.addSeparator();
            contextMenu.addAction(transformToTrackpoint);
            contextMenu.addSeparator();
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
        } else if (activatedItem->type() == Hierarchyexit::Type) {
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
            contextMenu.addSeparator();
            contextMenu.addAction(transformToTrackpoint);
            contextMenu.addSeparator();
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
        } else if (activatedItem->type() == TranslineText::Type) {
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
            contextMenu.addSeparator();
            contextMenu.addAction(actionresetposition);
        } else {
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Front);
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionMove_to_Back);
            contextMenu.addSeparator();
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionCu_t);
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Copy);
            contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Delete);
        }
    } else {
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Paste);
        contextMenu.addSeparator();
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_State);
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Transition);
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Hypertrans);
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Link);
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->actionconnect_to_Link);
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_Join);
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))->action_New_Hierarchy);
        contextMenu.addSeparator();
        contextMenu.addAction((dynamic_cast<MainguiWindow*>(wid))-> action_Grid);
    }
    contextMenu.exec(e->globalPos());
    //QGraphicsView::ContextMenuEvent(e);
}

void View::keyPressEvent(QKeyEvent* ke) {
    //if(ke->key() == Qt::Key_Control) {
    if(ke->key() == Qt::Key_A) {
        lock = LOCKED; //Succesive drawing of several Items
        //cout << "LOCKED" << endl;
    }
    if(ke->key() == Qt::Key_Control) {
        ctrlSet = true;
        //cout << "Control Key pressed down" << endl;
    }
    if(ke->key() == Qt::Key_Alt) {
        altSet = true;
        //cout << "Alt Key pressed down" << endl;
    }
    if(ke->key() == Qt::Key_G) {
        gSet = true;
        //cout << "G Key pressed down" << endl;
    }
    if(ke->key() == Qt::Key_X) {
        xSet = true;
        //cout << "X Key pressed down" << endl;
    }
    if ( (gSet == true) ){
            showFixedGrid = true;
        }
    if ( (xSet == true) ){
            snapToGrid = true;
        }

}

void View::keyReleaseEvent(QKeyEvent* ke) {  
    //if(ke->key() == Qt::Key_Control) {
    if(ke->key() == Qt::Key_A) {
        //cout << "UNLOCKED" << endl;
        lock = UNLOCKED;
    }
    if(ke->key() == Qt::Key_Control) {
        ctrlSet = false;
        //cout << "Control Key released" << endl;
    }
    if(ke->key() == Qt::Key_Alt) {
        altSet = false;
        //cout << "Alt Key released" << endl;
    }
    if(ke->key() == Qt::Key_G) {
        gSet = false;
        //cout << "G Key released" << endl;
    }
    if(ke->key() == Qt::Key_X) {
        xSet = false;
        //cout << "X Key released" << endl;
    }
    if ( (gSet == false) ){
        showFixedGrid = false;
    }
    if ( (xSet == false) ){
        snapToGrid = false;
    }
}

void View::setunlocked(){
    lock = UNLOCKED;
}

void View::setlocked() {
    lock = LOCKED;
}

void View::setmode(Workbenchmode w) {
    if(w == TRANS) {
        transbegin = true;
    }
    objectmode = w;
}

void View::cut() {
    copy();
    remove();
}

void View::copy() { 
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    if(activatedItem) {
        posx = 50;
        posy = 50;
        QString s;
        string str = "";
        string str2 = "";
        int id;
        
        switch(activatedItem->type()) {
        case NewStateItem::Type:
            id = dynamic_cast<NewStateItem*>(activatedItem)->id;
            f.getStatebyID(id);
            s = QString("State %1 %2")
                .arg(QString::fromStdString(f.getStateOutput()))
                .arg(QString::number(f.getStateColor(), 10));
            break;
        case Leveltransfer::Type:
            s = QString("Leveltransfer");
            break;
        case Hypertransition::Type:
            id = dynamic_cast<Hypertransition*>(activatedItem)->id;
            f.getHypertransbyID(id);
            if(f.getHypertransFirstCondition()) {
                do {
                    str = str + f.getHypertransConditionValue();
                    str2 = str2 + "+" + f.getHypertransConditionName();
                } while (f.getHypertransNextCondition());
                str2.assign(str2, 1, str2.length() - 1);
            }
            s = QString("Hypertransition %1 %2")
                .arg(QString::fromStdString(str))
                .arg(QString::fromStdString(str2));
            break;
        case LinkItem::Type:
            s = QString("Link");
            break;
        case Linkentrance::Type:
            s = QString("Linkentrance");
            break;
        }
        QApplication::clipboard()->setText(s);
    }
}

void View::paste() {
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    int id;
    QString s = QApplication::clipboard()->text();
    QTextStream in(&s);
    QString tag;
    in >> tag;
    if ( tag == "State" ) {
    	addState();
        newItem->setPos(posx,posy);
        placeItem(newItem);
        int color;
        QString output;
        in >> output >> color;
        id = dynamic_cast<NewStateItem*>(newItem)->id;
        f.getStatebyID(id);
        f.modifyState(id, color);
        f.stateOutput(id, output.toStdString());
        newItem = 0;
    } else if ( tag == "Link" ) {
        addLink();
        newItem->setPos(posx,posy);
        placeItem(newItem);
        newItem = 0;
    } else if ( tag == "Leveltransfer" ) {
        addHierarchy();
        newItem->setPos(posx,posy);
        placeItem(newItem);
        newItem = 0;
    } else if ( tag == "Hypertransition" ) {
        addHypertrans();
        newItem->setPos(posx,posy);
        placeItem(newItem);
        QString allcon;
        QString conname;
        in >> allcon >> conname;
        string allcons = allcon.toStdString();
        string connames = conname.toStdString();
        if(!allcons.empty()) {
            id = dynamic_cast<Hypertransition*>(newItem)->id;
            f.getHypertransbyID(id);
            string str = "";
            string str2 = "";
            while (allcons.length() > f.getNumberOfInputs()) {
                str.assign(allcons, allcons.length() - f.getNumberOfInputs(), allcons.length() - 1);
                allcons.assign(allcons, 0, allcons.length()  - f.getNumberOfInputs());
                f.addConditionToHypertrans(id);
                f.modifyHypertransConditionValue(id, f.getHypertransConditionID(), str);
                int len = conname.length();
                if(len > 0) {
                    do {
                        len--;
                    } while ( (conname.at(len) != '+') && (len != 0));
                    conname.remove(len, conname.length() - 1);
                    str2.assign(connames, len+ 1, connames.length() - len);
                    connames.assign(connames, 0, len);
                }
                
                f.modifyHypertrans(id, f.getHypertransConditionID(), str2);
            }
            f.addConditionToHypertrans(id);
            f.modifyHypertransConditionValue(id, f.getHypertransConditionID(), allcons);
            f.modifyHypertrans(id, f.getHypertransConditionID(), connames);
        }
        activateItem(0);
        newItem = 0;
    } else if ( tag == "Link" ) {
        addLinkentrance();
        newItem->setPos(posx,posy);
        placeItem(newItem);
        newItem = 0;
    }
}

void View::remove() {

	// Get FSM
	Core_I& c = CoreFactory::getCore_I();
	Fsm_I& f = c.getCurrentFsm();

	// remove based on selection in scene
	QList<QGraphicsItem*> selectedItems(this->scene()->selectedItems());
	for (int index=0;index<selectedItems.size();index++) {

		// Get Current Item
		QGraphicsItem* i = selectedItems.at(index);
		//if (i==NULL)
		//this->activateItem(i);
		this->activatedItem = i;

		// Remove
		//------------
		(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
		int id;
		        int tid;
		        bool go;
		        switch(activatedItem->type()) {
		        case NewStateItem::Type:
		            id = (dynamic_cast<NewStateItem*>(activatedItem))->id;
		            f.deleteState(id);
		            break;
		        case Leveltransfer::Type:
		            {
		                id = (dynamic_cast<Leveltransfer*>(activatedItem))->id;
		                tid = (dynamic_cast<Leveltransfer*>(activatedItem))->transid;
		                testJoin((dynamic_cast<Leveltransfer*>(activatedItem))->transid, (dynamic_cast<Leveltransfer*>(activatedItem))->id);
		                f.getTransbyID(tid);
		                f.getTransTrackpointbyID(id);
		                int goalState = f.getTransEnd();
		                int l = f.getTransStart();
		                if(f.getTransNextTrackpoint()) {
		                    f.addTrans(-1, goalState);
		                    int newtransid = f.getTransID();
		                    f.getTransbyID(tid);
		                    f.getTransTrackpointbyID(id);
		                    f.getTransNextTrackpoint();
		                    do {
		                        int hl = f.getTransTrackpointLevel();
		                        double x = f.getTransTrackpointPosx();
		                        double y = f.getTransTrackpointPosy();
		                        f.getTransbyID(newtransid);
		                        f.addTrackpointToTrans(newtransid, 1000, true, x, y, hl);
		                        int trid = f.getTransTrackpointID();
		                        f.getTransbyID(tid);
		                        f.getTransTrackpointbyID(id);
		                        f.getTransNextTrackpoint();
		                        if(f.getTransTrackpointLink()) {
		                            f.modifyTrans(newtransid, trid, true, f.getTransTrackpointLinkID());
		                        }
		                        if(f.getTransTrackpointJoin()) {
		                            f.modifyTrans(newtransid, trid, true) ;
		                        }
		                        if(f.getTransTrackpointLevel() != f.getTransTrackpointHldirection()) {
		                            f.transLevel(newtransid, trid, f.getTransTrackpointHldirection());
		                        }
		                        int deleteid = f.getTransTrackpointID();
		                        f.deleteTrackpointOfTrans(tid, deleteid);
		                        f.getTransTrackpointbyID(id);
		                    } while (f.getTransNextTrackpoint());
		                }
		                f.getTransbyID(tid);
		                f.getTransFirstTrackpoint();
		                if(f.getTransTrackpointID()==id) {
		                    f.deleteTrans(tid);
		                } else {
		                    f.deleteTrackpointOfTrans(tid, id);
		                }
		                if(testjoin) {
		                    refreshOtherJoins((dynamic_cast<Leveltransfer*>(activatedItem))->transid, (dynamic_cast<Leveltransfer*>(activatedItem))->id);
		                    testjoin = true;
		                }
		                if(testjoin) {
		                    changeJoinTarget(tid, 1000, -1);
		                } else {
		                    f.modifyTrans(tid, l, -1);
		                }
		            }
		            break;
		        case Hypertransition::Type:
		            id = (dynamic_cast<Hypertransition*>(activatedItem))->id;
		            f.deleteHypertrans(id);
		            break;
		        case LinkItem::Type:
		            id = (dynamic_cast<LinkItem*>(activatedItem))->id;
		            f.deleteLink(id);
		            break;
		        case TrackpointItem::Type:
		            id = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
		            tid = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
		            if (!f.getTransbyID(tid))
		            	break;
		            go = true;
		            if(f.getTransStart() == f.getTransEnd()) {
		                if(f.getTransFirstTrackpoint()) {
		                    if(!f.getTransNextTrackpoint()) {
		                        go = false;
		                        f.deleteTrans(tid);
		                    }
		                }
		            }
		            if((f.getTransStart() == -1) || (f.getTransEnd() == -1)) {
		                if(f.getTransFirstTrackpoint()) {
		                    if(!f.getTransNextTrackpoint()) {
		                        go = false;
		                        f.deleteTrans(tid);
		                    }
		                }
		            }
		            if(go) {
		                testJoin((dynamic_cast<TrackpointItem*>(activatedItem))->transid, (dynamic_cast<TrackpointItem*>(activatedItem))->id);
		                f.deleteTrackpointOfTrans(tid, id);
		                if(testjoin) {
		                    refreshOtherJoins((dynamic_cast<TrackpointItem*>(activatedItem))->transid, (dynamic_cast<TrackpointItem*>(activatedItem))->id);
		                }
		            }
		            break;
		        case Linkentrance::Type:
		            {
		                id = (dynamic_cast<Linkentrance*>(activatedItem))->id;
		                tid = (dynamic_cast<Linkentrance*>(activatedItem))->transid;
		                testJoin((dynamic_cast<Linkentrance*>(activatedItem))->transid, (dynamic_cast<Linkentrance*>(activatedItem))->id);
		                f.getTransbyID(tid);
		                if (!f.getTransbyID(tid))
							break;
		                int transstart = f.getTransStart();
		                f.deleteTrackpointOfTrans(tid, id);
		                if(testjoin) {
		                    refreshOtherJoins((dynamic_cast<Linkentrance*>(activatedItem))->transid, (dynamic_cast<Linkentrance*>(activatedItem))->id);
		                    testjoin = true;
		                }
		                if(testjoin) {
		                    changeJoinTarget(tid, 1000, -1);
		                } else {
		                    f.modifyTrans(tid, transstart, -1);
		                }
		            }
		            break;
		        case Hierarchyexit::Type:
		            {
		                id = (dynamic_cast<Hierarchyexit*>(activatedItem))->id;
		                tid = (dynamic_cast<Hierarchyexit*>(activatedItem))->transid;
		                testJoin((dynamic_cast<Hierarchyexit*>(activatedItem))->transid, (dynamic_cast<Hierarchyexit*>(activatedItem))->id);
		                f.getTransbyID(tid);
		                if (!f.getTransbyID(tid) || !f.getTransTrackpointbyID(id))
							break;

		                int goalS = f.getTransEnd();
		                int k = f.getTransStart();
		                if(f.getTransNextTrackpoint()) {
		                    f.addTrans(-1, goalS);
		                    int newtransid = f.getTransID();
		                    f.getTransbyID(tid);
		                    f.getTransTrackpointbyID(id);
		                    f.getTransNextTrackpoint();
		                    do {
		                        int hl = f.getTransTrackpointLevel();
		                        double x = f.getTransTrackpointPosx();
		                        double y = f.getTransTrackpointPosy();
		                        f.getTransbyID(newtransid);
		                        f.addTrackpointToTrans(newtransid, 1000, true, x, y, hl);
		                        int trid = f.getTransTrackpointID();
		                        f.getTransbyID(tid);
		                        f.getTransTrackpointbyID(id);
		                        f.getTransNextTrackpoint();
		                        if(f.getTransTrackpointLink()) {
		                            f.modifyTrans(newtransid, trid, true, f.getTransTrackpointLinkID());
		                        }
		                        if(f.getTransTrackpointJoin()) {
		                            f.modifyTrans(newtransid, trid, true) ;
		                        }
		                        if(f.getTransTrackpointLevel() != f.getTransTrackpointHldirection()) {
		                            f.transLevel(newtransid, trid, f.getTransTrackpointHldirection());
		                        }
		                        int deleteid = f.getTransTrackpointID();
		                        f.deleteTrackpointOfTrans(tid, deleteid);
		                        f.getTransTrackpointbyID(id);
		                    } while (f.getTransNextTrackpoint());
		                }
		                f.deleteTrackpointOfTrans(tid, id);
		                if(testjoin) {
		                    refreshOtherJoins((dynamic_cast<Hierarchyexit*>(activatedItem))->transid, (dynamic_cast<Hierarchyexit*>(activatedItem))->id);
		                    testjoin = true;
		                }
		                if(testjoin) {
		                    changeJoinTarget(tid, 1000, -1);
		                } else {
		                    f.modifyTrans(tid, k, -1);
		                }
		            }
		            break;
		        case Joinpoint::Type:
		            {
		                id = (dynamic_cast<Joinpoint*>(activatedItem))->id;
		                tid = (dynamic_cast<Joinpoint*>(activatedItem))->transid;
		                f.deleteTrackpointOfTrans(tid, id);
		            }
		            break;
		        case Transline::Type:
		            {
		                id = (dynamic_cast<Transline*>(activatedItem))->id;
		                if(id > -1 && f.getTransbyID(id)) {
		                    f.deleteTrans(id);
		                }
		            }
		            break;
		        }
		        activateItem(NULL);
		        delete i;


	} // END OF LOOP

	(dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
	(dynamic_cast<MainguiWindow*>(wid))->undo();
	(dynamic_cast<MainguiWindow*>(wid))->nomoreredo();

}

void View::movetoback() {
    if(activatedItem){
        //--back;
        --back;
        activatedItem->setZValue(back);
        scene()->update();
    }
}

void View::movetofront() {
    if(activatedItem){
        //++front;
        ++front;
        activatedItem->setZValue(front);
        scene()->update();
    }
}


void View::addState() {

    NewStateItem* st2;
    //li = new LinkItem(30,20,can);
    //li = new LinkItem(QRectF((qreal)x(),(qreal)y(), 30, 20));
    //li = new LinkItem(QRectF((qreal)scenePosX,(qreal)scenePosY, 30, 20));
    qreal st2Width = stateSize;
    qreal st2Height = stateSize;
    st2 = new NewStateItem(QRectF(-(st2Width/2) ,-(st2Height/2), st2Width, st2Height));
    scene()->addItem(st2);

    //-- Create State
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    f.addState();
    int a;
    a = f.getStateID();
    f.stateLevel(a, getCurrentLevel());

    //-- Initial params
    //unsigned int col = f.getStateColor();
    st2->setBrush(statecolor); // Sets State Item color
    st2->setStateItemColor(statecolor); // Sets State Item color
    f.modifyState(a, (unsigned int) statecolor.rgb());// propagates color to fsm core
    string s;
    s = f.getStateName();
    st2->setText(QString::fromStdString(s));
    st2->id = a;

    if (f.getResetState()==f.getStateID()) {
    	st2->resetState = true;
    }

    //st2->modifyPos(posx, posy);

    string s2; //See also View::drawState
    string sns;
    QString qs; QString si; QString sn; QString swo;
    swo = " with Output :";
    sns = f.getStateName();
    sn = QString::fromStdString(sns);
    si = sn;
    si.append(swo);
    s2 = f.getStateOutput();
    qs = QString::fromStdString(s2);
    si.append(qs);
    st2->setToolTip(si);

    newItem = st2;    
}

void View::activateItem(QGraphicsItem* itemToActivate) {

	// Only do something if item to activate
	//is different from the one already activated
	if (itemToActivate==activatedItem)
		return;

    // If there were an activated item, deactivate it
	//----------
	if(activatedItem!=NULL){

		/* ???
		if(!paintmode) {
			if(activatedItem->type() == NewStateItem::Type) {
				(dynamic_cast<NewStateItem*>(activatedItem))->setBrush(QColor(Qt::red));
			}
		}
		*/

		// Deselect
		//activatedItem->setActive(false);
		//activatedItem->setSelected(false);
	}

	// Set Property display to nothing
	(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(0,0);

	// Register new activated item
	activatedItem = itemToActivate;

	// Do something if it isn't NULL (NULL means all deselected then)
	//----------
	if (activatedItem){

		// Set Selected
		//activatedItem->setSelected(true);

		// Update properties depending on type
		switch(activatedItem->type()) {
			case NewStateItem::Type:
				(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(1, (dynamic_cast<NewStateItem*>(activatedItem))->id);
				/* ???
				  if(!paintmode) {
					(dynamic_cast<NewStateItem*>(activatedItem))->setBrush(QColor(Qt::green));
				}
				*/
				break;
			case Transline::Type:

				(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(2, (dynamic_cast<Transline*>(activatedItem))->id);
				(dynamic_cast<MainguiWindow*>(wid))->fillcNameCombobox();

				break;
			case LinkItem::Type:
				(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(3, (dynamic_cast<LinkItem*>(activatedItem))->id);
				break;
			case Hypertransition::Type:
				(dynamic_cast<MainguiWindow*>(wid))->setPropertyStatus(4, (dynamic_cast<Hypertransition*>(activatedItem))->id);
				break;
		}
	}
	//scene()->update();
}

void View::placeItem(QGraphicsItem* i) {
    //activateItem(i);
    //i->setSelected(true);
    movetofront();
    i->show();
    scene()->update();
}

void View::addHypertrans() {
    activateItem(0);
    helpvar = -1;
    NewHyper* nh;
    nh = new NewHyper(this);
    nh->exec();
    if(helpvar > -1) {
        Hypertransition* hy;
        //hy = new Hypertransition(20,5,can);
        //hy = new Hypertransition(QRectF((qreal)x(),(qreal)y(), 20, 5));
        qreal wwidth = 20;
        qreal hheight = 5;
        hy = new Hypertransition(QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight));
        scene()->addItem(hy);
        hy->setBrush(Qt::yellow);
        Core_I& c = CoreFactory::getCore_I();
        int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        Fsm_I& f = c.getFsm(currentidMaingui);
        //Fsm_I& f = c.getCurrentFsm();
        f.addHypertrans(true, posx, posy, helpvar, getCurrentLevel());
        int a;
        a = f.getHypertransID();
        hy->id = a;
        Transline* tl;
        //tl= new Transline(can);
        tl = new Transline();
        scene()->addItem(tl);
        a = f.getHypertransGoal();
        f.getStatebyID(a);
        endx = f.getStatePosx();
        endy = f.getStatePosy();
        //tl->setBrush(Qt::black);
        //tl->setPen(QPen( Qt::black));
        QPen translinePen = QPen(Qt::black);
        translinePen.setWidth(translineWidth);
        tl->setPen(translinePen);
        int distance = 29;
        double cx = (posx - endx);
        double cy = (posy - endy);
        double length = sqrt(cx*cx+cy*cy);
        if(length != 0) {
            int sx = (int)((cx/(length/distance)) + endx);
            int sy = (int)((cy/(length/distance)) + endy);
            //tl->setPoints((int)posx, (int)posy, sx, sy);
            tl->setLine((int)posx, (int)posy, sx, sy);
            tl->id = -1;
            placeItem(tl);
            drawArrow(posx, posy, endx, endy);
        }
        endx = -1;
        endy = -1;
        newItem = hy;
    }
}

void View::addLink() {
    helpvar = -1;
    NewLink* nl;
    nl = new NewLink(this);
    nl->exec();
    if(helpvar > -1) {
        LinkItem* li;
        //li = new LinkItem(30,20,can);
        //li = new LinkItem(QRectF((qreal)x(),(qreal)y(), 30, 20));
        //li = new LinkItem(QRectF((qreal)scenePosX,(qreal)scenePosY, 30, 20));
        qreal liWidth = 30;
        qreal liHeight = 20;
        li = new LinkItem(QRectF(-(liWidth/2) ,-(liHeight/2), liWidth, liHeight));
        scene()->addItem(li);
        Core_I& c = CoreFactory::getCore_I();
        int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        Fsm_I& f = c.getFsm(currentidMaingui);
        //Fsm_I& f = c.getCurrentFsm();
        f.addLink(posx, posy, helpvar, getCurrentLevel());
        int a;
        a = f.getLinkID();
        li->id = a;
        //li->setBrush(Qt::green);
        li->setBrush(linkcolor); // Sets Link Item color
        li->linkItemColor = linkcolor; // Sets Link Item color
        f.modifyLink(a, (unsigned int) linkcolor.rgb());// propagates color to fsm core
        Transline* tl;
        //tl= new Transline(can);
        tl = new Transline();
        scene()->addItem(tl);
        a = f.getLinkGoal();
        f.getStatebyID(a);
        endx = f.getStatePosx();
        endy = f.getStatePosy();
        //tl->setBrush(Qt::black);
        //tl->setPen(QPen( Qt::black));
        QPen translinePen = QPen(Qt::black);
        translinePen.setWidth(translineWidth);
        tl->setPen(translinePen);
        int distance = 29;
        double cx = (posx - endx);
        double cy = (posy - endy);
        double length = sqrt(cx*cx+cy*cy);
        if(length != 0) {
            int sx = (int)((cx/(length/distance)) + endx);
            int sy = (int)((cy/(length/distance)) + endy);
            tl->setLine((int)posx, (int)posy, sx, sy);
            tl->id = -1;
            placeItem(tl);
            drawArrow(posx, posy, endx, endy);
        }
        endx = -1;
        endy = -1;
        newItem = li;
    }
}


void View::addHierarchy() {
    Leveltransfer* lt;
    //lt = new Leveltransfer(30,10,can);
    qreal ltWidth = 30;
    qreal ltHeight = 10;
    lt = new Leveltransfer(QRectF(-(ltWidth/2) ,-(ltHeight/2), ltWidth, ltHeight));
    scene()->addItem(lt);
    lt->setBrush(Qt::blue);
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    f.addTrans(-1,-1);
    int id =  f.getTransID();
    lt->transid = id;
    f.addTrackpointToTrans(id, 0, true, posx, posy, getCurrentLevel());
    int trid = f.getTransTrackpointID();
    lt->id = trid;
    helpvar = getCurrentLevel() + 1;
    f.transLevel(id, trid, helpvar);
    newItem = lt;
}


void View::addTransline(int sx, int sy, int ex, int ey) {
    Transline* tl;
    //tl = new Transline(can);
    tl = new Transline();
    scene()->addItem(tl);
    tl->setLine(sx, sy, ex, ey);
    QPen translinePen = QPen(Qt::black);
    translinePen.setWidth(translineWidth);
    tl->setPen(translinePen);
    tl->id = helpvar;

    tl->setToolTip(transConditionString);

    placeItem(tl);
}


void View::addTrans() {     

    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    //Fsm_I& f = c.getFsm(currentidMaingui);
    Fsm_I& f = c.getCurrentFsm();

    f.addTrans(transstart, transend);
    helpvar = f.getTransID();
    string s2; //See also View::drawTrans
    QString qs;
    transConditionString = "Transition Item with Condition :";
    s2 = f.getTransConditionbyID(helpvar);
    qs = QString::fromStdString(s2);
    transConditionString.append(qs);
}

void View::addTrackpoint() {
    TrackpointItem* tp;
    //tp = new TrackpointItem(can);
    tp = new TrackpointItem();
    scene()->addItem(tp);
    tp->setBrush(Qt::black);
    qreal tpWidth = 5;
    qreal tpHeight = 5;
    //tp->setSize(5,5);
    //tp->setRect(  x(),  y(), 5, 5);
    //tp->setRect( QRectF(-(tpWidth/2)+1 ,-(tpHeight/2)+1, tpWidth, tpHeight));
    tp->setRect( QRectF(0 ,0, tpWidth, tpHeight));
    tp->transid = helpvar;
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    f.addTrackpointToTrans(tp->transid, 1000, true, posx, posy, getCurrentLevel());
    tp->id = f.getTransTrackpointID();
    newItem = tp;
}

void View::addJoin() {
    Joinpoint* j;
    //j = new Joinpoint(can);
    j = new Joinpoint();
    scene()->addItem(j);
    j->setBrush(Qt::red);
    qreal jWidth = 10;
    qreal jHeight = 10;
    //j->setSize(10,10);
    //j->setRect(  x(),  y(), 10, 10);
    j->setRect(  QRectF(-(jWidth/2) ,-(jHeight/2), jWidth, jHeight));
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    f.addTrans(-1,-1);
    int id =  f.getTransID();
    j->transid = id;
    f.addTrackpointToTrans(id, 0, true, posx, posy, getCurrentLevel());
    int trid = f.getTransTrackpointID();
    j->id = trid;
    f.modifyTrans(id, trid, true);
    newItem = j;
}

void View::addLinkentrance() {
    Linkconnector* lc;
    lc = new Linkconnector(this);
    lc ->exec();
    if(helpvar > -1) {
        Linkentrance* le;
        le = new Linkentrance();
        Core_I& c = CoreFactory::getCore_I();
        int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        Fsm_I& f = c.getFsm(currentidMaingui);
        //Fsm_I& f = c.getCurrentFsm();
        scene()->addItem(le);
        qreal leWidth = 15;
        qreal leHeight = 8;
        le->setRect( QRectF(-(leWidth/2) ,-(leHeight/2), leWidth, leHeight));
        f.addTrans(-1,-1);
        int id =  f.getTransID();
        le->transid = id;
        f.addTrackpointToTrans(id, 0, true, posx, posy, getCurrentLevel());
        int trid = f.getTransTrackpointID();
        le->id = trid;
        f.modifyTrans(id, trid, true, helpvar);
        f.getLinkbyID(helpvar);
        int target = f.getLinkGoal();
        int start = f.getTransStart();
        f.modifyTrans(id, start, target);
        newItem = le;
        int linkid = f.getTransTrackpointLinkID();
        if (f.getLinkbyID(linkid)== true)
        {
            unsigned int color = f.getLinkColor();
            le->setBrush(QColor(color));
        }
        else
        {
            le->setBrush(Qt::green);
        }
    }
}

void View::grid() {
    if(usegrid) {
        usegrid = false;
        scene()->setBackgroundBrush(QPixmap());
    } else {
        usegrid = true;
        scene()->setBackgroundBrush(*bgGrid);
    }
}

void View::zoomIn() {
    //QMatrix wm=worldMatrix();
    QMatrix wm=matrix();
    wm.scale(1.1,1.1);
    setMatrix(wm);
}

void View::zoomOut() {
    //  QMatrix wm=worldMatrix();
    //  wm.scale(1/1.1,1/1.1);
    //  setWorldMatrix(wm);
    QMatrix wm=matrix();
    wm.scale(1/1.1,1/1.1);
    setMatrix(wm);
}

void View::createActionsforContextMenu() {
    Straight =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    Straight->setText(QApplication::translate("View", "Straight", 0, QApplication::UnicodeUTF8));
    Straight->setObjectName(QString::fromUtf8("Straight"));
    setDefault =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    setDefault->setObjectName(QString::fromUtf8("setDefault"));
    setDefault->setText(QApplication::translate("View", "Default", 0, QApplication::UnicodeUTF8));
    change_Start =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    change_Start->setObjectName(QString::fromUtf8("change_Start"));
    change_Start->setText(QApplication::translate("View", "change Start", 0, QApplication::UnicodeUTF8));
    change_Target =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    change_Target->setObjectName(QString::fromUtf8("change_Target"));
    change_Target->setText(QApplication::translate("View", "change Target", 0, QApplication::UnicodeUTF8));
    insertTrackpointbefore  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    insertTrackpointbefore->setObjectName(QString::fromUtf8("insertTrackpointbefore"));
    insertTrackpointbefore->setText(QApplication::translate("View", "New Trackpoint before", 0, QApplication::UnicodeUTF8));
    insertTrackpointafter  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    insertTrackpointafter->setObjectName(QString::fromUtf8("insertTrackpointafter"));
    insertTrackpointafter->setText(QApplication::translate("View", "New Trackpoint after", 0, QApplication::UnicodeUTF8));
    insertTrackpoint  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    insertTrackpoint->setObjectName(QString::fromUtf8("insertTrackpoint"));
    insertTrackpoint->setText(QApplication::translate("View", "New Trackpoint", 0, QApplication::UnicodeUTF8));
    connecttoLink =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    connecttoLink->setObjectName(QString::fromUtf8("connecttoLink"));
    connecttoLink->setText(QApplication::translate("View", "connect to Link", 0, QApplication::UnicodeUTF8));
    changetoHierarchy  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    changetoHierarchy->setObjectName(QString::fromUtf8("changetoHierarchy"));
    changetoHierarchy->setText(QApplication::translate("View", "change to Hierarchy", 0, QApplication::UnicodeUTF8));
    createJoin  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    createJoin->setObjectName(QString::fromUtf8("createJoin"));
    createJoin->setText(QApplication::translate("View", "change to Join", 0, QApplication::UnicodeUTF8));
    transformToTrackpoint  =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    transformToTrackpoint->setObjectName(QString::fromUtf8("transformToTrackpoint"));
    transformToTrackpoint->setText(QApplication::translate("View", "Transform to Trackpoint", 0, QApplication::UnicodeUTF8));
    connecttoHierarchy =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    connecttoHierarchy->setObjectName(QString::fromUtf8("connecttoHierarchy"));
    connecttoHierarchy->setText(QApplication::translate("View", "connect to Hierarchy", 0, QApplication::UnicodeUTF8));
    actionresetposition =  new QAction((dynamic_cast<MainguiWindow*>(wid)));
    actionresetposition->setObjectName(QString::fromUtf8("resetposition"));
    actionresetposition->setText(QApplication::translate("View", "Reset Position", 0, QApplication::UnicodeUTF8));
}

void View::straighten() { 
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    if(!((dynamic_cast<Transline*>(activatedItem))->id < 0)) {
        Core_I& c = CoreFactory::getCore_I();
        int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        Fsm_I& f = c.getFsm(currentidMaingui);
        //Fsm_I& f = c.getCurrentFsm();
        f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
        if((f.getTransStart() < 0) || (f.getTransEnd() < 0)) {
            f.deleteTrans((dynamic_cast<Transline*>(activatedItem))->id);
        } else {
            while(f.getTransFirstTrackpoint()) {
                int i = f.getTransTrackpointID();
                f.deleteTrackpointOfTrans((dynamic_cast<Transline*>(activatedItem))->id, i);
            }
            redraw();
            scene()->update();
        }
    }
}

void View::setdefault() {
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    if(!((dynamic_cast<Transline*>(activatedItem))->id < 0)) {
        Core_I& c = CoreFactory::getCore_I();
        int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        Fsm_I& f = c.getFsm(currentidMaingui);
        //Fsm_I& f = c.getCurrentFsm();
        f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
        if(f.getTransDefault()) {
            f.modifyTrans((dynamic_cast<Transline*>(activatedItem))->id, false);
        } else {
            f.modifyTrans((dynamic_cast<Transline*>(activatedItem))->id, true);
        }
        newItem = activatedItem;
        activateItem(0);
        activateItem(newItem);
        newItem = 0;
    }
}

void View::changestart() {
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    if(!((dynamic_cast<Transline*>(activatedItem))->id < 0)) {
        Core_I& c = CoreFactory::getCore_I();
        int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        Fsm_I& f = c.getFsm(currentidMaingui);
        //Fsm_I& f = c.getCurrentFsm();
        f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
        int end;
        end = f.getTransEnd();
        ChangeStart* cs;
        cs = new ChangeStart(this);
        cs->exec();
        if(helpvar > -1) {
            f.getTransbyID((dynamic_cast<Transline*>(activatedItem))->id);
            if(f.getTransFirstTrackpoint()) {
                f.modifyTrans((dynamic_cast<Transline*>(activatedItem))->id, helpvar, end);
            } else {
                if(end != helpvar) {
                    f.modifyTrans((dynamic_cast<Transline*>(activatedItem))->id, helpvar, end);
                }
            }
        }
        redraw();
        //update();
    }
}

void View::changetarget() {
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    if(!((dynamic_cast<Transline*>(activatedItem))->id < 0)) {
        Core_I& c = CoreFactory::getCore_I();
        int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        Fsm_I& f = c.getFsm(currentidMaingui);
        //Fsm_I& f = c.getCurrentFsm();
        int id = (dynamic_cast<Transline*>(activatedItem))->id;
        f.getTransbyID(id);
        int start;
        start = f.getTransStart();
        ChangeEnd* ce;
        ce = new ChangeEnd(this);
        ce->exec();
        if(helpvar > -1) {
            if(testJoin(id, 1000)) {
                changeJoinTarget(id, 1000, helpvar);
                refreshOtherJoins(id, 1000);
            } else {
                f.getTransbyID(id);
                if(f.getTransFirstTrackpoint()) {
                    f.modifyTrans(id, start, helpvar);
                } else {
                    if(start != helpvar) {
                        f.modifyTrans(id, start, helpvar);
                    }
                }
            }
        }
        redraw();
        //update();
    }
}

void View::inserttrackpoint() {  
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    bool ok = true;
    int id = (dynamic_cast<Transline*>(activatedItem))->id;
    testJoin(id, 1000);
    f.getTransbyID(id);
    if(f.getTransFirstTrackpoint()) {
        if((f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) && (f.getTransTrackpointLevel() == getCurrentLevel())) {
            ok = false;
        }
        while(f.getTransNextTrackpoint()){
            if((f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) && (f.getTransTrackpointLevel() == getCurrentLevel())) {
                ok = false;
            }
        }
    }
    if(ok) {
        objectmode =  TRACKPOINT;
        f.addTrackpointToTrans(id, 1000, true, 10, 10, getCurrentLevel());
    }
}

void View::inserttrackpointbefore() {  
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    objectmode =  TRACKPOINT;
    int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
    int posid = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
    testJoin(id, posid);
    f.getTransbyID(id);
    f.getTransTrackpointbyID(posid);
    f.addTrackpointToTrans(id, posid, true, 10, 10, getCurrentLevel());
}

void View::inserttrackpointafter() {  
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    objectmode =  TRACKPOINT;
    int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
    int posid = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
    testJoin(id, posid);
    f.getTransbyID(id);
    f.getTransTrackpointbyID(posid);
    f.addTrackpointToTrans(id, posid, false, 10, 10, getCurrentLevel());
}

void View::connecttolink() { 
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
    int trid = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
    f.getTransbyID(id);
    f.getTransTrackpointbyID(trid);
    Linkconnector* lc;
    lc = new Linkconnector(this);
    lc ->exec();
    if(helpvar > -1) {
        testJoin(id, trid);
        f.modifyTrans(id, trid, true, helpvar);
        f.getLinkbyID(helpvar);
        int target = f.getLinkGoal();
        int start = f.getTransStart();
        if(testjoin) {
            refreshOtherJoins(id, trid);
            testjoin = true;
        }
        if(testjoin) {
            changeJoinTarget(id, trid, target);
        } else {
            f.modifyTrans(id, start, target);
        }
        redraw();
        //update();
    }
}

void View::changetohierarchy() { 
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
    int trid = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
    testJoin(id, trid);
    f.getTransbyID(id);
    f.getTransTrackpointbyID(trid);
    f.transLevel(id, trid, getCurrentLevel() + 1);
    while(f.getTransNextTrackpoint()) {
        f.deleteTrackpointOfTrans(id,f.getTransTrackpointID());
        f.getTransTrackpointbyID(trid);
    }
    f.getTransTrackpointbyID(trid);
    if(testjoin) {
        refreshOtherJoins(id, trid);
    }
    redraw();
    //update();
}

void View::createjoin() {
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    int id = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
    int trid = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
    testJoin(id, trid);
    f.getTransbyID(id);
    f.getTransTrackpointbyID(trid);
    f.modifyTrans((dynamic_cast<TrackpointItem*>(activatedItem))->transid, (dynamic_cast<TrackpointItem*>(activatedItem))->id, true);
    QGraphicsItem* i = activatedItem;
    activateItem(0);
    delete i;
    drawJoin(); 
    if(testjoin) {
        refreshOtherJoins(id, trid);
    }
}

void View::transformtotrackpoint() {
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    if (activatedItem->type() == Joinpoint::Type) {
        int id = (dynamic_cast<Joinpoint*>(activatedItem))->transid;
        int trid = (dynamic_cast<Joinpoint*>(activatedItem))->id;
        testJoin(id, trid);
        activateItem(0);
        f.modifyTrans(id, trid, false);
        if(testjoin) {
            refreshOtherJoins(id, trid);
        }
    } else if (activatedItem->type() == Linkentrance::Type) {
        int id = (dynamic_cast<Linkentrance*>(activatedItem))->transid;
        int trid = (dynamic_cast<Linkentrance*>(activatedItem))->id;
        testJoin(id, trid);
        activateItem(0);
        f.modifyTrans(id, trid, false, 0);
        f.getTransbyID(id);
        int k = f.getTransStart();
        int l = -1;
        if(testjoin) {
            refreshOtherJoins(id, trid);
            testjoin = true;
        }
        if(testjoin) {
            changeJoinTarget(id, trid, l);
        } else {
            f.modifyTrans(id, k, l);
        }
    } else if (activatedItem->type() == Hierarchyexit::Type) {
        int id = (dynamic_cast<Hierarchyexit*>(activatedItem))->id;
        int tid = (dynamic_cast<Hierarchyexit*>(activatedItem))->transid;
        testJoin(tid, id);
        activateItem(0);
        f.getTransbyID(tid);
        f.getTransTrackpointbyID(id);
        int goalState = f.getTransEnd();
        int l = f.getTransStart();
        f.addTrans(-1, goalState);
        int newtransid = f.getTransID();
        f.getTransbyID(tid);
        f.getTransTrackpointbyID(id);
        while (f.getTransNextTrackpoint()) {
            int hl = f.getTransTrackpointLevel();
            double x = f.getTransTrackpointPosx();
            double y = f.getTransTrackpointPosy();
            f.getTransbyID(newtransid);
            f.addTrackpointToTrans(newtransid, 1000, true, x, y, hl);
            int trid = f.getTransTrackpointID();
            f.getTransbyID(tid);
            f.getTransTrackpointbyID(id);
            f.getTransNextTrackpoint();
            if(f.getTransTrackpointLink()) {
                f.modifyTrans(newtransid, trid, true, f.getTransTrackpointLinkID());
            }
            if(f.getTransTrackpointJoin()) {
                f.modifyTrans(newtransid, trid, true) ;
            }
            if(f.getTransTrackpointLevel() != f.getTransTrackpointHldirection()) {
                f.transLevel(newtransid, trid, f.getTransTrackpointHldirection());
            }
            int deleteid = f.getTransTrackpointID();
            f.deleteTrackpointOfTrans(tid, deleteid);
            f.getTransTrackpointbyID(id);
        }
        f.getTransbyID(tid);
        f.getTransTrackpointbyID(id);
        int hl = f.getTransTrackpointLevel();
        double x = f.getTransTrackpointPosx();
        double y = f.getTransTrackpointPosy();
        f.getTransbyID(newtransid);
        f.addTrackpointToTrans(newtransid, -10, true, x, y, hl);
        if(testJoin(tid, id)) {
            changeJoinTarget(tid, id, -1);
            f.deleteTrackpointOfTrans(tid, id);
            testjoin = true;
        } else {
            int curid = f.getTransStart();
            f.modifyTrans(tid, curid, -1);
            f.deleteTrackpointOfTrans(tid, id);
        }
        if(testjoin) {
            refreshOtherJoins(tid, 1000);
        }
    }
    redraw();
    //update();
}

void View::connecttohierarchy() {
    (dynamic_cast<MainguiWindow*>(wid))->saveforunredo();
    Hierarchyconnector* hc;
    hc = new Hierarchyconnector(this);
    hc ->exec();
    if(helpvar > -1) {
        Core_I& c = CoreFactory::getCore_I();
        int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
        Fsm_I& f = c.getFsm(currentidMaingui);
        //Fsm_I& f = c.getCurrentFsm();
        int tid = (dynamic_cast<TrackpointItem*>(activatedItem))->transid;
        int id = (dynamic_cast<TrackpointItem*>(activatedItem))->id;
        int trid;
        f.getTransbyID(tid);
        f.getTransTrackpointbyID(id);
        int hl = f.getTransTrackpointLevel();
        double x = f.getTransTrackpointPosx();
        double y = f.getTransTrackpointPosy();
        f.addTrackpointToTrans(helpvar, 1000, true, x, y, hl);
        f.getTransbyID(tid);
        f.getTransTrackpointbyID(id);
        while(f.getTransNextTrackpoint()) {
            hl = f.getTransTrackpointLevel();
            x = f.getTransTrackpointPosx();
            y = f.getTransTrackpointPosy();
            f.addTrackpointToTrans(helpvar, 1000, true, x, y, hl);
            trid = f.getTransTrackpointID();
            f.getTransbyID(tid);
            f.getTransTrackpointbyID(id);
            f.getTransNextTrackpoint();
            if(f.getTransTrackpointLink()) {
                f.modifyTrans(helpvar, trid, true, f.getTransTrackpointLinkID());
            }
            if(f.getTransTrackpointJoin()) {
                f.modifyTrans(helpvar, trid, true) ;
            }
            if(f.getTransTrackpointLevel() != f.getTransTrackpointHldirection()) {
                f.transLevel(helpvar, trid, f.getTransTrackpointHldirection());
            }
            int deleteid = f.getTransTrackpointID();
            f.deleteTrackpointOfTrans(tid, deleteid);
            f.getTransTrackpointbyID(id);
        }
        f.getTransbyID(tid);
        int e = f.getTransEnd();
        f.getTransbyID(helpvar);
        int s = f.getTransStart();
        f.deleteTrans(tid);
        if(testJoin(helpvar, 1000)) {
            refreshOtherJoins(helpvar, trid);
            testjoin = true;
        }
        if(testjoin) {
            changeJoinTarget(helpvar, 1000, e);
        } else {
            f.modifyTrans(helpvar, s, e);
        }
        redraw();
        //update();
    }
}

void View::connectContextMenuActions() {
    connect(Straight,SIGNAL(activated()),this,SLOT(straighten()));
    connect(setDefault,SIGNAL(activated()),this,SLOT(setdefault()));
    connect(change_Start,SIGNAL(activated()),this,SLOT(changestart()));
    connect(change_Target,SIGNAL(activated()),this,SLOT(changetarget()));
    connect(insertTrackpointbefore,SIGNAL(activated()),this,SLOT(inserttrackpointbefore()));
    connect(insertTrackpointafter,SIGNAL(activated()),this,SLOT(inserttrackpointafter()));
    connect(connecttoLink,SIGNAL(activated()),this,SLOT(connecttolink()));
    connect(changetoHierarchy,SIGNAL(activated()),this,SLOT(changetohierarchy()));
    connect(createJoin,SIGNAL(activated()),this,SLOT(createjoin()));
    connect(transformToTrackpoint,SIGNAL(activated()),this,SLOT(transformtotrackpoint()));
    connect(insertTrackpoint,SIGNAL(activated()),this,SLOT(inserttrackpoint()));
    connect(connecttoHierarchy,SIGNAL(activated()),this,SLOT(connecttohierarchy()));
    connect(actionresetposition,SIGNAL(activated()),this,SLOT(resetposition()));
}

int View::getCurrentLevel() {
    return (dynamic_cast<MainguiWindow*>(wid))->getcurrenthierarchylevel();
}

void View::sethelpvar(int i) {
    helpvar = i;
}

void View::setPaintmode(bool b) {
    paintmode = b;
    if(!b) {
        activateItem(0);
    }
}

void View::redraw() {

	this->setUpdatesEnabled(false);

	// Deactivate any activated
    activateItem(NULL);
    //Q3CanvasItemList cil = canvas()->allItems();
    QList <QGraphicsItem *> cil = can->items()  ;
    while (!cil.isEmpty()) {
        QGraphicsItem* i = cil.takeFirst();
//        cerr << "Deleting type " << i->type() << endl;
//        if (i!=NULL && sizeof(*i) < sizeof(QGraphicsItem))
        if (i!=NULL) {
        	can->removeItem(i);
        	//delete i;
        }
    }


    // Draw FSM
    //---------
    back = 0;
    front = 0;
    drawBoundingBox();
    if(showFixedGrid == true)
    {drawFixedGrid();}
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Core_I& c = CoreFactory::getCore_I();
    Fsm_I& f = c.getCurrentFsm();
    //Fsm_I& f = c.getFsm(currentidMaingui);
    int currentid = f.getFsmID();
#ifdef DEBUGVIEW2
    qDebug()<<"View redraw, current FSMID : "<<currentid <<" currentfsmid from MainguiWindow : "<<currentidMaingui ;
#endif
    if(f.getFirstTrans()) {

    	// Draw Trans
    	Transline * lastTransline = NULL;
        drawTrans(&lastTransline);

        // Draw Trans
        TranslineText * lastTranslineText;
        lastTranslineText = drawTransText();

        //  Add last added transline to text
        if (lastTranslineText!=NULL)
        	lastTranslineText->setLastTransline(lastTransline);

        while(f.getNextTrans()) {

        	// Draw Trans
            drawTrans(&lastTransline);

            // Draw Trans
			TranslineText * lastTranslineText;
			lastTranslineText = drawTransText();

			//  Add last added transline to text
			if (lastTranslineText!=NULL)
				lastTranslineText->setLastTransline(lastTransline);
        }
    }
    if(f.getFirstTrans()) {
        drawTransPoints();
        while(f.getNextTrans()) {
            drawTransPoints();
        }
    }
    if(f.getFirstLink()) {
        if(f.getLinkLevel() == getCurrentLevel()) {
            drawLink();
        }
        while(f.getNextLink()) {
            if (f.getLinkLevel() == getCurrentLevel()) {
                drawLink();
            }
        }
    }
    if(f.getFirstHypertrans()) {
        if(f.getHypertransLevel() == getCurrentLevel()) {
            drawHypertrans();
        }
        while(f.getNextHypertrans()) {
            if(f.getHypertransLevel() == getCurrentLevel()) {
                drawHypertrans();
            }
        }
    }
    if(f.getFirstState()) {
        if(f.getStateLevel() == getCurrentLevel()) {
            drawState();
        }
        while(f.getNextState()) {
            if(f.getStateLevel() == getCurrentLevel()) {
                drawState();
            }
        }
    }
    activateItem(0);
    this->setUpdatesEnabled(true);
    scene()->update();
}

void View::drawState() {
    // StateItem* st;
    NewStateItem* st;
    //st = new StateItem(56,56,can);
    //st = new StateItem(QRectF( (qreal)x(), (qreal)y(), 56, 56) );
    int wwidth = stateSize;
    int hheight = stateSize;
    st = new NewStateItem(QRectF( -(wwidth/2), -(hheight/2), wwidth, hheight) );
    scene()->addItem(st);
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    // Fsm_I& f = c.getCurrentFsm();
    int a;
    a = f.getStateID();
    string s;
    s = f.getStateName();
    st->setText(QString::fromStdString(s));
    st->id = a;
    posx = f.getStatePosx();
    posy = f.getStatePosy();
    unsigned int color = f.getStateColor();
#ifdef DEBUGVIEW
    // qDebug()<<"drawState with color :"<<color<<"and Pos x :"<<posx<<"y : "<<posy ;
#endif
    QPen pen;  // creates a pen
    if ((f.getResetState() == a) )
    {
        //pen.setStyle(Qt::DashLine);
        //pen.setWidth(3);
        pen.setBrush(Qt::black);
        st->setPen(pen);
        //color = 4294967295;
        st->resetState = true;
    } //4289462090 == light yellow , 4294967040 other yellow , 4294967295 white, 4294901760 == Qt::red
    else
    {
        st->resetState = false;
    }
    st->setStateItemColor(color);
    st->setBrush(QColor(color));
    st->setPos(posx,posy);

    string s2; //See also addNewState
    string sns;
    QString qs; QString si; QString sn; QString swo;
    swo = " with Output :";
    sns = f.getStateName();
    sn = QString::fromStdString(sns);
    si = sn;
    si.append(swo);
    s2 = f.getStateOutput();
    qs = QString::fromStdString(s2);
    si.append(qs);
    st->setToolTip(si); // State Output bits as Tooltip

    placeItem(st);
}

void View::drawHypertrans() {
    Hypertransition* hy;
    //hy = new Hypertransition(20,5,can);
    int wwidth = 20;
    int hheight = 5;
    hy = new Hypertransition(QRectF( -(wwidth/2) , -(hheight/2), wwidth, hheight));
    scene()->addItem(hy);
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    int a;
    a = f.getHypertransID();
    hy->id = a;
    posx = f.getHypertransPosx();
    posy = f.getHypertransPosy();
    hy->setBrush(Qt::yellow);
    hy->setPos(posx,posy);
    //hy->setPos(200, 200);
    Transline* tl;
    //tl= new Transline(can);
    tl = new Transline();
    scene()->addItem(tl);
    a = f.getHypertransGoal();
    f.getStatebyID(a);
    endx = f.getStatePosx();
    endy = f.getStatePosy();
    //tl->setBrush(Qt::black);
    QPen translinePen = QPen(Qt::black);
    translinePen.setWidth(translineWidth);
    tl->setPen(translinePen);
    int distance = 29;
    double cx = (posx - endx);
    double cy = (posy - endy);
    double length = sqrt(cx*cx+cy*cy);
    if(length != 0) {
        int sx = (int)((cx/(length/distance)) + endx);
        int sy = (int)((cy/(length/distance)) + endy);
        tl->setLine((int)posx, (int)posy, sx, sy);
        tl->id = -1;
        placeItem(tl);
        drawArrow(posx, posy, endx, endy);
    }
    placeItem(hy);
    endx = -1;
    endy = -1;
}

void View::drawLink() {
    LinkItem* li;
    //li = new LinkItem(30,20,can);
    int wwidth = 30;
    int hheight = 20;
    li = new LinkItem(QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight));
    scene()->addItem(li);
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    int a;
    a = f.getLinkID();
    li->id = a;
    posx = f.getLinkPosx();
    posy = f.getLinkPosy();
    //li->setBrush(Qt::green);

    unsigned int color = f.getLinkColor();
    #ifdef DEBUGVIEW
    // qDebug()<<"draw Link with color :"<<color<<"and Pos x :"<<posx<<"y : "<<posy ;
    #endif
    li->linkItemColor = color;
    li->setBrush(QColor(color));

    li->setPos(posx,posy);
    Transline* tl;
    //tl= new Transline(can);
    tl = new Transline();
    scene()->addItem(tl);
    a = f.getLinkGoal();
    f.getStatebyID(a);
    endx = f.getStatePosx();
    endy = f.getStatePosy();
    //tl->setBrush(Qt::black);
    QPen translinePen = QPen(Qt::black);
    translinePen.setWidth(translineWidth);
    tl->setPen(translinePen);
    int distance = 29;
    double cx = (posx - endx);
    double cy = (posy - endy);
    double length = sqrt(cx*cx+cy*cy);
    if(length != 0) {
        int sx = (int)((cx/(length/distance)) + endx);
        int sy = (int)((cy/(length/distance)) + endy);
        //tl->setPoints((int)posx, (int)posy, sx, sy);
        tl->setLine((int)posx, (int)posy, sx, sy);
        tl->id = -1;
        placeItem(tl);
        drawArrow(posx, posy, endx, endy);
    }
    placeItem(li);
    endx = -1;
    endy = -1;
}

void View::drawHierarchy() { 
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    Leveltransfer* lt;
    //lt = new Leveltransfer(30,10,can);
    int wwidth = 30;
    int hheight = 10;
    lt = new Leveltransfer(QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight));
    scene()->addItem(lt);
    lt->setBrush(Qt::blue);
    int id =  f.getTransID();
    lt->transid = id;
    int trid = f.getTransTrackpointID();
    lt->id = trid;
    posx = f.getTransTrackpointPosx();
    posy = f.getTransTrackpointPosy();
    lt->setPos(posx, posy);
    placeItem(lt);
}

void View::drawTrans(Transline **  returnPointer  ) {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    bool connecttoend = true;
    int a;
    int x;
    int y;
    int transid;
    int trackid;
    double posxx;
    double posyy;
    bool draw = false;
    //Q3CanvasItemList cil;
    QList <QGraphicsItem *> cil;
    a = f.getTransID();

    string s2; //See also View::addTrans
    QString qs;
    transConditionString = "Transition Item with Condition :";
    if (f.getTransConditionbyID(a))
    {
        s2 = f.getTransConditionValue();
        qs = QString::fromStdString(s2);
        transConditionString.append(qs);
    }
    if((f.getTransStart() < 0) && (f.getTransEnd() < 0)) {
        if(f.getTransFirstTrackpoint()) {
            if(f.getTransTrackpointJoin()) {
            } else if(f.getTransTrackpointLink()) {
            } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
            }
            if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                draw = true;
            } else {
                draw = false;
            }
            posxx = f.getTransTrackpointPosx();
            posyy = f.getTransTrackpointPosy();
            while(f.getTransNextTrackpoint()) {
                if(f.getTransTrackpointLevel() != getCurrentLevel()) {
                    draw = false;
                }
                posx = f.getTransTrackpointPosx();
                posy = f.getTransTrackpointPosy();
                if(draw) {
                    Transline* tl = new Transline();
                    if(returnPointer!=NULL)
                    	*returnPointer = tl;
                    if (tl->isSelected())
                    { tl->selectedTransline = true; }
                    else
                    { tl->selectedTransline = false; }
                    scene()->addItem(tl);
                    tl->setLine((int)posxx, (int)posyy, (int)posx, (int)posy);
                    //tl->setBrush(Qt::black);
                    QPen translinePen = QPen(Qt::black);
                    translinePen.setWidth(translineWidth);
                    tl->setPen(translinePen);
                    tl->id = a;
                     tl->setToolTip(transConditionString);
                    placeItem(tl);
                }
                if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                    draw = true;
                }
                posxx = posx;
                posyy = posy;
                if(f.getTransTrackpointJoin()) {
                    transid = f.getTransID();
                    trackid = f.getTransTrackpointID();
                    int goal = f.getTransEnd();
                    x = (int) f.getTransTrackpointPosx();
                    y = (int) f.getTransTrackpointPosy();
                    //cil = canvas()->collisions(QPoint(x, y));
                    cil = can->items (QPoint(x, y));
                    
                    while (!cil.isEmpty()) {
                        //Q3CanvasItem* i = cil.takeFirst();
                        QGraphicsItem* i = cil.takeFirst();
                        if(i->type() == Joinpoint::Type) {
                            f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
                            if(f.getTransEnd() == goal) {
                                f.getTransbyID(transid);
                                f.getTransTrackpointbyID(trackid);
                                while(f.getTransNextTrackpoint()) {}
                            } else {
                                f.getTransbyID(transid);
                                f.getTransTrackpointbyID(trackid);
                            }
                        }
                    }
                } else if(f.getTransTrackpointLink()) {
                    while(f.getTransNextTrackpoint()) {}
                } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                        while(f.getTransNextTrackpoint()) {}
                    }
                }
            }
        } else {
            f.deleteTrans(a);
        }
    } else {
        if(f.getTransStart() < 0) {
            if(f.getTransFirstTrackpoint()) {
                if(f.getTransTrackpointJoin()) {
                } else if(f.getTransTrackpointLink()) {
                    while(f.getTransNextTrackpoint()) {}
                    connecttoend = false;
                } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                        while(f.getTransNextTrackpoint()) {}
                        connecttoend = false;
                    }
                }
                if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                    draw = true;
                } else {
                    draw = false;
                }
                posxx = f.getTransTrackpointPosx();
                posyy = f.getTransTrackpointPosy();
                while(f.getTransNextTrackpoint()) {
                    if(!(f.getTransTrackpointLevel() == getCurrentLevel())) {
                        draw = false;
                    }
                    posx = f.getTransTrackpointPosx();
                    posy = f.getTransTrackpointPosy();
                    if(draw) {
                        Transline* tl= new Transline();
                        if(returnPointer!=NULL)
							*returnPointer = tl;
                        scene()->addItem(tl);
                        tl->setLine((int)posxx, (int)posyy, (int)posx, (int)posy);
                        //tl->setBrush(Qt::black);
                        QPen translinePen = QPen(Qt::black);
                        translinePen.setWidth(translineWidth);
                        tl->setPen(translinePen);
                        tl->id = a;
                         tl->setToolTip(transConditionString);
                        placeItem(tl);
                    }
                    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                        draw = true;
                    }
                    posxx = posx;
                    posyy = posy;
                    if(f.getTransTrackpointJoin()) {
                        transid = f.getTransID();
                        trackid = f.getTransTrackpointID();
                        int goal = f.getTransEnd();
                        x = (int) f.getTransTrackpointPosx();
                        y = (int) f.getTransTrackpointPosy();
                        //cil = canvas()->collisions(QPoint(x, y));
                        cil =  can->items (QPoint(x, y));
                        while (!cil.isEmpty()) {
                            QGraphicsItem* i = cil.takeFirst();
                            if(i->type() == Joinpoint::Type) {
                                f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
                                if(f.getTransEnd() == goal) {
                                    f.getTransbyID(transid);
                                    f.getTransTrackpointbyID(trackid);
                                    while(f.getTransNextTrackpoint()) {}
                                    connecttoend = false;
                                } else {
                                    f.getTransbyID(transid);
                                    f.getTransTrackpointbyID(trackid);
                                }
                            }
                        }
                    } else if(f.getTransTrackpointLink()) {
                        while(f.getTransNextTrackpoint()) {}
                        connecttoend = false;
                    } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                            while(f.getTransNextTrackpoint()) {}
                            connecttoend = false;
                        }
                    }
                }
                if(!(f.getTransEnd() < 0)  && connecttoend) {
                    f.getStatebyID(f.getTransEnd());
                    posx = f.getStatePosx();
                    posy = f.getStatePosy();
                    if(!(f.getStateLevel() == getCurrentLevel())) {
                        draw = false;
                    }
                    if(draw){
                        Transline* tl = new Transline();
                        if(returnPointer!=NULL)
							*returnPointer = tl;
                        scene()->addItem(tl);
                        int distance = 29;
                        double cx = (posxx - posx);
                        double cy = (posyy - posy);
                        double length = sqrt(cx*cx+cy*cy);
                        if(length != 0) {
                            int sx = (int)((cx/(length/distance)) + posx);
                            int sy = (int)((cy/(length/distance)) + posy);
                            tl->setLine((int)posxx, (int)posyy, sx, sy);
                            //tl->setBrush(Qt::black);
                            QPen translinePen = QPen(Qt::black);
                            translinePen.setWidth(translineWidth);
                            tl->setPen(translinePen);
                            tl->id = a;
                             tl->setToolTip(transConditionString);
                            placeItem(tl);

                           // If length (of what??) ==0, don't arrow
						   if (sqrt((posxx-posx)*(posxx-posx)+(posyy-posy)*(posyy-posy))!=0) {
							   tl->drawArrow = true;
							  // drawArrow(posxx, posyy, posx, posy);
						   }
                        }
                    }
                }
            } else {
                f.deleteTrans(a);
            }
        } else {
            f.getStatebyID(f.getTransStart());
            posxx = f.getStatePosx();
            posyy = f.getStatePosy();
            if(f.getStateLevel() == getCurrentLevel()) {
                draw = true;
            } else {
                draw = false;
            }
            if(f.getTransFirstTrackpoint()) {
                if(f.getTransTrackpointLevel() != getCurrentLevel()) {
                    draw = false;
                }
                if(f.getTransTrackpointJoin()) {
                    transid = f.getTransID();
                    trackid = f.getTransTrackpointID();
                    int goal = f.getTransEnd();
                    posx = f.getTransTrackpointPosx();
                    posy = f.getTransTrackpointPosy();
                    x = (int) posx;
                    y = (int) posy;
                    //cil = canvas()->collisions(QPoint(x, y));
                    cil = can->items (QPoint(x, y));
                    while (!cil.isEmpty()) {
                        QGraphicsItem* i = cil.takeFirst();
                        if(i->type() == Joinpoint::Type) {
                            f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
                            if(f.getTransEnd() == goal) {
                                f.getTransbyID(transid);
                                f.getTransTrackpointbyID(trackid);
                                while(f.getTransNextTrackpoint()) {}
                                connecttoend = false;
                            } else {
                                f.getTransbyID(transid);
                                f.getTransTrackpointbyID(trackid);
                            }
                        }
                    }
                } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                    posx = f.getTransTrackpointPosx();
                    posy = f.getTransTrackpointPosy();
                    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                        while(f.getTransNextTrackpoint()) {}
                        connecttoend = false;
                    }
                } else if(f.getTransTrackpointLink()) {
                    posx = f.getTransTrackpointPosx();
                    posy = f.getTransTrackpointPosy();
                    while(f.getTransNextTrackpoint()) {}
                    connecttoend = false;
                } else {
                    posx = f.getTransTrackpointPosx();
                    posy = f.getTransTrackpointPosy();
                }
                if(draw) {
                    Transline* tl = new Transline();
                    if(returnPointer!=NULL)
						*returnPointer = tl;
                    scene()->addItem(tl);
                    int distance = 29;
                    double cx = (posx - posxx);
                    double cy = (posy - posyy);
                    double length = sqrt(cx*cx+cy*cy);
                    if(length != 0) {
                        posxx = (int)((cx/(length/distance)) + posxx);
                        posyy = (int)((cy/(length/distance)) + posyy);
                        tl->setLine((int)posxx, (int)posyy, (int)posx, (int)posy);
                        //tl->setBrush(Qt::black);
                        QPen translinePen = QPen(Qt::black);
                        translinePen.setWidth(translineWidth);
                        tl->setPen(translinePen);
                        tl->id = a;
                         tl->setToolTip(transConditionString);
                        placeItem(tl);
                    }
                }
                if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                    draw = true;
                }
                posxx = posx;
                posyy = posy;
                while(f.getTransNextTrackpoint()) {
                    posx = f.getTransTrackpointPosx();
                    posy = f.getTransTrackpointPosy();
                    if(f.getTransTrackpointLevel() != getCurrentLevel()) {
                        draw = false;
                    }
                    if(draw) {
                        Transline* tl = new Transline();
                        if(returnPointer!=NULL)
							*returnPointer = tl;
                        scene()->addItem(tl);
                        tl->setLine((int)posxx, (int)posyy, (int)posx, (int)posy);
                        //tl->setBrush(Qt::black);
                        QPen translinePen = QPen(Qt::black);
                        translinePen.setWidth(translineWidth);
                        tl->setPen(translinePen);
                        tl->id = a;
                         tl->setToolTip(transConditionString);
                        placeItem(tl);
                    }
                    if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                        draw = true;
                    }
                    posxx = posx;
                    posyy = posy;
                    if(f.getTransTrackpointJoin()) {
                        transid = f.getTransID();
                        trackid = f.getTransTrackpointID();
                        int goal = f.getTransEnd();
                        x = (int) f.getTransTrackpointPosx();
                        y = (int) f.getTransTrackpointPosy();
                        //cil = canvas()->collisions(QPoint(x, y));
                        cil = can->items (QPoint(x, y));
                        while (!cil.isEmpty()) {
                            QGraphicsItem* i = cil.takeFirst();
                            if(i->type() == Joinpoint::Type) {
                                f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
                                if(f.getTransEnd() == goal) {
                                    f.getTransbyID(transid);
                                    
                                    
                                    f.getTransTrackpointbyID(trackid);
                                    while(f.getTransNextTrackpoint()) {}
                                    connecttoend = false;
                                } else {
                                    f.getTransbyID(transid);
                                    f.getTransTrackpointbyID(trackid);
                                }
                            }
                        }
                    } else if(f.getTransTrackpointLink()) {
                        while(f.getTransNextTrackpoint()) {}
                        connecttoend = false;
                    } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()) {
                            while(f.getTransNextTrackpoint()) {}
                            connecttoend = false;
                        }
                    }
                }
                if(!(f.getTransEnd() < 0) && connecttoend) {
                    f.getStatebyID(f.getTransEnd());
                    posx = f.getStatePosx();
                    posy = f.getStatePosy();
                    if(f.getStateLevel() != getCurrentLevel()) {
                        draw = false;
                    }
                    if(draw) {
                        Transline* tl = new Transline();
                        if(returnPointer!=NULL)
							*returnPointer = tl;
                        scene()->addItem(tl);
                        int distance = 29;
                        double cx = (posxx - posx);
                        double cy = (posyy - posy);
                        double length = sqrt(cx*cx+cy*cy);
                        if(length != 0) {
                            int sx = (int)((cx/(length/distance)) + posx);
                            int sy = (int)((cy/(length/distance)) + posy);
                            tl->setLine((int)posxx, (int)posyy, sx, sy);
                            //tl->setBrush(Qt::black);
                            QPen translinePen = QPen(Qt::black);
                            translinePen.setWidth(translineWidth);
                            tl->setPen(translinePen);
                            tl->id = a;
                             tl->setToolTip(transConditionString);
                            placeItem(tl);

                            // If length (of what??) ==0, don't arrow
                            if (sqrt((posxx-posx)*(posxx-posx)+(posyy-posy)*(posyy-posy))!=0){
 							   tl->drawArrow = true;
 							   //drawArrow(posxx, posyy, posx, posy);
 						   }
                        }
                    }
                }
            } else {
                if(f.getTransEnd() < 0) {
                    f.deleteTrans(a);
                } else {
                    f.getStatebyID(f.getTransEnd());
                    posx = f.getStatePosx();
                    posy = f.getStatePosy();
                    if(!(f.getStateLevel() == getCurrentLevel())) {
                        draw = false;
                    }
                    int distance = 29;
                    double cx = (posx - posxx);
                    double cy = (posy - posyy);
                    double length = sqrt(cx*cx+cy*cy);
                    if(length != 0) {
                        posxx = (int)((cx/(length/distance)) + posxx);
                        posyy = (int)((cy/(length/distance)) + posyy);
                    }
                    if(draw) {
                        Transline* tl = new Transline();
                        if(returnPointer!=NULL)
							*returnPointer = tl;
                        scene()->addItem(tl);
                        int distance = 29;
                        double cx = (posxx - posx);
                        double cy = (posyy - posy);
                        double length = sqrt(cx*cx+cy*cy);
                        //if(length != 0) {tp->setRect( x(), y(), 5, 5);
                        if(length != 0) {
                            int sx = (int)((cx/(length/distance)) + posx);
                            int sy = (int)((cy/(length/distance)) + posy);
                            tl->setLine((int)posxx, (int)posyy, sx, sy);
                            //tl->setBrush(Qt::black);
                            QPen translinePen = QPen(Qt::black);
                            translinePen.setWidth(translineWidth);
                            tl->setPen(translinePen);
                            tl->id = a;
                             tl->setToolTip(transConditionString);
                            placeItem(tl);

                            // If length (of what??) ==0, don't arrow
 						    if (sqrt((posxx-posx)*(posxx-posx)+(posyy-posy)*(posyy-posy))!=0){
 							   tl->drawArrow = true;
 							   //drawArrow(posxx, posyy, posx, posy);
 						   }
                        }
                    }
                }
            }
        }
    }
}

void View::drawTransPoints() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    bool hierarchy = false;
    int x;
    int y;
    int transid;
    int trackid;
    //Q3CanvasItemList cil;
    QList <QGraphicsItem *> cil;
    if((f.getTransStart() < 0) && (f.getTransEnd() < 0)) {
        if(f.getTransFirstTrackpoint()) {
            if(f.getTransTrackpointJoin()) {
                if(f.getTransTrackpointLevel() == getCurrentLevel()){
                    drawJoin();
                }
            } else if(f.getTransTrackpointLink()) {
                if(f.getTransTrackpointLevel() == getCurrentLevel()){
                    drawLinkentrance();
                }
                while(f.getTransNextTrackpoint()) {}
                
            } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                if(f.getTransTrackpointLevel() == getCurrentLevel()){
                    drawHierarchy();
                    while(f.getTransNextTrackpoint()) {}
                } else {
                    hierarchy = true;
                }
            } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
                hierarchy = true;
            } else {
                if(f.getTransTrackpointLevel() == getCurrentLevel()){
                    drawTrackpoint();
                }
            }
            while(f.getTransNextTrackpoint()) {
                if(f.getTransTrackpointJoin()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        transid = f.getTransID();
                        trackid = f.getTransTrackpointID();
                        int goal = f.getTransEnd();
                        bool drawjoin = true;
                        x = (int) f.getTransTrackpointPosx();
                        y = (int) f.getTransTrackpointPosy();
                        //cil = canvas()->collisions(QPoint(x, y));
                        cil = can->items (QPoint(x, y));
                        while (!cil.isEmpty()) {
                            QGraphicsItem* i = cil.takeFirst();
                            if(i->type() == Joinpoint::Type) {
                                f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
                                if(f.getTransEnd() == goal) {
                                    f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
                                    double newtrackposx = f.getTransTrackpointPosx();
                                    double newtrackposy = f.getTransTrackpointPosy();
                                    f.getTransbyID(transid);
                                    f.getTransTrackpointbyID(trackid);
                                    f.modifyTrans(transid, trackid, newtrackposx, newtrackposy);
                                    while(f.getTransNextTrackpoint()) {}
                                    drawjoin = false;
                                } else {
                                    f.getTransbyID(transid);
                                    f.getTransTrackpointbyID(trackid);
                                }
                            }
                        }
                        if(drawjoin) {
                            drawJoin();
                        }
                    }
                } else if(f.getTransTrackpointLink()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawLinkentrance();
                    }
                    while(f.getTransNextTrackpoint()) {}
                } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawHierarchy();
                        while(f.getTransNextTrackpoint()) {}
                    } else {
                        hierarchy = true;
                    }
                } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
                    hierarchy = true;
                } else {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        if(hierarchy) {
                            drawHierarchyexit();
                            hierarchy = false;
                        } else {
                            drawTrackpoint();
                        }
                    }
                }
            }
        }
    } else {
        if(f.getTransStart() < 0) {
            if(f.getTransFirstTrackpoint()) {
                if(f.getTransTrackpointJoin()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawJoin();
                    }
                }  else if(f.getTransTrackpointLink()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawLinkentrance();
                    }
                    while(f.getTransNextTrackpoint()) {}
                } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawHierarchy();
                        while(f.getTransNextTrackpoint()) {}
                    } else {
                        hierarchy = true;
                    }
                } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
                    hierarchy = true;
                } else {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawTrackpoint();
                    }
                }
                while(f.getTransNextTrackpoint()) {
                    if(f.getTransTrackpointJoin()) {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()){
                            transid = f.getTransID();
                            trackid = f.getTransTrackpointID();
                            int goal = f.getTransEnd();
                            bool drawjoin = true;
                            x = (int) f.getTransTrackpointPosx();
                            y = (int) f.getTransTrackpointPosy();
                            //cil = canvas()->collisions(QPoint(x, y));
                            cil = can->items (QPoint(x, y));
                            while (!cil.isEmpty()) {
                                QGraphicsItem* i = cil.takeFirst();
                                if(i->type() == Joinpoint::Type) {
                                    f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
                                    if(f.getTransEnd() == goal) {
                                        f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
                                        double newtrackposx = f.getTransTrackpointPosx();
                                        double newtrackposy = f.getTransTrackpointPosy();
                                        f.getTransbyID(transid);
                                        f.getTransTrackpointbyID(trackid);
                                        f.modifyTrans(transid, trackid, newtrackposx, newtrackposy);
                                        while(f.getTransNextTrackpoint()) {}
                                        drawjoin = false;
                                    } else {
                                        f.getTransbyID(transid);
                                        f.getTransTrackpointbyID(trackid);
                                    }
                                }
                            }
                            if(drawjoin) {
                                drawJoin();
                            }
                        }
                    } else if(f.getTransTrackpointLink()) {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()){
                            drawLinkentrance();
                        }
                        while(f.getTransNextTrackpoint()) {}
                    } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()){
                            drawHierarchy();
                            while(f.getTransNextTrackpoint()) {}
                        } else {
                            hierarchy = true;
                        }
                    } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
                        hierarchy = true;
                    } else {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()){
                            if(hierarchy) {
                                drawHierarchyexit();
                                hierarchy = false;
                            } else {
                                drawTrackpoint();
                            }
                        }
                    }
                }
            }
        } else {
            if(f.getTransFirstTrackpoint()) {
                if(f.getTransTrackpointJoin()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        transid = f.getTransID();
                        trackid = f.getTransTrackpointID();
                        int goal = f.getTransEnd();
                        bool drawjoin = true;
                        x = (int) f.getTransTrackpointPosx();
                        y = (int) f.getTransTrackpointPosy();
                        //cil = canvas()->collisions(QPoint(x, y));
                        cil = can->items (QPoint(x, y));
                        while (!cil.isEmpty()) {
                            //Q3CanvasItem* i = cil.takeFirst();
                            QGraphicsItem* i = cil.takeFirst();
                            if(i->type() == Joinpoint::Type) {
                                f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
                                if(f.getTransEnd() == goal) {
                                    f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
                                    double newtrackposx = f.getTransTrackpointPosx();
                                    double newtrackposy = f.getTransTrackpointPosy();
                                    f.getTransbyID(transid);
                                    f.getTransTrackpointbyID(trackid);
                                    f.modifyTrans(transid, trackid, newtrackposx, newtrackposy);
                                    while(f.getTransNextTrackpoint()) {}
                                    drawjoin = false;
                                } else {
                                    f.getTransbyID(transid);
                                    f.getTransTrackpointbyID(trackid);
                                }
                            }
                        }
                        if(drawjoin) {
                            drawJoin();
                        }
                    }
                } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawHierarchy();
                        while(f.getTransNextTrackpoint()) {}
                    } else {
                        hierarchy = true;
                    }
                } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
                    hierarchy = true;
                } else if(f.getTransTrackpointLink()) {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawLinkentrance();
                    }
                    while(f.getTransNextTrackpoint()) {}
                } else {
                    if(f.getTransTrackpointLevel() == getCurrentLevel()){
                        drawTrackpoint();
                    }
                }
                while(f.getTransNextTrackpoint()) {
                    if(f.getTransTrackpointJoin()) {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()){
                            transid = f.getTransID();
                            trackid = f.getTransTrackpointID();
                            int goal = f.getTransEnd();
                            bool drawjoin = true;
                            x = (int) f.getTransTrackpointPosx();
                            y = (int) f.getTransTrackpointPosy();
                            //cil = canvas()->collisions(QPoint(x, y));
                            cil = can->items (QPoint(x, y));
                            while (!cil.isEmpty()) {
                                QGraphicsItem* i = cil.takeFirst();
                                if(i->type() == Joinpoint::Type) {
                                    f.getTransbyID((dynamic_cast<Joinpoint*>(i))->transid);
                                    if(f.getTransEnd() == goal) {
                                        f.getTransTrackpointbyID((dynamic_cast<Joinpoint*>(i))->id);
                                        double newtrackposx = f.getTransTrackpointPosx();
                                        double newtrackposy = f.getTransTrackpointPosy();
                                        f.getTransbyID(transid);
                                        f.getTransTrackpointbyID(trackid);
                                        f.modifyTrans(transid, trackid, newtrackposx, newtrackposy);
                                        while(f.getTransNextTrackpoint()) {}
                                        drawjoin = false;
                                    } else {
                                        f.getTransbyID(transid);
                                        f.getTransTrackpointbyID(trackid);
                                    }
                                }
                            }
                            if(drawjoin) {
                                drawJoin();
                            }
                        }
                    } else if(f.getTransTrackpointLink()) {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()){
                            drawLinkentrance();
                        }
                        while(f.getTransNextTrackpoint()) {}
                    } else if(f.getTransTrackpointHldirection() != getCurrentLevel()) {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()){
                            drawHierarchy();
                            while(f.getTransNextTrackpoint()) {}
                        } else {
                            hierarchy = true;
                        }
                    } else if(f.getTransTrackpointHldirection() != f.getTransTrackpointLevel()) {
                        hierarchy = true;
                    } else {
                        if(f.getTransTrackpointLevel() == getCurrentLevel()){
                            if(hierarchy) {
                                drawHierarchyexit();
                                hierarchy = false;
                            } else {
                                drawTrackpoint();
                            }
                        }
                    }
                }
            }
        }
    }
}

void View::drawTrackpoint() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    TrackpointItem* tp;
    //tp = new TrackpointItem(can);
    tp = new TrackpointItem();
    scene()->addItem(tp);
    tp->setBrush(Qt::black);
    //tp->setSize(5,5);
    //tp->setRect( x(), y(), 5, 5);
    qreal wwidth = 5;
    qreal hheight = 5;
    tp->setRect( QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight) );
    int id =  f.getTransID();
    tp->transid = id;
    int trid = f.getTransTrackpointID();
    tp->id = trid;
    posx = f.getTransTrackpointPosx();
    posy = f.getTransTrackpointPosy();
    //tp->move(posx - 2.5, posy - 2.5);
    tp->setPos(posx, posy);
    //tp->setPos((qreal) 240, (qreal) 220);
    placeItem(tp);
    scene()->update();
}

void View::drawJoin() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    Joinpoint* j;
    //j = new Joinpoint(can);
    j = new Joinpoint();
    scene()->addItem(j);
    j->setBrush(Qt::red);
    //j->setSize(10,10);
    qreal wwidth = 10;
    qreal hheight = 10;
    j->setRect( QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight) );
    int id =  f.getTransID();
    j->transid = id;
    int trid = f.getTransTrackpointID();
    j->id = trid;
    posx = f.getTransTrackpointPosx();
    posy = f.getTransTrackpointPosy();
    //j->setPos(posx - 5, posy - 5);
    //j->setPos(70,70);
    //j->setPos(posx+7, posy+7); // omits bug due to empty cil
    j->setPos(posx, posy);
    placeItem(j);
}

void View::drawLinkentrance() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    Linkentrance* le;
    le = new Linkentrance();
    scene()->addItem(le);
    qreal wwidth = 15;
    qreal hheight = 8;
    le->setRect( QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight) );
    int id =  f.getTransID();
    le->transid = id;
    int trid = f.getTransTrackpointID();
    le->id = trid;
    //le->setBrush(Qt::green);
    int linkid = f.getTransTrackpointLinkID();
    if (f.getLinkbyID(linkid)== true)
    {
        unsigned int color = f.getLinkColor();
        le->setBrush(QColor(color));
    }
    else
    {
        le->setBrush(Qt::green);
    }
    posx = f.getTransTrackpointPosx();
    posy = f.getTransTrackpointPosy();
    le->setPos(posx, posy);
    placeItem(le);
}

void View::drawHierarchyexit() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    Hierarchyexit* he;
    //he = new Hierarchyexit(can);
    he = new Hierarchyexit();
    scene()->addItem(he);
    he->setBrush(Qt::blue);
    //he->setSize(10,5);
    qreal wwidth = 10;
    qreal hheight = 5;
    he->setRect( QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight) );
    int id =  f.getTransID();
    he->transid = id;
    int trid = f.getTransTrackpointID();
    he->id = trid;
    posx = f.getTransTrackpointPosx();
    posy = f.getTransTrackpointPosy();
    he->setPos(posx, posy);
    placeItem(he);
}

void View::drawArrow(double ax, double ay, double bx, double by) {
    int distance = 29;
    int distance2 = 42;
    int arrowwidth = 8;
    double factor;
    double cx = (ax - bx);
    double cy = (ay - by);
    double length = sqrt(cx*cx+cy*cy);
    if(length != 0){
        int sx = (int)((cx/(length/distance)) + bx);
        int sy = (int)((cy/(length/distance)) + by);
        double dx = (cx/(length/distance2)) + bx;
        double dy = (cy/(length/distance2)) + by;
        double nx;
        double ny;
        if(cx == 0){
            nx = 1;
            ny = 0;
            factor = arrowwidth;
        } else if (cy == 0){
            ny = 1;
            nx = 0;
            factor = arrowwidth;
        } else {
            nx = -cy/cx;
            ny = 1;
            factor = sqrt(arrowwidth*arrowwidth/(nx*nx+ny*ny));
        }
        
        int e1x = (int)(dx + factor*nx);
        int e1y = (int)(dy + factor*ny);
        int e2x = (int)(dx + (-1)*factor*nx);
        int e2y = (int)(dy + (-1)*factor*ny);
        Transline* tl;
        //tl = new Transline(can);
        tl = new Transline();
        scene()->addItem(tl);
        //tl->setPoints(sx, sy, e1x, e1y);
        tl->setLine(sx, sy, e1x, e1y);
        //tl->setBrush(Qt::black);
        QPen translinePen = QPen(Qt::black);
        translinePen.setWidth(0);
        tl->setPen(translinePen);
        tl->id = -1;
        placeItem(tl);
        Transline* tl2;
        //tl2 = new Transline(can);
        tl2 = new Transline();
        scene()->addItem(tl2);
        tl2->setLine(sx, sy, e2x, e2y);
        //tl2->setBrush(Qt::black);
        QPen translinePen2 = QPen(Qt::black);
        translinePen2.setWidth(0);
        tl->setPen(translinePen2);
        tl2->id = -1;
        placeItem(tl2);
    }
}

void View::drawBoundingBox() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    BoundingBox* bb;
    bb = new BoundingBox();
    scene()->addItem(bb);
    //bb->setBrush(QColor(255,255,255,0));//Transparent, white
    bb->setBrush(QColor(Qt::transparent));//Transparent, white
    qreal wwidth = dynamic_cast<MainguiWindow*>(wid)->getsceneWidth();
    qreal hheight = dynamic_cast<MainguiWindow*>(wid)->getsceneHeight();
    //bb->setRect( QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight) );
    bb->setRect( QRectF(0 , 0, wwidth, hheight) );
//    j->setPos(posx, posy);
    bb->setZValue(-1000);
    placeItem(bb);
}

void View::drawFixedGrid() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    FixedGrid* fg;
    qreal wwidth = dynamic_cast<MainguiWindow*>(wid)->getsceneWidth();
    qreal hheight = dynamic_cast<MainguiWindow*>(wid)->getsceneHeight();
    int i=0;
    int j=0;
    int gridInterval = 200;
    qreal maxHeight = hheight/gridInterval;
    qreal maxWidth = wwidth/gridInterval;
    QBrush brush= QBrush(QColor(0,0,0,128));
    QPen pen(brush , 0, Qt::DotLine);
    for (i; i<maxHeight; i++)
    {
        fg = new FixedGrid();
        scene()->addItem(fg);
        //bb->setBrush(QColor(255,255,255,0));//Transparent, white
        //fg->setBrush(QColor(Qt::transparent));//Transparent, white
        //QBrush &brush= QBrush(QColor(255,255,255,128), style = Qt::SolidPattern);        
        //QPen pen(Qt::black, 0, Qt::DotLine);        
        fg->setPen(pen);
        //bb->setRect( QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight) );
        //fg->setLine( 0 , 30, wwidth, 30 );
        int cury= gridInterval*i;
        fg->setLine( 0, cury, wwidth, cury);
        //    j->setPos(posx, posy);
        fg->setZValue(-999);
        placeItem(fg);
    }
    for (j; j<maxWidth; j++)
    {
        fg = new FixedGrid();
        scene()->addItem(fg);
        //bb->setBrush(QColor(255,255,255,0));//Transparent, white
        //fg->setBrush(QColor(Qt::transparent));//Transparent, white
        //QBrush &brush= QBrush(QColor(255,255,255,128), style = Qt::SolidPattern);
        //QPen pen(Qt::black, 0, Qt::DotLine);    
        fg->setPen(pen);
        //bb->setRect( QRectF(-(wwidth/2) , -(hheight/2), wwidth, hheight) );
        //fg->setLine( 0 , 30, wwidth, 30 );
        int curx = gridInterval*j;
        fg->setLine( curx, 0, curx, hheight);
        //    j->setPos(posx, posy);
        fg->setZValue(-990);
        placeItem(fg);
    }
}

bool View::testJoin(int transid, int trackid) {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    testjoin = false;
    f.getTransbyID(transid);
    if(f.getTransFirstTrackpoint()) {
        do {
            if(f.getTransTrackpointID() == trackid) {
                return testjoin;
            } else {
                if(f.getTransTrackpointJoin()) {
                    testjoin = true;
                }
            }
        } while(f.getTransNextTrackpoint());
    }
    if(trackid < 1000){
        f.getTransTrackpointbyID(trackid);
    }
    return testjoin;
}

void View::refreshOtherJoins(int transid, int trackid) { 
    testjoin = false;
    int curhelp = -1;
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    f.getTransbyID(transid);
    f.getTransTrackpointbyID(trackid);
    if(f.getTransFirstTrackpoint()) {
        do {
            if(f.getTransTrackpointID() == trackid) {
                while (f.getTransNextTrackpoint()) {}
            } else {
                if(f.getTransTrackpointJoin()) {
                    curhelp = f.getTransTrackpointID();
                }
            }
        } while (f.getTransNextTrackpoint());
    }
    if(curhelp > -1) {
        trackid = curhelp;
        f.getTransTrackpointbyID(trackid);
        int goal = f.getTransEnd();
        double jposx = f.getTransTrackpointPosx();
        double jposy = f.getTransTrackpointPosy();
        if(f.getFirstTrans()) {
            do {
                if (f.getTransEnd() == goal) {
                    if(f.getTransID() != transid){
                        if(f.getTransFirstTrackpoint()) {
                            do {
                                if(f.getTransTrackpointJoin()) {
                                    if((f.getTransTrackpointPosx() == jposx) && (f.getTransTrackpointPosy() == jposy)) {
                                        int deleteid;
                                        double curposx;
                                        double curposy;
                                        int curtransid = f.getTransID();
                                        int curtrackid = f.getTransTrackpointID();
                                        while(f.getTransNextTrackpoint()) {
                                            deleteid = f.getTransTrackpointID();
                                            f.deleteTrackpointOfTrans(curtransid, deleteid);
                                            f.getTransTrackpointbyID(curtrackid);
                                        }
                                        f.getTransbyID(transid);
                                        f.getTransTrackpointbyID(trackid);
                                        while(f.getTransNextTrackpoint()) {
                                            curposx = f.getTransTrackpointPosx();
                                            curposy = f.getTransTrackpointPosy();
                                            curhelp = f.getTransTrackpointLevel();
                                            f.addTrackpointToTrans(curtransid, 1000, true, curposx, curposy, curhelp);
                                            f.getTransbyID(curtransid);
                                            curtrackid = f.getTransTrackpointID();
                                            f.getTransbyID(transid);
                                            curhelp = f.getTransTrackpointColor();
                                            f.modifyTransTrackpointColor(curtransid, curtrackid, curhelp);
                                            if(f.getTransTrackpointJoin()) {
                                                f.modifyTrans(curtransid, curtrackid, true);
                                            }
                                            if(f.getTransTrackpointLink()) {
                                                curhelp = f.getTransTrackpointLinkID();
                                                f.modifyTrans(curtransid, curtrackid, true, curhelp);
                                            }
                                            curhelp = f.getTransTrackpointHldirection();
                                            f.transLevel(curtransid, curtrackid, curhelp);
                                        }
                                        f.getTransbyID(curtransid);
                                    }
                                }
                            } while (f.getTransNextTrackpoint());
                        }
                    }
                }
            } while (f.getNextTrans());
        }
    }
}

void View::changeJoinTarget(int transid, int trackid, int target) {
    testjoin = false;
    int curhelp = -1;
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    f.getTransbyID(transid);
    f.getTransTrackpointbyID(trackid);
    if(f.getTransFirstTrackpoint()) {
        do {
            if(f.getTransTrackpointID() == trackid) {
                while (f.getTransNextTrackpoint()) {}
            } else {
                if(f.getTransTrackpointJoin()) {
                    curhelp = f.getTransTrackpointID();
                }
            }
        } while (f.getTransNextTrackpoint());
    }
    if(curhelp > -1) {
        trackid = curhelp;
        f.getTransTrackpointbyID(trackid);
        int goal = f.getTransEnd();
        double jposx = f.getTransTrackpointPosx();
        double jposy = f.getTransTrackpointPosy();
        if(f.getFirstTrans()) {
            do {
                if (f.getTransEnd() == goal) {
                    if(f.getTransFirstTrackpoint()) {
                        do {
                            if(f.getTransTrackpointJoin()) {
                                if((f.getTransTrackpointPosx() == jposx) && (f.getTransTrackpointPosy() == jposy)) {
                                    int curid = f.getTransID();
                                    int curstart = f.getTransStart();
                                    f.modifyTrans(curid, curstart, target);
                                    while (f.getTransNextTrackpoint()) {}
                                }
                            }
                        } while (f.getTransNextTrackpoint());
                    }
                }
            } while (f.getNextTrans());
        }
    }
}

TranslineText* View::drawTransText() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    if(f.getTransLevel() == getCurrentLevel()) {
        if(f.getTransStart() > -1 || f.getTransEnd() > -1 || (f.getTransFirstTrackpoint() && f.getTransNextTrackpoint())) {
            if(!(f.getTransFirstTrackpoint() && f.getTransTrackpointLink())) {
                TranslineText* tlt;
                //tlt = new TranslineText(QString::fromStdString(f.getTransName()), can);
                tlt = new TranslineText(QString::fromStdString(f.getTransName()), NULL);
                scene()->addItem(tlt);
                tlt->id = f.getTransID();
                qreal px = f.getTransTextPosx();
                qreal py = f.getTransTextPosy();
                //tlt->move(px, py);
                tlt->setPos(px, py);
                placeItem(tlt);
                return tlt;
            }
        }
    }
    return NULL;
}

void View::resetposition() {
    Core_I& c = CoreFactory::getCore_I();
    int currentidMaingui = dynamic_cast<MainguiWindow*>(wid)->getcurrentfsmid();
    Fsm_I& f = c.getFsm(currentidMaingui);
    //Fsm_I& f = c.getCurrentFsm();
    int id = dynamic_cast<TranslineText*>(activatedItem)->id;
    f.getTransbyID(id);
    int start = f.getTransStart();
    double px;
    double py;
    if(start > -1) {
        f.getStatebyID(start);
        px = f.getStatePosx();
        py = f.getStatePosy();
        if(f.getTransFirstTrackpoint()) {
            px = f.getTransTrackpointPosx();
            py = f.getTransTrackpointPosy();
        } else {
            int distancetxt = 70;
            px = px + 15;
            py = py + 15;
            f.getStatebyID(f.getTransEnd());
            double gx = f.getStatePosx();
            double gy = f.getStatePosy();
            double cx = (gx - px-15);
            double cy = (gy - px-15);
            double length = sqrt(cx*cx+cy*cy);
            if(length != 0) {
                double px = ((cx/(length/distancetxt)) + px);
                double py = ((cy/(length/distancetxt)) + py);
            }
            if(length > 80) {
                f.modifyTrans(helpvar , (double) px + 5, (double) py + 5, getCurrentLevel());
            } else {
                f.modifyTrans(helpvar , (double) px + 20, (double) py + 20, getCurrentLevel());
            }
        }
    } else {
        if(f.getTransFirstTrackpoint()) {
            px = f.getTransTrackpointPosx();
            py = f.getTransTrackpointPosy();
        } else {
            px = 10;
            py = 10;
        }
    }
    f.modifyTrans(id, px + 5, py + 5, getCurrentLevel());
    redraw();
    //update();
}

void View::simulationMark(int type, int id) {
    activateItem(0);
    //Q3CanvasItemList cil = canvas()->allItems();
    QList <QGraphicsItem *> cil =  can->items()  ;
    while (!cil.isEmpty()) {
        QGraphicsItem* i = cil.takeFirst();
        switch(type) {
        case 1:
            if(i->type() == NewStateItem::Type) {
                if((dynamic_cast<NewStateItem*>(i))->id == id) {
                    activateItem(i);
                    return;
                }
            }
            break;
        case 2:
            if(i->type() == Transline::Type) {
                if((dynamic_cast<Transline*>(i))->id == id) {
                    activateItem(i);
                    return;
                }
            }
            break;
        case 3:
            if(i->type() == Hypertransition::Type) {
                if((dynamic_cast<Hypertransition*>(i))->id == id) {
                    activateItem(i);
                    return;
                }
            }
            break;
        default:
            
            break;
        }
    }
    
}


QColor View::getInitStateColor() {
    return statecolor;
}

void View::initStateColor(QColor c) {
    statecolor = c;
}

QColor View::getInitLinkColor() {
    return linkcolor;
}

void View::initLinkColor(QColor c) {
    linkcolor = c;
}

