/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#ifndef VIEW_H_
#define VIEW_H_

// #include <Qt3Support/Q3Canvas>
// #include <Qt3Support/Q3CanvasView>
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QGraphicsSceneMouseEvent>

#include "selectionanimator.h"
#include "scene.h"
#include "items.h"

typedef enum {CHOOSE, STATE, TRANS, HTRANS, LINK, LINKCONNECT, JOIN, REMOVE, HIERARCHY, TRACKPOINT} Workbenchmode;

typedef enum {LOCKED, UNLOCKED} Lockmode;

class View : public QGraphicsView {

    Q_OBJECT;

    //QGraphicsScene* can;
    Scene* can;

public:
    QWidget* wid;

private:
    QGraphicsItem* activatedItem;
    QGraphicsItem* newItem;
    QPixmap* bgGrid;
    QColor statecolor;
    QColor linkcolor;
    Workbenchmode objectmode;
    Qt::ItemSelectionMode mode;
    Lockmode lock; // Used for drawing of multiple objects when locked
    bool ctrlSet;   //Control key is pressed down
    bool altSet;    //Alt key is pressed down
    bool gSet;      //G key is pressed down
    bool xSet;      //X key is pressed down
    bool paintmode; //FSMDesigner is not in simulation Mode
    bool snapToGrid;    //moved object snaps to grid
    int front;
    int back;
    int helpvar;
    int transstart;
    int transend;
    bool mousemoved;
    bool drag;
    bool usegrid;
    bool transbegin;
    //bool transinserted;
    bool testjoin;
    bool foundstate;
    qreal posx;
    qreal posy;
    qreal scenePosX;
    qreal scenePosY;
    qreal mousePosX;
    qreal mousePosY;
    QString transConditionString;
    double startx;
    double starty;
    double endx;
    double endy;
    bool showFixedGrid;
    int translineWidth;
    qreal stateSize;



    QAction* Straight;
    QAction* setDefault;
    QAction* change_Start;
    QAction* change_Target;

    QAction* insertTrackpointbefore;
    QAction* insertTrackpointafter;
    QAction* connecttoLink;
    QAction* changetoHierarchy;
    QAction* createJoin;
    QAction* insertTrackpoint;
    QAction* connecttoHierarchy;
    QAction* actionresetposition;

    QAction* transformToTrackpoint;

    // Animator on selection
    SelectionAnimator * selectionAnimator;

    // Selection area control
    QPointF * areaSelectionInitial;


public:
    View (Scene* c, QWidget* w);
    //~View();

protected:
    void wheelEvent(QWheelEvent *event);    //Zoom in and out with the  mouse Wheel
    void mousePressEvent(QMouseEvent* e);
    void mouseMoveEvent(QMouseEvent* e);
    void mouseReleaseEvent(QMouseEvent* e);
    void contextMenuEvent(QContextMenuEvent* e);
    void keyPressEvent(QKeyEvent* ke);
    void keyReleaseEvent(QKeyEvent* ke);
    void drawForeground ( QPainter * painter, const QRectF & rect );

private:
    void placeItem(QGraphicsItem* i);
    void createActionsforContextMenu();
    void connectContextMenuActions();
    void initvariables();


private slots:
    void straighten();
    void setdefault();
    void changestart();
    void changetarget();
    void inserttrackpoint();
    void inserttrackpointbefore();
    void inserttrackpointafter();
    void connecttolink();
    void changetohierarchy();
    void createjoin();
    void transformtotrackpoint();
    void connecttohierarchy();
    void resetposition();

public slots:
    void setunlocked();
    void setlocked();


public:
    int getCurrentLevel();
    void activateItem(QGraphicsItem* i);
    void sethelpvar(int i);
    void setmode(Workbenchmode w);
    void cut();
    void copy();
    void paste();
    void remove();
    void movetoback();
    void movetofront();
    void addState();
    void addHypertrans();
    void addLink();
    void addHierarchy();

    /// Add a transition in FSM
    void addTrans();

    /// Add A transition line on the view
    void addTransline(int sx, int sy, int ex, int ey);

    void addTrackpoint();
    void addJoin();
    void addLinkentrance();
    void grid();
    void zoomIn();
    void zoomOut();
    void setPaintmode(bool b);
    void redraw();
    void simulationMark(int type, int id);
    QColor getInitStateColor();
    void initStateColor(QColor c);
    QColor getInitLinkColor();
    void initLinkColor(QColor c);

private:
    void drawState();
    void drawHypertrans();
    void drawLink();
    void drawHierarchy();
    void drawTrans(Transline**  returnPointer = NULL );
    void drawTransPoints();
    void drawTrackpoint();
    void drawJoin();
    void drawLinkentrance();
    void drawHierarchyexit();
    void drawArrow(double ax, double ay, double bx, double by);
    TranslineText* drawTransText();
    void drawBoundingBox();
    void drawFixedGrid();

    bool testJoin(int transid, int trackid);
    void refreshOtherJoins(int transid, int trackid);
    void changeJoinTarget(int transid, int trackid, int target);
};



#endif
