/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <stdio.h>
#include <string>
#include <iostream>

#include <QtGui/QApplication>

#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/util/XMLString.hpp>

#include "testbenchhandler.h"

using namespace std;

int main(int argc, char** argv) {
  QApplication a( argc, argv );

  if (argc<2) {
		cout<<"Usage: createtestbench [gui] infile [outdir]"<<endl;
    return -1;
  }

  /// using sax parser for for mmap generation
  try {
    XMLPlatformUtils::Initialize();
  }
  catch (const XMLException& toCatch) {
    char* message = XMLString::transcode(toCatch.getMessage());
    cout << "Error during initialization! :\n";
    cout << "Exception message is: \n"
	 << message << "\n";
    XMLString::release(&message);
    return -1; 
  }
 

  ///extract filename and options
  TestbenchHandler* defaultHandler;
 bool gui=false;
 string filename;
 if(strcmp(argv[1],"gui")==0)
 {
	gui=true;  
        if(argc==3)
	{
        	filename=argv[2];
		defaultHandler= new TestbenchHandler();
	}
	else
		cout<<"Usage: createtestbench [gui] infile [outdir]"<<endl;
 }
 else 
 {
  	filename = argv[1];
        if(argc==3)
	{
        	string outfile=argv[2];
		defaultHandler= new TestbenchHandler(outfile);
	}
	else
		cout<<"Usage: createtestbench [gui] infile [outdir]"<<endl;
 };
  
  
  
 
  const char* xmlFile = filename.c_str();
  SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();
  parser->setFeature(XMLUni::fgSAX2CoreValidation, true);   
  parser->setFeature(XMLUni::fgSAX2CoreNameSpaces, false);   
  
  parser->setContentHandler(defaultHandler);
  parser->setErrorHandler(defaultHandler);
  
  try {
    parser->parse(xmlFile);
  }
  catch (const XMLException& toCatch) {
    char* message = XMLString::transcode(toCatch.getMessage());
    cout << "Exception message is: \n"
	 << message << "\n";
    XMLString::release(&message);
    return -1;
  }
  catch (const SAXParseException& toCatch) {
    char* message = XMLString::transcode(toCatch.getMessage());
    cout << "Exception message is: \n"
	 << message << "\n";
    XMLString::release(&message);
    return -1;
  }
  catch (...) {
    cout << "Unexpected Exception \n" ;
    return -1;
  }
  
  delete parser;
  delete defaultHandler;
  XMLPlatformUtils::Terminate();
  return 0;
}
