/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <cstdlib>
#include <iostream>
#include <string>
#include <fstream>

#include <QtCore/QVariant>

#include <QtCore/QDir>
#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/sax/SAXParseException.hpp>

#include "mmaphandler.h"


MmapHandler::MmapHandler() { 
  QDir dir = QDir::home();
  name = dir.path().toStdString() + "/.simvisionrc";
  statename = false;
  sname = "";
  write = false;
  col = "";
}

void MmapHandler::startDocument() {
  const char*const filename = name.c_str();
  ofstream out(filename, ios_base::out|ios_base::trunc);
}

void MmapHandler::startElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname, const Attributes& att) { 
  char* help = XMLString::transcode(qname);
  const char*const filename = name.c_str();
  ofstream out(filename, ios_base::app); 
  if(out.is_open()) {
    if(strcmp(help,"fsm") == 0) {  
      string mapname = XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0));
      out << "mmap new -name " +  mapname + " -contents {" << endl;   
      numberofoutputs = XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 3));
    } else if  (strcmp(help,"state") == 0) {
      col = QColor::fromRgb((unsigned long)(atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2)))));
      sname = "";
    } else if (strcmp(help,"sname") == 0) {
      out << "{ ";
      statename = true;
    } else if (strcmp(help,"output") == 0) {
      out << "'b" ;
      write = true;
    }
    out.close();   
  }
}

void MmapHandler::endElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname) {
  char* help = XMLString::transcode(qname);
  const char*const filename = name.c_str();
  ofstream out(filename, ios_base::app); 
  if(out.is_open()) { 
    if(strcmp(help,"fsm") == 0) {
      out << "}" << endl;
    } else if (strcmp(help,"sname") == 0) {
      statename = false;
    } else if (strcmp(help,"output") == 0) {
      write = false;
      QVariant v = col;
      string hexcolstr = v.toString().toStdString();
      out << " -label \"" + sname + "\" -linecolor " + hexcolstr  +"}" << endl; 
    }
    out.close();
  }
}

void MmapHandler::characters (const XMLCh* const chars, const unsigned long length) {
  char* help = XMLString::transcode(chars);
  const char*const filename = name.c_str();
  ofstream out(filename, ios_base::app); 
  if(out.is_open()) { 
    if(write) {
      out << help;
    } else if (statename) {
      sname.append(help);
    } 
    out.close();
  }
}

void MmapHandler::endDocument() {

}

void MmapHandler::fatalError(const SAXParseException& exc) {
  char* message = XMLString::transcode(exc.getMessage());
  cout << "Fatal Error: " << message
       << " at line: " << exc.getLineNumber()
       << endl;
}

void MmapHandler::warning(const SAXParseException& exc) {
  char* message = XMLString::transcode(exc.getMessage());
  cout << "Warning: " << message
       << " at line: " << exc.getLineNumber()
       << endl;
}

void MmapHandler::error(const SAXParseException& exc) {
  char* message = XMLString::transcode(exc.getMessage());
  cout << "Error: " << message
       << " at line: " << exc.getLineNumber()
       << endl;
}
