/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <cstdlib>
#include <ctime>
#include <iostream>
#include <string>
#include <fstream>

#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtCore/QString>

#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/sax/SAXParseException.hpp>

#include "testbenchhandler.h"


TestbenchHandler::TestbenchHandler() : QDialog(){
  inflag = false;
  outflag = false;
  write =false;
  project = "";
  fsmname = "";
  directory = ""; 
  names = "";
  QString qs; 
  QWidget w;
  qs = QFileDialog::getExistingDirectory(&w,
				    "Choose a directory",
				    "/home",
				    QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
  directory = qs.toStdString();
}

TestbenchHandler::TestbenchHandler(string outdir)
{
  inflag = false;
  outflag = false;
  write =false;
  project = "";
  fsmname = "";
  directory = outdir; 
  names = "";
};

void TestbenchHandler::startDocument() {

}

void TestbenchHandler::startElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname, const Attributes& att) {
  char* help = XMLString::transcode(qname);
  if(strcmp(help,"project") == 0) { 
    project = XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0));
    cout << project << endl;
  } else if(strcmp(help,"fsm") == 0) {  
    fsmname =  XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0));
    string fsmwithdir =  directory + fsmname + "_testbench.v";
    const char*const filename = fsmwithdir.c_str();
    ofstream out(filename, ios_base::trunc); 
    if(out.is_open()) {
      out << "`resetall" << endl << endl;   
      out << "`timescale 1ns / 1ps" << endl << endl;   
      out << "//////////////////////////////////////////////////////////////////////////" << endl;
      out << "// Company: " << endl;
      out << "// Engineer: " << endl;
      out << "//" << endl;
      time_t now;
      time(&now);
      out << "// Create Date: " << ctime(&now);
      out << "// Design Name: " << fsmname << endl;
      out << "// Module Name: " << fsmname << "_testbench.v" << endl;
      out << "// Project Name:" << project << endl;
      out << "// Target Device: " << endl;
      out << "// Description: " << endl;
      out << "//" << endl;
      out << "// Created by FSMDesigner4 tools." << endl;
      out << "//" << endl;
      out << "// Additional Comments: " << endl;
      out << "//" << endl;
      out << "//////////////////////////////////////////////////////////////////////////" << endl << endl;
      out << "module "<< fsmname << "_testbench;"<< endl << endl;
      
      out.close();
    } 
  } else if  (strcmp(help,"globals") == 0) {
    string fsmwithdir =  directory + fsmname + "_testbench.v";
    const char*const filename = fsmwithdir.c_str();
    ofstream out(filename, ios_base::app); 
    if(out.is_open()) {
      out << "//Inputs" << endl;
      out.close();
    }
      inflag = true;
  } else if  (strcmp(help,"inputnames") == 0) {
    inflag = true;
  } else if  (strcmp(help,"outputnames") == 0) {
    string fsmwithdir =  directory + fsmname + "_testbench.v";
    const char*const filename = fsmwithdir.c_str();
    ofstream out(filename, ios_base::app); 
    if(out.is_open()) {
      out << endl << "//Outputs" << endl;
      out.close();
    } 
    outflag = true;
  }  else if  (strcmp(help,"name") == 0) {
    if(inflag || outflag){
      string fsmwithdir =  directory + fsmname + "_testbench.v";
      const char*const filename = fsmwithdir.c_str();
      ofstream out(filename, ios_base::app); 
      if(out.is_open()) {
	if(inflag) {
	  out << "reg ";
	} else {
	  out << "wire ";
	}
	out.close();
      }
      write = true;
    }
  }
}

void TestbenchHandler::endElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname) {
  char* help = XMLString::transcode(qname);
  if(strcmp(help,"fsm") == 0) {
    string fsmwithdir =  directory + fsmname + "_testbench.v";
    const char*const filename = fsmwithdir.c_str();
    ofstream out(filename, ios_base::app); 
    if(out.is_open()) { 
      out << endl << "// Instantiate the Unit Under Test (uut)" << endl;
      out << fsmname << " uut (" << endl;
      vector <string>::iterator i;
      for(i = ins.begin(); i!=ins.end(); i++) { 
	out << "."  << (*i) << "("  << (*i) << ")," << endl;
      }
      for(i = outs.begin(); i!=outs.end(); i++) {
	if(i == (--(outs.end()))) {
	  out << "."  << (*i) << "("  << (*i) << ")" << endl;
	} else {
	  out << "."  << (*i) << "("  << (*i) << ")," << endl;
	}
      }
      out << ");" << endl;
      out << endl << "always" << endl;
      out << "begin" << endl;
      out << "#10 " << ins[0] << "=!" << ins[0] << ";" << endl;
      out << "end" << endl;
      out << endl << "initial begin" << endl << endl;
      out << "// Initialize Inputs" << endl;
      for(i = ins.begin(); i!=ins.end(); i++) {
	out << (*i) << " = 0;" << endl;
      }
      out << endl;
      out << endl << "#100" << endl << "$finish;" << endl << endl;;
      out << "end" << endl << endl;;
      out << "endmodule" << endl;
      out.close();
    }
    while(ins.begin() != ins.end()) {
      ins.pop_back();
    } 
    while(ins.begin() != ins.end()) {
      outs.pop_back();
    }
    } else if (strcmp(help,"name") == 0) {
      if(inflag || outflag){
	string fsmwithdir =  directory + fsmname + "_testbench.v";
	const char*const filename = fsmwithdir.c_str();
	ofstream out(filename, ios_base::app); 
	if(out.is_open()) {
	    out << ";" << endl;
	  out.close();
	}
	if(inflag) {
	  ins.push_back(names);
	} else {
	  outs.push_back(names);
	} 
	names = "";
	write = false;
      }
    } else if (strcmp(help,"outputnames") == 0) {
      outflag = false;
    } else if (strcmp(help,"inputnames") == 0) {
      inflag = false;
    } else if (strcmp(help,"globals") == 0) {
      inflag = false;
    }
}


void TestbenchHandler::characters (const XMLCh* const chars, const unsigned long length) {
  char* help = XMLString::transcode(chars);
  string fsmwithdir =  directory + fsmname + "_testbench.v";
  const char*const filename = fsmwithdir.c_str();
  ofstream out(filename, ios_base::app); 
  if(out.is_open()) { 
    if(write) {
      out << help;
      names.append(help);
    } 
    out.close();
  }
}

void TestbenchHandler::endDocument() {
  cout << "end" << endl;
}

void TestbenchHandler::fatalError(const SAXParseException& exc) {
  char* message = XMLString::transcode(exc.getMessage());
  cout << "Fatal Error: " << message
       << " at line: " << exc.getLineNumber()
       << endl;
}

void TestbenchHandler::warning(const SAXParseException& exc) {
  char* message = XMLString::transcode(exc.getMessage());
  cout << "Warning: " << message
       << " at line: " << exc.getLineNumber()
       << endl;
}

void TestbenchHandler::error(const SAXParseException& exc) {
  char* message = XMLString::transcode(exc.getMessage());
  cout << "Error: " << message
       << " at line: " << exc.getLineNumber()
       << endl;
}
