/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <cstdlib>
#include <iostream>
#include <string>
#include <fstream>


#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"
#include "projecthandler.h"
#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/sax/SAXParseException.hpp>

using namespace std;


ProjectHandler::ProjectHandler() {
  numberofinputs = 0;
  numberofoutputs = 0;
  count = 0;
  count2 = 0;
  init = true;
  level = 0;
  towrite = "";
  towrite2 = "";
  param = false;
  trans = false;
  transname = false;
  hypertrans = false;
  firstState = true;
  firstFsm = true;
  hvalue = 0;
  firstTrackpoint = true;
  firstHypertrans = true;
  firstLink = true;
  global = true;
  posx = 0.1; 
  posy = 0.1;
}

void ProjectHandler::startDocument() {
  Core_I& c = CoreFactory::getCore_I(); 
  c.addFsm();
  Fsm_I& f = c.getCurrentFsm();
  fsmid = f.getFsmID();
}

void ProjectHandler::startElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname, const Attributes& att) {
  char* help = XMLString::transcode(qname);
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getFsm(fsmid);

  switch(level) {
  case 0: 
    if (strcmp(help,"trans") == 0) {
      level = 1;
      count = 0;
      trans = true;
      transname = true;
      f.addTrans(0,0);
      f.modifyTrans(count2,atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0))));
      f.modifyTrans(count2, atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 1))), atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2))), atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 3))));
    } else {
      if (strcmp(help,"state") == 0) {
	level = 1;
	if(firstState) {
	  firstState = false;
	  count = 0;
	} else {
	  count++;
	}
	f.addState();
	f.modifyState(count, atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0))), atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 1))));
	f.modifyState(count,  (unsigned long) strtoul(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2)),NULL,0));
	f.stateLevel(count, atoi(XMLString::transcode(att.getValue(3))));
      } else {
	if (strcmp(help,"hypertrans") == 0) {
	  count = 0;
	  level = 1;
	  hypertrans = true;
	  if(firstHypertrans) {
	    count2 = 0;
	    firstHypertrans = false;
	  }
	  posx = atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0)));
	  posy = atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 1)));
	  f.addHypertrans(false, posx, posy, 0, atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 3))));
	  f.modifyHypertrans(count2, atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2))));
	} else {
	  if (strcmp(help,"link") == 0) {
	    if(firstLink) {
	      count = 0;
	      firstLink = false;
	    }
	    f.addLink(atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0))), atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 1))), atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 4))), atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 3))));
	    f.modifyLink(count, atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2))));
	    count++;
	    level = 1;
	  } else {
	    if (strcmp(help,"parameter") == 0) {
	      param = true;
	      count = 0;
	      level = 1;
	    } else { 
	      if (strcmp(help,"globals") == 0) {
		count = 0;
		level = 1;
	      } else {
		if (strcmp(help,"inputnames") == 0) {
		  count = 0;
		  level = 1;
		} else {
		  if (strcmp(help,"outputnames") == 0) {
		    count = 0;
		    level = 1;
		  } else {
		    if (strcmp(help,"fsm") == 0) {
		      firstState = true;
		      if(firstFsm) {
			f.fsmName(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0)));
			f.resetState(atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 1))));
			numberofinputs = atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2)));
                        f.numberOfIns(numberofinputs, -1000);
			numberofoutputs = atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 3)));
                        f.numberOfOuts(numberofoutputs, false);
			firstFsm = false;
		      } else {
			c.addFsm();
			Fsm_I& ff = c.getCurrentFsm();
			fsmid = ff.getFsmID();
			ff.fsmName(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0)));
			ff.resetState(atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 1))));
			numberofinputs = atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2)));
                        ff.numberOfIns(numberofinputs, -1000);
			numberofoutputs = atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 3)));
                        ff.numberOfOuts(numberofoutputs, false);
		      }
		    } else {
		      if(strcmp(help,"project") == 0) {
			c.projectName(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0)));
		      }
		    }
		  }
		}
	      }
	    }	  
	  }
	}
      }
    }
    break;
  case 1: 
    if (strcmp(help,"condition") == 0) {
      level = 2;
      if(param) {
	f.addParameter();
      } 
      if(trans){
	f.addConditionToTrans(count2);
      }
      if(hypertrans) {
	f.addConditionToHypertrans(count2);
      }
    } else {
      if (strcmp(help,"trackpoint") == 0) {
	if(firstTrackpoint) {
	  count = 0;
	  firstTrackpoint = false;
	}
	f.addTrackpointToTrans(count2, 1000, false, atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 0))), atof(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 1))), atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 3))));
	f.transLevel(count2, count, atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 2))));
	if(atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 4))) == 0) {
	  b = false;
	} else {
	  b = true;
	}
	f.modifyTrans(count2, count, b, atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 5))));
	if(atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 6))) == 0) {
	  b = false;
	} else {
	  b = true;
	}
	f.modifyTrans(count2, count, b);
	f.modifyTransTrackpointColor(count2, count, atoi(XMLString::transcode(att.getValue((xerces_xmlsize_t_cast) 7))) );
	count++;
      } else if (strcmp(help,"sname") == 0) {
	write = true;
      } else if (strcmp(help,"output") == 0) {
	write = true;
      } else if (strcmp(help,"default") == 0) {
	write = true;
      } else if (strcmp(help,"start") == 0) {
	write = true;
      } else if (strcmp(help,"end") == 0) {
	write = true;
      } else if (strcmp(help,"name") == 0) {
	write = true;
      } else if (strcmp(help,"type") == 0) {
	write = true;
      } else if (strcmp(help,"goal") == 0) {
	write = true;
      }
    }
    break;
  case 2: 
    if (strcmp(help,"cname") == 0) {
      write = true;
    } else {
      if (strcmp(help,"input") == 0) {
	write = true;
      }
    }
    break;
  }
}

void ProjectHandler::endElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname) {
  char* help = XMLString::transcode(qname);
  Core_I& c = CoreFactory::getCore_I();
  Fsm_I& f = c.getFsm(fsmid);

  if (strcmp(help,"trans") == 0) {
    level = 0;
    count2++;
    trans = false;
    firstTrackpoint = true;
  } else if (strcmp(help,"state") == 0) {
    level = 0;
  } else if (strcmp(help,"sname") == 0) {
    f.modifyState(count, towrite);
    towrite = "";
  } else if (strcmp(help,"output") == 0) {
    for(int i = 0; i < numberofoutputs; i++) {
      f.modifyState(count, i, towrite[i]);
    }
    towrite = "";
  } else if (strcmp(help,"condition") == 0) {
    level = 1;
    count++;
  } else if (strcmp(help,"cname") == 0) {
    if(write) {
      if(param) {
	f.modifyParameter(count, towrite);
      } else if (trans) {
	f.modifyTrans(count2, count, towrite);
      } else if (hypertrans) {
	f.modifyHypertrans(count2, count, towrite);
      }
      towrite = "";
      write = false;
    }
  } else if (strcmp(help,"input") == 0) {
    if(write) {
      if(param) {
	for(int i = 0; i < numberofinputs; i++) {
	  f.modifyParameter(count, i, towrite[i]);
	}
      } else if (trans) { 
	for(int i = 0; i < numberofinputs; i++) {
	    f.modifyTrans(count2, count, i, towrite[i]);
	}
      } else if (hypertrans) {
	for(int i = 0; i < numberofinputs; i++) {
	   f.modifyHypertrans(count2, count, i, towrite[i]);
	}
      }
      towrite = "";
      write = false;
    }
  } else if (strcmp(help,"type") == 0) {
    f.modifyHypertrans(count2, !((strcmp("0", towrite.c_str()) == 0)));
    write = false;
    towrite = "";
  } else if (strcmp(help,"goal") == 0) {
    f.modifyHypertrans(count2, posx, posy, atoi(towrite.c_str()));
    write = false;
    towrite = "";
  } else if (strcmp(help,"default") == 0) {
    f.modifyTrans(count2, !((strcmp("0", towrite.c_str()) == 0)));
    write = false;
    towrite = "";
  } else if (strcmp(help,"start") == 0) {
     char* c;
     c = new char[towrite.length()+1];
     strcpy(c,towrite.c_str());
     if (strcmp(c, "non") == 0) {
       hvalue = -1;
       towrite = "";
     } else {
       hvalue = atoi(towrite.c_str());
       write = false;
       towrite = "";
     }
  } else if (strcmp(help,"end") == 0) {
     char* c;
     c = new char[towrite.length()+1];
     strcpy(c,towrite.c_str());
     if (strcmp(c, "non") == 0) {
       int l = -1;
       f.modifyTrans(count2, hvalue, l); 
       towrite = "";
     } else {
       f.modifyTrans(count2, hvalue, atoi(towrite.c_str()));
       write = false;
       towrite = "";
     }
  } else if (strcmp(help,"hypertrans") == 0) {
    level = 0;
    hypertrans = false;
    count2++;
  } else if (strcmp(help,"link") == 0) {
    level = 0;
  } else if (strcmp(help,"parameter") == 0) {
    level = 0;
    param = false;
  } else if (strcmp(help,"globals") == 0) {
    global = false;
    level = 0;
  } else if (strcmp(help,"inputnames") == 0) {
    init = false;
    level = 0;
  } else if (strcmp(help,"outputnames") == 0) {
    level = 0;
  } else if (strcmp(help,"name") == 0) {
    if(transname) {
      f.modifyTrans(count2, towrite);
      towrite = "";
      write = false;
      transname = false;
    } else {
      if(global) {
	if(count == 0) {
	  f.clockName(towrite);
	  towrite = "";
	  write = false;
	  count++;
	} else {
	  f.resetName(towrite);
	  towrite = "";
	  write = false;
	}
      } else { 
	if (init) {
	  f.inputName(count, towrite);
	  towrite = "";
	  write = false;
	  count++;
      } else {
	f.outputName(count, towrite);
	towrite = "";
	write = false;
	count++;
      }
      }
    }
  } else if (strcmp(help,"fsm") == 0) {
    numberofinputs = 0;
    numberofoutputs = 0;
    count = 0;
    count2 = 0;
    init = true;
    level = 0;
    towrite = "";
    towrite2 = "";
    param = false;
    trans = false;
    transname = false;
    hypertrans = false;
    firstState = true;
    hvalue = 0;
    firstTrackpoint = true;
    firstHypertrans = true;
    firstLink = true;
    global = true;
    posx = 0.1; 
    posy = 0.1;
  }
}
  
 
 void ProjectHandler::endDocument() {

}

void ProjectHandler::characters(const XMLCh* const chars , const xerces_xmlsize_t_cast length) {
  char* help = XMLString::transcode(chars);
  if(write) {
    towrite.append(help);
  }
}

void ProjectHandler::fatalError(const SAXParseException& exc) {
    char* message = XMLString::transcode(exc.getMessage());
    cout << "Fatal Error: " << message
         << " at line: " << exc.getLineNumber()
         << endl;
}

void ProjectHandler::warning(const SAXParseException& exc) {
 char* message = XMLString::transcode(exc.getMessage());
    cout << "Warning: " << message
         << " at line: " << exc.getLineNumber()
         << endl;
}

void ProjectHandler::error(const SAXParseException& exc) {
 char* message = XMLString::transcode(exc.getMessage());
    cout << "Error: " << message
         << " at line: " << exc.getLineNumber()
         << endl;
}
