/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/
/*!
  \file projecthandler.h
  \brief ProjectHandler

  \author Frank Lemke
  \date 2006-03-02
*/


#ifndef PROJECTHANDLER_H_
#define PROJECTHANDLER_H_

#include <string>
#include <cstdlib>
#include <xercesc/sax2/DefaultHandler.hpp>

#include "../XercesDefinitions.hpp"

#include "../core_I.h"
#include "../fsm_I.h" 

XERCES_CPP_NAMESPACE_USE

using namespace std;

/*!
  \brief ProjectHandler
  
  Implementation of default handler for sax-parser. Needed for loading previously
  saved project with all FSMs into Core.
*/

class ProjectHandler : public DefaultHandler {
  
 private:
  int numberofinputs;
  int numberofoutputs;
  int count;
  int count2;
  bool init;
  int level;
  bool write;
  string towrite;
  string towrite2;
  bool param;
  bool trans;
  bool hypertrans;
  bool firstState;
  bool firstTrackpoint;
  bool firstHypertrans;
  bool firstLink;
  bool firstFsm;
  bool b;
  bool global;
  bool transname;
  int hvalue;
  double posx;
  double posy;
  int fsmid;

 public:
  ProjectHandler();
  void startElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname, const Attributes& att);
  void endElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname);
  void startDocument();
  void endDocument();
  void characters (const XMLCh* const chars, const xerces_xmlsize_t_cast length);
  void fatalError(const SAXParseException& exc);
  void warning(const SAXParseException& exc);
  void error(const SAXParseException& exc);
};


#endif
