/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <fstream>
#include <iostream>
#include <string>
#include <string.h>

#include "../core_I.h"
#include "../fsm_I.h"
#include "xmlsave.h" 
#include "../corefactory.h"

using namespace std;


Xmlsave::Xmlsave() {
}

void Xmlsave::createDTD(string dirname) {
  string dtdname = dirname + "fsm.dtd";
  const char*const dateiname = dtdname.c_str();
  ofstream out(dateiname, ios_base::out|ios_base::trunc);
  if(out.is_open()) {
    out << "<!ELEMENT fsm (globals, inputnames, outputnames, parameter*, state+, trans*, hypertrans*, link*)>" << endl;
    out << "<!ATTLIST fsm" << endl;
    out << "fname CDATA #REQUIRED" << endl;
    out << "resetstate CDATA #REQUIRED" << endl;
    out << "numberofinputs CDATA #REQUIRED" << endl;
    out << "numberofoutputs CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT globals (name+)>" << endl;
    out << "<!ELEMENT inputnames (name*)>" << endl;
    out << "<!ELEMENT outputnames (name*)>" << endl;
    out << "<!ELEMENT name (#PCDATA)>" << endl;
    out << "<!ELEMENT parameter (condition+)>" << endl;
    out << "<!ELEMENT condition (cname,input)>" << endl;
    out << "<!ELEMENT cname (#PCDATA)>" << endl;
    out << "<!ELEMENT input (#PCDATA)>" << endl;
    out << "<!ELEMENT state (sname,output)>" << endl;
    out << "<!ATTLIST state" << endl;
    out << "posx CDATA #REQUIRED" << endl;
    out << "posy CDATA #REQUIRED" << endl;
    out << "color CDATA #REQUIRED" << endl;
    out << "hlevel CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT sname (#PCDATA)>" << endl;
    out << "<!ELEMENT output (#PCDATA)>" << endl;
    out << "<!ELEMENT trans (name, default, start, end, condition*, trackpoint*)>" << endl;
    out << "<!ATTLIST trans" << endl; 
    out << "color CDATA #REQUIRED" << endl;  
    out << "textposx CDATA #REQUIRED" << endl;
    out << "textposy CDATA #REQUIRED" << endl;
    out << "level CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT default (#PCDATA)>" << endl;
    out << "<!ELEMENT start (#PCDATA)>" << endl;
    out << "<!ELEMENT end (#PCDATA)>" << endl;
    out << "<!ELEMENT trackpoint EMPTY>" << endl;
    out << "<!ATTLIST trackpoint" << endl;  
    out << "posx CDATA #REQUIRED" << endl;
    out << "posy CDATA #REQUIRED" << endl;
    out << "hlevel CDATA #REQUIRED" << endl;
    out << "hldirection CDATA #REQUIRED" << endl;
    out << "link CDATA #REQUIRED" << endl;
    out << "linkid CDATA #REQUIRED" << endl;
    out << "join CDATA #REQUIRED" << endl;
    out << "color CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT hypertrans (type, goal, condition*)>" << endl;
    out << "<!ATTLIST hypertrans" << endl;
    out << "posx CDATA #REQUIRED" << endl;
    out << "posy CDATA #REQUIRED" << endl;
    out << "color CDATA #REQUIRED" << endl;
    out << "hlevel CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT type (#PCDATA)>" << endl;
    out << "<!ELEMENT goal (#PCDATA)>" << endl;
    out << "<!ELEMENT link EMPTY>" << endl;
    out << "<!ATTLIST link" << endl;
    out << "posx CDATA #REQUIRED" << endl;
    out << "posy CDATA #REQUIRED" << endl;
    out << "color CDATA #REQUIRED" << endl;
    out << "hlevel CDATA #REQUIRED" << endl;
    out << "goal CDATA #REQUIRED>" << endl;
    out.close();
  }
}

void Xmlsave::createProjectDTD(string dirname) {
  string dtdname = dirname + "project.dtd";
  const char*const dateiname = dtdname.c_str();
  ofstream out(dateiname, ios_base::out|ios_base::trunc);
  if(out.is_open()) {
    out << "<!ELEMENT project (fsm*)>" << endl; 
    out << "<!ATTLIST project" <<endl;
    out << "pname CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT fsm (globals, inputnames, outputnames, parameter*, state+, trans*, hypertrans*, link*)>" << endl;
    out << "<!ATTLIST fsm" << endl;
    out << "fname CDATA #REQUIRED" << endl;
    out << "resetstate CDATA #REQUIRED" << endl;
    out << "numberofinputs CDATA #REQUIRED" << endl;
    out << "numberofoutputs CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT globals (name+)>" << endl;
    out << "<!ELEMENT inputnames (name*)>" << endl;
    out << "<!ELEMENT outputnames (name*)>" << endl;
    out << "<!ELEMENT name (#PCDATA)>" << endl;
    out << "<!ELEMENT parameter (condition+)>" << endl;
    out << "<!ELEMENT condition (cname,input)>" << endl;
    out << "<!ELEMENT cname (#PCDATA)>" << endl;
    out << "<!ELEMENT input (#PCDATA)>" << endl;
    out << "<!ELEMENT state (sname,output)>" << endl;
    out << "<!ATTLIST state" << endl;
    out << "posx CDATA #REQUIRED" << endl;
    out << "posy CDATA #REQUIRED" << endl;
    out << "color CDATA #REQUIRED" << endl;
    out << "hlevel CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT sname (#PCDATA)>" << endl;
    out << "<!ELEMENT output (#PCDATA)>" << endl;
    out << "<!ELEMENT trans (name, default, start, end, condition*, trackpoint*)>" << endl;
    out << "<!ATTLIST trans" << endl;
    out << "color CDATA #REQUIRED" << endl;
    out << "textposx CDATA #REQUIRED" << endl;
    out << "textposy CDATA #REQUIRED" << endl;
    out << "level CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT default (#PCDATA)>" << endl;
    out << "<!ELEMENT start (#PCDATA)>" << endl;
    out << "<!ELEMENT end (#PCDATA)>" << endl;
    out << "<!ELEMENT trackpoint EMPTY>" << endl;
    out << "<!ATTLIST trackpoint" << endl;
    out << "posx CDATA #REQUIRED" << endl;
    out << "posy CDATA #REQUIRED" << endl;
    out << "hlevel CDATA #REQUIRED" << endl;
    out << "hldirection CDATA #REQUIRED" << endl;
    out << "link CDATA #REQUIRED" << endl;
    out << "linkid CDATA #REQUIRED" << endl;
    out << "join CDATA #REQUIRED" << endl;
    out << "color CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT hypertrans (type, goal, condition*)>" << endl;
    out << "<!ATTLIST hypertrans" << endl;
    out << "posx CDATA #REQUIRED" << endl;
    out << "posy CDATA #REQUIRED" << endl;
    out << "color CDATA #REQUIRED" << endl;
    out << "hlevel CDATA #REQUIRED>" << endl;
    out << "<!ELEMENT type (#PCDATA)>" << endl;
    out << "<!ELEMENT goal (#PCDATA)>" << endl;
    out << "<!ELEMENT link EMPTY>" << endl;
    out << "<!ATTLIST link" << endl;
    out << "posx CDATA #REQUIRED" << endl;
    out << "posy CDATA #REQUIRED" << endl;
    out << "color CDATA #REQUIRED" << endl;
    out << "hlevel CDATA #REQUIRED" << endl;
    out << "goal CDATA #REQUIRED>" << endl;
    out.close();
  }

}

void Xmlsave::save( string filename, string fsmname) {
  string dirname = filename;
  int len = dirname.length();
  if (len > 0) {
    string s;
    s = dirname;
    do {
      len--;
    } while ( (s[len] != '/') && (len != 0));
    if (len == 0) {
      dirname = "";
    } else {
      dirname.assign(s, 0, (len + 1));
    }
  }
  const char* const dateiname = filename.c_str();
  createDTD(dirname);
  ofstream out(dateiname, ios_base::out|ios_base::trunc);
  if(out.is_open()) {
    Core_I& c = CoreFactory::getCore_I();
    if(c.getFirstFsm()) {
      while(!(strcmp(c.getCurrentFsm().getFsmName().c_str(), fsmname.c_str()) == 0)) {
	if(!(c.getNextFsm())) {
	  cout << "This FSM does not exist!" << endl;
	  out.close();
	  return;
	}
      }
      if(strcmp(c.getCurrentFsm().getFsmName().c_str(), fsmname.c_str()) == 0) {
	Fsm_I& f = c.getCurrentFsm();
	f.optimize();
	out << "<?xml version='1.0' encoding='ISO-8859-1' ?>" << endl; 
	out << "<!DOCTYPE fsm SYSTEM 'fsm.dtd'>" << endl; 
	out << "<fsm fname='" << f.getFsmName() << "' resetstate='" << f.getResetState();
	out << "' numberofinputs='" << f.getNumberOfInputs() << "' numberofoutputs='";
	out << f.getNumberOfOutputs() << "' >"  << endl;
	out << "    <globals>" <<endl;
	out << "        <name>" << f.getClockName() << "</name>" << endl;
	out << "        <name>" << f.getResetName() << "</name>" << endl;
	out << "    </globals>" <<endl;
	out << "    <inputnames>" <<endl;
	for (int i = 0; i<f.getNumberOfInputs(); i++) {
	  out << "        <name>" << f.getInputName(i) << "</name>" << endl;
	}
	out << "    </inputnames>" <<endl;
	out << "    <outputnames>" <<endl;
	for (int i = 0; i<f.getNumberOfOutputs(); i++) {
	  out << "        <name>" << f.getOutputName(i) << "</name>" << endl;
	}
	out << "    </outputnames>" <<endl;

	if(f.getFirstParameter()) {
	   out << "    <parameter>" <<endl;
	  do {
	    out << "        <condition>" <<endl;
	    out << "            <cname>" << f.getParameterName() << "</cname>" <<endl;
	    out << "            <input>" << f.getParameterValue() << "</input>" <<endl;
	    out << "        </condition>" <<endl;
	  } while(f.getNextParameter());
	    out << "    </parameter>" <<endl;
	}

	if(f.getFirstState()) {
	  do {
	    out << "    <state posx='" << f.getStatePosx() << "' posy='" << f.getStatePosy() << "' color='";
	    out << f.getStateColor() << "' hlevel='" << f.getStateLevel() << "' >" << endl;
	    out << "        <sname>" << f.getStateName() << "</sname>" << endl;
	    out << "        <output>" << f.getStateOutput() << "</output>" << endl;
	    out << "    </state>" << endl;
	  } while(f.getNextState());
	}

	if(f.getFirstTrans()) {
	  do {
	    out << "    <trans color='" << f.getTransColor() << "' textposx='" << f.getTransTextPosx() << "' textposy='" << f.getTransTextPosy() << "' level='" << f.getTransLevel() << "' >" <<endl;
	    out << "        <name>" << f.getTransName() << "</name>" << endl;
	    out << "        <default>" << f.getTransDefault() << "</default>" <<endl;
	    if(f.getTransStart() < 0) {
	      out << "        <start>" << "non" << "</start>" <<endl;
	    } else {
	      out << "        <start>" << f.getTransStart() << "</start>" <<endl;
	    }
	    if(f.getTransEnd() < 0) { 
	      out << "        <end>" << "non" << "</end>" <<endl;
	    } else {
	      out << "        <end>" << f.getTransEnd() << "</end>" <<endl;
	    }
	    if(f.getTransFirstCondition()){
	      do {
		out << "        <condition>" <<endl;
		out << "            <cname>" << f.getTransConditionName() << "</cname>" <<endl;
		out << "            <input>" << f.getTransConditionValue() << "</input>" <<endl;
		out << "        </condition>" <<endl;
	      } while(f.getTransNextCondition());
	    }
	    if(f.getTransFirstTrackpoint()){
	      do {
		out << "        <trackpoint posx='" << f.getTransTrackpointPosx() <<"' posy='" << f.getTransTrackpointPosy();
		out << "' hldirection='" << f.getTransTrackpointHldirection() << "' hlevel='" << f.getTransTrackpointLevel();
		out << "' link='" << f.getTransTrackpointLink() << "' linkid='" << f.getTransTrackpointLinkID();
		out << "' join='" << f.getTransTrackpointJoin() << "' color='" << f.getTransTrackpointColor();
		out << "' />" <<endl;
	      } while(f.getTransNextTrackpoint());
	    }
	    out << "    </trans>" <<endl;
	  } while(f.getNextTrans());
	}
	if(f.getFirstHypertrans()) {
	  do {
	  out << "    <hypertrans posx='" << f.getHypertransPosx() << "' posy='" << f.getHypertransPosy() << "' color='" ;
	  out << f.getHypertransColor() << "' hlevel='" << f.getHypertransLevel() << "' >" <<endl;
	  out << "        <type>" << f.getHypertransType() << "</type>" <<endl;
	  out << "        <goal>" << f.getHypertransGoal() << "</goal>" <<endl;
	  if(f.getHypertransFirstCondition()) {
	    do {
	      out << "        <condition>" <<endl;
	      out << "            <cname>" << f.getHypertransConditionName() << "</cname>" <<endl;
	      out << "            <input>" << f.getHypertransConditionValue() << "</input>" <<endl;
	      out << "        </condition>" <<endl;
	    } while(f.getHypertransNextCondition());
	  }
	  out << "    </hypertrans>" <<endl;
	  } while(f.getNextHypertrans());
	}
	if(f.getFirstLink()) {
	  do {
	    out << "    <link posx='" << f.getLinkPosx() << "' posy='" << f.getLinkPosy() << "' color='";
	    out << f.getLinkColor() << "' hlevel='" << f.getLinkLevel() << "' goal='" << f.getLinkGoal();
	    out << "' />" << endl;
	  } while(f.getNextLink());
	}
	out << "</fsm>" << endl;
      }
    }
   out.close();
  }
} 
 
void Xmlsave::saveProject(string filename) {
  string dirname = filename;
  int len = dirname.length();
  if(len > 0) { 
    string s;
    s = dirname;
    do {
      len--;
    } while ( (s[len] != '/') && (len != 0));
    if (len == 0) { 
      dirname = "";
    } else {
      dirname.assign(s, 0, (len + 1));
    }
  }
  const char* const dateiname = filename.c_str();
  createProjectDTD(dirname);
  ofstream out(dateiname, ios_base::out|ios_base::trunc);
  if(out.is_open()) {
    Core_I& c = CoreFactory::getCore_I();
    if(c.getFirstFsm()) {
      out << "<?xml version='1.0' encoding='ISO-8859-1' ?>" << endl; 
      out << "<!DOCTYPE project SYSTEM 'project.dtd'>" << endl; 
      out << "<project pname='" << c.getProjectName() << "'>" <<endl;
      do {
	Fsm_I& f = c.getCurrentFsm();	
	f.optimize();
	out << "  <fsm fname='" << f.getFsmName() << "' resetstate='" << f.getResetState();
	out << "' numberofinputs='" << f.getNumberOfInputs() << "' numberofoutputs='";
	out << f.getNumberOfOutputs() << "' >"  << endl;
	out << "    <globals>" <<endl;
	out << "        <name>" << f.getClockName() << "</name>" << endl;
	out << "        <name>" << f.getResetName() << "</name>" << endl;
	out << "    </globals>" <<endl;
	out << "    <inputnames>" <<endl;
	for (int i = 0; i<f.getNumberOfInputs(); i++) {
	  out << "        <name>" << f.getInputName(i) << "</name>" << endl;
	}
	out << "    </inputnames>" <<endl;
	out << "    <outputnames>" <<endl;
	for (int i = 0; i<f.getNumberOfOutputs(); i++) {
	  out << "        <name>" << f.getOutputName(i) << "</name>" << endl;
	}
	out << "    </outputnames>" <<endl;
	
	if(f.getFirstParameter()) {
	  out << "    <parameter>" <<endl;
	  do {
	    out << "        <condition>" <<endl;
	    out << "            <cname>" << f.getParameterName() << "</cname>" <<endl;
	    out << "            <input>" << f.getParameterValue() << "</input>" <<endl;
	    out << "        </condition>" <<endl;
	  } while(f.getNextParameter());
	  out << "    </parameter>" <<endl;
	}
	
	if(f.getFirstState()) {
	  do {
	    out << "    <state posx='" << f.getStatePosx() << "' posy='" << f.getStatePosy() << "' color='";
	    out << f.getStateColor() << "' hlevel='" << f.getStateLevel() << "' >" << endl;
	    out << "        <sname>" << f.getStateName() << "</sname>" << endl;
	    out << "        <output>" << f.getStateOutput() << "</output>" << endl;
	    out << "    </state>" << endl;
	  } while(f.getNextState());
	}
	
	if(f.getFirstTrans()) {
	  do {
	    out << "    <trans color='" << f.getTransColor() << "' textposx='" << f.getTransTextPosx() << "' textposy='" << f.getTransTextPosy() << "' level='" << f.getTransLevel() << "' >" <<endl;
	    out << "        <name>" << f.getTransName() << "</name>" << endl;
	    out << "        <default>" << f.getTransDefault() << "</default>" <<endl;
	    if(f.getTransStart() < 0) {
	      out << "        <start>" << "non" << "</start>" <<endl;
	    } else {
	    out << "        <start>" << f.getTransStart() << "</start>" <<endl;
	    }
	    if(f.getTransEnd() < 0) { 
	      out << "        <end>" << "non" << "</end>" <<endl;
	    } else {
	      out << "        <end>" << f.getTransEnd() << "</end>" <<endl;
	    }
	    if(f.getTransFirstCondition()){
	      do {
		out << "        <condition>" <<endl;
		out << "            <cname>" << f.getTransConditionName() << "</cname>" <<endl;
		out << "            <input>" << f.getTransConditionValue() << "</input>" <<endl;
		out << "        </condition>" <<endl;
	      } while(f.getTransNextCondition());
	    }
	    if(f.getTransFirstTrackpoint()){
	      do {
		out << "        <trackpoint posx='" << f.getTransTrackpointPosx() <<"' posy='" << f.getTransTrackpointPosy();
		out << "' hldirection='" << f.getTransTrackpointHldirection() << "' hlevel='" << f.getTransTrackpointLevel();
		out << "' link='" << f.getTransTrackpointLink() << "' linkid='" << f.getTransTrackpointLinkID();
		out << "' join='" << f.getTransTrackpointJoin() << "' color='" << f.getTransTrackpointColor();
		out << "' />" <<endl;
	      } while(f.getTransNextTrackpoint());
	    }
	    out << "    </trans>" <<endl;
	  } while(f.getNextTrans());
	}
	if(f.getFirstHypertrans()) {
	  do {
	    out << "    <hypertrans posx='" << f.getHypertransPosx() << "' posy='" << f.getHypertransPosy() << "' color='" ;
	    out << f.getHypertransColor() << "' hlevel='" << f.getHypertransLevel() << "' >" <<endl;
	    out << "        <type>" << f.getHypertransType() << "</type>" <<endl;
	    out << "        <goal>" << f.getHypertransGoal() << "</goal>" <<endl;
	    if(f.getHypertransFirstCondition()) {
	      do {
		out << "        <condition>" <<endl;
		out << "            <cname>" << f.getHypertransConditionName() << "</cname>" <<endl;
		out << "            <input>" << f.getHypertransConditionValue() << "</input>" <<endl;
		out << "        </condition>" <<endl;
	      } while(f.getHypertransNextCondition());
	    }
	    out << "    </hypertrans>" <<endl;
	  } while(f.getNextHypertrans());
	}
	if(f.getFirstLink()) {
	  do {
	    out << "    <link posx='" << f.getLinkPosx() << "' posy='" << f.getLinkPosy() << "' color='";
	    out << f.getLinkColor() << "' hlevel='" << f.getLinkLevel() << "' goal='" << f.getLinkGoal();
	    out << "' />" << endl;
	  } while(f.getNextLink());
	}
	out << "  </fsm>" << endl; ;
      } while (c.getNextFsm());
      out << "</project>" << endl;
    } 
    out.close();
  }
}
