/***********************************************************************
*                                                                      *
* (C) 2006, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/
 
/*!
  \file xmlsave.h
  \brief Xmlsave
  
  \author Frank Lemke
  \date 2006-02-28
*/

#ifndef XMLSAVE_H_
#define XMLSAVE_H_

#include <string>
#include <cstdlib>

#include "../save_I.h"

using namespace std;

/*!
  \brief Xmlsave

  Class to allocate functions for saving FSMs in xml-files and creating
  an external dtd to discribe the xml-file structure.

*/

class Xmlsave : public Save_I{  

 public:

  Xmlsave();
  ~Xmlsave(){};

  /*!
    \brief createdtd

    A function to create a Document Type Definition(DTD) file to
    describe the saved FSMs.
  */

  void createDTD(string dirname);

  /*!
    \brief createprojectdtd

    A function to create a Document Type Definition(DTD) for
    a project containing many FSMs.
  */

  void createProjectDTD(string dirname);
  
  /*!
    \brief save 

    A function to save finite state maschines in XML-files with a
    separated dtd-file.
    \param filename    Filename with relative path to save FSM without extension.
    \param fsmid       ID of a FSM to know which FSM to save.
  */

  void save(string filename, string fsmname);

/*!
    \brief saveProject 

    A function to save all finite state maschines of a project
    in XML-files with a separated dtd-file.
    \param filename    Filename with relative path to save project without extension.
  */

  void saveProject(string filename);
};

#endif
